@@ -16,19 +16,25 @@ else
 	AUTOMAKE=automake-${AM_VERSION}
 fi
 
-if test -f /opt/local/bin/glibtoolize ; then
-        # darwin
-        LIBTOOLIZE=/opt/local/bin/glibtoolize
-else
-        LIBTOOLIZE=libtoolize
-fi
+LIBTOOLIZE="libtoolize"
+for lt in glibtoolize libtoolize15 libtoolize14 libtoolize13 ; do
+        if test -x /usr/bin/$lt ; then
+                LIBTOOLIZE=$lt ; break
+        fi
+        if test -x /usr/local/bin/$lt ; then
+                LIBTOOLIZE=$lt ; break
+        fi
+        if test -x /opt/local/bin/$lt ; then
+                LIBTOOLIZE=$lt ; break
+        fi
+done
 
 if test -d /opt/local/share/aclocal ; then
-        ACLOCAL_ARGS="-I /opt/local/share/aclocal"
+		ACLOCAL_ARGS="-I /opt/local/share/aclocal"
 fi
 
 if test -d /share/aclocal ; then
-        ACLOCAL_ARGS="$ACLOCAL_ARGS -I /share/aclocal"
+		ACLOCAL_ARGS="$ACLOCAL_ARGS -I /share/aclocal"
 fi
 
 INTLTOOLIZE=$(which intltoolize)
@@ -49,6 +55,13 @@ $AUTOMAKE --force-missing --add-missing --copy
 autoconf
 
 set +x
+
+#install git pre-commit hooks if possible
+if [ -d .git/hooks ] && [ ! -f .git/hooks/pre-commit ]; then
+		cp .git-pre-commit .git/hooks/pre-commit
+		chmod +x .git/hooks/pre-commit
+fi
+
 if [ "$srcdir" = "." ]; then
 	if [ -x oRTP/autogen.sh ]; then
 		echo "Generating build scripts in oRTP..."
