@@ -19,7 +19,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
  ****************************************************************************/
 
@@ -31,9 +31,9 @@
 #endif /*_WIN32_WCE*/
 #include <limits.h>
 #include <ctype.h>
-#include <linphonecore.h>
+#include <linphone/core.h>
 #include "linphonec.h"
-#include "lpconfig.h"
+#include <linphone/lpconfig.h>
 
 #ifndef _WIN32
 #include <sys/wait.h>
@@ -133,8 +133,8 @@ static LPC_COMMAND *lpc_find_command(const char *name);
 
 void linphonec_out(const char *fmt,...);
 
-VideoParams lpc_video_params={-1,-1,-1,-1,0,TRUE};
-VideoParams lpc_preview_params={-1,-1,-1,-1,0,TRUE};
+VideoParams lpc_video_params={-1,-1,-1,-1,NULL,TRUE,FALSE};
+VideoParams lpc_preview_params={-1,-1,-1,-1,NULL,TRUE,FALSE};
 
 /***************************************************************************
  *
@@ -212,7 +212,8 @@ static LPC_COMMAND commands[] = {
 		"'ipv6 disable' : do not use ipv6 network."
 	},
 	{ "mute", lpc_cmd_mute_mic,
-	  "Mute microphone and suspend voice transmission."
+		"Mute microphone and suspend voice transmission.",
+		NULL
 	},
 	{ "nat", lpc_cmd_nat, "Set nat address",
 		"'nat'        : show nat settings.\n"
@@ -228,7 +229,8 @@ static LPC_COMMAND commands[] = {
 		"'play <wav file>'    : play a wav file."
 	},
 	{ "playbackgain", lpc_cmd_playback_gain,
-		"Adjust playback gain."
+		"Adjust playback gain.",
+		NULL
 	},
 	{ "proxy", lpc_cmd_proxy, "Manage proxies",
 		"'proxy list' : list all proxy setups.\n"
@@ -269,7 +271,8 @@ static LPC_COMMAND commands[] = {
 		"'transfer <call id1> --to-call <call id2>': transfers the call with 'id1' to the destination of call 'id2' (attended transfer)\n"
 	},
 	{ "unmute", lpc_cmd_unmute_mic,
-		"Unmute microphone and resume voice transmission."
+		"Unmute microphone and resume voice transmission.",
+		NULL
 	},
 	{ "webcam", lpc_cmd_webcam, "Manage webcams",
 		"'webcam list' : list all known devices.\n"
@@ -327,8 +330,9 @@ static LPC_COMMAND advanced_commands[] = {
 	{ "preview-snapshot", lpc_cmd_preview_snapshot, "Take a snapshot of currently captured video stream",
 		"'preview-snapshot <file path>': take a snapshot and records it in jpeg format into the supplied path\n"
 	},
-	{ "vfureq", lpc_cmd_vfureq, "Request the other side to send VFU for the current call"
-},
+	{ "vfureq", lpc_cmd_vfureq, "Request the other side to send VFU for the current call",
+		NULL
+	},
 #endif
 	{ "states", lpc_cmd_states, "Show internal states of liblinphone, registrations and calls, according to linphonecore.h definitions",
 		"'states global': shows global state of liblinphone \n"
@@ -459,7 +463,7 @@ linphonec_command_generator(const char *text, int state)
 	{
 		index=0;
 		adv=0;
-		len=strlen(text);
+		len=(int)strlen(text);
 	}
 	/*
  	 * Return the next name which partially matches
@@ -593,7 +597,7 @@ lpc_cmd_call(LinphoneCore *lc, char *args)
 		{
 			snprintf(callee_name,sizeof(callee_name),"%s",args);
 		}
-		linphone_call_params_destroy(cp);
+		linphone_call_params_unref(cp);
 	}
 	return 1;
 }
@@ -673,13 +677,13 @@ lpc_cmd_transfer(LinphoneCore *lc, char *args)
 				linphonec_out("No active call, please specify a call id among the ones listed by 'calls' command.\n");
 				return 0;
 			}
-			linphone_core_transfer_call(lc, call, refer_to);
+			linphone_call_transfer(call, refer_to);
 		}else if (n==2){
 			long id=atoi(arg1);
 			refer_to=args+strlen(arg1)+1;
 			call=linphonec_get_call(id);
 			if (call==NULL) return 0;
-			linphone_core_transfer_call(lc, call, refer_to);
+			linphone_call_transfer(call, refer_to);
 		}else if (n==3){
 			long id=atoi(arg1);
 			call=linphonec_get_call(id);
@@ -689,7 +693,7 @@ lpc_cmd_transfer(LinphoneCore *lc, char *args)
 				return 0;
 			}
 			linphonec_out("Performing attended transfer of call %i to call %i",id,id2);
-			linphone_core_transfer_call_to_another (lc,call,call2);
+			linphone_call_transfer_to_another(call,call2);
 		}else return 0;
 	}else{
 		linphonec_out("Transfer command requires at least one argument\n");
@@ -722,7 +726,7 @@ lpc_cmd_terminate(LinphoneCore *lc, char *args)
 		long id=atoi(args);
 		LinphoneCall *call=linphonec_get_call(id);
 		if (call){
-			if (linphone_core_terminate_call(lc,call)==-1){
+			if (linphone_call_terminate(call)==-1){
 				linphonec_out("Could not stop the call with id %li\n",id);
 			}
 		}else return 0;
@@ -745,7 +749,7 @@ lpc_cmd_redirect(LinphoneCore *lc, char *args){
 		while(elem!=NULL){
 			LinphoneCall *call=(LinphoneCall*)elem->data;
 			if (linphone_call_get_state(call)==LinphoneCallIncomingReceived){
-				if (linphone_core_redirect_call(lc,call,args+4) != 0) {
+				if (linphone_call_redirect(call,args+4) != 0) {
 					linphonec_out("Could not redirect call.\n");
 					elem=elem->next;
 				} else {
@@ -767,7 +771,7 @@ lpc_cmd_redirect(LinphoneCore *lc, char *args){
 			if ( call != NULL ) {
 				if (linphone_call_get_state(call)!=LinphoneCallIncomingReceived) {
 					linphonec_out("The state of the call is not incoming, can't be redirected.\n");
-				} else if (linphone_core_redirect_call(lc,call,args+charRead) != 0) {
+				} else if (linphone_call_redirect(call,args+charRead) != 0) {
 					linphonec_out("Could not redirect call.\n");
 				}
 			}
@@ -780,7 +784,7 @@ static int
 lpc_cmd_answer(LinphoneCore *lc, char *args){
 	if (!args)
 	{
-		int nb=bctbx_list_size(linphone_core_get_calls(lc));
+		int nb=(int)bctbx_list_size(linphone_core_get_calls(lc));
 		if (nb==1){
 			//if just one call is present answer the only one in passing NULL to the linphone_core_accept_call ...
 			if ( -1 == linphone_core_accept_call(lc, NULL) )
@@ -798,7 +802,7 @@ lpc_cmd_answer(LinphoneCore *lc, char *args){
 		long id;
 		if (sscanf(args,"%li",&id)==1){
 			LinphoneCall *call=linphonec_get_call (id);
-			if (linphone_core_accept_call (lc,call)==-1){
+			if (linphone_call_accept(call)==-1){
 				linphonec_out("Fail to accept call %i\n",id);
 			}
 		}else return 0;
@@ -1450,7 +1454,7 @@ static int lpc_cmd_pause(LinphoneCore *lc, char *args){
 
 	if(linphone_core_in_call(lc))
 	{
-		linphone_core_pause_call(lc,linphone_core_get_current_call(lc));
+		linphone_call_pause(linphone_core_get_current_call(lc));
 		return 1;
 	}
 	linphonec_out("you can only pause when a call is in process\n");
@@ -1471,7 +1475,7 @@ static int lpc_cmd_resume(LinphoneCore *lc, char *args){
 		if (n == 1){
 			LinphoneCall *call=linphonec_get_call (id);
 			if (call){
-				if(linphone_core_resume_call(lc,call)==-1){
+				if(linphone_call_resume(call)==-1){
 					linphonec_out("There was a problem to resume the call check the remote address you gave %s\n",args);
 				}
 			}
@@ -1481,10 +1485,10 @@ static int lpc_cmd_resume(LinphoneCore *lc, char *args){
 	else
 	{
 		const bctbx_list_t *calls = linphone_core_get_calls(lc);
-		int nbcalls=bctbx_list_size(calls);
+		int nbcalls=(int)bctbx_list_size(calls);
 		if( nbcalls == 1)
 		{
-			if(linphone_core_resume_call(lc,calls->data) < 0)
+			if(linphone_call_resume(calls->data) < 0)
 			{
 				linphonec_out("There was a problem to resume the unique call.\n");
 			}
@@ -1829,13 +1833,13 @@ linphonec_proxy_use(LinphoneCore *lc, int index)
 static void
 linphonec_friend_display(LinphoneFriend *fr)
 {
-	LinphoneAddress *uri=linphone_address_clone(linphone_friend_get_address(fr));
-	char *str;
+	const LinphoneAddress *addr = linphone_friend_get_address(fr);
+	char *str = NULL;
 
-	linphonec_out("name: %s\n", linphone_address_get_display_name(uri));
-	linphone_address_set_display_name(uri,NULL);
-	str=linphone_address_as_string(uri);
+	linphonec_out("name: %s\n", linphone_friend_get_name(fr));
+	if (addr) str = linphone_address_as_string_uri_only(addr);
 	linphonec_out("address: %s\n", str);
+	if (str) ms_free(str);
 }
 
 static int
@@ -1853,9 +1857,8 @@ linphonec_friend_list(LinphoneCore *lc, char *pat)
 	for(n=0; friend!=NULL; friend=bctbx_list_next(friend), ++n )
 	{
 		if ( pat ) {
-			const char *name = linphone_address_get_display_name(
-			    linphone_friend_get_address((LinphoneFriend*)friend->data));
-			if (name && ! strstr(name, pat) ) continue;
+			const char *name = linphone_friend_get_name((LinphoneFriend *)friend->data);
+			if (name && !strstr(name, pat)) continue;
 		}
 		linphonec_out("****** Friend %i *******\n",n);
 		linphonec_friend_display((LinphoneFriend*)friend->data);
@@ -1869,17 +1872,22 @@ linphonec_friend_call(LinphoneCore *lc, unsigned int num)
 {
 	const bctbx_list_t *friend = linphone_core_get_friend_list(lc);
 	unsigned int n;
-	char *addr;
+	char *addr_str;
 
 	for(n=0; friend!=NULL; friend=bctbx_list_next(friend), ++n )
 	{
 		if ( n == num )
 		{
 			int ret;
-			addr = linphone_address_as_string(linphone_friend_get_address((LinphoneFriend*)friend->data));
-			ret=lpc_cmd_call(lc, addr);
-			ms_free(addr);
-			return ret;
+			const LinphoneAddress *addr = linphone_friend_get_address((LinphoneFriend*)friend->data);
+			if (addr) {
+				addr_str = linphone_address_as_string(addr);
+				ret=lpc_cmd_call(lc, addr_str);
+				ms_free(addr_str);
+				return ret;
+			} else {
+				linphonec_out("Friend %u does not have an address\n", num);
+			}
 		}
 	}
 	linphonec_out("No such friend %u\n", num);
@@ -1905,7 +1913,7 @@ static int
 linphonec_friend_delete(LinphoneCore *lc, int num)
 {
 	const bctbx_list_t *friend = linphone_core_get_friend_list(lc);
-	unsigned int n;
+	int n;
 
 	for(n=0; friend!=NULL; friend=bctbx_list_next(friend), ++n )
 	{
@@ -1918,13 +1926,13 @@ linphonec_friend_delete(LinphoneCore *lc, int num)
 
 	if (-1 == num)
 	{
-		unsigned int i;
+		int i;
 		for (i = 0 ; i < n ; i++)
 			linphonec_friend_delete(lc, 0);
 		return 0;
 	}
 
-	linphonec_out("No such friend %u\n", num);
+	linphonec_out("No such friend %i\n", num);
 	return 1;
 }
 
@@ -1973,8 +1981,8 @@ static int lpc_cmd_register(LinphoneCore *lc, char *args){
 		if ((from=linphone_address_new(identity))!=NULL){
 			info=linphone_auth_info_new(linphone_address_get_username(from),NULL,passwd,NULL,NULL,linphone_address_get_username(from));
 			linphone_core_add_auth_info(lc,info);
-			linphone_address_destroy(from);
-			linphone_auth_info_destroy(info);
+			linphone_address_unref(from);
+			linphone_auth_info_unref(info);
 		}
 	}
 	elem=linphone_core_get_proxy_config_list(lc);
@@ -2062,7 +2070,7 @@ static int lpc_cmd_status(LinphoneCore *lc, char *args)
 				linphonec_out("hook=paused sip:%s\n",linphonec_get_callee());
 			break;
 			case LinphoneCallIdle:
-				linphonec_out("hook=offhook\n");
+				linphonec_out("hook=on-hook\n");
 			break;
 			case LinphoneCallStreamsRunning:
 			case LinphoneCallConnected:
@@ -2483,7 +2491,7 @@ static void lpc_display_call_states(LinphoneCore *lc){
 			tmp=linphone_call_get_remote_address_as_string (call);
 			flag=in_conference ? "conferencing" : "";
 			flag=linphone_call_has_transfer_pending(call) ? "transfer pending" : flag;
-			linphonec_out("%-2i | %-35s | %-15s | %s\n",(int)(long)linphone_call_get_user_pointer(call),
+			linphonec_out("%-2i | %-35s | %-15s | %s\n",VOIDPTR_TO_INT(linphone_call_get_user_pointer(call)),
 						  tmp,linphone_call_state_to_string(linphone_call_get_state(call))+strlen("LinphoneCall"),flag);
 			ms_free(tmp);
 		}
@@ -2565,8 +2573,8 @@ static int lpc_cmd_camera(LinphoneCore *lc, char *args){
 					/*update the call to add the video stream*/
 					LinphoneCallParams *ncp=linphone_call_params_copy(cp);
 					linphone_call_params_enable_video(ncp,TRUE);
-					linphone_core_update_call(lc,call,ncp);
-					linphone_call_params_destroy (ncp);
+					linphone_call_update(call,ncp);
+					linphone_call_params_unref (ncp);
 					linphonec_out("Trying to bring up video stream...\n");
 				}
 			}
