@@ -66,7 +66,7 @@ void sipomatic_process_event(Sipomatic *obj,eXosip_event_t *ev)
 }
 
 
-void endoffile_cb(void *ud, unsigned int ev,void * arg){
+void endoffile_cb(void *ud, MSFilter *f, unsigned int ev,void * arg){
 	Call*call=(Call*)ud;
 	call->eof=1;
 }
@@ -76,7 +76,7 @@ void call_accept(Call *call)
 	sdp_context_t *ctx;
 	PayloadType *payload;
 	char *hellofile;
-	static int call_count=0;	
+	static int call_count=0;
 	char record_file[250];
 	osip_message_t *msg=NULL;
 	sprintf(record_file,"/tmp/sipomatic%i.wav",call_count);
@@ -97,7 +97,7 @@ void call_accept(Call *call)
 	osip_message_set_content_type(msg,"application/sdp");
 	osip_message_set_body(msg,call->sdpc->answerstr,strlen(call->sdpc->answerstr));
 	eXosip_call_send_answer(call->tid,200,msg);
- 	call->audio_stream=audio_stream_new(call->audio.localport,call->root->ipv6);
+	call->audio_stream=audio_stream_new(call->audio.localport,call->audio.localport+1,call->root->ipv6);
 	audio_stream_start_with_files(call->audio_stream, call->profile,
 				call->audio.remaddr,call->audio.remoteport,call->audio.remoteport+1,
 				 call->audio.pt,20,hellofile,record_file);
@@ -105,7 +105,7 @@ void call_accept(Call *call)
 #ifdef VIDEO_ENABLED
 	if (call->video.remoteport!=0){
 		video_stream_send_only_start(call->video_stream,call->profile,
-			call->video.remaddr,call->video.remoteport,call->video.remoteport+1,call->video.pt, 60, 
+			call->video.remaddr,call->video.remoteport,call->video.remoteport+1,call->video.pt, 60,
 			ms_web_cam_manager_get_default_cam(ms_web_cam_manager_get()));
 	}
 #endif
@@ -124,7 +124,7 @@ PayloadType * sipomatic_payload_is_supported(sdp_payload_t *payload,RtpProfile *
 		localpt=payload->pt;
 		ms_warning("payload has no rtpmap.");
 	}
-	
+
 	if (localpt>=0){
 		/* this payload is supported in our local rtp profile, so add it to the dialog rtp
 		profile */
@@ -160,7 +160,7 @@ int sipomatic_accept_audio_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 	Call *call=(Call*)sdp_context_get_user_pointer(ctx);
 	PayloadType *supported;
 	struct stream_params *params=&call->audio;
-	
+
 	/* see if this codec is supported in our local rtp profile*/
 	supported=sipomatic_payload_is_supported(payload,&av_profile,call->profile);
 	if (supported==NULL) {
@@ -191,7 +191,7 @@ int sipomatic_accept_video_offer(sdp_context_t *ctx,sdp_payload_t *payload)
 	Call *call=(Call*)sdp_context_get_user_pointer(ctx);
 	PayloadType *supported;
 	struct stream_params *params=&call->video;
-	
+
 	/* see if this codec is supported in our local rtp profile*/
 	supported=sipomatic_payload_is_supported(payload,&av_profile,call->profile);
 	if (supported==NULL) {
@@ -221,9 +221,9 @@ void sipomatic_init(Sipomatic *obj, char *url, bool_t ipv6)
 {
 	osip_uri_t *uri=NULL;
 	int port=5064;
-	
+
 	obj->ipv6=ipv6;
-	
+
 	if (url==NULL){
 		url=getenv("SIPOMATIC_URL");
 		if (url==NULL){
@@ -237,7 +237,7 @@ void sipomatic_init(Sipomatic *obj, char *url, bool_t ipv6)
 			if (uri->port!=NULL) port=atoi(uri->port);
 		}else{
 			ms_warning("Invalid identity uri:%s",url);
-		}	
+		}
 	}
 	ms_message("Starting using url %s",url);
 	ms_mutex_init(&obj->lock,NULL);
@@ -271,8 +271,8 @@ void sipomatic_uninit(Sipomatic *obj)
 
 void sipomatic_iterate(Sipomatic *obj)
 {
-	MSList *elem;
-	MSList *to_be_destroyed=NULL;
+	bctbx_list_t *elem;
+	bctbx_list_t *to_be_destroyed=NULL;
 	Call *call;
 	double elapsed;
 	eXosip_event_t *ev;
@@ -293,13 +293,13 @@ void sipomatic_iterate(Sipomatic *obj)
 			case CALL_STATE_RUNNING:
 				if (elapsed>obj->max_call_time || call->eof){
 					call_release(call);
-					to_be_destroyed=ms_list_append(to_be_destroyed,call);
+					to_be_destroyed=bctbx_list_append(to_be_destroyed,call);
 				}
 			break;
 		}
-		elem=ms_list_next(elem);
+		elem=bctbx_list_next(elem);
 	}
-	for(;to_be_destroyed!=NULL; to_be_destroyed=ms_list_next(to_be_destroyed)){
+	for(;to_be_destroyed!=NULL; to_be_destroyed=bctbx_list_next(to_be_destroyed)){
 		call_destroy((Call*)to_be_destroyed->data);
 	}
 }
@@ -307,9 +307,9 @@ void sipomatic_iterate(Sipomatic *obj)
 
 Call* sipomatic_find_call(Sipomatic *obj,int did)
 {
-	MSList *it;
+	bctbx_list_t *it;
 	Call *call=NULL;
-	for (it=obj->calls;it!=NULL;it=ms_list_next(it)){
+	for (it=obj->calls;it!=NULL;it=bctbx_list_next(it)){
 		call=(Call*)it->data;
 		if ( call->did==did) return call;
 	}
@@ -324,7 +324,7 @@ Call * call_new(Sipomatic *root, eXosip_event_t *ev)
 	int status;
 	sdp_message_t *sdp;
 	sdp_context_t *sdpc;
-	
+
 	sdp=eXosip_get_sdp_info(ev->request);
 	sdpc=sdp_handler_create_context(&sipomatic_sdp_handler,NULL,"sipomatic",NULL);
 	obj=ms_new0(Call,1);
@@ -334,7 +334,7 @@ Call * call_new(Sipomatic *root, eXosip_event_t *ev)
 	sdpans=sdp_context_get_answer(sdpc,sdp);
 	if (sdpans!=NULL){
 		eXosip_call_send_answer(ev->tid,180,NULL);
-		
+
 	}else{
 		status=sdp_context_get_status(sdpc);
 		eXosip_call_send_answer(ev->tid,status,NULL);
@@ -351,7 +351,7 @@ Call * call_new(Sipomatic *root, eXosip_event_t *ev)
 	obj->state=CALL_STATE_INIT;
 	obj->eof=0;
 	obj->root=root;
-	root->calls=ms_list_append(root->calls,obj);
+	root->calls=bctbx_list_append(root->calls,obj);
 	return obj;
 }
 
@@ -367,7 +367,7 @@ void call_release(Call *call)
 
 void call_destroy(Call *obj)
 {
-	obj->root->calls=ms_list_remove(obj->root->calls,obj);
+	obj->root->calls=bctbx_list_remove(obj->root->calls,obj);
 	rtp_profile_destroy(obj->profile);
 	sdp_context_free(obj->sdpc);
 	ms_free(obj);
@@ -409,7 +409,7 @@ int main(int argc, char *argv[])
 	char *url=NULL;
 	bool_t ipv6=FALSE;
 	int i;
-	
+
 	for(i=1;i<argc;i++){
 		if ( (strcmp(argv[i],"-h")==0) || (strcmp(argv[i],"--help")==0) ){
 			display_help();
@@ -441,7 +441,7 @@ int main(int argc, char *argv[])
 			continue;
 		}
 	}
-	
+
 	signal(SIGINT,stop_handler);
 	ortp_init();
 	ms_init();
@@ -453,18 +453,18 @@ int main(int argc, char *argv[])
 	rtp_profile_set_payload(&av_profile,101,&payload_type_telephone_event);
 	rtp_profile_set_payload(&av_profile,116,&payload_type_truespeech);
 	rtp_profile_set_payload(&av_profile,98,&payload_type_h263_1998);
-	
+
 	sipomatic_init(&sipomatic,url,ipv6);
 	if (file!=NULL) sipomatic_set_annouce_file(&sipomatic,file);
-	
+
 	while (run_cond){
 		sipomatic_iterate(&sipomatic);
-#ifndef WIN32
+#ifndef _WIN32
 		usleep(20000);
 #else
 		Sleep(20);
 #endif
 	}
-	
+
 	return(0);
 }
