@@ -20,7 +20,7 @@ call.
 
 #include "../coreapi/linphonecore.h"
 #include "../coreapi/sdphandler.h"
-
+#include <eXosip2/eXosip.h>
 #undef PACKAGE
 #undef VERSION
 #include "mediastreamer2/mediastream.h"
@@ -68,6 +68,7 @@ struct _Call
 	sdp_context_t *sdpc;
 	int time;
 	int did;
+	int tid;
 	AudioStream *audio_stream;
 #ifdef VIDEO_ENABLED
 	VideoStream *video_stream;
@@ -87,7 +88,7 @@ struct _Call
 typedef struct _Call Call;
 
 	
-Call * call_new(Sipomatic *obj,int cid,char *sdp);
+Call * call_new(Sipomatic *obj, eXosip_event_t *ev);
 void call_accept(Call *call);
 void call_release(Call *call);
 void call_destroy(Call *call);
