@@ -14,11 +14,11 @@
 #include "TunnelManager.hh"
 
 #include "ortp/rtpsession.h"
-#include "linphonecore.h"
-#include "linphonecore_utils.h"
+#include "linphone/core.h"
+#include "linphone/core_utils.h"
 #include "private.h"
 
-#ifdef ANDROID
+#ifdef __ANDROID__
 #include <android/log.h>
 #endif
 
@@ -29,8 +29,8 @@ using namespace ::std;
 
 void TunnelManager::addServer(const char *ip, int port,unsigned int udpMirrorPort,unsigned int delay) {
 	if (ip == NULL) {
-		ip = "";
 		ms_warning("Adding tunnel server with empty ip, it will not work!");
+		return;
 	}
 	addServer(ip,port);
 	mUdpMirrorClients.push_back(UdpMirrorClient(ServerAddr(ip,udpMirrorPort),delay));
@@ -38,15 +38,48 @@ void TunnelManager::addServer(const char *ip, int port,unsigned int udpMirrorPor
 
 void TunnelManager::addServer(const char *ip, int port) {
 	if (ip == NULL) {
-		ip = "";
 		ms_warning("Adding tunnel server with empty ip, it will not work!");
+		return;
 	}
+	if (mUseDualClient) {
+		ms_warning("TunnelManager is configured in dual mode, use addServerPair instead");
+		return;
+	}
+	
 	mServerAddrs.push_back(ServerAddr(ip,port));
-	if (mTunnelClient) mTunnelClient->addServer(ip,port);
+	if (mTunnelClient && !mUseDualClient) {
+		static_cast<TunnelClient*>(mTunnelClient)->addServer(ip,port);
+	}
+}
+
+void TunnelManager::addServerPair(const char *ip1, int port1, const char *ip2, int port2, unsigned int udpMirrorPort, unsigned int delay) {
+	if (ip1 == NULL || ip2 == NULL) {
+		ms_warning("Adding tunnel server with empty ip, it will not work!");
+		return;
+	}
+	addServerPair(ip1, port1, ip2, port2);
+	mUdpMirrorClients.push_back(UdpMirrorClient(ServerAddr(ip1, udpMirrorPort), delay));
+}
+
+void TunnelManager::addServerPair(const char *ip1, int port1, const char *ip2, int port2) {
+	if (ip1 == NULL || ip2 == NULL) {
+		ms_warning("Adding tunnel server with empty ip, it will not work!");
+		return;
+	}
+	if (!mUseDualClient) {
+		ms_warning("TunnelManager is configured in single mode, use addServer instead");
+		return;
+	}
+	
+	mDualServerAddrs.push_back(DualServerAddr(ip1, port1, ip2, port2));
+	if (mTunnelClient && mUseDualClient) {
+		static_cast<DualTunnelClient*>(mTunnelClient)->addServerPair(ip1, port1, ip2, port2);
+	}
 }
 
 void TunnelManager::cleanServers() {
 	mServerAddrs.clear();
+	mDualServerAddrs.clear();
 	if (mLongRunningTaskId > 0) {
 		sal_end_background_task(mLongRunningTaskId);
 		mLongRunningTaskId = 0;
@@ -57,18 +90,31 @@ void TunnelManager::cleanServers() {
 		s.stop();
 	}
 	mUdpMirrorClients.clear();
+	mCurrentUdpMirrorClient = mUdpMirrorClients.end();
 	if (mTunnelClient) mTunnelClient->cleanServers();
 }
 
+void TunnelManager::enableDualMode(bool enable) {
+	mUseDualClient = enable;
+}
+
+bool TunnelManager::isDualModeEnabled() {
+	return mUseDualClient;
+}
+
 void TunnelManager::reconnect(){
 	if (mTunnelClient)
 		mTunnelClient->reconnect();
 }
 
 static void sCloseRtpTransport(RtpTransport *t){
-	TunnelSocket *s=(TunnelSocket*)t->data;
-	TunnelManager *manager=(TunnelManager*)s->getUserPointer();
-	manager->closeRtpTransport(t, s);
+	DualSocket *ds = (DualSocket *)t->data;
+	TunnelSocket *sendSocket = ds->sendSocket;
+	TunnelSocket *recvSocket = ds->recvSocket;
+	TunnelManager *manager=(TunnelManager*)sendSocket->getUserPointer();
+	manager->closeRtpTransport(t, sendSocket);
+	manager->closeRtpTransport(t, recvSocket);
+	ms_free(ds);
 }
 void TunnelManager::closeRtpTransport(RtpTransport *t, TunnelSocket *s){
 	mTunnelClient->closeSocket(s);
@@ -83,23 +129,47 @@ void sDestroyRtpTransport(RtpTransport *t){
 }
 
 RtpTransport *TunnelManager::createRtpTransport(int port){
-	TunnelSocket *socket=mTunnelClient->createSocket(port);
-	socket->setUserPointer(this);
-	RtpTransport *t=ms_new0(RtpTransport,1);
+	DualSocket *dualSocket = ms_new0(DualSocket, 1);
+	if (!mUseDualClient) {
+		TunnelSocket *socket = ((TunnelClient *)mTunnelClient)->createSocket(port);
+		socket->setUserPointer(this);
+		dualSocket->sendSocket = socket;
+		dualSocket->recvSocket = socket;
+	} else {
+		dualSocket->sendSocket = ((DualTunnelClient *)mTunnelClient)->createSocket(TunnelSendOnly, port);
+		dualSocket->sendSocket->setUserPointer(this);
+		dualSocket->recvSocket = ((DualTunnelClient *)mTunnelClient)->createSocket(TunnelRecvOnly, port);
+		dualSocket->recvSocket->setUserPointer(this);
+	}
+	
+	RtpTransport *t = ms_new0(RtpTransport,1);
 	t->t_getsocket=NULL;
 	t->t_recvfrom=customRecvfrom;
 	t->t_sendto=customSendto;
 	t->t_close=sCloseRtpTransport;
 	t->t_destroy=sDestroyRtpTransport;
-	t->data=socket;
+	t->data=dualSocket;
 	ms_message("Creating tunnel RTP transport for local virtual port %i", port);
 	return t;
 }
 
 void TunnelManager::startClient() {
 	ms_message("TunnelManager: Starting tunnel client");
-	mTunnelClient = new TunnelClient(TRUE);
-	mTunnelClient->setCallback((TunnelClientController::StateCallback)tunnelCallback,this);
+	if (!mTunnelClient) {
+		if (mUseDualClient) {
+			mTunnelClient = DualTunnelClient::create(TRUE);
+		} else {
+			mTunnelClient = TunnelClient::create(TRUE);
+		}
+		
+		sal_set_tunnel(mCore->sal, mTunnelClient);
+		if (!mUseDualClient) {
+			static_cast<TunnelClient*>(mTunnelClient)->setCallback(tunnelCallback,this);
+		} else {
+			static_cast<DualTunnelClient*>(mTunnelClient)->setCallback(tunnelCallback2,this);
+		}
+	}
+	
 	if (mVerifyServerCertificate) {
 		const char *rootCertificatePath = linphone_core_get_root_ca(mCore);
 		if (rootCertificatePath != NULL) {
@@ -109,14 +179,41 @@ void TunnelManager::startClient() {
 			ms_warning("TunnelManager is set to verify server certificate but no root certificate is available in linphoneCore");
 		}
 	}
-	list<ServerAddr>::iterator it;
-	for(it=mServerAddrs.begin();it!=mServerAddrs.end();++it){
-		const ServerAddr &addr=*it;
-		mTunnelClient->addServer(addr.mAddr.c_str(), addr.mPort);
+	mTunnelClient->cleanServers();
+	if (mUseDualClient) {
+		list<DualServerAddr>::iterator it;
+		for(it=mDualServerAddrs.begin();it!=mDualServerAddrs.end();++it){
+			const DualServerAddr &addr=*it;
+			static_cast<DualTunnelClient*>(mTunnelClient)->addServerPair(addr.mAddr1.c_str(), addr.mPort1, addr.mAddr2.c_str(), addr.mPort2);
+		}
+	} else {
+		list<ServerAddr>::iterator it;
+		for(it=mServerAddrs.begin();it!=mServerAddrs.end();++it){
+			const ServerAddr &addr=*it;
+			static_cast<TunnelClient*>(mTunnelClient)->addServer(addr.mAddr.c_str(), addr.mPort);
+		}
 	}
+	
 	mTunnelClient->setHttpProxy(mHttpProxyHost.c_str(), mHttpProxyPort, mHttpUserName.c_str(), mHttpPasswd.c_str());
-	mTunnelClient->start();
-	sal_set_tunnel(mCore->sal, mTunnelClient);
+	if (!mTunnelClient->isStarted()) {
+		ms_message("Starting tunnel client");
+		mTunnelClient->start();
+	}
+	else {
+		ms_message("Reconnecting tunnel client");
+		mTunnelClient->reconnect(); /*force a reconnection to take into account new parameters*/
+	}
+}
+
+void TunnelManager::stopClient(){
+	if (linphone_core_get_calls_nb(mCore) == 0){
+		/*if no calls are running, we can decide to stop the client completely, so that the connection to the tunnel server is terminated.*/
+		if (mTunnelClient) {
+			ms_message("TunnelManager: stoppping tunnel client");
+			mTunnelClient->stop();
+		}
+		/*otherwise, it doesn't really matter if the tunnel connection is kept alive even if it is not used anymore by the liblinphone.*/
+	}
 }
 
 bool TunnelManager::isConnected() const {
@@ -125,19 +222,21 @@ bool TunnelManager::isConnected() const {
 
 int TunnelManager::customSendto(struct _RtpTransport *t, mblk_t *msg , int flags, const struct sockaddr *to, socklen_t tolen){
 	int size;
+	DualSocket *ds = (DualSocket *)t->data;
 	msgpullup(msg,-1);
 	size=msgdsize(msg);
-	((TunnelSocket*)t->data)->sendto(msg->b_rptr,size,to,tolen);
+	ds->sendSocket->sendto(msg->b_rptr,size,to,tolen);
 	return size;
 }
 
 int TunnelManager::customRecvfrom(struct _RtpTransport *t, mblk_t *msg, int flags, struct sockaddr *from, socklen_t *fromlen){
+	DualSocket *ds = (DualSocket *)t->data;
 	memset(&msg->recv_addr,0,sizeof(msg->recv_addr));
-	int err=((TunnelSocket*)t->data)->recvfrom(msg->b_wptr,msg->b_datap->db_lim-msg->b_datap->db_base,from,*fromlen);
+	int err=ds->recvSocket->recvfrom(msg->b_wptr,dblk_lim(msg->b_datap)-dblk_base(msg->b_datap),from,*fromlen);
 	//to make ice happy
-	inet_aton(((TunnelManager*)((TunnelSocket*)t->data)->getUserPointer())->mLocalAddr,&msg->recv_addr.addr.ipi_addr);
+	inet_aton(((TunnelManager*)(ds->recvSocket)->getUserPointer())->mLocalAddr,&msg->recv_addr.addr.ipi_addr);
 	msg->recv_addr.family = AF_INET;
-	msg->recv_addr.port = htons((unsigned short)((TunnelSocket*)t->data)->getPort());
+	msg->recv_addr.port = htons((unsigned short)(ds->recvSocket)->getPort());
 	if (err>0) return err;
 	return 0;
 }
@@ -145,13 +244,12 @@ int TunnelManager::customRecvfrom(struct _RtpTransport *t, mblk_t *msg, int flag
 TunnelManager::TunnelManager(LinphoneCore* lc) :
 	mCore(lc),
 	mMode(LinphoneTunnelModeDisable),
-	mState(disabled),
-	mTunnelizeSipPackets(true),
 	mTunnelClient(NULL),
 	mHttpProxyPort(0),
 	mVTable(NULL),
 	mLongRunningTaskId(0),
-	mSimulateUdpLoss(false)
+	mSimulateUdpLoss(false),
+	mUseDualClient(false)
 {
 	linphone_core_add_iterate_hook(mCore,(LinphoneCoreIterateHook)sOnIterate,this);
 	mTransportFactories.audio_rtcp_func=sCreateRtpTransport;
@@ -166,6 +264,11 @@ TunnelManager::TunnelManager(LinphoneCore* lc) :
 	mVTable->network_reachable = networkReachableCb;
 	linphone_core_add_listener(mCore, mVTable);
 	linphone_core_get_local_ip_for(AF_INET, NULL, mLocalAddr);
+	mAutodetectionRunning = false;
+	mState = Off;
+	mTargetState = Off;
+	mStarted = false;
+	mTunnelizeSipPackets = true;
 }
 
 TunnelManager::~TunnelManager(){
@@ -176,7 +279,11 @@ TunnelManager::~TunnelManager(){
 	for(UdpMirrorClientList::iterator udpMirror = mUdpMirrorClients.begin(); udpMirror != mUdpMirrorClients.end(); udpMirror++) {
 		udpMirror->stop();
 	}
-	if(mTunnelClient) delete mTunnelClient;
+	stopClient();
+	if (mTunnelClient) {
+		mTunnelClient->stop();
+		delete mTunnelClient;
+	}
 	sal_set_tunnel(mCore->sal,NULL);
 	linphone_core_remove_listener(mCore, mVTable);
 	linphone_core_v_table_destroy(mVTable);
@@ -199,80 +306,103 @@ void TunnelManager::doUnregistration() {
 	}
 }
 
+void TunnelManager::tunnelizeLiblinphone(){
+	ms_message("LinphoneCore goes into tunneled mode.");
+	mState = On; /*do this first because _linphone_core_apply_transports() will use it to know if tunnel listening point is to be used*/
+	linphone_core_set_rtp_transport_factories(mCore,&mTransportFactories);
+	if (mTunnelizeSipPackets) {
+		doUnregistration();
+		_linphone_core_apply_transports(mCore);
+		doRegistration();
+	}
+}
+
+void TunnelManager::untunnelizeLiblinphone(){
+	ms_message("LinphoneCore leaves tunneled mode.");
+	mState = Off;
+	linphone_core_set_rtp_transport_factories(mCore, NULL);
+	if (mTunnelizeSipPackets) {
+		doUnregistration();
+		_linphone_core_apply_transports(mCore);
+		doRegistration();
+	}
+}
+
+void TunnelManager::applyState() {
+	if (!linphone_core_is_network_reachable(mCore)) return;
+	if (mTargetState == On && mState == Off){
+		if (!mTunnelClient || !mTunnelClient->isStarted()){
+			startClient();
+		}
+		if (mTunnelClient->isReady()) tunnelizeLiblinphone();
+	}else if (mTargetState == Off && mState == On){
+		untunnelizeLiblinphone();
+		stopClient();
+	}
+}
+
+void TunnelManager::setState ( TunnelManager::State state ) {
+	mTargetState = state;
+	applyState();
+}
+
 void TunnelManager::processTunnelEvent(const Event &ev){
 	if (ev.mData.mConnected){
 		ms_message("TunnelManager: tunnel is connected");
-		if(mState == connecting) {
-			linphone_core_set_rtp_transport_factories(mCore,&mTransportFactories);
-			mState = ready;
-			if(mTunnelizeSipPackets) {
-				doUnregistration();
-				_linphone_core_apply_transports(mCore);
-				doRegistration();
-			}
-
-		}
+		applyState();
 	} else {
 		ms_error("TunnelManager: tunnel has been disconnected");
 	}
 }
 
-void TunnelManager::setMode(LinphoneTunnelMode mode) {
-	if(mMode == mode) return;
-	if((mode==LinphoneTunnelModeDisable && mState==disabled)
-			|| (mode==LinphoneTunnelModeEnable && mState==ready)) {
-		return;
-	}
-	ms_message("TunnelManager: switching mode from %s to %s",
-			   linphone_tunnel_mode_to_string(mMode),
-			   linphone_tunnel_mode_to_string(mode));
-	switch(mode) {
+void TunnelManager::applyMode() {
+	switch(mMode) {
 	case LinphoneTunnelModeEnable:
-		if(mState == disabled) {
-			startClient();
-			mState = connecting;
-			mMode = mode;
-		} else {
-			ms_error("TunnelManager: could not change mode. Bad state");
-		}
+		stopAutoDetection();
+		setState(On);
 		break;
 	case LinphoneTunnelModeDisable:
-		if(mState == ready) {
-			linphone_core_set_rtp_transport_factories(mCore,NULL);
-			mState = disabled;
-			mMode = mode;
-			if(mTunnelizeSipPackets) {
-				doUnregistration();
-				_linphone_core_apply_transports(mCore);
-			}
-			sal_set_tunnel(mCore->sal,NULL);
-			delete mTunnelClient;
-			mTunnelClient=NULL;
-		} else {
-			ms_error("TunnelManager: could not change mode. Bad state");
-		}
+		stopAutoDetection();
+		setState(Off);
 		break;
 	case LinphoneTunnelModeAuto:
-		if(mState == disabled || mState == ready) {
-			if(startAutoDetection()) {
-				mState = autodetecting;
-				mMode = mode;
-			}
-		} else {
-			ms_error("TunnelManager: could not change mode. Bad state");
-		}
+		if (linphone_core_is_network_reachable(mCore)) startAutoDetection();
 		break;
 	default:
-		ms_error("TunnelManager::setMode(): invalid mode (%d)", mode);
+		ms_error("TunnelManager::setMode(): invalid mode (%d)", (int)mMode);
 	}
 }
 
-void TunnelManager::tunnelCallback(bool connected, TunnelManager *zis){
-	Event ev;
-	if (zis->mLongRunningTaskId > 0) {
-		sal_end_background_task(zis->mLongRunningTaskId);
-		zis->mLongRunningTaskId = 0;
+void TunnelManager::setMode(LinphoneTunnelMode mode) {
+	if(mMode == mode) return;
+	ms_message("TunnelManager: switching mode from %s to %s",
+			   linphone_tunnel_mode_to_string(mMode),
+			   linphone_tunnel_mode_to_string(mode));
+	mMode = mode;
+	applyMode();
+	
+}
+
+void TunnelManager::stopLongRunningTask() {
+	if (mLongRunningTaskId != 0) {
+		sal_end_background_task(mLongRunningTaskId);
+		mLongRunningTaskId = 0;
 	}
+}
+
+void TunnelManager::tunnelCallback(bool connected, void *user_pointer){
+	TunnelManager *zis = static_cast<TunnelManager*>(user_pointer);
+	Event ev;
+	
+	ev.mType=TunnelEvent;
+	ev.mData.mConnected=connected;
+	zis->postEvent(ev);
+}
+
+void TunnelManager::tunnelCallback2(TunnelDirection direction, bool connected, void *user_pointer){
+	TunnelManager *zis = static_cast<TunnelManager*>(user_pointer);
+	Event ev;
+	
 	ev.mType=TunnelEvent;
 	ev.mData.mConnected=connected;
 	zis->postEvent(ev);
@@ -299,7 +429,7 @@ void TunnelManager::sOnIterate(TunnelManager *zis){
 	zis->onIterate();
 }
 
-#ifdef ANDROID
+#ifdef __ANDROID__
 extern void linphone_android_log_handler(int prio, char *str);
 static void linphone_android_tunnel_log_handler(int lev, const char *fmt, va_list args) {
 	char str[4096];
@@ -317,7 +447,7 @@ static void linphone_android_tunnel_log_handler(int lev, const char *fmt, va_lis
 	}
 	linphone_android_log_handler(prio, str);
 }
-#endif /*ANDROID*/
+#endif /* __ANDROID__ */
 
 void TunnelManager::enableLogs(bool value) {
 	enableLogs(value,NULL);
@@ -325,7 +455,7 @@ void TunnelManager::enableLogs(bool value) {
 
 void TunnelManager::enableLogs(bool isEnabled,LogHandler logHandler) {
 	if (logHandler != NULL)	SetLogHandler(logHandler);
-#ifdef ANDROID
+#ifdef __ANDROID__
 	else SetLogHandler(linphone_android_tunnel_log_handler);
 #else
 	else SetLogHandler(tunnel_default_log_handler);
@@ -338,13 +468,12 @@ void TunnelManager::enableLogs(bool isEnabled,LogHandler logHandler) {
 	}
 }
 
-
 LinphoneTunnelMode TunnelManager::getMode() const {
 	return mMode;
 }
 
 void TunnelManager::processUdpMirrorEvent(const Event &ev){
-	if(mState != autodetecting) return;
+	if (mAutodetectionRunning == false) return; /*auto detection was cancelled, for example by switching to disabled state*/
 	if (mSimulateUdpLoss || !ev.mData.mHaveUdp) {
 		if (mSimulateUdpLoss) {
 			ms_message("TunnelManager: simulate UDP lost on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
@@ -354,30 +483,20 @@ void TunnelManager::processUdpMirrorEvent(const Event &ev){
 		mCurrentUdpMirrorClient++;
 		if (mCurrentUdpMirrorClient !=mUdpMirrorClients.end()) {
 			ms_message("TunnelManager: trying another UDP mirror on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
-			if (mLongRunningTaskId == 0)
-				mLongRunningTaskId = sal_begin_background_task("Tunnel auto detect", NULL, NULL);
 			UdpMirrorClient &lUdpMirrorClient=*mCurrentUdpMirrorClient;
 			lUdpMirrorClient.start(TunnelManager::sUdpMirrorClientCallback,(void*)this);
+			mAutodetectionRunning = true;
+			return;
 		} else {
 			ms_message("TunnelManager: all UDP mirror tests failed");
-			if(mTunnelClient==NULL) {
-				startClient();
-				mState = connecting;
-			} else {
-				mState = ready;
-			}
+			setState(On);
 		}
 	} else {
-		ms_message("TunnelManager: UDP mirror test succeed on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
-		if(mTunnelClient) {
-			if(mTunnelizeSipPackets) doUnregistration();
-			sal_set_tunnel(mCore->sal,NULL);
-			delete mTunnelClient;
-			mTunnelClient = NULL;
-			if(mTunnelizeSipPackets) doRegistration();
-		}
-		mState = disabled;
+		ms_message("TunnelManager: UDP mirror test success on %s:%d", mCurrentUdpMirrorClient->getServerAddress().mAddr.c_str(), mCurrentUdpMirrorClient->getServerAddress().mPort);
+		setState(Off);
 	}
+	mAutodetectionRunning = false;
+	stopLongRunningTask();
 }
 
 void TunnelManager::postEvent(const Event &ev){
@@ -396,18 +515,32 @@ void TunnelManager::sUdpMirrorClientCallback(bool isUdpAvailable, void* data) {
 
 void TunnelManager::networkReachableCb(LinphoneCore *lc, bool_t reachable) {
 	TunnelManager *tunnel = bcTunnel(linphone_core_get_tunnel(lc));
-	if(reachable && tunnel->getMode() == LinphoneTunnelModeAuto && tunnel->mState != connecting && tunnel->mState != autodetecting) {
-		LOGI("TunnelManager: Network is now reachable, starting auto detection");
-		tunnel->startAutoDetection();
-		tunnel->mState = autodetecting;
-	} else if (!reachable && tunnel->mState == autodetecting) {
-		// if network is no more reachable, cancel autodetection
-		for(UdpMirrorClientList::iterator udpMirror = tunnel->mUdpMirrorClients.begin(); udpMirror != tunnel->mUdpMirrorClients.end(); udpMirror++) {
+	
+	if (reachable) {
+		linphone_core_get_local_ip_for(AF_INET, NULL,tunnel->mLocalAddr);
+		if (tunnel->getMode() == LinphoneTunnelModeAuto){
+			tunnel->startAutoDetection();
+			/*autodetection will call applyState() when finished*/
+		}else{
+			tunnel->applyState();
+		}
+	} else if (!reachable) {
+		// if network is no more reachable, cancel autodetection if any
+		tunnel->stopAutoDetection();
+		//turn off the tunnel connection
+		tunnel->stopClient();
+		tunnel->untunnelizeLiblinphone();
+	}
+}
+
+void TunnelManager::stopAutoDetection(){
+	if (mAutodetectionRunning){
+		for(UdpMirrorClientList::iterator udpMirror = mUdpMirrorClients.begin(); udpMirror != mUdpMirrorClients.end(); udpMirror++) {
 			udpMirror->stop();
 		}
-		tunnel->mState = disabled;
+		mAutodetectionRunning = false;
+		stopLongRunningTask();
 	}
-	linphone_core_get_local_ip_for(AF_INET, NULL,tunnel->mLocalAddr);
 }
 
 bool TunnelManager::startAutoDetection() {
@@ -420,20 +553,13 @@ bool TunnelManager::startAutoDetection() {
 	if (mLongRunningTaskId == 0)
 		 mLongRunningTaskId = sal_begin_background_task("Tunnel auto detect", NULL, NULL);
 	UdpMirrorClient &lUdpMirrorClient=*mCurrentUdpMirrorClient;
+	mAutodetectionRunning = true;
 	lUdpMirrorClient.start(TunnelManager::sUdpMirrorClientCallback,(void*)this);
 	return true;
 }
 
 bool TunnelManager::isActivated() const{
-	switch(getMode()){
-		case LinphoneTunnelModeAuto:
-			return !(mState==disabled);
-		case LinphoneTunnelModeDisable:
-			return false;
-		case LinphoneTunnelModeEnable:
-			return true;
-	}
-	return false;
+	return mState == On;
 }
 
 void TunnelManager::setHttpProxyAuthInfo(const char* username,const char* passwd) {
