@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #ifndef SAL_IMPL_H_
@@ -37,6 +37,7 @@ struct Sal{
 	int session_expires;
 	unsigned int keep_alive;
 	char *root_ca;
+	char *root_ca_data;
 	char *uuid;
 	int refresher_retry_after; /*retry after value for refresher*/
 	MSList *supported_tags;/*list of char * */
@@ -54,6 +55,7 @@ struct Sal{
 	SalOpSDPHandling default_sdp_handling;
 	bool_t pending_trans_checking; /*testing purpose*/
 	void *ssl_config;
+	bctbx_list_t *supported_content_types; /* list of char* */
 };
 
 typedef enum SalOpState {
@@ -85,6 +87,7 @@ struct SalOp{
 	SalOpBase base;
 	const belle_sip_listener_callbacks_t *callbacks;
 	SalErrorInfo error_info;
+	SalErrorInfo reason_error_info;
 	belle_sip_client_transaction_t *pending_auth_transaction;
 	belle_sip_server_transaction_t* pending_server_trans;
 	belle_sip_server_transaction_t* pending_update_server_trans;
@@ -129,6 +132,7 @@ SalOp* sal_op_ref(SalOp* op);
 void* sal_op_unref(SalOp* op);
 void sal_op_release_impl(SalOp *op);
 
+void sal_op_set_replaces(SalOp* op,belle_sip_header_replaces_t* replaces);
 void sal_op_set_remote_ua(SalOp*op,belle_sip_message_t* message);
 int sal_op_send_request(SalOp* op, belle_sip_request_t* request);
 int sal_op_send_request_with_expires(SalOp* op, belle_sip_request_t* request,int expires);
@@ -146,7 +150,7 @@ belle_sip_header_contact_t* sal_op_create_contact(SalOp *op) ;
 
 bool_t _sal_compute_sal_errors(belle_sip_response_t* response, SalReason* sal_reason, char* reason, size_t reason_size);
 SalReason _sal_reason_from_sip_code(int code);
-
+void sal_op_set_reason_error_info(SalOp *op, belle_sip_message_t *msg);
 void sal_op_set_error_info_from_response(SalOp *op, belle_sip_response_t *response);
 /*presence*/
 void sal_op_presence_fill_cbs(SalOp*op);
@@ -176,4 +180,5 @@ void _sal_op_add_custom_headers(SalOp *op, belle_sip_message_t *msg);
 
 SalSubscribeStatus belle_sip_message_get_subscription_state(const belle_sip_message_t *msg);
 
+
 #endif /* SAL_IMPL_H_ */
