@@ -14,11 +14,13 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include "sal_impl.h"
 #include "offeranswer.h"
 
+#include <bctoolbox/defs.h>
+
 static int extract_sdp(SalOp* op,belle_sip_message_t* message,belle_sdp_session_description_t** session_desc, SalReason *error);
 
 /*used for calls terminated before creation of a dialog*/
@@ -37,11 +39,11 @@ static void call_set_error(SalOp* op,belle_sip_response_t* response, bool_t fata
 	if (fatal) op->state = SalOpStateTerminating;
 	op->base.root->callbacks.call_failure(op);
 }
-static void set_addr_to_0000(char value[]) {
+static void set_addr_to_0000(char value[], size_t sz) {
 	if (ms_is_ipv6(value)) {
-		strcpy(value,"::0");
+		strncpy(value,"::0", sz);
 	} else {
-		strcpy(value,"0.0.0.0");
+		strncpy(value,"0.0.0.0", sz);
 	}
 	return;
 }
@@ -66,11 +68,11 @@ static void sdp_process(SalOp *h){
 		offer_answer_initiate_incoming(h->base.root->factory, h->base.local_media,h->base.remote_media,h->result,h->base.root->one_matching_codec);
 		/*for backward compatibility purpose*/
 		if(h->cnx_ip_to_0000_if_sendonly_enabled && sal_media_description_has_dir(h->result,SalStreamSendOnly)) {
-			set_addr_to_0000(h->result->addr);
+			set_addr_to_0000(h->result->addr, sizeof(h->result->addr));
 			for(i=0;i<SAL_MEDIA_DESCRIPTION_MAX_STREAMS;++i){
 				if (h->result->streams[i].dir == SalStreamSendOnly) {
-					set_addr_to_0000(h->result->streams[i].rtp_addr);
-					set_addr_to_0000(h->result->streams[i].rtcp_addr);
+					set_addr_to_0000(h->result->streams[i].rtp_addr, sizeof(h->result->streams[i].rtp_addr));
+					set_addr_to_0000(h->result->streams[i].rtcp_addr, sizeof(h->result->streams[i].rtcp_addr));
 				}
 			}
 		}
@@ -142,21 +144,28 @@ static int set_sdp_from_desc(belle_sip_message_t *msg, const SalMediaDescription
 	return err;
 
 }
-static void call_process_io_error(void *user_ctx, const belle_sip_io_error_event_t *event){
-	SalOp* op=(SalOp*)user_ctx;
 
-	if (op->state==SalOpStateTerminated) return;
+static void call_process_io_error(void *user_ctx, const belle_sip_io_error_event_t *event) {
+	SalOp *op = (SalOp *)user_ctx;
 
-	if (!op->dialog)  {
-		/*call terminated very early*/
-		sal_error_info_set(&op->error_info,SalReasonIOError,503,"IO error",NULL);
+	if (op->state == SalOpStateTerminated) return;
+
+	if (op->pending_client_trans && (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(op->pending_client_trans)) == BELLE_SIP_TRANSACTION_INIT)) {
+
+		sal_error_info_set(&op->error_info, SalReasonIOError, "SIP", 503, "IO error", NULL);
 		op->base.root->callbacks.call_failure(op);
-		op->state = SalOpStateTerminating;
-		call_set_released(op);
+
+		if (!op->dialog || belle_sip_dialog_get_state(op->dialog) != BELLE_SIP_DIALOG_CONFIRMED){
+			/* Call terminated very very early, before INVITE is even sent, probably DNS resolution timeout. */
+			op->state = SalOpStateTerminating;
+			call_set_released(op);
+		}
 	} else {
-		/*dialog will terminated shortly, nothing to do*/
+		/* Nothing to be done. If the error comes from a connectivity loss,
+		 * the call will be marked as broken, and an attempt to repair it will be done. */
 	}
 }
+
 static void process_dialog_terminated(void *ctx, const belle_sip_dialog_terminated_event_t *event) {
 	SalOp* op=(SalOp*)ctx;
 
@@ -208,27 +217,12 @@ static void handle_sdp_from_response(SalOp* op,belle_sip_response_t* response) {
 	if (op->base.local_media) sdp_process(op);
 }
 
-static void cancelling_invite(SalOp* op ){
-	belle_sip_request_t* cancel;
-	ms_message("Cancelling INVITE request from [%s] to [%s] ",sal_op_get_from(op), sal_op_get_to(op));
-	cancel = belle_sip_client_transaction_create_cancel(op->pending_client_trans);
-	if (cancel){
-		sal_op_send_request(op,cancel);
-	}else if (op->dialog){
-		belle_sip_dialog_state_t state = belle_sip_dialog_get_state(op->dialog);;
-		/*case where the response received is invalid (could not establish a dialog), but the transaction is not cancellable 
-		 * because already terminated*/
-		switch(state){
-			case BELLE_SIP_DIALOG_EARLY:
-			case BELLE_SIP_DIALOG_NULL:
-				/*force kill the dialog*/
-				ms_warning("op [%p]: force kill of dialog [%p]", op, op->dialog);
-				belle_sip_dialog_delete(op->dialog);
-			break;
-			default:
-			break;
-		}
-	}
+void sal_call_cancel_invite(SalOp *op) {
+	sal_call_cancel_invite_with_info(op,NULL);
+}
+
+static void cancelling_invite(SalOp *op, const SalErrorInfo *info) {
+	sal_call_cancel_invite_with_info(op, info);
 	op->state=SalOpStateTerminating;
 }
 
@@ -271,7 +265,7 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 					if (strcmp("CANCEL",belle_sip_request_get_method(belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(op->pending_client_trans))))!=0) {
 						/*it wasn't sent */
 						if (code<200) {
-							cancelling_invite(op);
+							cancelling_invite(op, NULL);
 						}else{
 							/* no need to send the INVITE because the UAS rejected the INVITE*/
 							if (op->dialog==NULL) call_set_released(op);
@@ -293,11 +287,13 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 					call_set_error(op, response, TRUE);
 					if (op->dialog==NULL) call_set_released(op);
 				}
-			} else if (code >=200
-						&& code<300
-						&& strcmp("UPDATE",belle_sip_request_get_method(req))==0) {
+			} else if (code >=200 && code<300) {
+				if (strcmp("UPDATE",method)==0) {
 					handle_sdp_from_response(op,response);
 					op->base.root->callbacks.call_accepted(op);
+				} else if (strcmp("CANCEL", method) == 0) {
+					op->base.root->callbacks.call_cancel_done(op);
+				}
 			}
 		}
 		break;
@@ -319,8 +315,9 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 								op->sdp_answer=NULL;
 							}
 							belle_sip_message_add_header(BELLE_SIP_MESSAGE(ack),BELLE_SIP_HEADER(op->base.root->user_agent));
-							belle_sip_dialog_send_ack(op->dialog,ack);
 							op->base.root->callbacks.call_accepted(op); /*INVITE*/
+							op->base.root->callbacks.call_ack_being_sent(op, (SalCustomHeader*)ack);
+							belle_sip_dialog_send_ack(op->dialog,ack);
 							op->state=SalOpStateActive;
 						}else if (code >= 300){
 							call_set_error(op,response, FALSE);
@@ -339,6 +336,8 @@ static void call_process_response(void *op_base, const belle_sip_response_event_
 						}
 					}else if (strcmp("UPDATE",method)==0){
 						op->base.root->callbacks.call_accepted(op); /*INVITE*/
+					}else if (strcmp("CANCEL",method)==0){
+						op->base.root->callbacks.call_cancel_done(op);
 					}
 				break;
 				case SalOpStateTerminating:
@@ -371,7 +370,7 @@ static void call_process_timeout(void *user_ctx, const belle_sip_timeout_event_t
 
 	if (!op->dialog)  {
 		/*call terminated very early*/
-		sal_error_info_set(&op->error_info,SalReasonRequestTimeout,408,"Request timeout",NULL);
+		sal_error_info_set(&op->error_info, SalReasonRequestTimeout, "SIP", 408, "Request timeout", NULL);
 		op->base.root->callbacks.call_failure(op);
 		op->state = SalOpStateTerminating;
 		call_set_released(op);
@@ -397,7 +396,7 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 		resp=belle_sip_transaction_get_response(BELLE_SIP_TRANSACTION(server_transaction));
 	}
 	if (resp) code = belle_sip_response_get_status_code(resp);
-	
+
 	if (op->state == SalOpStateTerminating
 			&& strcmp("BYE",belle_sip_request_get_method(req))==0
 			&& (!resp || (belle_sip_response_get_status_code(resp) != 401
@@ -406,7 +405,7 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 		release_call=TRUE;
 	}else if (op->state == SalOpStateEarly && code < 200){
 		/*call terminated early*/
-		sal_error_info_set(&op->error_info,SalReasonIOError,503,"I/O error",NULL);
+		sal_error_info_set(&op->error_info, SalReasonIOError, "SIP", 503, "I/O error", NULL);
 		op->state = SalOpStateTerminating;
 		op->base.root->callbacks.call_failure(op);
 		release_call=TRUE;
@@ -424,12 +423,14 @@ static void call_process_transaction_terminated(void *user_ctx, const belle_sip_
 	if (release_call) call_set_released(op);
 }
 
-static void call_terminated(SalOp* op,belle_sip_server_transaction_t* server_transaction, belle_sip_request_t* request,int status_code) {
+static void call_terminated(SalOp* op,belle_sip_server_transaction_t* server_transaction, int status_code, belle_sip_request_t* cancel_request) {
 	belle_sip_response_t* resp;
+	belle_sip_request_t* server_req = belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(server_transaction));
 	op->state = SalOpStateTerminating;
-	op->base.root->callbacks.call_terminated(op,op->dir==SalOpDirIncoming?sal_op_get_from(op):sal_op_get_to(op));
-	resp=sal_op_create_response_from_request(op,request,status_code);
+	sal_op_set_reason_error_info(op, BELLE_SIP_MESSAGE(cancel_request ? cancel_request : server_req));
+	resp=sal_op_create_response_from_request(op,server_req,status_code);
 	belle_sip_server_transaction_send_response(server_transaction,resp);
+	op->base.root->callbacks.call_terminated(op,op->dir==SalOpDirIncoming?sal_op_get_from(op):sal_op_get_to(op));
 }
 
 static void unsupported_method(belle_sip_server_transaction_t* server_transaction,belle_sip_request_t* request) {
@@ -492,10 +493,11 @@ static int is_media_description_acceptable(SalMediaDescription *md){
 	return TRUE;
 }
 
-static int process_sdp_for_invite(SalOp* op,belle_sip_request_t* invite) {
+static SalReason process_sdp_for_invite(SalOp* op,belle_sip_request_t* invite) {
 	belle_sdp_session_description_t* sdp;
-	int err=0;
-	SalReason reason;
+	SalReason reason = SalReasonNone;
+	SalErrorInfo sei = {0};
+
 	if (extract_sdp(op,BELLE_SIP_MESSAGE(invite),&sdp,&reason)==0) {
 		if (sdp){
 			op->sdp_offering=FALSE;
@@ -503,17 +505,18 @@ static int process_sdp_for_invite(SalOp* op,belle_sip_request_t* invite) {
 			sdp_to_media_description(sdp,op->base.remote_media);
 			/*make some sanity check about the SDP received*/
 			if (!is_media_description_acceptable(op->base.remote_media)){
-				err=-1;
 				reason=SalReasonNotAcceptable;
 			}
 			belle_sip_object_unref(sdp);
 		}else op->sdp_offering=TRUE; /*INVITE without SDP*/
-	}else err=-1;
-
-	if (err==-1){
-		sal_call_decline(op,reason,NULL);
 	}
-	return err;
+
+	if (reason != SalReasonNone){
+		sal_error_info_set(&sei, reason,"SIP", 0, NULL, NULL);
+		sal_call_decline_with_error_info(op, &sei,NULL);
+		sal_error_info_reset(&sei);
+	}
+	return reason;
 }
 
 static void sal_op_reset_descriptions(SalOp *op) {
@@ -526,8 +529,15 @@ static void sal_op_reset_descriptions(SalOp *op) {
 		op->result=NULL;
 	}
 }
+
+static bool_t is_a_pending_invite_incoming_transaction(belle_sip_transaction_t *tr){
+	return BELLE_SIP_OBJECT_IS_INSTANCE_OF(tr, belle_sip_ist_t) && belle_sip_transaction_state_is_transient(
+		belle_sip_transaction_get_state(tr));
+}
+
 static void process_request_event(void *op_base, const belle_sip_request_event_t *event) {
 	SalOp* op = (SalOp*)op_base;
+	SalReason reason;
 	belle_sip_server_transaction_t* server_transaction=NULL;
 	belle_sdp_session_description_t* sdp;
 	belle_sip_request_t* req = belle_sip_request_event_get_request(event);
@@ -538,7 +548,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 	bool_t is_update=FALSE;
 	bool_t drop_op = FALSE;
 
-	if (strcmp("ACK",method)!=0){  /*ACK does'nt create srv transaction*/
+	if (strcmp("ACK",method)!=0){  /*ACK doesn't create a server transaction*/
 		server_transaction = belle_sip_provider_create_server_transaction(op->base.root->prov,belle_sip_request_event_get_request(event));
 		belle_sip_object_ref(server_transaction);
 		belle_sip_transaction_set_application_data(BELLE_SIP_TRANSACTION(server_transaction),sal_op_ref(op));
@@ -572,7 +582,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 				ms_warning("replace header already set");
 			}
 
-			if (process_sdp_for_invite(op,req) == 0) {
+			if ( (reason = process_sdp_for_invite(op,req)) == SalReasonNone) {
 				if ((call_info=belle_sip_message_get_header(BELLE_SIP_MESSAGE(req),"Call-Info"))) {
 					if( strstr(belle_sip_header_get_unparsed_value(call_info),"answer-after=") != NULL) {
 						op->auto_answer_asked=TRUE;
@@ -581,11 +591,13 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 				}
 				op->base.root->callbacks.call_received(op);
 			}else{
+				sal_error_info_set(&op->error_info, reason, "SIP", 0, NULL, NULL);
+				op->base.root->callbacks.call_rejected(op);
 				/*the INVITE was declined by process_sdp_for_invite(). As we are not inside an established dialog, we can drop the op immediately*/
 				drop_op = TRUE;
 			}
 			break;
-		} /* else same behavior as for EARLY state, thus NO BREAK*/
+		}BCTBX_NO_BREAK; /* else same behavior as for EARLY state, thus NO BREAK*/
 	}
 	case BELLE_SIP_DIALOG_EARLY: {
 		if (strcmp("CANCEL",method)==0) {
@@ -594,11 +606,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 				belle_sip_server_transaction_send_response(server_transaction
 						,sal_op_create_response_from_request(op,req,200));
 				/*terminate invite transaction*/
-				call_terminated(op
-						,op->pending_server_trans
-						,belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(op->pending_server_trans)),487);
-
-
+				call_terminated(op,op->pending_server_trans,487,req);
 			} else {
 				/*call leg does not exist*/
 				belle_sip_server_transaction_send_response(server_transaction
@@ -609,7 +617,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 			belle_sip_server_transaction_send_response(server_transaction,resp);
 		} else if (strcmp("UPDATE",method)==0) {
 			sal_op_reset_descriptions(op);
-			if (process_sdp_for_invite(op,req)==0)
+			if (process_sdp_for_invite(op,req)==SalReasonNone)
 				op->base.root->callbacks.call_updating(op,TRUE);
 		} else {
 			belle_sip_error("Unexpected method [%s] for dialog state BELLE_SIP_DIALOG_EARLY",belle_sip_request_get_method(req));
@@ -620,7 +628,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 	case BELLE_SIP_DIALOG_CONFIRMED:
 		/*great ACK received*/
 		if (strcmp("ACK",method)==0) {
-			if (!op->pending_client_trans || 
+			if (!op->pending_client_trans ||
 				!belle_sip_transaction_state_is_transient(belle_sip_transaction_get_state((belle_sip_transaction_t*)op->pending_client_trans))){
 				if (op->sdp_offering){
 					SalReason reason;
@@ -637,15 +645,12 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 						}
 					}
 				}
-				op->base.root->callbacks.call_ack(op);
+				op->base.root->callbacks.call_ack_received(op, (SalCustomHeader*)req);
 			}else{
 				ms_message("Ignored received ack since a new client transaction has been started since.");
 			}
 		} else if(strcmp("BYE",method)==0) {
-			resp=sal_op_create_response_from_request(op,req,200);
-			belle_sip_server_transaction_send_response(server_transaction,resp);
-			op->base.root->callbacks.call_terminated(op,op->dir==SalOpDirIncoming?sal_op_get_from(op):sal_op_get_to(op));
-			op->state=SalOpStateTerminating;
+			call_terminated(op,server_transaction,200,req);
 			/*call end not notified by dialog deletion because transaction can end before dialog*/
 		} else if(strcmp("INVITE",method)==0 || (is_update=(strcmp("UPDATE",method)==0)) ) {
 			if (is_update && !belle_sip_message_get_body(BELLE_SIP_MESSAGE(req))) {
@@ -658,7 +663,7 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 			} else {
 				/*re-invite*/
 				sal_op_reset_descriptions(op);
-				if (process_sdp_for_invite(op,req)==0)
+				if (process_sdp_for_invite(op,req)==SalReasonNone)
 					op->base.root->callbacks.call_updating(op,is_update);
 			}
 		} else if (strcmp("INFO",method)==0){
@@ -698,8 +703,16 @@ static void process_request_event(void *op_base, const belle_sip_request_event_t
 			resp=sal_op_create_response_from_request(op,req,200);
 			belle_sip_server_transaction_send_response(server_transaction,resp);
 		} else if (strcmp("CANCEL",method)==0) {
-			/*call leg does not exist because 200ok already sent*/
-			belle_sip_server_transaction_send_response(server_transaction,sal_op_create_response_from_request(op,req,481));
+			belle_sip_transaction_t *last_transaction = belle_sip_dialog_get_last_transaction(op->dialog);
+			if (last_transaction == NULL || !is_a_pending_invite_incoming_transaction(last_transaction) ) {
+				/*call leg does not exist because 200ok already sent*/
+				belle_sip_server_transaction_send_response(server_transaction,sal_op_create_response_from_request(op,req,481));
+			} else {
+				/* CANCEL on re-INVITE for which a 200ok has not been sent yet */
+				belle_sip_server_transaction_send_response(server_transaction, sal_op_create_response_from_request(op, req, 200));
+				belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(last_transaction),
+					sal_op_create_response_from_request(op, belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(last_transaction)), 487));
+			}
 		} else if (strcmp("MESSAGE",method)==0){
 			sal_process_incoming_message(op,event);
 		}else{
@@ -878,9 +891,9 @@ int sal_call_accept(SalOp*h){
 		return -1;
 	}
 	ms_message("Accepting server transaction [%p] on op [%p]", transaction, h);
+
 	/* sends a 200 OK */
 	response = sal_op_create_response_from_request(h,belle_sip_transaction_get_request(BELLE_SIP_TRANSACTION(transaction)),200);
-
 	if (response==NULL){
 		ms_error("Fail to build answer for call");
 		return -1;
@@ -912,6 +925,44 @@ int sal_call_accept(SalOp*h){
 	return 0;
 }
 
+static belle_sip_header_reason_t *sal_call_make_reason_header( const SalErrorInfo *info){
+	if (info != NULL){
+		belle_sip_header_reason_t* reason = BELLE_SIP_HEADER_REASON(belle_sip_header_reason_new());
+		belle_sip_header_reason_set_text(reason, info->status_string);
+		belle_sip_header_reason_set_protocol(reason,info->protocol);
+		belle_sip_header_reason_set_cause(reason,info->protocol_code);
+		return reason;
+	}
+	return NULL;
+}
+
+void sal_call_cancel_invite_with_info(SalOp* op, const SalErrorInfo *info) {
+	belle_sip_request_t* cancel;
+	ms_message("Cancelling INVITE request from [%s] to [%s] ",sal_op_get_from(op), sal_op_get_to(op));
+	cancel = belle_sip_client_transaction_create_cancel(op->pending_client_trans);
+	if (cancel){
+		if (info != NULL){
+			belle_sip_header_reason_t* reason = sal_call_make_reason_header(info);
+			belle_sip_message_add_header(BELLE_SIP_MESSAGE(cancel),BELLE_SIP_HEADER(reason));
+		}
+		sal_op_send_request(op,cancel);
+	}else if (op->dialog){
+		belle_sip_dialog_state_t state = belle_sip_dialog_get_state(op->dialog);;
+		/*case where the response received is invalid (could not establish a dialog), but the transaction is not cancellable
+		 * because already terminated*/
+		switch(state){
+			case BELLE_SIP_DIALOG_EARLY:
+			case BELLE_SIP_DIALOG_NULL:
+				/*force kill the dialog*/
+				ms_warning("op [%p]: force kill of dialog [%p]", op, op->dialog);
+				belle_sip_dialog_delete(op->dialog);
+			break;
+			default:
+			break;
+		}
+	}
+}
+
 int sal_call_decline(SalOp *op, SalReason reason, const char *redirection /*optional*/){
 	belle_sip_response_t* response;
 	belle_sip_header_contact_t* contact=NULL;
@@ -940,9 +991,54 @@ int sal_call_decline(SalOp *op, SalReason reason, const char *redirection /*opti
 	return 0;
 }
 
+int sal_call_decline_with_error_info(SalOp *op,  const SalErrorInfo *info, const char *redirection /*optional*/){
+	belle_sip_response_t* response;
+	belle_sip_header_contact_t* contact=NULL;
+	int status = info->protocol_code;
+	belle_sip_transaction_t *trans;
+
+	if (info->reason==SalReasonRedirect){
+		if (redirection!=NULL) {
+			if (strstr(redirection,"sip:")!=0) status=302;
+			else status=380;
+			contact= belle_sip_header_contact_new();
+			belle_sip_header_address_set_uri(BELLE_SIP_HEADER_ADDRESS(contact),belle_sip_uri_parse(redirection));
+		} else {
+			ms_error("Cannot redirect to null");
+		}
+	}
+	trans=(belle_sip_transaction_t*)op->pending_server_trans;
+	if (!trans) trans=(belle_sip_transaction_t*)op->pending_update_server_trans;
+	if (!trans){
+		ms_error("sal_call_decline_with_error_info(): no pending transaction to decline.");
+		return -1;
+	}
+	response = sal_op_create_response_from_request(op,belle_sip_transaction_get_request(trans),status);
+	belle_sip_header_reason_t* reason_header = sal_call_make_reason_header(info->sub_sei);
+	if (reason_header) {
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(reason_header));
+	}
+
+	if (contact) {
+		belle_sip_message_add_header(BELLE_SIP_MESSAGE(response),BELLE_SIP_HEADER(contact));
+	}
+	belle_sip_server_transaction_send_response(BELLE_SIP_SERVER_TRANSACTION(trans),response);
+	return 0;
+}
+
 int sal_call_update(SalOp *op, const char *subject, bool_t no_user_consent){
 	belle_sip_request_t *update;
-	belle_sip_dialog_state_t state=belle_sip_dialog_get_state(op->dialog);
+	belle_sip_dialog_state_t state;
+
+	if (op->dialog == NULL) {
+		/* If the dialog does not exist, this is that we are trying to recover from a connection loss
+			during a very early state of outgoing call initiation (the dialog has not been created yet). */
+		const char *from = sal_op_get_from(op);
+		const char *to = sal_op_get_to(op);
+		return sal_call(op, from, to);
+	}
+
+	state = belle_sip_dialog_get_state(op->dialog);
 	belle_sip_dialog_enable_pending_trans_checking(op->dialog,op->base.root->pending_trans_checking);
 
 	/*check for dialog state*/
@@ -964,14 +1060,14 @@ int sal_call_update(SalOp *op, const char *subject, bool_t no_user_consent){
 	}
 	/*it failed why ?*/
 	if (belle_sip_dialog_request_pending(op->dialog))
-		sal_error_info_set(&op->error_info,SalReasonRequestPending,491,NULL,NULL);
+		sal_error_info_set(&op->error_info,SalReasonRequestPending, "SIP", 491,NULL,NULL);
 	else
-		sal_error_info_set(&op->error_info,SalReasonUnknown,500,NULL,NULL);
+		sal_error_info_set(&op->error_info,SalReasonUnknown, "SIP", 500,NULL,NULL);
 	return -1;
 }
 
 SalMediaDescription * sal_call_get_remote_media_description(SalOp *h){
-	return h->base.remote_media;;
+	return h->base.remote_media;
 }
 
 SalMediaDescription * sal_call_get_final_media_description(SalOp *h){
@@ -985,7 +1081,7 @@ int sal_call_send_dtmf(SalOp *h, char dtmf){
 	if (h->dialog && (belle_sip_dialog_get_state(h->dialog) == BELLE_SIP_DIALOG_CONFIRMED || belle_sip_dialog_get_state(h->dialog) == BELLE_SIP_DIALOG_EARLY)){
 		belle_sip_request_t *req=belle_sip_dialog_create_queued_request(h->dialog,"INFO");
 		if (req){
-			int bodylen;
+			size_t bodylen;
 			char dtmf_body[128]={0};
 
 			snprintf(dtmf_body, sizeof(dtmf_body)-1, "Signal=%c\r\nDuration=250\r\n", dtmf);
@@ -999,25 +1095,44 @@ int sal_call_send_dtmf(SalOp *h, char dtmf){
 	return 0;
 }
 
-int sal_call_terminate(SalOp *op){
-	belle_sip_dialog_state_t dialog_state=op->dialog?belle_sip_dialog_get_state(op->dialog):BELLE_SIP_DIALOG_NULL;
+
+int sal_call_terminate_with_error(SalOp *op, const SalErrorInfo *info){
+	SalErrorInfo sei = { 0 };
+	const SalErrorInfo *p_sei;
+	belle_sip_dialog_state_t dialog_state = op->dialog ? belle_sip_dialog_get_state(op->dialog) : BELLE_SIP_DIALOG_NULL;
+	int ret = 0;
+	
+	if (info == NULL && dialog_state != BELLE_SIP_DIALOG_CONFIRMED && op->dir == SalOpDirIncoming){
+		/*the purpose of this line is to set a default SalErrorInfo for declining an incoming call (not yet established of course) */
+		sal_error_info_set(&sei,SalReasonDeclined, "SIP", 0, NULL, NULL);
+		p_sei = &sei;
+	} else{
+		p_sei = info;
+	}
 	if (op->state==SalOpStateTerminating || op->state==SalOpStateTerminated) {
 		ms_error("Cannot terminate op [%p] in state [%s]",op,sal_op_state_to_string(op->state));
-		return -1;
+		ret = -1;
+		goto end;
 	}
 	switch(dialog_state) {
 		case BELLE_SIP_DIALOG_CONFIRMED: {
-			sal_op_send_request(op,belle_sip_dialog_create_request(op->dialog,"BYE"));
+			belle_sip_request_t * req = belle_sip_dialog_create_request(op->dialog,"BYE");
+			if (info != NULL){
+				belle_sip_header_reason_t* reason = sal_call_make_reason_header(info);
+				belle_sip_message_add_header(BELLE_SIP_MESSAGE(req),BELLE_SIP_HEADER(reason));
+			}
+			sal_op_send_request(op,req);
 			op->state=SalOpStateTerminating;
 			break;
 		}
+
 		case BELLE_SIP_DIALOG_NULL: {
 			if (op->dir == SalOpDirIncoming) {
-				sal_call_decline(op, SalReasonDeclined,NULL);
+				sal_call_decline_with_error_info(op, p_sei, NULL);
 				op->state=SalOpStateTerminated;
 			} else if (op->pending_client_trans){
 				if (belle_sip_transaction_get_state(BELLE_SIP_TRANSACTION(op->pending_client_trans)) == BELLE_SIP_TRANSACTION_PROCEEDING){
-					cancelling_invite(op);
+					cancelling_invite(op, p_sei);
 				}else{
 					/* Case where the CANCEL cannot be sent because no provisional response was received so far.
 					 * The Op must be kept for the time of the transaction in case a response is received later.
@@ -1030,21 +1145,30 @@ int sal_call_terminate(SalOp *op){
 		}
 		case BELLE_SIP_DIALOG_EARLY: {
 			if (op->dir == SalOpDirIncoming) {
-				sal_call_decline(op, SalReasonDeclined,NULL);
+				sal_call_decline_with_error_info(op, p_sei,NULL);
 				op->state=SalOpStateTerminated;
 			} else  {
-				cancelling_invite(op);
+				cancelling_invite(op, p_sei);
 			}
 			break;
 		}
 		default: {
 			ms_error("sal_call_terminate not implemented yet for dialog state [%s]",belle_sip_dialog_state_to_string(dialog_state));
-			return -1;
+			ret = -1;
+			goto end;
 		}
 	}
-	return 0;
+end:
+	sal_error_info_reset(&sei);
+	return ret;
 }
 
+
+int sal_call_terminate(SalOp *op){
+	return sal_call_terminate_with_error(op, NULL);
+}
+
+
 bool_t sal_call_autoanswer_asked(SalOp *op){
 	return op->auto_answer_asked;
 }
@@ -1086,6 +1210,24 @@ int sal_call_is_offerer(const SalOp *h){
 	return h->sdp_offering;
 }
 
+bool_t sal_call_compare_op(const SalOp *op1, const SalOp *op2) {
+	if (strcmp(op1->base.call_id, op2->base.call_id) == 0) return TRUE;
+	return FALSE;
+}
 
+bool_t sal_call_dialog_request_pending(const SalOp *op) {
+	return belle_sip_dialog_request_pending(op->dialog) ? TRUE : FALSE;
+}
 
+const char * sal_call_get_local_tag(SalOp *op) {
+	return belle_sip_dialog_get_local_tag(op->dialog);
+}
 
+const char * sal_call_get_remote_tag(SalOp *op) {
+	return belle_sip_dialog_get_remote_tag(op->dialog);
+}
+
+void sal_call_set_replaces(SalOp *op, const char *call_id, const char *from_tag, const char *to_tag) {
+	belle_sip_header_replaces_t *replaces = belle_sip_header_replaces_create(call_id, from_tag, to_tag);
+	sal_op_set_replaces(op, replaces);
+}
