@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include "sal_impl.h"
 
@@ -30,6 +30,11 @@ SalOp * sal_op_new(Sal *sal){
 	return op;
 }
 
+void sal_op_kill_dialog(SalOp *op) {
+	ms_warning("op [%p]: force kill of dialog [%p]", op, op->dialog);
+	belle_sip_dialog_delete(op->dialog);
+}
+
 void sal_op_release(SalOp *op){
 	/*if in terminating state, keep this state because it means we are waiting for a response to be able to terminate the operation.*/
 	if (op->state!=SalOpStateTerminating)
@@ -154,6 +159,7 @@ belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
 	belle_sip_request_t *req;
 	belle_sip_uri_t* req_uri;
 	belle_sip_uri_t* to_uri;
+	belle_sip_header_call_id_t *call_id_header;
 
 	const SalAddress* to_address;
 	const MSList *elem=sal_op_get_route_addresses(op);
@@ -184,11 +190,15 @@ belle_sip_request_t* sal_op_build_request(SalOp *op,const char* method) {
 	belle_sip_uri_set_secure(req_uri,sal_op_is_secure(op));
 
 	to_header = belle_sip_header_to_create(BELLE_SIP_HEADER_ADDRESS(to_address),NULL);
+	call_id_header = belle_sip_provider_create_call_id(prov);
+	if (sal_op_get_call_id(op)) {
+		belle_sip_header_call_id_set_call_id(call_id_header, sal_op_get_call_id(op));
+	}
 
 	req=belle_sip_request_create(
 					req_uri,
 					method,
-					belle_sip_provider_create_call_id(prov),
+					call_id_header,
 					belle_sip_header_cseq_create(20,method),
 					from_header,
 					to_header,
@@ -235,6 +245,14 @@ int sal_ping(SalOp *op, const char *from, const char *to){
 	return sal_op_send_request(op,sal_op_build_request(op,"OPTIONS"));
 }
 
+void sal_op_set_replaces(SalOp* op,belle_sip_header_replaces_t* replaces) {
+	if (op->replaces){
+		belle_sip_object_unref(op->replaces);
+	}
+	op->replaces=replaces;
+	belle_sip_object_ref(op->replaces);
+}
+
 void sal_op_set_remote_ua(SalOp*op,belle_sip_message_t* message) {
 	belle_sip_header_user_agent_t* user_agent=belle_sip_message_get_header_by_type(message,belle_sip_header_user_agent_t);
 	char user_agent_string[256];
@@ -342,6 +360,9 @@ static int _sal_op_send_request_with_contact(SalOp* op, belle_sip_request_t* req
 			}
 #endif
 		}
+		/*because in case of tunnel, transport can be changed*/
+		transport=belle_sip_uri_get_transport_param(next_hop_uri);
+		
 		if ((strcmp(method,"REGISTER")==0 || strcmp(method,"SUBSCRIBE")==0) && transport &&
 			(strcasecmp(transport,"TCP")==0 || strcasecmp(transport,"TLS")==0)){
 			/*RFC 5923: add 'alias' parameter to tell the server that we want it to keep the connection for future requests*/
@@ -540,7 +561,8 @@ const SalErrorInfo *sal_error_info_none(void){
 		"Ok",
 		200,
 		NULL,
-		NULL
+		NULL,
+		
 	};
 	return &none;
 }
@@ -559,17 +581,28 @@ void sal_error_info_reset(SalErrorInfo *ei){
 		ms_free(ei->full_string);
 		ei->full_string=NULL;
 	}
+	if (ei->protocol){
+		ms_free(ei->protocol);
+		ei->protocol = NULL;
+	}
 	ei->protocol_code=0;
 	ei->reason=SalReasonNone;
+	ei->sub_sei = NULL;
 }
 
-void sal_error_info_set(SalErrorInfo *ei, SalReason reason, int code, const char *status_string, const char *warning){
+void sal_error_info_set(SalErrorInfo *ei, SalReason reason, const char *protocol, int code, const char *status_string, const char *warning){
 	sal_error_info_reset(ei);
-	if (reason==SalReasonUnknown) ei->reason=_sal_reason_from_sip_code(code);
-	else ei->reason=reason;
+	if (reason==SalReasonUnknown && strcmp(protocol, "SIP") == 0 && code != 0) ei->reason=_sal_reason_from_sip_code(code);
+	else{
+		ei->reason=reason;
+		if (code == 0) {
+			code = sal_reason_to_sip_code(reason);
+		}
+	}
 	ei->protocol_code=code;
 	ei->status_string=status_string ? ms_strdup(status_string) : NULL;
 	ei->warnings=warning ? ms_strdup(warning) : NULL;
+	ei->protocol = protocol ? ms_strdup(protocol) : NULL;
 	if (ei->status_string){
 		if (ei->warnings)
 			ei->full_string=ms_strdup_printf("%s %s",ei->status_string,ei->warnings);
@@ -577,24 +610,40 @@ void sal_error_info_set(SalErrorInfo *ei, SalReason reason, int code, const char
 	}
 }
 
+void sal_op_set_reason_error_info(SalOp *op, belle_sip_message_t *msg){
+	belle_sip_header_reason_t* reason_header = belle_sip_message_get_header_by_type(msg,belle_sip_header_reason_t);
+	if (reason_header){
+		SalErrorInfo *ei=&op->reason_error_info; // ?//
+		const char *protocol = belle_sip_header_reason_get_protocol(reason_header);
+		int code = belle_sip_header_reason_get_cause(reason_header);
+		const char *text = belle_sip_header_reason_get_text(reason_header);
+		sal_error_info_set(ei, SalReasonUnknown, protocol, code, text, NULL);
+	}
+}
+
 void sal_op_set_error_info_from_response(SalOp *op, belle_sip_response_t *response){
 	int code = belle_sip_response_get_status_code(response);
 	const char *reason_phrase=belle_sip_response_get_reason_phrase(response);
-	/*Remark: the reason header is to be used mainly in SIP requests, thus the use and prototype of this function should be changed.*/
-	belle_sip_header_t* reason_header = belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Reason");
 	belle_sip_header_t *warning=belle_sip_message_get_header(BELLE_SIP_MESSAGE(response),"Warning");
 	SalErrorInfo *ei=&op->error_info;
 	const char *warnings;
 
 	warnings=warning ? belle_sip_header_get_unparsed_value(warning) : NULL;
-	if (warnings==NULL) warnings=reason_header ? belle_sip_header_get_unparsed_value(reason_header) : NULL;
-	sal_error_info_set(ei,SalReasonUnknown,code,reason_phrase,warnings);
+	sal_error_info_set(ei,SalReasonUnknown,"SIP", code,reason_phrase,warnings);
+	sal_op_set_reason_error_info(op, BELLE_SIP_MESSAGE(response));
 }
 
 const SalErrorInfo *sal_op_get_error_info(const SalOp *op){
 	return &op->error_info;
 }
 
+const SalErrorInfo * sal_op_get_reason_error_info(const SalOp *op){
+	return &op->reason_error_info;
+}
+
+
+
+
 static void unlink_op_with_dialog(SalOp *op, belle_sip_dialog_t* dialog){
 	belle_sip_dialog_set_application_data(dialog,NULL);
 	sal_op_unref(op);
@@ -738,7 +787,7 @@ void sal_op_set_manual_refresher_mode(SalOp *op, bool_t enabled){
 	op->manual_refresher=enabled;
 }
 
-bool_t sal_op_is_ipv6(SalOp *op){
+int sal_op_get_address_family(SalOp *op){
 	belle_sip_transaction_t *tr=NULL;
 	belle_sip_header_address_t *contact;
 	
@@ -753,7 +802,7 @@ bool_t sal_op_is_ipv6(SalOp *op){
 	
 	if (tr==NULL){
 		ms_error("Unable to determine IP version from signaling operation.");
-		return FALSE;
+		return AF_UNSPEC;
 	}
 	
 	
@@ -762,16 +811,16 @@ bool_t sal_op_is_ipv6(SalOp *op){
 		belle_sip_header_via_t *via = resp ?belle_sip_message_get_header_by_type(resp,belle_sip_header_via_t):NULL;
 		if (!via){
 			ms_error("Unable to determine IP version from signaling operation, no via header found.");
-			return FALSE;
+			return AF_UNSPEC;
 		}
-		return strchr(belle_sip_header_via_get_host(via),':') != NULL;
+		return (strchr(belle_sip_header_via_get_host(via),':') != NULL) ? AF_INET6 : AF_INET;
 	} else {
 		belle_sip_request_t *req = belle_sip_transaction_get_request(tr);
 		contact=(belle_sip_header_address_t*)belle_sip_message_get_header_by_type(req,belle_sip_header_contact_t);
 		if (!contact){
 			ms_error("Unable to determine IP version from signaling operation, no contact header found.");
 		}
-		return sal_address_is_ipv6((SalAddress*)contact);
+		return sal_address_is_ipv6((SalAddress*)contact) ? AF_INET6 : AF_INET;
 	}
 }
 
@@ -821,3 +870,26 @@ void sal_op_set_event(SalOp *op, const char *eventname){
 	}
 	op->event = header;
 }
+
+const char* sal_op_get_public_address(SalOp *op, int *port) {
+	if (op && op->refresher) {
+		return belle_sip_refresher_get_public_address(op->refresher, port);
+	}
+	return NULL;
+}
+
+const char* sal_op_get_local_address(SalOp *op, int *port) {
+	if (op && op->refresher) {
+		return belle_sip_refresher_get_local_address(op->refresher, port);
+	}
+	return NULL;
+}
+
+char* sal_op_get_dialog_id(const SalOp *op) {
+	if (op->dialog != NULL) {
+		return ms_strdup_printf("%s;to-tag=%s;from-tag=%s", ((SalOpBase*)op)->call_id,
+			belle_sip_dialog_get_remote_tag(op->dialog), belle_sip_dialog_get_local_tag(op->dialog));
+	}
+	return NULL;
+}
+
