@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include "sal_impl.h"
 
@@ -83,7 +83,7 @@ static void presence_process_dialog_terminated(void *ctx, const belle_sip_dialog
 	}/* else client dialog is managed by refresher*/
 }
 
-static void presence_refresher_listener(belle_sip_refresher_t* refresher, void* user_pointer, unsigned int status_code, const char* reason_phrase){
+static void presence_refresher_listener(belle_sip_refresher_t* refresher, void* user_pointer, unsigned int status_code, const char* reason_phrase, int will_retry){
 	SalOp* op = (SalOp*)user_pointer;
 	if (status_code >= 300) {
 		ms_message("The SUBSCRIBE dialog no longer works. Let's restart a new one.");
@@ -153,7 +153,7 @@ static void presence_response_event(void *op_base, const belle_sip_response_even
 					belle_sip_object_unref(op->refresher);
 					op->refresher=NULL;
 				}
-				if (expires>0){
+				if ((expires != NULL) && (belle_sip_header_expires_get_expires(expires) > 0)) {
 					op->refresher=belle_sip_client_transaction_create_refresher(client_transaction);
 					belle_sip_refresher_set_listener(op->refresher,presence_refresher_listener,op);
 					belle_sip_refresher_set_realm(op->refresher,op->base.realm);
