@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include "sal_impl.h"
 #define keywordcmp(key,b) strncmp(key,b,sizeof(key))
@@ -139,7 +139,7 @@ static void add_rtcp_fb_attributes(belle_sdp_media_description_t *media_desc, co
 	for (pt_it = stream->payloads; pt_it != NULL; pt_it = pt_it->next) {
 		pt = (PayloadType *)pt_it->data;
 
-		/* AVPF/SAVPF profile is used so enable AVPF for all paylad types. */
+		/* AVPF/SAVPF profile is used so enable AVPF for all payload types. */
 		payload_type_set_flag(pt, PAYLOAD_TYPE_RTCP_FEEDBACK_ENABLED);
 		avpf_params = payload_type_get_avpf_params(pt);
 
@@ -539,7 +539,7 @@ static void sdp_parse_media_ice_parameters(belle_sdp_media_description_t *media_
 		att_name = belle_sdp_attribute_get_name(attribute);
 		value = belle_sdp_attribute_get_value(attribute);
 
-		if (	(nb_ice_candidates < sizeof (stream->ice_candidates)/sizeof(SalIceCandidate))
+		if ((nb_ice_candidates < (int)(sizeof(stream->ice_candidates)/sizeof(SalIceCandidate)))
 				&& (keywordcmp("candidate", att_name) == 0)
 				&& (value != NULL)) {
 			SalIceCandidate *candidate = &stream->ice_candidates[nb_ice_candidates];
@@ -548,7 +548,10 @@ static void sdp_parse_media_ice_parameters(belle_sdp_media_description_t *media_
 				candidate->foundation, &candidate->componentID, proto, &candidate->priority, candidate->addr, &candidate->port,
 				candidate->type, candidate->raddr, &candidate->rport);
 			if (strcasecmp("udp",proto)==0 && ((nb == 7) || (nb == 9))) nb_ice_candidates++;
-			else memset(candidate, 0, sizeof(*candidate));
+			else {
+				ms_error("ice: Failed parsing a=candidate SDP attribute");
+				memset(candidate, 0, sizeof(*candidate));
+			}
 		} else if ((keywordcmp("remote-candidates", att_name) == 0) && (value != NULL)) {
 			SalIceRemoteCandidate candidate;
 			unsigned int componentID;
