@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #define _XOPEN_SOURCE 700 /*required for strptime of GNU libc*/
@@ -24,7 +24,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
 #ifdef SQLITE_STORAGE_ENABLED
 #ifndef _WIN32
-#if !defined(ANDROID) && !defined(__QNXNTO__)
+#if !defined(__ANDROID__) && !defined(__QNXNTO__)
 #	include <langinfo.h>
 #	include <iconv.h>
 #	include <string.h>
@@ -37,6 +37,11 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "sqlite3.h"
 #endif
 
+typedef struct _CallLogStorageResult {
+	LinphoneCore *core;
+	bctbx_list_t *result;
+} CallLogStorageResult;
+
 /*******************************************************************************
  * Internal functions                                                          *
  ******************************************************************************/
@@ -76,11 +81,13 @@ static void set_call_log_date(LinphoneCallLog *cl, time_t start_time){
 void call_logs_write_to_config_file(LinphoneCore *lc){
 	bctbx_list_t *elem;
 	char logsection[32];
-	unsigned int i;
+	int i;
 	char *tmp;
 	LpConfig *cfg=lc->config;
 
 	if (linphone_core_get_global_state (lc)==LinphoneGlobalStartup) return;
+	
+	if (lc->max_call_logs == LINPHONE_MAX_CALL_HISTORY_UNLIMITED) return;
 
 	for(i=0,elem=lc->call_logs;elem!=NULL;elem=elem->next,++i){
 		LinphoneCallLog *cl=(LinphoneCallLog*)elem->data;
@@ -109,12 +116,14 @@ void call_logs_write_to_config_file(LinphoneCore *lc){
 	}
 }
 
-void call_logs_read_from_config_file(LinphoneCore *lc){
+bctbx_list_t * call_logs_read_from_config_file(LinphoneCore *lc){
 	char logsection[32];
 	int i;
 	const char *tmp;
 	uint64_t sec;
 	LpConfig *cfg=lc->config;
+	bctbx_list_t *call_logs = NULL;
+
 	for(i=0;;++i){
 		snprintf(logsection,sizeof(logsection),"call_log_%i",i);
 		if (lp_config_has_section(cfg,logsection)){
@@ -147,9 +156,10 @@ void call_logs_read_from_config_file(LinphoneCore *lc){
 			cl->video_enabled=lp_config_get_int(cfg,logsection,"video_enabled",0);
 			tmp=lp_config_get_string(cfg,logsection,"call_id",NULL);
 			if (tmp) cl->call_id=ms_strdup(tmp);
-			lc->call_logs=bctbx_list_append(lc->call_logs,cl);
+			call_logs=bctbx_list_append(call_logs,cl);
 		}else break;
 	}
+	return call_logs;
 }
 
 
@@ -252,6 +262,10 @@ bool_t linphone_call_log_was_conference(LinphoneCallLog *cl) {
 	return cl->was_conference;
 }
 
+const LinphoneErrorInfo *linphone_call_log_get_error_info(LinphoneCallLog *cl){
+	return cl->error_info;
+}
+
 
 /*******************************************************************************
  * Reference and user data handling functions                                  *
@@ -279,13 +293,14 @@ void linphone_call_log_unref(LinphoneCallLog *cl) {
  ******************************************************************************/
 
 static void _linphone_call_log_destroy(LinphoneCallLog *cl) {
-	if (cl->from!=NULL) linphone_address_destroy(cl->from);
-	if (cl->to!=NULL) linphone_address_destroy(cl->to);
+	if (cl->from!=NULL) linphone_address_unref(cl->from);
+	if (cl->to!=NULL) linphone_address_unref(cl->to);
 	if (cl->refkey!=NULL) ms_free(cl->refkey);
 	if (cl->call_id) ms_free(cl->call_id);
 	if (cl->reporting.reports[LINPHONE_CALL_STATS_AUDIO]!=NULL) linphone_reporting_destroy(cl->reporting.reports[LINPHONE_CALL_STATS_AUDIO]);
 	if (cl->reporting.reports[LINPHONE_CALL_STATS_VIDEO]!=NULL) linphone_reporting_destroy(cl->reporting.reports[LINPHONE_CALL_STATS_VIDEO]);
 	if (cl->reporting.reports[LINPHONE_CALL_STATS_TEXT]!=NULL) linphone_reporting_destroy(cl->reporting.reports[LINPHONE_CALL_STATS_TEXT]);
+	if (cl->error_info) linphone_error_info_unref(cl->error_info);
 }
 
 LinphoneCallLog * linphone_call_log_new(LinphoneCallDir dir, LinphoneAddress *from, LinphoneAddress *to) {
@@ -306,6 +321,7 @@ LinphoneCallLog * linphone_call_log_new(LinphoneCallDir dir, LinphoneAddress *fr
 	return cl;
 }
 
+
 /* DEPRECATED */
 void linphone_call_log_destroy(LinphoneCallLog *cl) {
 	belle_sip_object_unref(cl);
@@ -327,7 +343,7 @@ BELLE_SIP_INSTANCIATE_VPTR(LinphoneCallLog, belle_sip_object_t,
 
 #ifdef SQLITE_STORAGE_ENABLED
 
-static void linphone_create_table(sqlite3* db) {
+static void linphone_create_call_log_table(sqlite3* db) {
 	char* errmsg=NULL;
 	int ret;
 	ret=sqlite3_exec(db,"CREATE TABLE IF NOT EXISTS call_history ("
@@ -384,7 +400,7 @@ void linphone_core_call_log_storage_init(LinphoneCore *lc) {
 		return;
 	}
 
-	linphone_create_table(db);
+	linphone_create_call_log_table(db);
 	linphone_update_call_log_table(db);
 	lc->logs_db = db;
 
@@ -399,6 +415,15 @@ void linphone_core_call_log_storage_close(LinphoneCore *lc) {
 	}
 }
 
+static LinphoneCallLog * find_call_log_by_storage_id(bctbx_list_t *call_logs, unsigned int storage_id) {
+	bctbx_list_t *item;
+	for (item = call_logs; item != NULL; item = bctbx_list_next(item)) {
+		LinphoneCallLog *call_log = bctbx_list_get_data(item);
+		if (call_log->storage_id == storage_id) return call_log;
+	}
+	return NULL;
+}
+
 /* DB layout:
  * | 0  | storage_id
  * | 1  | from
@@ -414,13 +439,20 @@ void linphone_core_call_log_storage_close(LinphoneCore *lc) {
  * | 11 | refkey
  */
 static int create_call_log(void *data, int argc, char **argv, char **colName) {
-	bctbx_list_t **list = (bctbx_list_t **)data;
+	CallLogStorageResult *clsres = (CallLogStorageResult *)data;
 	LinphoneAddress *from;
 	LinphoneAddress *to;
 	LinphoneCallDir dir;
 	LinphoneCallLog *log;
 
 	unsigned int storage_id = (unsigned int)atoi(argv[0]);
+
+	log = find_call_log_by_storage_id(clsres->core->call_logs, storage_id);
+	if (log != NULL) {
+		clsres->result = bctbx_list_append(clsres->result, linphone_call_log_ref(log));
+		return 0;
+	}
+
 	from = linphone_address_new(argv[1]);
 	to = linphone_address_new(argv[2]);
 	
@@ -447,24 +479,24 @@ static int create_call_log(void *data, int argc, char **argv, char **colName) {
 		}
 	}
 
-	*list = bctbx_list_append(*list, log);
+	clsres->result = bctbx_list_append(clsres->result, log);
 	return 0;
 	
 error:
 	if (from){
-		linphone_address_destroy(from);
+		linphone_address_unref(from);
 	}
 	if (to){
-		linphone_address_destroy(to);
+		linphone_address_unref(to);
 	}
 	ms_error("Bad call log at storage_id %u", storage_id);
 	return 0;
 }
 
-static void linphone_sql_request_call_log(sqlite3 *db, const char *stmt, bctbx_list_t **list) {
+static void linphone_sql_request_call_log(sqlite3 *db, const char *stmt, CallLogStorageResult *clsres) {
 	char* errmsg = NULL;
 	int ret;
-	ret = sqlite3_exec(db, stmt, create_call_log, list, &errmsg);
+	ret = sqlite3_exec(db, stmt, create_call_log, clsres, &errmsg);
 	if (ret != SQLITE_OK) {
 		ms_error("linphone_sql_request: statement %s -> error sqlite3_exec(): %s.", stmt, errmsg);
 		sqlite3_free(errmsg);
@@ -515,47 +547,29 @@ void linphone_core_store_call_log(LinphoneCore *lc, LinphoneCallLog *log) {
 	}
 }
 
-static void copy_user_data_from_existing_log(bctbx_list_t *existing_logs, LinphoneCallLog *log) {
-	while (existing_logs) {
-		LinphoneCallLog *existing_log = (LinphoneCallLog *)existing_logs->data;
-		if (existing_log->storage_id == log->storage_id) {
-			log->user_data = existing_log->user_data;
-			break;
-		}
-		existing_logs = bctbx_list_next(existing_logs);
-	}
-}
-
-static void copy_user_data_from_existing_logs(bctbx_list_t *existing_logs, bctbx_list_t *new_logs) {
-	while (new_logs) {
-		LinphoneCallLog *new_log = (LinphoneCallLog *)new_logs->data;
-		copy_user_data_from_existing_log(existing_logs, new_log);
-		new_logs = bctbx_list_next(new_logs);
-	}
-}
-
 const bctbx_list_t *linphone_core_get_call_history(LinphoneCore *lc) {
 	char *buf;
 	uint64_t begin,end;
-	bctbx_list_t *result = NULL;
+	CallLogStorageResult clsres;
 
 	if (!lc || lc->logs_db == NULL) return NULL;
+    if (lc->call_logs != NULL) return lc->call_logs;
 
-	buf = sqlite3_mprintf("SELECT * FROM call_history ORDER BY id DESC LIMIT %u", lc->max_call_logs);
+	if (lc->max_call_logs != LINPHONE_MAX_CALL_HISTORY_UNLIMITED){
+		buf = sqlite3_mprintf("SELECT * FROM call_history ORDER BY id DESC LIMIT %i", lc->max_call_logs);
+	}else{
+		buf = sqlite3_mprintf("SELECT * FROM call_history ORDER BY id DESC");
+	}
 
+	clsres.core = lc;
+	clsres.result = NULL;
 	begin = ortp_get_cur_time_ms();
-	linphone_sql_request_call_log(lc->logs_db, buf, &result);
+	linphone_sql_request_call_log(lc->logs_db, buf, &clsres);
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
 
-	if (lc->call_logs) {
-		copy_user_data_from_existing_logs(lc->call_logs, result);
-	}
-
-	lc->call_logs = bctbx_list_free_with_data(lc->call_logs, (void (*)(void*))linphone_call_log_unref);
-	lc->call_logs = result;
-
+	lc->call_logs = clsres.result;
 	return lc->call_logs;
 }
 
@@ -604,7 +618,7 @@ bctbx_list_t * linphone_core_get_call_history_for_address(LinphoneCore *lc, cons
 	char *buf;
 	char *sipAddress;
 	uint64_t begin,end;
-	bctbx_list_t *result = NULL;
+	CallLogStorageResult clsres;
 
 	if (!lc || lc->logs_db == NULL || addr == NULL) return NULL;
 
@@ -612,43 +626,39 @@ bctbx_list_t * linphone_core_get_call_history_for_address(LinphoneCore *lc, cons
 	sipAddress = linphone_address_as_string_uri_only(addr);
 	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE caller LIKE '%%%q%%' OR callee LIKE '%%%q%%' ORDER BY id DESC", sipAddress, sipAddress); // The '%%%q%%' takes care of the eventual presence of a display name
 
+	clsres.core = lc;
+	clsres.result = NULL;
 	begin = ortp_get_cur_time_ms();
-	linphone_sql_request_call_log(lc->logs_db, buf, &result);
+	linphone_sql_request_call_log(lc->logs_db, buf, &clsres);
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
 	ms_free(sipAddress);
 
-	if (lc->call_logs) {
-		copy_user_data_from_existing_logs(lc->call_logs, result);
-	}
-
-	return result;
+	return clsres.result;
 }
 
 LinphoneCallLog * linphone_core_get_last_outgoing_call_log(LinphoneCore *lc) {
 	char *buf;
 	uint64_t begin,end;
-	bctbx_list_t *list = NULL;
-	LinphoneCallLog* result = NULL;
+	CallLogStorageResult clsres;
+	LinphoneCallLog *result = NULL;
 
 	if (!lc || lc->logs_db == NULL) return NULL;
 
 	/*since we want to append query parameters depending on arguments given, we use malloc instead of sqlite3_mprintf*/
 	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE direction = 0 ORDER BY id DESC LIMIT 1");
 
+	clsres.core = lc;
+	clsres.result = NULL;
 	begin = ortp_get_cur_time_ms();
-	linphone_sql_request_call_log(lc->logs_db, buf, &list);
+	linphone_sql_request_call_log(lc->logs_db, buf, &clsres);
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
 
-	if (list) {
-		result = (LinphoneCallLog*)list->data;
-	}
-
-	if (lc->call_logs && result) {
-		copy_user_data_from_existing_log(lc->call_logs, result);
+	if (clsres.result != NULL) {
+		result = (LinphoneCallLog *)bctbx_list_get_data(clsres.result);
 	}
 
 	return result;
@@ -657,7 +667,7 @@ LinphoneCallLog * linphone_core_get_last_outgoing_call_log(LinphoneCore *lc) {
 LinphoneCallLog * linphone_core_find_call_log_from_call_id(LinphoneCore *lc, const char *call_id) {
 	char *buf;
 	uint64_t begin,end;
-	bctbx_list_t *list = NULL;
+	CallLogStorageResult clsres;
 	LinphoneCallLog* result = NULL;
 
 	if (!lc || lc->logs_db == NULL) return NULL;
@@ -665,18 +675,16 @@ LinphoneCallLog * linphone_core_find_call_log_from_call_id(LinphoneCore *lc, con
 	/*since we want to append query parameters depending on arguments given, we use malloc instead of sqlite3_mprintf*/
 	buf = sqlite3_mprintf("SELECT * FROM call_history WHERE call_id = '%q' ORDER BY id DESC LIMIT 1", call_id);
 
+	clsres.core = lc;
+	clsres.result = NULL;
 	begin = ortp_get_cur_time_ms();
-	linphone_sql_request_call_log(lc->logs_db, buf, &list);
+	linphone_sql_request_call_log(lc->logs_db, buf, &clsres);
 	end = ortp_get_cur_time_ms();
 	ms_message("%s(): completed in %i ms",__FUNCTION__, (int)(end-begin));
 	sqlite3_free(buf);
 
-	if (list) {
-		result = (LinphoneCallLog*)list->data;
-	}
-
-	if (lc->call_logs && result) {
-		copy_user_data_from_existing_log(lc->call_logs, result);
+	if (clsres.result != NULL) {
+		result = (LinphoneCallLog *)bctbx_list_get_data(clsres.result);
 	}
 
 	return result;
