@@ -14,9 +14,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
+#include "linphone/call_params.h"
 #include "private.h"
 
 
@@ -128,21 +129,7 @@ void linphone_call_params_clear_custom_sdp_media_attributes(LinphoneCallParams *
 }
 
 LinphoneCallParams * linphone_call_params_copy(const LinphoneCallParams *cp){
-	unsigned int i;
-	LinphoneCallParams *ncp=linphone_call_params_new();
-	memcpy(ncp,cp,sizeof(LinphoneCallParams));
-	if (cp->record_file) ncp->record_file=ms_strdup(cp->record_file);
-	if (cp->session_name) ncp->session_name=ms_strdup(cp->session_name);
-	/*
-	 * The management of the custom headers is not optimal. We copy everything while ref counting would be more efficient.
-	 */
-	if (cp->custom_headers) ncp->custom_headers=sal_custom_header_clone(cp->custom_headers);
-	if (cp->custom_sdp_attributes) ncp->custom_sdp_attributes = sal_custom_sdp_attribute_clone(cp->custom_sdp_attributes);
-	for (i = 0; i < (unsigned int)LinphoneStreamTypeUnknown; i++) {
-		if (cp->custom_sdp_media_attributes[i]) ncp->custom_sdp_media_attributes[i] = sal_custom_sdp_attribute_clone(cp->custom_sdp_media_attributes[i]);
-	}
-
-	return ncp;
+	return (LinphoneCallParams *)belle_sip_object_clone((const belle_sip_object_t *)cp);
 }
 
 bool_t linphone_call_params_early_media_sending_enabled(const LinphoneCallParams *cp){
@@ -163,7 +150,7 @@ void linphone_call_params_enable_audio(LinphoneCallParams *cp, bool_t enabled){
 		cp->audio_dir=LinphoneMediaDirectionSendRecv;
 }
 
-int linphone_call_params_enable_realtime_text(LinphoneCallParams *params, bool_t yesno) {
+LinphoneStatus linphone_call_params_enable_realtime_text(LinphoneCallParams *params, bool_t yesno) {
 	params->realtimetext_enabled=yesno;
 	return 0;
 }
@@ -206,6 +193,10 @@ MSVideoSize linphone_call_params_get_received_video_size(const LinphoneCallParam
 	return cp->recv_vsize;
 }
 
+const LinphoneVideoDefinition * linphone_call_params_get_received_video_definition(const LinphoneCallParams *cp) {
+	return cp->recv_vdef;
+}
+
 const char *linphone_call_params_get_record_file(const LinphoneCallParams *cp){
 	return cp->record_file;
 }
@@ -222,19 +213,35 @@ MSVideoSize linphone_call_params_get_sent_video_size(const LinphoneCallParams *c
 	return cp->sent_vsize;
 }
 
+const LinphoneVideoDefinition * linphone_call_params_get_sent_video_definition(const LinphoneCallParams *cp) {
+	return cp->sent_vdef;
+}
+
 const char *linphone_call_params_get_session_name(const LinphoneCallParams *cp){
 	return cp->session_name;
 }
 
-const LinphonePayloadType* linphone_call_params_get_used_audio_codec(const LinphoneCallParams *cp) {
+LinphonePayloadType *linphone_call_params_get_used_audio_payload_type(const LinphoneCallParams *cp) {
+	return cp->audio_codec ? linphone_payload_type_new(NULL, cp->audio_codec) : NULL;
+}
+
+LinphonePayloadType *linphone_call_params_get_used_video_payload_type(const LinphoneCallParams *cp) {
+	return cp->video_codec ? linphone_payload_type_new(NULL, cp->video_codec) : NULL;
+}
+
+LinphonePayloadType *linphone_call_params_get_used_text_payload_type(const LinphoneCallParams *cp) {
+	return cp->text_codec ? linphone_payload_type_new(NULL, cp->text_codec) : NULL;
+}
+
+const OrtpPayloadType *linphone_call_params_get_used_audio_codec(const LinphoneCallParams *cp) {
 	return cp->audio_codec;
 }
 
-const LinphonePayloadType* linphone_call_params_get_used_video_codec(const LinphoneCallParams *cp) {
+const OrtpPayloadType *linphone_call_params_get_used_video_codec(const LinphoneCallParams *cp) {
 	return cp->video_codec;
 }
 
-const LinphonePayloadType* linphone_call_params_get_used_text_codec(const LinphoneCallParams *cp) {
+const OrtpPayloadType *linphone_call_params_get_used_text_codec(const LinphoneCallParams *cp) {
 	return cp->text_codec;
 }
 
@@ -340,7 +347,7 @@ bool_t linphone_call_params_video_multicast_enabled(const LinphoneCallParams *pa
  * Constructor and destructor functions                                        *
  ******************************************************************************/
 
-static void _linphone_call_params_destroy(LinphoneCallParams *cp){
+static void _linphone_call_params_uninit(LinphoneCallParams *cp){
 	unsigned int i;
 	if (cp->record_file) ms_free(cp->record_file);
 	if (cp->custom_headers) sal_custom_header_free(cp->custom_headers);
@@ -349,6 +356,32 @@ static void _linphone_call_params_destroy(LinphoneCallParams *cp){
 		if (cp->custom_sdp_media_attributes[i]) sal_custom_sdp_attribute_free(cp->custom_sdp_media_attributes[i]);
 	}
 	if (cp->session_name) ms_free(cp->session_name);
+	if (cp->sent_vdef != NULL) linphone_video_definition_unref(cp->sent_vdef);
+	if (cp->recv_vdef != NULL) linphone_video_definition_unref(cp->recv_vdef);
+}
+
+static void _linphone_call_params_clone(LinphoneCallParams *dst, const LinphoneCallParams *src) {
+	unsigned int i;
+	
+	/*
+	 * Save the belle_sip_object_t part, copy the entire structure and restore the belle_sip_object_t part
+	 */
+	belle_sip_object_t tmp = dst->base;
+	memcpy(dst, src, sizeof(LinphoneCallParams));
+	dst->base = tmp;
+
+	if (src->sent_vdef) dst->sent_vdef = linphone_video_definition_ref(src->sent_vdef);
+	if (src->recv_vdef) dst->recv_vdef = linphone_video_definition_ref(src->recv_vdef);
+	if (src->record_file) dst->record_file=ms_strdup(src->record_file);
+	if (src->session_name) dst->session_name=ms_strdup(src->session_name);
+	/*
+	 * The management of the custom headers is not optimal. We copy everything while ref counting would be more efficient.
+	 */
+	if (src->custom_headers) dst->custom_headers=sal_custom_header_clone(src->custom_headers);
+	if (src->custom_sdp_attributes) dst->custom_sdp_attributes = sal_custom_sdp_attribute_clone(src->custom_sdp_attributes);
+	for (i = 0; i < (unsigned int)LinphoneStreamTypeUnknown; i++) {
+		if (src->custom_sdp_media_attributes[i]) dst->custom_sdp_media_attributes[i] = sal_custom_sdp_attribute_clone(src->custom_sdp_media_attributes[i]);
+	}
 }
 
 LinphoneCallParams * linphone_call_params_new(void) {
@@ -368,8 +401,8 @@ void linphone_call_params_destroy(LinphoneCallParams *cp) {
 BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneCallParams);
 
 BELLE_SIP_INSTANCIATE_VPTR(LinphoneCallParams, belle_sip_object_t,
-	(belle_sip_object_destroy_t)_linphone_call_params_destroy,
-	NULL, // clone
+	(belle_sip_object_destroy_t)_linphone_call_params_uninit,
+	(belle_sip_object_clone_t)_linphone_call_params_clone, // clone
 	NULL, // marshal
 	FALSE
 );
