@@ -14,25 +14,25 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 
 LinphoneCardDavContext* linphone_carddav_context_new(LinphoneFriendList *lfl) {
 	LinphoneCardDavContext *carddav_context = NULL;
-	
+
+	if (!linphone_core_vcard_supported()) {
+		ms_error("[carddav] vCard isn't available (maybe it wasn't compiled), can't do CardDAV sync");
+		return NULL;
+	}
 	if (!lfl || !lfl->uri) {
 		return NULL;
 	}
-	
-#ifdef VCARD_ENABLED
+
 	carddav_context = (LinphoneCardDavContext *)ms_new0(LinphoneCardDavContext, 1);
 	carddav_context->friend_list = linphone_friend_list_ref(lfl);
-#else
-	ms_error("[carddav] vCard isn't available (maybe it wasn't compiled), can't do CardDAV sync");
-#endif
 	return carddav_context;
 }
 
@@ -43,7 +43,7 @@ void linphone_carddav_context_destroy(LinphoneCardDavContext *cdc) {
 			cdc->friend_list = NULL;
 		}
 		if (cdc->auth_info) {
-			linphone_auth_info_destroy(cdc->auth_info);
+			linphone_auth_info_unref(cdc->auth_info);
 			cdc->auth_info = NULL;
 		}
 		ms_free(cdc);
@@ -129,6 +129,7 @@ static void linphone_carddav_vcards_pulled(LinphoneCardDavContext *cdc, bctbx_li
 					ms_debug("Downloaded vCard etag/url are %s and %s", vCard->etag, full_url);
 
 					lf = linphone_friend_new_from_vcard(lvc);
+					linphone_vcard_unref(lvc); /*ref is now owned by friend*/
 					if (lf) {
 						local_friend = bctbx_list_find_custom(friends, (int (*)(const void*, const void*))find_matching_friend, lf);
 						
@@ -764,4 +765,4 @@ static LinphoneCardDavQuery* linphone_carddav_create_addressbook_multiget_query(
 void linphone_carddav_pull_vcards(LinphoneCardDavContext *cdc, bctbx_list_t *vcards_to_pull) {
 	LinphoneCardDavQuery *query = linphone_carddav_create_addressbook_multiget_query(cdc, vcards_to_pull);
 	linphone_carddav_send_query(query);
-}
\ No newline at end of file
+}
