@@ -23,6 +23,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include "linphone/core.h"
 #include "private.h"
 #include "conference_private.h"
 #include <mediastreamer2/msvolume.h>
@@ -32,6 +33,15 @@
 
 namespace Linphone {
 
+template <typename _type>
+inline std::list<_type> toStd(const bctbx_list_t *l){
+	std::list<_type> ret;
+	for(; l != NULL; l = l->next){
+		ret.push_back(static_cast<_type>(l->data));
+	}
+	return ret;
+}
+	
 class Conference {
 public:
 	class Participant {
@@ -81,7 +91,7 @@ public:
 		void enableVideo(bool enable) {m_enableVideo = enable;}
 		bool videoRequested() const {return m_enableVideo;}
 		void setStateChangedCallback(LinphoneConferenceStateChangedCb cb, void *userData) {
-			m_stateChangedCb =cb;
+			m_stateChangedCb = cb;
 			m_userData = userData;
 		}
 		
@@ -93,11 +103,12 @@ public:
 		friend class Conference;
 	};
 	
-	Conference(LinphoneCore *core, const Params *params = NULL);
+	Conference(LinphoneCore *core, LinphoneConference *conf, const Params *params = NULL);
 	virtual ~Conference() {};
 	
 	const Params &getCurrentParams() const {return m_currentParams;}
 	
+	virtual int inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params) = 0;
 	virtual int addParticipant(LinphoneCall *call) = 0;
 	virtual int removeParticipant(LinphoneCall *call) = 0;
 	virtual int removeParticipant(const LinphoneAddress *uri) = 0;
@@ -112,7 +123,7 @@ public:
 	bool microphoneIsMuted() const {return m_isMuted;}
 	float getInputVolume() const;
 	
-	virtual int getSize() const {return m_participants.size() + (isIn()?1:0);}
+	virtual int getSize() const {return (int)m_participants.size() + (isIn()?1:0);}
 	const std::list<Participant *> &getParticipants() const {return m_participants;}
 	
 	virtual int startRecording(const char *path) = 0;
@@ -123,6 +134,9 @@ public:
 	virtual void onCallTerminating(LinphoneCall *call) {};
 	
 	LinphoneConferenceState getState() const {return m_state;}
+	LinphoneCore *getCore()const{
+		return m_core;
+	}
 	static const char *stateToString(LinphoneConferenceState state);
 	
 protected:
@@ -137,13 +151,15 @@ protected:
 	std::list<Participant *> m_participants;
 	Params m_currentParams;
 	LinphoneConferenceState m_state;
+	LinphoneConference *m_conference;
 };
 
 class LocalConference: public Conference {
 public:
-	LocalConference(LinphoneCore *core, const Params *params = NULL);
+	LocalConference(LinphoneCore *core, LinphoneConference *conf, const Params *params = NULL);
 	virtual ~LocalConference();
 	
+	virtual int inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params);
 	virtual int addParticipant(LinphoneCall *call);
 	virtual int removeParticipant(LinphoneCall *call);
 	virtual int removeParticipant(const LinphoneAddress *uri);
@@ -178,9 +194,10 @@ private:
 
 class RemoteConference: public Conference {
 public:
-	RemoteConference(LinphoneCore *core, const Params *params = NULL);
+	RemoteConference(LinphoneCore *core, LinphoneConference *conf, const Params *params = NULL);
 	virtual ~RemoteConference();
 	
+	virtual int inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params);
 	virtual int addParticipant(LinphoneCall *call);
 	virtual int removeParticipant(LinphoneCall *call) {return -1;}
 	virtual int removeParticipant(const LinphoneAddress *uri);
@@ -208,7 +225,7 @@ private:
 	const char *m_focusAddr;
 	char *m_focusContact;
 	LinphoneCall *m_focusCall;
-	LinphoneCoreVTable *m_vtable;
+	LinphoneCoreCbs *m_coreCbs;
 	std::list<LinphoneCall *> m_pendingCalls;
 	std::list<LinphoneCall *> m_transferingCalls;
 };
@@ -220,19 +237,20 @@ using namespace Linphone;
 using namespace std;
 
 
-Conference::Conference(LinphoneCore *core, const Conference::Params *params):
+Conference::Conference(LinphoneCore *core, LinphoneConference *conf, const Conference::Params *params):
 	m_core(core),
 	m_localParticipantStream(NULL),
 	m_isMuted(false),
 	m_currentParams(),
-	m_state(LinphoneConferenceStopped) {
+	m_state(LinphoneConferenceStopped),
+	m_conference(conf) {
 	if(params) m_currentParams = *params;
 }
 
 int Conference::addParticipant(LinphoneCall *call) {
 	Participant *p =new Participant(call);
 	m_participants.push_back(p);
-	call->conf_ref = (LinphoneConference *)this;
+	call->conf_ref = m_conference;
 	return 0;
 }
 
@@ -301,7 +319,7 @@ void Conference::setState(LinphoneConferenceState state) {
 		ms_message("Switching conference [%p] into state '%s'", this, stateToString(state));
 		m_state = state;
 		if(m_currentParams.m_stateChangedCb) {
-			m_currentParams.m_stateChangedCb((LinphoneConference *)this, state, m_currentParams.m_userData);
+			m_currentParams.m_stateChangedCb(m_conference, state, m_currentParams.m_userData);
 		}
 	}
 }
@@ -323,8 +341,8 @@ Conference::Participant *Conference::findParticipant(const LinphoneAddress *uri)
 
 
 
-LocalConference::LocalConference(LinphoneCore *core, const Conference::Params *params):
-	Conference(core, params),
+LocalConference::LocalConference(LinphoneCore *core, LinphoneConference *conf, const Conference::Params *params):
+	Conference(core, conf, params),
 	m_conf(NULL),
 	m_localEndpoint(NULL),
 	m_recordEndpoint(NULL),
@@ -376,9 +394,38 @@ void LocalConference::addLocalEndpoint() {
 	_post_configure_audio_stream(st,m_core,FALSE);
 	m_localParticipantStream=st;
 	m_localEndpoint=ms_audio_endpoint_get_from_stream(st,FALSE);
+	ms_message("conference: adding local endpoint");
 	ms_audio_conference_add_member(m_conf,m_localEndpoint);
 }
 
+int LocalConference::inviteAddresses(const std::list<const LinphoneAddress*> &addresses, const LinphoneCallParams *params){
+	
+	for (std::list<const LinphoneAddress*>::const_iterator it = addresses.begin(); it != addresses.end(); ++it){
+		const LinphoneAddress *addr = *it;
+		LinphoneCall * call = linphone_core_get_call_by_remote_address2(m_core, addr);
+		if (!call){
+			/*start a new call by indicating that it has to be put into the conference directlly*/
+			LinphoneCallParams * new_params = params ? linphone_call_params_copy(params) : linphone_core_create_call_params(m_core, NULL);
+			LinphoneCall *call;
+			/*toggle this flag so the call is immediately added to the conference upon acceptance*/
+			new_params->in_conference = TRUE;
+			linphone_call_params_enable_video(new_params, FALSE); /*turn off video as it is not supported for conferencing at this time*/
+			call = linphone_core_invite_address_with_params(m_core, addr, new_params);
+			if (!call){
+				ms_error("LocalConference::inviteAddresses(): could not invite participant");
+			}
+			linphone_call_params_unref(new_params);
+		}else{
+			/*there is already a call to this address, so simply join it to the local conference if not already done*/
+			if (!call->current_params->in_conference)
+				addParticipant(call);
+		}
+		/*if the local participant is not yet created, created it and it to the conference */
+		if (!m_localEndpoint) addLocalEndpoint();
+	}
+	return 0;
+}
+
 int LocalConference::addParticipant(LinphoneCall *call) {
 	if (call->current_params->in_conference){
 		ms_error("Already in conference");
@@ -388,9 +435,9 @@ int LocalConference::addParticipant(LinphoneCall *call) {
 	if (call->state==LinphoneCallPaused){
 		call->params->in_conference=TRUE;
 		call->params->has_video=FALSE;
-		linphone_core_resume_call(m_core,call);
+		linphone_call_resume(call);
 	}else if (call->state==LinphoneCallStreamsRunning){
-		LinphoneCallParams *params=linphone_call_params_copy(linphone_call_get_current_params(call));
+		LinphoneCallParams *params = linphone_core_create_call_params(m_core, call);
 		params->in_conference=TRUE;
 		params->has_video=FALSE;
 
@@ -403,8 +450,8 @@ int LocalConference::addParticipant(LinphoneCall *call) {
 		}
 		/*this will trigger a reINVITE that will later redraw the streams */
 		/*FIXME probably a bit too much to just redraw streams !*/
-		linphone_core_update_call(m_core,call,params);
-		linphone_call_params_destroy(params);
+		linphone_call_update(call,params);
+		linphone_call_params_unref(params);
 		addLocalEndpoint();
 	}else{
 		ms_error("Call is in state %s, it cannot be added to the conference.",linphone_call_state_to_string(call->state));
@@ -440,11 +487,11 @@ int LocalConference::removeFromConference(LinphoneCall *call, bool_t active){
 			leave();
 		}
 		ms_message("Updating call to actually remove from conference");
-		err=linphone_core_update_call(m_core,call,params);
-		linphone_call_params_destroy(params);
+		err=linphone_call_update(call,params);
+		linphone_call_params_unref(params);
 	} else{
 		ms_message("Pausing call to actually remove from conference");
-		err=_linphone_core_pause_call(m_core,call);
+		err=_linphone_call_pause(call);
 	}
 	return err;
 }
@@ -513,7 +560,7 @@ int LocalConference::terminate() {
 		LinphoneCall *call=(LinphoneCall*)calls->data;
 		calls=calls->next;
 		if (call->current_params->in_conference) {
-			linphone_core_terminate_call(m_core, call);
+			linphone_call_terminate(call);
 		}
 	}
 	
@@ -528,7 +575,7 @@ int LocalConference::enter() {
 		return -1;
 	}
 	if (m_core->current_call != NULL) {
-		_linphone_core_pause_call(m_core, m_core->current_call);
+		_linphone_call_pause(m_core->current_call);
 	}
 	if (m_localParticipantStream==NULL) addLocalEndpoint();
 	return 0;
@@ -587,6 +634,7 @@ int LocalConference::stopRecording() {
 void LocalConference::onCallStreamStarting(LinphoneCall *call, bool isPausedByRemote) {
 	call->params->has_video = FALSE;
 	call->camera_enabled = FALSE;
+	ms_message("LocalConference::onCallStreamStarting(): joining AudioStream [%p] of call [%p] into conference.", call->audiostream, call);
 	MSAudioEndpoint *ep=ms_audio_endpoint_get_from_stream(call->audiostream,TRUE);
 	ms_audio_conference_add_member(m_conf,ep);
 	ms_audio_conference_mute_member(m_conf,ep,isPausedByRemote);
@@ -621,23 +669,28 @@ void LocalConference::onCallTerminating(LinphoneCall *call) {
 
 
 
-RemoteConference::RemoteConference(LinphoneCore *core, const Conference::Params *params): Conference(core, params) {
+RemoteConference::RemoteConference(LinphoneCore *core, LinphoneConference *conf, const Conference::Params *params): Conference(core, conf, params) {
 	m_focusAddr = NULL;
 	m_focusContact = NULL;
 	m_focusCall = NULL;
-	m_vtable = NULL;
+	m_coreCbs = NULL;
 	m_focusAddr = lp_config_get_string(m_core->config, "misc", "conference_focus_addr", "");
-	m_vtable = linphone_core_v_table_new();
-	m_vtable->call_state_changed = callStateChangedCb;
-	m_vtable->transfer_state_changed = transferStateChanged;
-	linphone_core_v_table_set_user_data(m_vtable, this);
-	_linphone_core_add_listener(m_core, m_vtable, FALSE, TRUE);
+	m_coreCbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	linphone_core_cbs_set_call_state_changed(m_coreCbs, callStateChangedCb);
+	linphone_core_cbs_set_transfer_state_changed(m_coreCbs, transferStateChanged);
+	linphone_core_cbs_set_user_data(m_coreCbs, this);
+	_linphone_core_add_callbacks(m_core, m_coreCbs, TRUE);
 }
 
 RemoteConference::~RemoteConference() {
 	terminate();
-	linphone_core_remove_listener(m_core, m_vtable);
-	linphone_core_v_table_destroy(m_vtable);
+	linphone_core_remove_callbacks(m_core, m_coreCbs);
+	linphone_core_cbs_unref(m_coreCbs);
+}
+
+int RemoteConference::inviteAddresses(const std::list<const LinphoneAddress *> &addresses, const LinphoneCallParams *params){
+	ms_error("RemoteConference::inviteAddresses() not implemented");
+	return -1;
 }
 
 int RemoteConference::addParticipant(LinphoneCall *call) {
@@ -724,7 +777,7 @@ int RemoteConference::terminate() {
 	switch(m_state) {
 		case LinphoneConferenceRunning:
 		case LinphoneConferenceStarting:
-			linphone_core_terminate_call(m_core, m_focusCall);
+			linphone_call_terminate(m_focusCall);
 			break;
 			
 		default: break;
@@ -741,7 +794,7 @@ int RemoteConference::enter() {
 	switch(callState) {
 		case LinphoneCallStreamsRunning: break;
 		case LinphoneCallPaused:
-			linphone_core_resume_call(m_core, m_focusCall);
+			linphone_call_resume(m_focusCall);
 			break;
 		default:
 			ms_error("Could not join the conference: bad focus call state (%s)", linphone_call_state_to_string(callState));
@@ -759,7 +812,7 @@ int RemoteConference::leave() {
 	switch(callState) {
 		case LinphoneCallPaused: break;
 		case LinphoneCallStreamsRunning:
-			linphone_core_pause_call(m_core, m_focusCall);
+			linphone_call_pause(m_focusCall);
 			break;
 		default:
 			ms_error("Could not leave the conference: bad focus call state (%s)", linphone_call_state_to_string(callState));
@@ -782,7 +835,7 @@ bool RemoteConference::focusIsReady() const {
 }
 
 bool RemoteConference::transferToFocus(LinphoneCall *call) {
-	if(linphone_core_transfer_call(m_core, call, m_focusContact) == 0) {
+	if(linphone_call_transfer(call, m_focusContact) == 0) {
 		m_transferingCalls.push_back(call);
 		return true;
 	} else {
@@ -846,7 +899,7 @@ void RemoteConference::onPendingCallStateChanged(LinphoneCall *call, LinphoneCal
 				if(m_state == LinphoneConferenceRunning) {
 					m_pendingCalls.remove(call);
 					m_transferingCalls.push_back(call);
-					linphone_core_transfer_call(m_core, call, m_focusContact);
+					linphone_call_transfer(call, m_focusContact);
 				}
 				break;
 				
@@ -911,106 +964,173 @@ const char *linphone_conference_state_to_string(LinphoneConferenceState state) {
 	return Conference::stateToString(state);
 }
 
+
+struct _LinphoneConferenceParams {
+	::belle_sip_object_t base;
+	Conference::Params *params;
+};
+
+static void _linphone_conference_params_uninit(LinphoneConferenceParams *params);
+static void _linphone_conference_params_clone(LinphoneConferenceParams *params, const LinphoneConferenceParams *orig);
+
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneConferenceParams);
+BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneConferenceParams);
+BELLE_SIP_INSTANCIATE_VPTR(LinphoneConferenceParams, belle_sip_object_t,
+	_linphone_conference_params_uninit, // uninit
+	_linphone_conference_params_clone, // clone
+	NULL, // marshal
+	FALSE // unown
+);
+
 LinphoneConferenceParams *linphone_conference_params_new(const LinphoneCore *core) {
-	return (LinphoneConferenceParams *)new Conference::Params(core);
+	LinphoneConferenceParams *obj = belle_sip_object_new(LinphoneConferenceParams);
+	obj->params = new Conference::Params(core);
+	return obj;
+}
+
+static void _linphone_conference_params_uninit(LinphoneConferenceParams *params) {
+	delete params->params;
+}
+
+LinphoneConferenceParams *linphone_conference_params_ref(LinphoneConferenceParams *params) {
+	return (LinphoneConferenceParams *)belle_sip_object_ref(params);
+}
+
+void linphone_conference_params_unref(LinphoneConferenceParams *params) {
+	belle_sip_object_unref(params);
 }
 
 void linphone_conference_params_free(LinphoneConferenceParams *params) {
-	delete (Conference::Params *)params;
+	linphone_conference_params_unref(params);
+}
+
+static void _linphone_conference_params_clone(LinphoneConferenceParams *params, const LinphoneConferenceParams *orig) {
+	params->params = new Conference::Params(*orig->params);
 }
 
 LinphoneConferenceParams *linphone_conference_params_clone(const LinphoneConferenceParams *params) {
-	return (LinphoneConferenceParams *)new Conference::Params(*(Conference::Params *)params);
+	return (LinphoneConferenceParams *)belle_sip_object_clone((const belle_sip_object_t *)params);
 }
 
 void linphone_conference_params_enable_video(LinphoneConferenceParams *params, bool_t enable) {
-	((Conference::Params *)params)->enableVideo((enable == TRUE) ? true : false);
+	params->params->enableVideo(enable ? true : false);
 }
 
 bool_t linphone_conference_params_video_requested(const LinphoneConferenceParams *params) {
-	return ((Conference::Params *)params)->videoRequested();
+	return params->params->videoRequested() ? TRUE : FALSE;
 }
 
 void linphone_conference_params_set_state_changed_callback(LinphoneConferenceParams *params, LinphoneConferenceStateChangedCb cb, void *user_data) {
-	((Conference::Params *)params)->setStateChangedCallback(cb, user_data);
+	params->params->setStateChangedCallback(cb, user_data);
 }
 
 
+struct _LinphoneConference {
+	belle_sip_object_t base;
+	Conference *conf;
+};
+
+static void _linphone_conference_uninit(LinphoneConference *conf);
+
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneConference);
+BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneConference);
+BELLE_SIP_INSTANCIATE_VPTR(LinphoneConference, belle_sip_object_t,
+	_linphone_conference_uninit, // uninit
+	NULL, // clone
+	NULL, // marshal
+	FALSE // unown
+);
 
 LinphoneConference *linphone_local_conference_new(LinphoneCore *core) {
-	return (LinphoneConference *) new LocalConference(core);
+	LinphoneConference *conf = belle_sip_object_new(LinphoneConference);
+	conf->conf = new LocalConference(core, conf);
+	return conf;
 }
 
 LinphoneConference *linphone_local_conference_new_with_params(LinphoneCore *core, const LinphoneConferenceParams *params) {
-	return (LinphoneConference *) new LocalConference(core, (Conference::Params *)params);
+	LinphoneConference *conf = belle_sip_object_new(LinphoneConference);
+	conf->conf = new LocalConference(core, conf, params->params);
+	return conf;
 }
 
 LinphoneConference *linphone_remote_conference_new(LinphoneCore *core) {
-	return (LinphoneConference *) new RemoteConference(core);
+	LinphoneConference *conf = belle_sip_object_new(LinphoneConference);
+	conf->conf = new RemoteConference(core, conf);
+	return conf;
 }
 
 LinphoneConference *linphone_remote_conference_new_with_params(LinphoneCore *core, const LinphoneConferenceParams *params) {
-	return (LinphoneConference *) new RemoteConference(core, (Conference::Params *)params);
+	LinphoneConference *conf = belle_sip_object_new(LinphoneConference);
+	conf->conf = new RemoteConference(core, conf, params->params);
+	return conf;
 }
 
-void linphone_conference_free(LinphoneConference *obj) {
-	delete (Conference *)obj;
+static void _linphone_conference_uninit(LinphoneConference *conf) {
+	delete conf->conf;
+}
+
+LinphoneConference *linphone_conference_ref(LinphoneConference *conf) {
+	return (LinphoneConference *)belle_sip_object_ref(conf);
+}
+
+void linphone_conference_unref(LinphoneConference *conf) {
+	belle_sip_object_unref(conf);
 }
 
 LinphoneConferenceState linphone_conference_get_state(const LinphoneConference *obj) {
-	return ((Conference *)obj)->getState();
+	return obj->conf->getState();
 }
 
 int linphone_conference_add_participant(LinphoneConference *obj, LinphoneCall *call) {
-	return ((Conference *)obj)->addParticipant(call);
+	return obj->conf->addParticipant(call);
 }
 
-int linphone_conference_remove_participant(LinphoneConference *obj, const LinphoneAddress *uri) {
-	return ((Conference *)obj)->removeParticipant(uri);
+LinphoneStatus linphone_conference_remove_participant(LinphoneConference *obj, const LinphoneAddress *uri) {
+	return obj->conf->removeParticipant(uri);
 }
 
 int linphone_conference_remove_participant_with_call(LinphoneConference *obj, LinphoneCall *call) {
-	return ((Conference *)obj)->removeParticipant(call);
+	return obj->conf->removeParticipant(call);
 }
 
 int linphone_conference_terminate(LinphoneConference *obj) {
-	return ((Conference *)obj)->terminate();
+	return obj->conf->terminate();
 }
 
 int linphone_conference_enter(LinphoneConference *obj) {
-	return ((Conference *)obj)->enter();
+	return obj->conf->enter();
 }
 
 int linphone_conference_leave(LinphoneConference *obj) {
-	return ((Conference *)obj)->leave();
+	return obj->conf->leave();
 }
 
 bool_t linphone_conference_is_in(const LinphoneConference *obj) {
-	return ((Conference *)obj)->isIn();
+	return obj->conf->isIn();
 }
 
 AudioStream *linphone_conference_get_audio_stream(const LinphoneConference *obj) {
-	return ((Conference *)obj)->getAudioStream();
+	return obj->conf->getAudioStream();
 }
 
 int linphone_conference_mute_microphone(LinphoneConference *obj, bool_t val) {
-	return ((Conference *)obj)->muteMicrophone((val == TRUE) ? true : false);
+	return obj->conf->muteMicrophone(val ? true : false);
 }
 
 bool_t linphone_conference_microphone_is_muted(const LinphoneConference *obj) {
-	return ((Conference *)obj)->microphoneIsMuted();
+	return obj->conf->microphoneIsMuted() ? TRUE : FALSE;
 }
 
 float linphone_conference_get_input_volume(const LinphoneConference *obj) {
-	return ((Conference *)obj)->getInputVolume();
+	return obj->conf->getInputVolume();
 }
 
 int linphone_conference_get_size(const LinphoneConference *obj) {
-	return ((Conference *)obj)->getSize();
+	return obj->conf->getSize();
 }
 
 bctbx_list_t *linphone_conference_get_participants(const LinphoneConference *obj) {
-	const list<Conference::Participant *> &participants = ((Conference *)obj)->getParticipants();
+	const list<Conference::Participant *> &participants = obj->conf->getParticipants();
 	bctbx_list_t *participants_list = NULL;
 	for(list<Conference::Participant *>::const_iterator it=participants.begin();it!=participants.end();it++) {
 		LinphoneAddress *uri = linphone_address_clone((*it)->getUri());
@@ -1020,29 +1140,33 @@ bctbx_list_t *linphone_conference_get_participants(const LinphoneConference *obj
 }
 
 int linphone_conference_start_recording(LinphoneConference *obj, const char *path) {
-	return ((Conference *)obj)->startRecording(path);
+	return obj->conf->startRecording(path);
 }
 
 int linphone_conference_stop_recording(LinphoneConference *obj) {
-	return ((Conference *)obj)->stopRecording();
+	return obj->conf->stopRecording();
 }
 
 void linphone_conference_on_call_stream_starting(LinphoneConference *obj, LinphoneCall *call, bool_t is_paused_by_remote) {
-	((Conference *)obj)->onCallStreamStarting(call, (is_paused_by_remote == TRUE) ? true : false);
+	obj->conf->onCallStreamStarting(call, is_paused_by_remote ? true : false);
 }
 
 void linphone_conference_on_call_stream_stopping(LinphoneConference *obj, LinphoneCall *call) {
-	((Conference *)obj)->onCallStreamStopping(call);
+	obj->conf->onCallStreamStopping(call);
 }
 
 void linphone_conference_on_call_terminating(LinphoneConference *obj, LinphoneCall *call) {
-	((Conference *)obj)->onCallTerminating(call);
+	obj->conf->onCallTerminating(call);
 }
 
 bool_t linphone_conference_check_class(LinphoneConference *obj, LinphoneConferenceClass _class) {
 	switch(_class) {
-		case LinphoneConferenceClassLocal: return typeid(obj) == typeid(LocalConference);
-		case LinphoneConferenceClassRemote: return typeid(obj) == typeid(RemoteConference);
+		case LinphoneConferenceClassLocal: return typeid(obj->conf) == typeid(LocalConference);
+		case LinphoneConferenceClassRemote: return typeid(obj->conf) == typeid(RemoteConference);
 		default: return FALSE;
 	}
 }
+
+LinphoneStatus linphone_conference_invite_participants(LinphoneConference *obj, const bctbx_list_t *addresses, const LinphoneCallParams *params){
+	return obj->conf->inviteAddresses(toStd<const LinphoneAddress*>(addresses), params);
+}
