@@ -15,7 +15,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "private.h"
@@ -23,7 +23,7 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "mediastreamer2/mstonedetector.h"
 #include "mediastreamer2/dtmfgen.h"
 
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 
 
@@ -44,8 +44,8 @@ static void ecc_init_filters(EcCalibrator *ecc){
 	ecc->read_resampler=ms_factory_create_filter(ecc->factory, MS_RESAMPLE_ID);
 	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_SAMPLE_RATE,&rate);
 	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_OUTPUT_SAMPLE_RATE,&ecc->rate);
-	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_NCHANNELS,&ecc_channels);
-	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_OUTPUT_NCHANNELS,&channels);
+	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_NCHANNELS,&channels);
+	ms_filter_call_method(ecc->read_resampler,MS_FILTER_SET_OUTPUT_NCHANNELS,&ecc_channels);
 	
 	
 	ecc->det=ms_factory_create_filter(ecc->factory, MS_TONE_DETECTOR_ID);
