@@ -14,11 +14,12 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
+#include "linphone/event.h"
 #include "private.h"
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 const char * linphone_subscription_dir_to_string(LinphoneSubscriptionDir dir){
 	switch(dir){
@@ -68,6 +69,14 @@ LINPHONE_PUBLIC const char *linphone_publish_state_to_string(LinphonePublishStat
 	return NULL;
 }
 
+static void linphone_event_release(LinphoneEvent *lev){
+	if (lev->op) {
+		/*this will stop the refresher*/
+		sal_op_stop_refreshing(lev->op);
+	}
+	linphone_event_unref(lev);
+}
+
 static LinphoneEvent * linphone_event_new_base(LinphoneCore *lc, LinphoneSubscriptionDir dir, const char *name, SalOp *op){
 	LinphoneEvent *lev=belle_sip_object_new(LinphoneEvent);
 	lev->lc=lc;
@@ -112,7 +121,7 @@ void linphone_event_set_state(LinphoneEvent *lev, LinphoneSubscriptionState stat
 		lev->subscription_state=state;
 		linphone_core_notify_subscription_state_changed(lev->lc,lev,state);
 		if (state==LinphoneSubscriptionTerminated || state == LinphoneSubscriptionError){
-			linphone_event_unref(lev);
+			linphone_event_release(lev);
 		}
 	}
 }
@@ -124,13 +133,13 @@ void linphone_event_set_publish_state(LinphoneEvent *lev, LinphonePublishState s
 		linphone_core_notify_publish_state_changed(lev->lc,lev,state);
 		switch(state){
 			case LinphonePublishCleared:
-				if (lev->expires!=-1)
-					linphone_event_unref(lev);
+				linphone_event_release(lev);
 				break;
 			case LinphonePublishOk:
+				if (lev->oneshot) linphone_event_release(lev);
+				break;
 			case LinphonePublishError:
-				if (lev->expires==-1)
-					linphone_event_unref(lev);
+				linphone_event_release(lev);
 				break;
 			case LinphonePublishNone:
 			case LinphonePublishProgress:
@@ -147,7 +156,9 @@ LinphonePublishState linphone_event_get_publish_state(const LinphoneEvent *lev){
 }
 
 const LinphoneErrorInfo *linphone_event_get_error_info(const LinphoneEvent *lev){
-	return linphone_error_info_from_sal_op(lev->op);
+	if (!lev->ei) ((LinphoneEvent*)lev)->ei = linphone_error_info_new();
+	linphone_error_info_from_sal_op(lev->ei, lev->op);
+	return lev->ei;
 }
 
 LinphoneReason linphone_event_get_reason(const LinphoneEvent *lev){
@@ -176,7 +187,7 @@ LinphoneEvent *linphone_core_subscribe(LinphoneCore *lc, const LinphoneAddress *
 	return lev;
 }
 
-int linphone_event_send_subscribe(LinphoneEvent *lev, const LinphoneContent *body){
+LinphoneStatus linphone_event_send_subscribe(LinphoneEvent *lev, const LinphoneContent *body){
 	SalBodyHandler *body_handler;
 	int err;
 
@@ -187,7 +198,7 @@ int linphone_event_send_subscribe(LinphoneEvent *lev, const LinphoneContent *bod
 	switch (lev->subscription_state){
 		case LinphoneSubscriptionIncomingReceived:
 		case LinphoneSubscriptionTerminated:
-		case LinphoneSubscriptionOutgoingInit:
+		case LinphoneSubscriptionOutgoingProgress:
 			ms_error("linphone_event_send_subscribe(): cannot update subscription while in state [%s]", linphone_subscription_state_to_string(lev->subscription_state));
 			return -1;
 		break;
@@ -210,20 +221,20 @@ int linphone_event_send_subscribe(LinphoneEvent *lev, const LinphoneContent *bod
 	err=sal_subscribe(lev->op,NULL,NULL,lev->name,lev->expires,body_handler);
 	if (err==0){
 		if (lev->subscription_state==LinphoneSubscriptionNone)
-			linphone_event_set_state(lev,LinphoneSubscriptionOutgoingInit);
+			linphone_event_set_state(lev,LinphoneSubscriptionOutgoingProgress);
 	}
 	return err;
 }
 
-int linphone_event_update_subscribe(LinphoneEvent *lev, const LinphoneContent *body){
+LinphoneStatus linphone_event_update_subscribe(LinphoneEvent *lev, const LinphoneContent *body){
 	return linphone_event_send_subscribe(lev,body);
 }
 
-int linphone_event_refresh_subscribe(LinphoneEvent *lev) {
+LinphoneStatus linphone_event_refresh_subscribe(LinphoneEvent *lev) {
 	return sal_op_refresh(lev->op);
 }
 
-int linphone_event_accept_subscription(LinphoneEvent *lev){
+LinphoneStatus linphone_event_accept_subscription(LinphoneEvent *lev){
 	int err;
 	if (lev->subscription_state!=LinphoneSubscriptionIncomingReceived){
 		ms_error("linphone_event_accept_subscription(): cannot accept subscription if subscription wasn't just received.");
@@ -236,7 +247,7 @@ int linphone_event_accept_subscription(LinphoneEvent *lev){
 	return err;
 }
 
-int linphone_event_deny_subscription(LinphoneEvent *lev, LinphoneReason reason){
+LinphoneStatus linphone_event_deny_subscription(LinphoneEvent *lev, LinphoneReason reason){
 	int err;
 	if (lev->subscription_state!=LinphoneSubscriptionIncomingReceived){
 		ms_error("linphone_event_deny_subscription(): cannot deny subscription if subscription wasn't just received.");
@@ -247,7 +258,7 @@ int linphone_event_deny_subscription(LinphoneEvent *lev, LinphoneReason reason){
 	return err;
 }
 
-int linphone_event_notify(LinphoneEvent *lev, const LinphoneContent *body){
+LinphoneStatus linphone_event_notify(LinphoneEvent *lev, const LinphoneContent *body){
 	SalBodyHandler *body_handler;
 	if (lev->subscription_state!=LinphoneSubscriptionActive && lev->subscription_state!=LinphoneSubscriptionIncomingReceived){
 		ms_error("linphone_event_notify(): cannot notify if subscription is not active.");
@@ -261,12 +272,40 @@ int linphone_event_notify(LinphoneEvent *lev, const LinphoneContent *body){
 	return sal_notify(lev->op, body_handler);
 }
 
-LinphoneEvent *linphone_core_create_publish(LinphoneCore *lc, const LinphoneAddress *resource, const char *event, int expires){
-	LinphoneEvent *lev=linphone_event_new(lc,LinphoneSubscriptionInvalidDir, event,expires);
-	linphone_configure_op(lc,lev->op,resource,NULL,lp_config_get_int(lc->config,"sip","publish_msg_with_contact",0));
+static LinphoneEvent *_linphone_core_create_publish(LinphoneCore *core, LinphoneProxyConfig *cfg, const LinphoneAddress *resource, const char *event, int expires){
+	LinphoneCore *lc = core;
+	LinphoneEvent *lev;
+	
+	if (!lc && cfg) {
+		if (cfg->lc)
+			lc = cfg->lc;
+		else  {
+			ms_error("Cannot create publish from proxy config [%p] not attached to any core",cfg);
+			return NULL;
+		}
+	}
+	if (!resource && cfg)
+		resource = linphone_proxy_config_get_identity_address(cfg);
+	
+	lev = linphone_event_new(lc,LinphoneSubscriptionInvalidDir, event,expires);
+	linphone_configure_op_with_proxy(lc,lev->op,resource,NULL,lp_config_get_int(lc->config,"sip","publish_msg_with_contact",0),cfg);
 	sal_op_set_manual_refresher_mode(lev->op,!lp_config_get_int(lc->config,"sip","refresh_generic_publish",1));
 	return lev;
 }
+LinphoneEvent *linphone_core_create_publish(LinphoneCore *lc, const LinphoneAddress *resource, const char *event, int expires){
+	return _linphone_core_create_publish(lc, NULL, resource, event, expires);
+}
+LinphoneEvent *linphone_proxy_config_create_publish(LinphoneProxyConfig *cfg, const char *event, int expires) {
+	
+	return _linphone_core_create_publish(NULL, cfg,NULL, event, expires);
+	
+		
+}
+LinphoneEvent *linphone_core_create_one_shot_publish(LinphoneCore *lc, const LinphoneAddress *resource, const char *event){
+	LinphoneEvent *lev = linphone_core_create_publish(lc, resource, event, -1);
+	lev->oneshot = TRUE;
+	return lev;
+}
 
 static int _linphone_event_send_publish(LinphoneEvent *lev, const LinphoneContent *body, bool_t notify_err){
 	SalBodyHandler *body_handler;
@@ -303,15 +342,15 @@ LinphoneEvent *linphone_core_publish(LinphoneCore *lc, const LinphoneAddress *re
 }
 
 
-int linphone_event_send_publish(LinphoneEvent *lev, const LinphoneContent *body){
+LinphoneStatus linphone_event_send_publish(LinphoneEvent *lev, const LinphoneContent *body){
 	return _linphone_event_send_publish(lev,body,TRUE);
 }
 
-int linphone_event_update_publish(LinphoneEvent* lev, const LinphoneContent* body ) {
+LinphoneStatus linphone_event_update_publish(LinphoneEvent* lev, const LinphoneContent* body ) {
 	return linphone_event_send_publish(lev,body);
 }
 
-int linphone_event_refresh_publish(LinphoneEvent *lev) {
+LinphoneStatus linphone_event_refresh_publish(LinphoneEvent *lev) {
 	return sal_op_refresh(lev->op);
 }
 void linphone_event_pause_publish(LinphoneEvent *lev) {
@@ -359,8 +398,8 @@ void linphone_event_terminate(LinphoneEvent *lev){
 
 	if (lev->publish_state!=LinphonePublishNone){
 		if (lev->publish_state==LinphonePublishOk && lev->expires!=-1){
-			sal_publish(lev->op,NULL,NULL,NULL,0,NULL);
-		}else sal_op_unpublish(lev->op);
+			sal_op_unpublish(lev->op);
+		}
 		linphone_event_set_publish_state(lev,LinphonePublishCleared);
 		return;
 	}
@@ -378,6 +417,7 @@ LinphoneEvent *linphone_event_ref(LinphoneEvent *lev){
 }
 
 static void linphone_event_destroy(LinphoneEvent *lev){
+	if (lev->ei) linphone_error_info_unref(lev->ei);
 	if (lev->op) sal_op_release(lev->op);
 	if (lev->send_custom_headers) sal_custom_header_free(lev->send_custom_headers);
 	ms_free(lev->name);
