@@ -24,26 +24,43 @@
  */
 
 #include "TunnelManager.hh"
-#include "linphone_tunnel.h"
-#include "linphonecore.h"
+#include "linphone/tunnel.h"
+#include "linphone/core.h"
 #include "private.h"
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 LinphoneTunnel* linphone_core_get_tunnel(const LinphoneCore *lc){
 	return lc->tunnel;
 }
 
 struct _LinphoneTunnel {
+	::belle_sip_object_t base;
 	belledonnecomm::TunnelManager *manager;
 	bctbx_list_t *config_list;
 };
 
+static void _linphone_tunnel_uninit(LinphoneTunnel *tunnel);
+
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneTunnel);
+BELLE_SIP_DECLARE_VPTR(LinphoneTunnel);
+BELLE_SIP_INSTANCIATE_VPTR(LinphoneTunnel, belle_sip_object_t,
+	_linphone_tunnel_uninit, // uninit
+	NULL, // clone
+	NULL, // marshal
+	FALSE // unowned
+)
+
 extern "C" LinphoneTunnel* linphone_core_tunnel_new(LinphoneCore *lc){
-	LinphoneTunnel* tunnel = ms_new0(LinphoneTunnel, 1);
+	LinphoneTunnel* tunnel = belle_sip_object_new(LinphoneTunnel);
 	tunnel->manager = new belledonnecomm::TunnelManager(lc);
 	return tunnel;
 }
 
+static void _linphone_tunnel_uninit(LinphoneTunnel *tunnel) {
+	delete tunnel->manager;
+	bctbx_list_free_with_data(tunnel->config_list, (bctbx_list_free_func)linphone_tunnel_config_unref);
+}
+
 belledonnecomm::TunnelManager *bcTunnel(const LinphoneTunnel *tunnel){
 	return tunnel->manager;
 }
@@ -52,24 +69,41 @@ static inline _LpConfig *config(const LinphoneTunnel *tunnel){
 	return tunnel->manager->getLinphoneCore()->config;
 }
 
-void linphone_tunnel_destroy(LinphoneTunnel *tunnel){
-	delete tunnel->manager;
+LinphoneTunnel *linphone_tunnel_ref(LinphoneTunnel *tunnel) {
+	return (LinphoneTunnel *)belle_sip_object_ref(tunnel);
+}
 
-	bctbx_list_free_with_data(tunnel->config_list, (void (*)(void *))linphone_tunnel_config_destroy);
-
-	ms_free(tunnel);
+void linphone_tunnel_unref(LinphoneTunnel *tunnel) {
+	belle_sip_object_unref(tunnel);
 }
 
 static char *linphone_tunnel_config_to_string(const LinphoneTunnelConfig *tunnel_config) {
 	char *str = NULL;
 	const char *host = linphone_tunnel_config_get_host(tunnel_config);
+	const char *host2 = linphone_tunnel_config_get_host2(tunnel_config);
 	if(host != NULL) {
 		if(linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config) != -1) {
-			str = ms_strdup_printf("%s:%d:%d:%d",
+			if (host2 != NULL) {
+				str = ms_strdup_printf("%s:%d:%d:%d/%s:%d",
+									linphone_tunnel_config_get_host(tunnel_config),
+									linphone_tunnel_config_get_port(tunnel_config),
+									linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
+									linphone_tunnel_config_get_delay(tunnel_config),
+									linphone_tunnel_config_get_host2(tunnel_config),
+									linphone_tunnel_config_get_port2(tunnel_config));
+			} else {
+				str = ms_strdup_printf("%s:%d:%d:%d",
+									linphone_tunnel_config_get_host(tunnel_config),
+									linphone_tunnel_config_get_port(tunnel_config),
+									linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
+									linphone_tunnel_config_get_delay(tunnel_config));
+			}
+		} else if (host2 != NULL) {
+			str = ms_strdup_printf("%s:%d/%s:%d",
 								   linphone_tunnel_config_get_host(tunnel_config),
 								   linphone_tunnel_config_get_port(tunnel_config),
-								   linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
-								   linphone_tunnel_config_get_delay(tunnel_config));
+								   linphone_tunnel_config_get_host2(tunnel_config),
+								   linphone_tunnel_config_get_port2(tunnel_config));
 		} else {
 			str = ms_strdup_printf("%s:%d",
 								   linphone_tunnel_config_get_host(tunnel_config),
@@ -88,8 +122,12 @@ static LinphoneTunnelConfig *linphone_tunnel_config_from_string(const char *str)
 	int delay = -1;
 	int pos = 0;
 	char *pch;
-	pch = strtok(dstr, ":");
-	while(pch != NULL) {
+	char *tok1, *tok2;
+	tok1 = strtok(dstr, "/");
+	tok2 = strtok(NULL, "/");
+	
+	pch = strtok(tok1, ":");
+	while (pch != NULL) {
 		switch(pos) {
 		case 0:
 			host = pch;
@@ -112,17 +150,51 @@ static LinphoneTunnelConfig *linphone_tunnel_config_from_string(const char *str)
 		++pos;
 		pch = strtok(NULL, ":");
 	}
-	if(pos >= 2) {
+	if (pos >= 2) {
 		tunnel_config = linphone_tunnel_config_new();
+		linphone_tunnel_config_set_remote_udp_mirror_port(tunnel_config, -1);
 		linphone_tunnel_config_set_host(tunnel_config, host);
 		linphone_tunnel_config_set_port(tunnel_config, port);
 	}
-	if(pos >= 3) {
+	if (pos >= 3) {
 		linphone_tunnel_config_set_remote_udp_mirror_port(tunnel_config, remote_udp_mirror_port);
 	}
-	if(pos == 4) {
+	if (pos == 4) {
 		linphone_tunnel_config_set_delay(tunnel_config, delay);
 	}
+	
+	if (tok2) {
+		pos = 0;
+		pch = strtok(tok2, ":");
+		while (pch != NULL) {
+			switch(pos) {
+			case 0:
+				host = pch;
+				break;
+			case 1:
+				port = atoi(pch);
+				break;
+			case 2:
+				remote_udp_mirror_port = atoi(pch);
+				break;
+			case 3:
+				delay = atoi(pch);
+				break;
+			default:
+				// Abort
+				pos = 0;
+				break;
+
+			}
+			++pos;
+			pch = strtok(NULL, ":");
+		}
+		if (pos >= 2 && tunnel_config) {
+			linphone_tunnel_config_set_host2(tunnel_config, host);
+			linphone_tunnel_config_set_port2(tunnel_config, port);
+		}
+	}
+	
 	ms_free(dstr);
 	return tunnel_config;
 }
@@ -153,14 +225,28 @@ static void linphone_tunnel_save_config(const LinphoneTunnel *tunnel) {
 
 
 static void linphone_tunnel_add_server_intern(LinphoneTunnel *tunnel, LinphoneTunnelConfig *tunnel_config) {
-	if(linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config) == -1) {
-		bcTunnel(tunnel)->addServer(linphone_tunnel_config_get_host(tunnel_config),
-			linphone_tunnel_config_get_port(tunnel_config));
+	if (linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config) != -1) {
+		if (linphone_tunnel_config_get_host2(tunnel_config) != NULL) {
+			bcTunnel(tunnel)->addServerPair(linphone_tunnel_config_get_host(tunnel_config),
+				linphone_tunnel_config_get_port(tunnel_config),
+				linphone_tunnel_config_get_host2(tunnel_config),
+				linphone_tunnel_config_get_port2(tunnel_config),
+				linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
+				linphone_tunnel_config_get_delay(tunnel_config));
+		} else {
+			bcTunnel(tunnel)->addServer(linphone_tunnel_config_get_host(tunnel_config),
+				linphone_tunnel_config_get_port(tunnel_config),
+				linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
+				linphone_tunnel_config_get_delay(tunnel_config));
+		}
+	} else if (linphone_tunnel_config_get_host2(tunnel_config) != NULL) {
+		bcTunnel(tunnel)->addServerPair(linphone_tunnel_config_get_host(tunnel_config),
+			linphone_tunnel_config_get_port(tunnel_config),
+			linphone_tunnel_config_get_host2(tunnel_config),
+			linphone_tunnel_config_get_port2(tunnel_config));
 	} else {
 		bcTunnel(tunnel)->addServer(linphone_tunnel_config_get_host(tunnel_config),
-			linphone_tunnel_config_get_port(tunnel_config),
-			linphone_tunnel_config_get_remote_udp_mirror_port(tunnel_config),
-			linphone_tunnel_config_get_delay(tunnel_config));
+			linphone_tunnel_config_get_port(tunnel_config));
 	}
 	tunnel->config_list = bctbx_list_append(tunnel->config_list, linphone_tunnel_config_ref(tunnel_config));
 }
@@ -241,6 +327,15 @@ LinphoneTunnelMode linphone_tunnel_get_mode(const LinphoneTunnel *tunnel){
 	return bcTunnel(tunnel)->getMode();
 }
 
+void linphone_tunnel_enable_dual_mode(LinphoneTunnel *tunnel, bool_t dual_mode_enabled) {
+	lp_config_set_int(config(tunnel), "tunnel", "dual_mode", (dual_mode_enabled ? TRUE : FALSE));
+	bcTunnel(tunnel)->enableDualMode(dual_mode_enabled);
+}
+
+bool_t linphone_tunnel_dual_mode_enabled(const LinphoneTunnel *tunnel) {
+	return bcTunnel(tunnel)->isDualModeEnabled();
+}
+
 bool_t linphone_tunnel_connected(const LinphoneTunnel *tunnel){
 	return bcTunnel(tunnel)->isConnected();
 }
@@ -353,10 +448,18 @@ void linphone_tunnel_configure(LinphoneTunnel *tunnel){
 	LinphoneTunnelMode mode = linphone_tunnel_mode_from_string(lp_config_get_string(config(tunnel), "tunnel", "mode", NULL));
 	bool_t tunnelizeSIPPackets = (bool_t)lp_config_get_int(config(tunnel), "tunnel", "sip", TRUE);
 	bool_t tunnelVerifyServerCertificate = (bool_t)lp_config_get_int(config(tunnel), "tunnel", "verify_cert", FALSE);
+	bool_t useDualMode = (bool_t)lp_config_get_int(config(tunnel), "tunnel", "dual_mode", FALSE);
+	const char *http_host, *http_username, *http_passwd;
+	int http_port;
+	linphone_tunnel_get_http_proxy(tunnel,&http_host, &http_port, &http_username, &http_passwd);
+	bcTunnel(tunnel)->setHttpProxy(http_host, http_port, http_username, http_passwd);
+	
+	linphone_tunnel_enable_dual_mode(tunnel, useDualMode);
 	linphone_tunnel_enable_logs_with_handler(tunnel,TRUE,my_ortp_logv);
 	linphone_tunnel_load_config(tunnel);
 	linphone_tunnel_enable_sip(tunnel, tunnelizeSIPPackets);
 	linphone_tunnel_verify_server_certificate(tunnel, tunnelVerifyServerCertificate);
+	
 	/*Tunnel is started here if mode equals true*/
 	linphone_tunnel_set_mode(tunnel, mode);
 
