@@ -23,10 +23,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "linphone_tunnel.h"
-#include "linphonecore.h"
+#include "linphone/tunnel.h"
+#include "linphone/core.h"
 #include "private.h"
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 
 LinphoneTunnel* linphone_core_get_tunnel(const LinphoneCore *lc){
@@ -40,25 +40,40 @@ void linphone_tunnel_destroy(LinphoneTunnel *tunnel){
 
 
 void linphone_tunnel_add_server(LinphoneTunnel *tunnel, LinphoneTunnelConfig *tunnel_config){
+	ms_warning("linphone_tunnel_add_server() - stubbed, no implementation");
 }
 
 void linphone_tunnel_remove_server(LinphoneTunnel *tunnel, LinphoneTunnelConfig *tunnel_config){
+	ms_warning("linphone_tunnel_remove_server() - stubbed, no implementation");
 }
 
 const MSList *linphone_tunnel_get_servers(const LinphoneTunnel *tunnel){
+	ms_warning("linphone_tunnel_get_servers() - stubbed, no implementation");
 	return NULL;
 }
 
 void linphone_tunnel_clean_servers(LinphoneTunnel *tunnel){
+	ms_warning("linphone_tunnel_clean_servers() - stubbed, no implementation");
 }
 
 void linphone_tunnel_set_mode(LinphoneTunnel *tunnel, LinphoneTunnelMode mode) {
+	ms_warning("linphone_tunnel_set_mode() - stubbed, no implementation");
 }
 
 LinphoneTunnelMode linphone_tunnel_get_mode(const LinphoneTunnel *tunnel){
+	ms_warning("linphone_tunnel_get_mode() - stubbed, no implementation");
 	return LinphoneTunnelModeDisable;
 }
 
+void linphone_tunnel_enable_dual_mode(LinphoneTunnel *tunnel, bool_t dual_mode_enabled) {
+	ms_warning("linphone_tunnel_set_dual_mode() - stubbed, no implementation");
+}
+
+bool_t linphone_tunnel_dual_mode_enabled(const LinphoneTunnel *tunnel) {
+	ms_warning("linphone_tunnel_get_dual_mode() - stubbed, no implementation");
+	return FALSE;
+}
+
 bool_t linphone_tunnel_connected(const LinphoneTunnel *tunnel){
 	return FALSE;
 }
@@ -83,9 +98,12 @@ void linphone_tunnel_reconnect(LinphoneTunnel *tunnel){
 }
 
 void linphone_tunnel_configure(LinphoneTunnel *tunnel){
+	ms_warning("linphone_tunnel_configure() - stubbed, no implementation");
 }
 
-void linphone_tunnel_enable_sip(LinphoneTunnel *tunnel, bool_t enable) {}
+void linphone_tunnel_enable_sip(LinphoneTunnel *tunnel, bool_t enable) {
+	ms_warning("linphone_tunnel_enable_sip() - stubbed, no implementation");
+}
 bool_t linphone_tunnel_sip_enabled(const LinphoneTunnel *tunnel) { return FALSE; }
 
 /* Deprecated functions */
