@@ -24,8 +24,10 @@
 
 #define MAX_LEN 16384
 
-#include "linphonecore.h"
+#include "private.h"
 #include "bctoolbox/vfs.h"
+#include "belle-sip/object.h"
+#include "xml2lpc.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -56,7 +58,7 @@
 
 #define lp_new0(type,n)	(type*)calloc(sizeof(type),n)
 
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 #include "lpc2xml.h"
 
 typedef struct _LpItem{
@@ -81,20 +83,22 @@ typedef struct _LpSection{
 } LpSection;
 
 struct _LpConfig{
-	int refcnt;
+	belle_sip_object_t base;
 	bctbx_vfs_file_t* pFile;
 	char *filename;
 	char *tmpfilename;
 	bctbx_list_t *sections;
-	int modified;
-	int readonly;
+	bool_t modified;
+	bool_t readonly;
 	bctbx_vfs_t* g_bctbx_vfs;
 };
 
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneConfig);
+BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneConfig);
 
 
 char* lp_realpath(const char* file, char* name) {
-#if defined(_WIN32) || defined(__QNX__) || defined(ANDROID)
+#if defined(_WIN32) || defined(__QNX__) || defined(__ANDROID__)
 	return ms_strdup(file);
 #else
 	char * output = realpath(file, name);
@@ -166,15 +170,15 @@ void lp_section_add_item(LpSection *sec,LpItem *item){
 	sec->items=bctbx_list_append(sec->items,(void *)item);
 }
 
-void lp_config_add_section(LpConfig *lpconfig, LpSection *section){
+void linphone_config_add_section(LpConfig *lpconfig, LpSection *section){
 	lpconfig->sections=bctbx_list_append(lpconfig->sections,(void *)section);
 }
 
-void lp_config_add_section_param(LpSection *section, LpSectionParam *param){
+void linphone_config_add_section_param(LpSection *section, LpSectionParam *param){
 	section->params = bctbx_list_append(section->params, (void *)param);
 }
 
-void lp_config_remove_section(LpConfig *lpconfig, LpSection *section){
+void linphone_config_remove_section(LpConfig *lpconfig, LpSection *section){
 	lpconfig->sections=bctbx_list_remove(lpconfig->sections,(void *)section);
 	lp_section_destroy(section);
 }
@@ -200,7 +204,7 @@ static int is_a_comment(const char *str){
 	return 0;
 }
 
-LpSection *lp_config_find_section(const LpConfig *lpconfig, const char *name){
+LpSection *linphone_config_find_section(const LpConfig *lpconfig, const char *name){
 	LpSection *sec;
 	bctbx_list_t *elem;
 	/*printf("Looking for section %s\n",name);*/
@@ -254,7 +258,7 @@ LpItem *lp_section_find_item(const LpSection *sec, const char *name){
 	return NULL;
 }
 
-static LpSection* lp_config_parse_line(LpConfig* lpconfig, const char* line, LpSection* cur) {
+static LpSection* linphone_config_parse_line(LpConfig* lpconfig, const char* line, LpSection* cur) {
 	LpSectionParam *params = NULL;
 	char *pos1,*pos2;
 	int nbs;
@@ -274,10 +278,10 @@ static LpSection* lp_config_parse_line(LpConfig* lpconfig, const char* line, LpS
 			nbs = sscanf(pos1+1, "%s", secname);
 			if (nbs >= 1) {
 				if (strlen(secname) > 0) {
-					cur = lp_config_find_section (lpconfig,secname);
+					cur = linphone_config_find_section (lpconfig,secname);
 					if (cur == NULL) {
 						cur = lp_section_new(secname);
-						lp_config_add_section(lpconfig, cur);
+						linphone_config_add_section(lpconfig, cur);
 					}
 
 					if (pos2 > pos1 + 1 + strlen(secname)) {
@@ -291,7 +295,7 @@ static LpSection* lp_config_parse_line(LpConfig* lpconfig, const char* line, LpS
 							*pos1 = ' ';
 							if (sscanf(pos2, "%s %s", key, value) == 2) {
 								params = lp_section_param_new(key, value);
-								lp_config_add_section_param(cur, params);
+								linphone_config_add_section_param(cur, params);
 
 								pos2 += strlen(key) + strlen(value) + 2; // Remove the = sign + the white space after each param
 								pos1 = strchr(pos2, '=');
@@ -361,46 +365,45 @@ static LpSection* lp_config_parse_line(LpConfig* lpconfig, const char* line, LpS
 	return cur;
 }
 
-void lp_config_parse(LpConfig *lpconfig, bctbx_vfs_file_t* pFile){
+void linphone_config_parse(LpConfig *lpconfig, bctbx_vfs_file_t* pFile){
 	char tmp[MAX_LEN]= {'\0'};
 	LpSection* current_section = NULL;
 	int size  =0;
 	if (pFile==NULL) return;
 	while(( size = bctbx_file_get_nxtline(pFile, tmp, MAX_LEN)) > 0){
 		//tmp[size] = '\0';
-		current_section = lp_config_parse_line(lpconfig, tmp, current_section);
+		current_section = linphone_config_parse_line(lpconfig, tmp, current_section);
 	}
 }
 
-LpConfig * lp_config_new(const char *filename){
-	return lp_config_new_with_factory(filename, NULL);
+LpConfig * linphone_config_new(const char *filename){
+	return linphone_config_new_with_factory(filename, NULL);
 }
 
-LpConfig * lp_config_new_from_buffer(const char *buffer){
-	LpConfig* conf = lp_new0(LpConfig,1);
+static void _linphone_config_init_from_buffer(LinphoneConfig *conf, const char *buffer) {
 	LpSection* current_section = NULL;
 
 	char* ptr = ms_strdup(buffer);
 	char* strtok_storage = NULL;
 	char* line = strtok_r(ptr, "\n", &strtok_storage);
 
-	conf->refcnt=1;
-
 	while( line != NULL ){
-		current_section = lp_config_parse_line(conf,line,current_section);
+		current_section = linphone_config_parse_line(conf,line,current_section);
 		line = strtok_r(NULL, "\n", &strtok_storage);
 	}
 
 	ms_free(ptr);
+}
 
+LpConfig * linphone_config_new_from_buffer(const char *buffer){
+	LpConfig* conf = belle_sip_object_new(LinphoneConfig);
+	_linphone_config_init_from_buffer(conf, buffer);
 	return conf;
 }
 
-LpConfig *lp_config_new_with_factory(const char *config_filename, const char *factory_config_filename) {
-	LpConfig *lpconfig=lp_new0(LpConfig,1);
+static int _linphone_config_init_from_files(LinphoneConfig *lpconfig, const char *config_filename, const char *factory_config_filename) {
 	lpconfig->g_bctbx_vfs = bctbx_vfs_get_default();
-	
-	lpconfig->refcnt=1;
+
 	if (config_filename!=NULL){
 		if(ortp_file_exist(config_filename) == 0) {
 			lpconfig->filename=lp_realpath(config_filename, NULL);
@@ -438,28 +441,37 @@ LpConfig *lp_config_new_with_factory(const char *config_filename, const char *fa
 		}
 #endif
 		if (lpconfig->pFile != NULL){
-		    lp_config_parse(lpconfig, lpconfig->pFile);
+		    linphone_config_parse(lpconfig, lpconfig->pFile);
 			bctbx_file_close(lpconfig->pFile);
 			lpconfig->pFile = NULL;
-			lpconfig->modified=0;
+			lpconfig->modified = FALSE;
 		}
 	}
 	if (factory_config_filename != NULL) {
-		lp_config_read_file(lpconfig, factory_config_filename);
+		linphone_config_read_file(lpconfig, factory_config_filename);
 	}
-	return lpconfig;
+	return 0;
 
 fail:
-	ms_free(lpconfig);
-	return NULL;
+	return -1;
 }
 
-int lp_config_read_file(LpConfig *lpconfig, const char *filename){
+LpConfig *linphone_config_new_with_factory(const char *config_filename, const char *factory_config_filename) {
+	LpConfig *lpconfig=belle_sip_object_new(LinphoneConfig);
+	if (_linphone_config_init_from_files(lpconfig, config_filename, factory_config_filename) == 0) {
+		return lpconfig;
+	} else {
+		ms_free(lpconfig);
+		return NULL;
+	}
+}
+
+LinphoneStatus linphone_config_read_file(LpConfig *lpconfig, const char *filename){
 	char* path = lp_realpath(filename, NULL);
 	bctbx_vfs_file_t* pFile = bctbx_file_open(lpconfig->g_bctbx_vfs, path, "r");
 	if (pFile != NULL){
 		ms_message("Reading config information from %s", path);
-		lp_config_parse(lpconfig, pFile);
+		linphone_config_parse(lpconfig, pFile);
 		bctbx_file_close(pFile);
 		ms_free(path);
 		return 0;
@@ -469,40 +481,110 @@ int lp_config_read_file(LpConfig *lpconfig, const char *filename){
 	return -1;
 }
 
+static const char *xml_to_lpc_failed = "xml to lpc failed";
+static const char *invalid_xml = "invalid xml";
+static const char *empty_xml = "empty provisioning file";
+
+static const char* _linphone_config_xml_convert(LpConfig *lpc, xml2lpc_context *context, int result) {
+	const char* error_msg = NULL;
+	if (result == 0) {
+		result = xml2lpc_convert(context, lpc);
+		if (result == 0) {
+			// if the remote provisioning added a proxy config and none was set before, set it
+			if (lp_config_has_section(lpc, "proxy_0") && lp_config_get_int(lpc, "sip", "default_proxy", -1) == -1){
+				lp_config_set_int(lpc, "sip", "default_proxy", 0);
+			}
+			lp_config_sync(lpc);
+		} else {
+			error_msg = xml_to_lpc_failed;
+		}
+	} else {
+		error_msg = invalid_xml;
+	}
+	return error_msg;
+}
+
+const char* linphone_config_load_from_xml_file(LinphoneConfig *lpc, const char *filename) {
+	xml2lpc_context *context = NULL;
+	char* path = lp_realpath(filename, NULL);
+	const char* error_msg = NULL;
+
+	if (path) {
+		context = xml2lpc_context_new(NULL, NULL);
+		error_msg = _linphone_config_xml_convert(lpc, context, xml2lpc_set_xml_file(context, path));
+	}
+	if (context) xml2lpc_context_destroy(context);
+	return error_msg;
+}
+
+static void xml2lpc_callback(void *ctx, xml2lpc_log_level level, const char *fmt, va_list list) {
+	BctbxLogLevel bctbx_level;
+	switch(level) {
+		case XML2LPC_DEBUG: bctbx_level = BCTBX_LOG_DEBUG; break;
+		case XML2LPC_MESSAGE: bctbx_level = BCTBX_LOG_MESSAGE;break;
+		case XML2LPC_WARNING: bctbx_level = BCTBX_LOG_WARNING;break;
+		case XML2LPC_ERROR: 
+		default:
+			bctbx_level = BCTBX_LOG_ERROR;break;
+	}
+	bctbx_logv(BCTBX_LOG_DOMAIN, bctbx_level,fmt,list);
+}
+
+const char* _linphone_config_load_from_xml_string(LpConfig *lpc, const char *buffer) {
+	xml2lpc_context *context = NULL;
+	const char* error_msg = NULL;
+
+	if (buffer != NULL) {
+		context = xml2lpc_context_new(xml2lpc_callback, NULL);
+		error_msg = _linphone_config_xml_convert(lpc, context, xml2lpc_set_xml_string(context, buffer));
+	}else{
+		error_msg = empty_xml;
+	}
+	if (context) xml2lpc_context_destroy(context);
+	return error_msg;
+}
+
+LinphoneStatus linphone_config_load_from_xml_string(LpConfig *lpc, const char *buffer) {
+	const char *status;
+	if ((status =_linphone_config_load_from_xml_string(lpc,buffer))) {
+		ms_error("%s",status);
+		return -1;
+	} else
+		return 0;
+}
+
 void lp_item_set_value(LpItem *item, const char *value){
-	char *prev_value=item->value;
-	item->value=ortp_strdup(value);
-	ortp_free(prev_value);
+	if (item->value != value) {
+		char *prev_value=item->value;
+		item->value=ortp_strdup(value);
+		ortp_free(prev_value);
+	}
 }
 
 
-static void _lp_config_destroy(LpConfig *lpconfig){
+static void _linphone_config_uninit(LpConfig *lpconfig){
 	if (lpconfig->filename!=NULL) ortp_free(lpconfig->filename);
 	if (lpconfig->tmpfilename) ortp_free(lpconfig->tmpfilename);
 	bctbx_list_for_each(lpconfig->sections,(void (*)(void*))lp_section_destroy);
 	bctbx_list_free(lpconfig->sections);
-	free(lpconfig);
 }
 
-LpConfig *lp_config_ref(LpConfig *lpconfig){
-	lpconfig->refcnt++;
-	return lpconfig;
+LpConfig *linphone_config_ref(LpConfig *lpconfig){
+	return (LinphoneConfig *)belle_sip_object_ref(BELLE_SIP_OBJECT(lpconfig));
 }
 
-void lp_config_unref(LpConfig *lpconfig){
-	lpconfig->refcnt--;
-	if (lpconfig->refcnt==0)
-		_lp_config_destroy(lpconfig);
+void linphone_config_unref(LpConfig *lpconfig){
+	belle_sip_object_unref(BELLE_SIP_OBJECT(lpconfig));
 }
 
-void lp_config_destroy(LpConfig *lpconfig){
-	lp_config_unref(lpconfig);
+void linphone_config_destroy(LpConfig *lpconfig){
+	linphone_config_unref(lpconfig);
 }
 
-const char *lp_config_get_section_param_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_value){
+const char *linphone_config_get_section_param_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_value){
 	LpSection *sec;
 	LpSectionParam *param;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		param = lp_section_find_param(sec, key);
 		if (param != NULL) return param->value;
@@ -510,10 +592,10 @@ const char *lp_config_get_section_param_string(const LpConfig *lpconfig, const c
 	return default_value;
 }
 
-const char *lp_config_get_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_string){
+const char *linphone_config_get_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_string){
 	LpSection *sec;
 	LpItem *item;
-	sec=lp_config_find_section(lpconfig,section);
+	sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		item=lp_section_find_item(sec,key);
 		if (item!=NULL) return item->value;
@@ -521,9 +603,9 @@ const char *lp_config_get_string(const LpConfig *lpconfig, const char *section,
 	return default_string;
 }
 
-bctbx_list_t * lp_config_get_string_list(const LpConfig *lpconfig, const char *section, const char *key, bctbx_list_t *default_list) {
+bctbx_list_t * linphone_config_get_string_list(const LpConfig *lpconfig, const char *section, const char *key, bctbx_list_t *default_list) {
 	LpItem *item;
-	LpSection *sec = lp_config_find_section(lpconfig, section);
+	LpSection *sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		item = lp_section_find_item(sec, key);
 		if (item != NULL) {
@@ -546,8 +628,8 @@ bctbx_list_t * lp_config_get_string_list(const LpConfig *lpconfig, const char *s
 	return default_list;
 }
 
-bool_t lp_config_get_range(const LpConfig *lpconfig, const char *section, const char *key, int *min, int *max, int default_min, int default_max) {
-	const char *str = lp_config_get_string(lpconfig, section, key, NULL);
+bool_t linphone_config_get_range(const LpConfig *lpconfig, const char *section, const char *key, int *min, int *max, int default_min, int default_max) {
+	const char *str = linphone_config_get_string(lpconfig, section, key, NULL);
 	if (str != NULL) {
 		char *minusptr = strchr(str, '-');
 		if ((minusptr == NULL) || (minusptr == str)) {
@@ -566,8 +648,8 @@ bool_t lp_config_get_range(const LpConfig *lpconfig, const char *section, const
 }
 
 
-int lp_config_get_int(const LpConfig *lpconfig,const char *section, const char *key, int default_value){
-	const char *str=lp_config_get_string(lpconfig,section,key,NULL);
+int linphone_config_get_int(const LpConfig *lpconfig,const char *section, const char *key, int default_value){
+	const char *str=linphone_config_get_string(lpconfig,section,key,NULL);
 	if (str!=NULL) {
 		int ret=0;
 
@@ -580,8 +662,8 @@ int lp_config_get_int(const LpConfig *lpconfig,const char *section, const char *
 	else return default_value;
 }
 
-int64_t lp_config_get_int64(const LpConfig *lpconfig,const char *section, const char *key, int64_t default_value){
-	const char *str=lp_config_get_string(lpconfig,section,key,NULL);
+int64_t linphone_config_get_int64(const LpConfig *lpconfig,const char *section, const char *key, int64_t default_value){
+	const char *str=linphone_config_get_string(lpconfig,section,key,NULL);
 	if (str!=NULL) {
 #ifdef _WIN32
 		return (int64_t)_atoi64(str);
@@ -592,18 +674,18 @@ int64_t lp_config_get_int64(const LpConfig *lpconfig,const char *section, const
 	else return default_value;
 }
 
-float lp_config_get_float(const LpConfig *lpconfig,const char *section, const char *key, float default_value){
-	const char *str=lp_config_get_string(lpconfig,section,key,NULL);
+float linphone_config_get_float(const LpConfig *lpconfig,const char *section, const char *key, float default_value){
+	const char *str=linphone_config_get_string(lpconfig,section,key,NULL);
 	float ret=default_value;
 	if (str==NULL) return default_value;
 	sscanf(str,"%f",&ret);
 	return ret;
 }
 
-bool_t lp_config_get_overwrite_flag_for_entry(const LpConfig *lpconfig, const char *section, const char *key) {
+bool_t linphone_config_get_overwrite_flag_for_entry(const LpConfig *lpconfig, const char *section, const char *key) {
 	LpSection *sec;
 	LpItem *item;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL){
 		item = lp_section_find_item(sec, key);
 		if (item != NULL) return item->overwrite;
@@ -611,19 +693,19 @@ bool_t lp_config_get_overwrite_flag_for_entry(const LpConfig *lpconfig, const ch
 	return FALSE;
 }
 
-bool_t lp_config_get_overwrite_flag_for_section(const LpConfig *lpconfig, const char *section) {
+bool_t linphone_config_get_overwrite_flag_for_section(const LpConfig *lpconfig, const char *section) {
 	LpSection *sec;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL){
 		return sec->overwrite;
 	}
 	return FALSE;
 }
 
-bool_t lp_config_get_skip_flag_for_entry(const LpConfig *lpconfig, const char *section, const char *key) {
+bool_t linphone_config_get_skip_flag_for_entry(const LpConfig *lpconfig, const char *section, const char *key) {
 	LpSection *sec;
 	LpItem *item;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL){
 		item = lp_section_find_item(sec, key);
 		if (item != NULL) return item->skip;
@@ -631,37 +713,40 @@ bool_t lp_config_get_skip_flag_for_entry(const LpConfig *lpconfig, const char *s
 	return FALSE;
 }
 
-bool_t lp_config_get_skip_flag_for_section(const LpConfig *lpconfig, const char *section) {
+bool_t linphone_config_get_skip_flag_for_section(const LpConfig *lpconfig, const char *section) {
 	LpSection *sec;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL){
 		return sec->skip;
 	}
 	return FALSE;
 }
 
-void lp_config_set_string(LpConfig *lpconfig,const char *section, const char *key, const char *value){
+void linphone_config_set_string(LpConfig *lpconfig,const char *section, const char *key, const char *value){
 	LpItem *item;
-	LpSection *sec=lp_config_find_section(lpconfig,section);
+	LpSection *sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		item=lp_section_find_item(sec,key);
 		if (item!=NULL){
-			if (value!=NULL && value[0] != '\0')
-				lp_item_set_value(item,value);
-			else lp_section_remove_item(sec,item);
+			if ((value != NULL) && (value[0] != '\0')) {
+				if (strcmp(value, item->value) == 0) return;
+				lp_item_set_value(item, value);
+			} else {
+				lp_section_remove_item(sec, item);
+			}
 		}else{
 			if (value!=NULL && value[0] != '\0')
 				lp_section_add_item(sec,lp_item_new(key,value));
 		}
 	}else if (value!=NULL && value[0] != '\0'){
 		sec=lp_section_new(section);
-		lp_config_add_section(lpconfig,sec);
+		linphone_config_add_section(lpconfig,sec);
 		lp_section_add_item(sec,lp_item_new(key,value));
 	}
-	lpconfig->modified++;
+	lpconfig->modified = TRUE;
 }
 
-void lp_config_set_string_list(LpConfig *lpconfig, const char *section, const char *key, const bctbx_list_t *value) {
+void linphone_config_set_string_list(LpConfig *lpconfig, const char *section, const char *key, const bctbx_list_t *value) {
 	char *strvalue = NULL;
 	char *tmp = NULL;
 	const bctbx_list_t *elem;
@@ -673,72 +758,72 @@ void lp_config_set_string_list(LpConfig *lpconfig, const char *section, const ch
 		}
 		else strvalue = ms_strdup((const char *)elem->data);
 	}
-	lp_config_set_string(lpconfig, section, key, strvalue);
+	linphone_config_set_string(lpconfig, section, key, strvalue);
 	if (strvalue) ms_free(strvalue);
 }
 
-void lp_config_set_range(LpConfig *lpconfig, const char *section, const char *key, int min_value, int max_value) {
+void linphone_config_set_range(LpConfig *lpconfig, const char *section, const char *key, int min_value, int max_value) {
 	char tmp[30];
 	snprintf(tmp, sizeof(tmp), "%i-%i", min_value, max_value);
-	lp_config_set_string(lpconfig, section, key, tmp);
+	linphone_config_set_string(lpconfig, section, key, tmp);
 }
 
-void lp_config_set_int(LpConfig *lpconfig,const char *section, const char *key, int value){
+void linphone_config_set_int(LpConfig *lpconfig,const char *section, const char *key, int value){
 	char tmp[30];
 	snprintf(tmp,sizeof(tmp),"%i",value);
-	lp_config_set_string(lpconfig,section,key,tmp);
+	linphone_config_set_string(lpconfig,section,key,tmp);
 }
 
-void lp_config_set_int_hex(LpConfig *lpconfig,const char *section, const char *key, int value){
+void linphone_config_set_int_hex(LpConfig *lpconfig,const char *section, const char *key, int value){
 	char tmp[30];
 	snprintf(tmp,sizeof(tmp),"0x%x",value);
-	lp_config_set_string(lpconfig,section,key,tmp);
+	linphone_config_set_string(lpconfig,section,key,tmp);
 }
 
-void lp_config_set_int64(LpConfig *lpconfig,const char *section, const char *key, int64_t value){
+void linphone_config_set_int64(LpConfig *lpconfig,const char *section, const char *key, int64_t value){
 	char tmp[30];
 	snprintf(tmp,sizeof(tmp),"%lli",(long long)value);
-	lp_config_set_string(lpconfig,section,key,tmp);
+	linphone_config_set_string(lpconfig,section,key,tmp);
 }
 
 
-void lp_config_set_float(LpConfig *lpconfig,const char *section, const char *key, float value){
+void linphone_config_set_float(LpConfig *lpconfig,const char *section, const char *key, float value){
 	char tmp[30];
 	snprintf(tmp,sizeof(tmp),"%f",value);
-	lp_config_set_string(lpconfig,section,key,tmp);
+	linphone_config_set_string(lpconfig,section,key,tmp);
 }
 
-void lp_config_set_overwrite_flag_for_entry(LpConfig *lpconfig, const char *section, const char *key, bool_t value) {
+void linphone_config_set_overwrite_flag_for_entry(LpConfig *lpconfig, const char *section, const char *key, bool_t value) {
 	LpSection *sec;
 	LpItem *item;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		item = lp_section_find_item(sec, key);
 		if (item != NULL) item->overwrite = value;
 	}
 }
 
-void lp_config_set_overwrite_flag_for_section(LpConfig *lpconfig, const char *section, bool_t value) {
+void linphone_config_set_overwrite_flag_for_section(LpConfig *lpconfig, const char *section, bool_t value) {
 	LpSection *sec;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		sec->overwrite = value;
 	}
 }
 
-void lp_config_set_skip_flag_for_entry(LpConfig *lpconfig, const char *section, const char *key, bool_t value) {
+void linphone_config_set_skip_flag_for_entry(LpConfig *lpconfig, const char *section, const char *key, bool_t value) {
 	LpSection *sec;
 	LpItem *item;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		item = lp_section_find_item(sec, key);
 		if (item != NULL) item->skip = value;
 	}
 }
 
-void lp_config_set_skip_flag_for_section(LpConfig *lpconfig, const char *section, bool_t value) {
+void linphone_config_set_skip_flag_for_section(LpConfig *lpconfig, const char *section, bool_t value) {
 	LpSection *sec;
-	sec = lp_config_find_section(lpconfig, section);
+	sec = linphone_config_find_section(lpconfig, section);
 	if (sec != NULL) {
 		sec->skip = value;
 	}
@@ -746,14 +831,14 @@ void lp_config_set_skip_flag_for_section(LpConfig *lpconfig, const char *section
 
 void lp_item_write(LpItem *item, LpConfig *lpconfig){
 	int ret =-1 ;
-	if (item->is_comment){	
+	if (item->is_comment){
 		ret =bctbx_file_fprintf(lpconfig->pFile, 0, "%s\n",item->value);
 
 	}
 	else if (item->value && item->value[0] != '\0' ){
 		ret =bctbx_file_fprintf(lpconfig->pFile, 0, "%s=%s\n",item->key,item->value);
 	}
-	
+
 	else {
 		ms_warning("Not writing item %s to file, it is empty", item->key);
 	}
@@ -780,10 +865,10 @@ void lp_section_write(LpSection *sec,LpConfig *lpconfig){
 	bctbx_list_for_each2(sec->items, (void (*)(void*, void*))lp_item_write, (void *)lpconfig);
 
 	if (bctbx_file_fprintf(lpconfig->pFile, 0, "\n")< 0) ms_error("lp_section_write : write error");
-	
+
 }
 
-int lp_config_sync(LpConfig *lpconfig){
+LinphoneStatus linphone_config_sync(LpConfig *lpconfig){
 	bctbx_vfs_file_t *pFile = NULL;
 	if (lpconfig->filename==NULL) return -1;
 	if (lpconfig->readonly) return 0;
@@ -796,10 +881,10 @@ int lp_config_sync(LpConfig *lpconfig){
 	lpconfig->pFile = pFile;
 	if (pFile == NULL){
 		ms_warning("Could not write %s ! Maybe it is read-only. Configuration will not be saved.",lpconfig->filename);
-		lpconfig->readonly=1;
+		lpconfig->readonly = TRUE;
 		return -1;
 	}
-	
+
 	bctbx_list_for_each2(lpconfig->sections,(void (*)(void *,void*))lp_section_write,(void *)lpconfig);
 	bctbx_file_close(pFile);
 
@@ -813,16 +898,16 @@ int lp_config_sync(LpConfig *lpconfig){
 	if (rename(lpconfig->tmpfilename,lpconfig->filename)!=0){
 		ms_error("Cannot rename %s into %s: %s",lpconfig->tmpfilename,lpconfig->filename,strerror(errno));
 	}
-	lpconfig->modified=0;
+	lpconfig->modified = FALSE;
 	return 0;
 }
 
-int lp_config_has_section(const LpConfig *lpconfig, const char *section){
-	if (lp_config_find_section(lpconfig,section)!=NULL) return 1;
+int linphone_config_has_section(const LpConfig *lpconfig, const char *section){
+	if (linphone_config_find_section(lpconfig,section)!=NULL) return 1;
 	return 0;
 }
 
-void lp_config_for_each_section(const LpConfig *lpconfig, void (*callback)(const char *section, void *ctx), void *ctx) {
+void linphone_config_for_each_section(const LpConfig *lpconfig, void (*callback)(const char *section, void *ctx), void *ctx) {
 	LpSection *sec;
 	bctbx_list_t *elem;
 	for (elem=lpconfig->sections;elem!=NULL;elem=bctbx_list_next(elem)){
@@ -831,10 +916,10 @@ void lp_config_for_each_section(const LpConfig *lpconfig, void (*callback)(const
 	}
 }
 
-void lp_config_for_each_entry(const LpConfig *lpconfig, const char *section, void (*callback)(const char *entry, void *ctx), void *ctx) {
+void linphone_config_for_each_entry(const LpConfig *lpconfig, const char *section, void (*callback)(const char *entry, void *ctx), void *ctx) {
 	LpItem *item;
 	bctbx_list_t *elem;
-	LpSection *sec=lp_config_find_section(lpconfig,section);
+	LpSection *sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		for (elem=sec->items;elem!=NULL;elem=bctbx_list_next(elem)){
 			item=(LpItem*)elem->data;
@@ -844,50 +929,50 @@ void lp_config_for_each_entry(const LpConfig *lpconfig, const char *section, voi
 	}
 }
 
-void lp_config_clean_section(LpConfig *lpconfig, const char *section){
-	LpSection *sec=lp_config_find_section(lpconfig,section);
+void linphone_config_clean_section(LpConfig *lpconfig, const char *section){
+	LpSection *sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
-		lp_config_remove_section(lpconfig,sec);
+		linphone_config_remove_section(lpconfig,sec);
 	}
-	lpconfig->modified++;
+	lpconfig->modified = TRUE;
 }
 
-int lp_config_needs_commit(const LpConfig *lpconfig){
-	return lpconfig->modified>0;
+bool_t linphone_config_needs_commit(const LpConfig *lpconfig){
+	return lpconfig->modified;
 }
 
 static const char *DEFAULT_VALUES_SUFFIX = "_default_values";
 
-int lp_config_get_default_int(const LpConfig *lpconfig, const char *section, const char *key, int default_value) {
+int linphone_config_get_default_int(const LpConfig *lpconfig, const char *section, const char *key, int default_value) {
 	char default_section[MAX_LEN];
 	strcpy(default_section, section);
 	strcat(default_section, DEFAULT_VALUES_SUFFIX);
 
-	return lp_config_get_int(lpconfig, default_section, key, default_value);
+	return linphone_config_get_int(lpconfig, default_section, key, default_value);
 }
 
-int64_t lp_config_get_default_int64(const LpConfig *lpconfig, const char *section, const char *key, int64_t default_value) {
+int64_t linphone_config_get_default_int64(const LpConfig *lpconfig, const char *section, const char *key, int64_t default_value) {
 	char default_section[MAX_LEN];
 	strcpy(default_section, section);
 	strcat(default_section, DEFAULT_VALUES_SUFFIX);
 
-	return lp_config_get_int64(lpconfig, default_section, key, default_value);
+	return linphone_config_get_int64(lpconfig, default_section, key, default_value);
 }
 
-float lp_config_get_default_float(const LpConfig *lpconfig, const char *section, const char *key, float default_value) {
+float linphone_config_get_default_float(const LpConfig *lpconfig, const char *section, const char *key, float default_value) {
 	char default_section[MAX_LEN];
 	strcpy(default_section, section);
 	strcat(default_section, DEFAULT_VALUES_SUFFIX);
 
-	return lp_config_get_float(lpconfig, default_section, key, default_value);
+	return linphone_config_get_float(lpconfig, default_section, key, default_value);
 }
 
-const char* lp_config_get_default_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_value) {
+const char* linphone_config_get_default_string(const LpConfig *lpconfig, const char *section, const char *key, const char *default_value) {
 	char default_section[MAX_LEN];
 	strcpy(default_section, section);
 	strcat(default_section, DEFAULT_VALUES_SUFFIX);
 
-	return lp_config_get_string(lpconfig, default_section, key, default_value);
+	return linphone_config_get_string(lpconfig, default_section, key, default_value);
 }
 
 /*
@@ -902,7 +987,7 @@ const char* lp_config_get_default_string(const LpConfig *lpconfig, const char *s
  *    the returned pointer.
  * 3. Do not feed it after midnight
  */
-static const char *_lp_config_dirname(char *path) {
+static const char *_linphone_config_dirname(char *path) {
 #ifdef _MSC_VER
 	char drive[_MAX_DRIVE];
 	char dir[_MAX_DIR];
@@ -917,13 +1002,13 @@ static const char *_lp_config_dirname(char *path) {
 #endif
 }
 
-bool_t lp_config_relative_file_exists(const LpConfig *lpconfig, const char *filename) {
+bool_t linphone_config_relative_file_exists(const LpConfig *lpconfig, const char *filename) {
 	bctbx_vfs_file_t *pFile;
 	if (lpconfig->filename == NULL) {
 		return FALSE;
 	} else {
 		char *conf_path = ms_strdup(lpconfig->filename);
-		const char *dir = _lp_config_dirname(conf_path);
+		const char *dir = _linphone_config_dirname(conf_path);
 		char *filepath = ms_strdup_printf("%s/%s", dir, filename);
 		char *realfilepath = lp_realpath(filepath, NULL);
 
@@ -941,7 +1026,7 @@ bool_t lp_config_relative_file_exists(const LpConfig *lpconfig, const char *file
 	}
 }
 
-void lp_config_write_relative_file(const LpConfig *lpconfig, const char *filename, const char *data) {
+void linphone_config_write_relative_file(const LpConfig *lpconfig, const char *filename, const char *data) {
 	char *dup_config_file = NULL;
 	const char *dir = NULL;
 	char *filepath = NULL;
@@ -956,7 +1041,7 @@ void lp_config_write_relative_file(const LpConfig *lpconfig, const char *filenam
 	}
 
 	dup_config_file = ms_strdup(lpconfig->filename);
-	dir = _lp_config_dirname(dup_config_file);
+	dir = _linphone_config_dirname(dup_config_file);
 	filepath = ms_strdup_printf("%s/%s", dir, filename);
 	realfilepath = lp_realpath(filepath, NULL);
 	if(realfilepath == NULL) {
@@ -978,7 +1063,7 @@ end:
 	if(realfilepath) ms_free(realfilepath);
 }
 
-int lp_config_read_relative_file(const LpConfig *lpconfig, const char *filename, char *data, size_t max_length) {
+LinphoneStatus linphone_config_read_relative_file(const LpConfig *lpconfig, const char *filename, char *data, size_t max_length) {
 	char *dup_config_file = NULL;
 	const char *dir = NULL;
 	char *filepath = NULL;
@@ -989,7 +1074,7 @@ int lp_config_read_relative_file(const LpConfig *lpconfig, const char *filename,
 	if (lpconfig->filename == NULL) return -1;
 
 	dup_config_file = ms_strdup(lpconfig->filename);
-	dir = _lp_config_dirname(dup_config_file);
+	dir = _linphone_config_dirname(dup_config_file);
 	filepath = ms_strdup_printf("%s/%s", dir, filename);
 	realfilepath = lp_realpath(filepath, NULL);
 	if(realfilepath == NULL) {
@@ -1006,7 +1091,7 @@ int lp_config_read_relative_file(const LpConfig *lpconfig, const char *filename,
 	if(bctbx_file_read(pFile, data, 1, (off_t)max_length) < 0){
 		ms_error("%s could not be loaded.", realfilepath);
 		goto err;
-		
+
 	}
 
 	bctbx_file_close(pFile);
@@ -1023,10 +1108,10 @@ err:
 	return -1;
 }
 
-const char** lp_config_get_sections_names(LpConfig *lpconfig) {
+const char** linphone_config_get_sections_names(LpConfig *lpconfig) {
 	const char **sections_names;
 	const bctbx_list_t *sections = lpconfig->sections;
-	int ndev;
+	size_t ndev;
 	int i;
 
 	ndev = bctbx_list_size(sections);
@@ -1041,7 +1126,7 @@ const char** lp_config_get_sections_names(LpConfig *lpconfig) {
 	return sections_names;
 }
 
-char* lp_config_dump_as_xml(const LpConfig *lpconfig) {
+char* linphone_config_dump_as_xml(const LpConfig *lpconfig) {
 	char *buffer;
 
 	lpc2xml_context *ctx = lpc2xml_context_new(NULL, NULL);
@@ -1060,7 +1145,7 @@ struct _entry_data {
 
 static void dump_entry(const char *entry, void *data) {
 	struct _entry_data *d = (struct _entry_data *) data;
-	const char *value = lp_config_get_string(d->conf, d->section, entry, "");
+	const char *value = linphone_config_get_string(d->conf, d->section, entry, "");
 	*d->buffer = ms_strcat_printf(*d->buffer, "\t%s=%s\n", entry, value);
 }
 
@@ -1068,21 +1153,21 @@ static void dump_section(const char *section, void *data) {
 	struct _entry_data *d = (struct _entry_data *) data;
 	d->section = section;
 	*d->buffer = ms_strcat_printf(*d->buffer, "[%s]\n", section);
-	lp_config_for_each_entry(d->conf, section, dump_entry, d);
+	linphone_config_for_each_entry(d->conf, section, dump_entry, d);
 }
 
-char* lp_config_dump(const LpConfig *lpconfig) {
+char* linphone_config_dump(const LpConfig *lpconfig) {
 	char* buffer = NULL;
 	struct _entry_data d = { lpconfig, NULL, &buffer };
-	lp_config_for_each_section(lpconfig, dump_section, &d);
+	linphone_config_for_each_section(lpconfig, dump_section, &d);
 
 	return buffer;
 }
 
-void lp_config_clean_entry(LpConfig *lpconfig, const char *section, const char *key) {
+void linphone_config_clean_entry(LpConfig *lpconfig, const char *section, const char *key) {
 	LpSection *sec;
 	LpItem *item;
-	sec=lp_config_find_section(lpconfig,section);
+	sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		item=lp_section_find_item(sec,key);
 		if (item!=NULL)
@@ -1090,12 +1175,21 @@ void lp_config_clean_entry(LpConfig *lpconfig, const char *section, const char *
 	}
 	return ;
 }
-int lp_config_has_entry(const LpConfig *lpconfig, const char *section, const char *key) {
+int linphone_config_has_entry(const LpConfig *lpconfig, const char *section, const char *key) {
 	LpSection *sec;
-	sec=lp_config_find_section(lpconfig,section);
+	sec=linphone_config_find_section(lpconfig,section);
 	if (sec!=NULL){
 		return lp_section_find_item(sec,key) != NULL;
 	} else
 		return FALSE;
-	
+
 }
+
+BELLE_SIP_INSTANCIATE_VPTR(
+	LinphoneConfig,
+	belle_sip_object_t,
+	_linphone_config_uninit, // uninit
+	NULL, // copy
+	NULL, // marshal
+	FALSE
+);
