@@ -14,11 +14,11 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "private.h"
-#include "linphonecore.h"
+#include "linphone/core.h"
 
 
 #ifdef SQLITE_STORAGE_ENABLED
@@ -28,7 +28,8 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #endif
 
 #ifndef _WIN32
-#if !defined(__QNXNTO__)
+#if !defined(__QNXNTO__) && !defined(__ANDROID__)
+#include <ctype.h>
 #include <langinfo.h>
 #include <locale.h>
 #include <iconv.h>
@@ -50,7 +51,7 @@ static char *utf8_convert(const char *filename){
 	wchar_t db_file_utf16[MAX_PATH_SIZE]={0};
 	MultiByteToWideChar(CP_ACP, MB_PRECOMPOSED, filename, -1, db_file_utf16, MAX_PATH_SIZE);
 	WideCharToMultiByte(CP_UTF8, 0, db_file_utf16, -1, db_file_utf8, sizeof(db_file_utf8), NULL, NULL);
-#elif defined(__QNXNTO__)
+#elif defined(__QNXNTO__) || defined(__ANDROID__)
 	strncpy(db_file_utf8, filename, MAX_PATH_SIZE - 1);
 #else
 	char db_file_locale[MAX_PATH_SIZE] = {'\0'};
@@ -101,7 +102,10 @@ int _linphone_sqlite3_open(const char *db_file, sqlite3 **db) {
 		ms_error("Cannot set sqlite3 temporary store to memory: %s.", errmsg);
 		sqlite3_free(errmsg);
 	}
-#if TARGET_OS_IPHONE
+
+	/* the lines below have been disabled because they are likely an
+	 * outdated hack */
+#if 0 && TARGET_OS_IPHONE
 	ret = sqlite3_exec(*db, "PRAGMA journal_mode = OFF", NULL, NULL, &errmsg);
 	if (ret != SQLITE_OK) {
 		ms_error("Cannot set sqlite3 journal_mode to off: %s.", errmsg);
@@ -117,6 +121,17 @@ int _linphone_sqlite3_open(const char *db_file, sqlite3 **db) {
 #ifdef SQLITE_STORAGE_ENABLED
 
 
+static LinphoneChatMessage * get_weak_message(LinphoneChatRoom *cr, unsigned int storage_id) {
+	LinphoneChatMessage *cm;
+	bctbx_list_t *item;
+	for (item = cr->weak_messages; item != NULL; item = bctbx_list_next(item)) {
+		cm = (LinphoneChatMessage *)bctbx_list_get_data(item);
+		if (linphone_chat_message_get_storage_id(cm) == storage_id)
+			return linphone_chat_message_ref(cm);
+	}
+	return NULL;
+}
+
 static ORTP_INLINE LinphoneChatMessage* get_transient_message(LinphoneChatRoom* cr, unsigned int storage_id){
 	bctbx_list_t* transients = cr->transient_messages;
 	LinphoneChatMessage* chat;
@@ -138,8 +153,8 @@ static ORTP_INLINE LinphoneChatMessage* get_transient_message(LinphoneChatRoom*
  * | 4  | encoding
  * | 5  | size
  * | 6  | data (currently not stored)
- * | 7  | size
- * | 8  | size
+ * | 7  | key size
+ * | 8  | key
  */
 // Callback for sql request when getting linphone content
 static int callback_content(void *data, int argc, char **argv, char **colName) {
@@ -183,7 +198,7 @@ static int callback_all(void *data, int argc, char **argv, char **colName){
 	LinphoneAddress *addr = linphone_address_new(address);
 	if (addr){
 		linphone_core_get_chat_room(lc, addr);
-		linphone_address_destroy(addr);
+		linphone_address_unref(addr);
 	}
 	return 0;
 }
@@ -192,23 +207,31 @@ static int callback_all(void *data, int argc, char **argv, char **colName){
  * | 0  | storage_id
  * | 1  | localContact
  * | 2  | remoteContact
- * | 3  | direction flag
- * | 4  | message
- * | 5  | time (unused now, used to be string-based timestamp)
- * | 6  | read flag
- * | 7  | status
- * | 8  | external body url
+ * | 3  | direction flag (LinphoneChatMessageDir)
+ * | 4  | message (text content of the message)
+ * | 5  | time (unused now, used to be string-based timestamp, replaced by the utc timestamp)
+ * | 6  | read flag (no longer used, replaced by the LinphoneChatMessageStateDisplayed state)
+ * | 7  | status (LinphoneChatMessageState)
+ * | 8  | external body url (deprecated file transfer system)
  * | 9  | utc timestamp
  * | 10 | app data text
- * | 11 | linphone content id
+ * | 11 | linphone content id (LinphoneContent describing a file transfer)
+ * | 12 | message id (used for IMDN)
+ * | 13 | content type (of the message field [must be text representable])
+ * | 14 | secured flag
  */
 static int create_chat_message(void *data, int argc, char **argv, char **colName){
 	LinphoneChatRoom *cr = (LinphoneChatRoom *)data;
 	unsigned int storage_id = (unsigned int)atoi(argv[0]);
+	LinphoneChatMessage* new_message;
 
-	// check if the message exists in the transient list, in which case we should return that one.
-	LinphoneChatMessage* new_message = get_transient_message(cr, storage_id);
-	if( new_message == NULL ){
+	/* Check if the message exists in the weak messages list, in which case we should return that one. */
+	new_message = get_weak_message(cr, storage_id);
+	if (new_message == NULL) {
+		/* Check if the message exists in the transient list, in which case we should return that one. */
+		new_message = get_transient_message(cr, storage_id);
+	}
+	if (new_message == NULL) {
 		new_message = linphone_chat_room_create_message(cr, argv[4]);
 
 		if(atoi(argv[3])==LinphoneChatMessageIncoming){
@@ -227,6 +250,9 @@ static int create_chat_message(void *data, int argc, char **argv, char **colName
 		new_message->storage_id=storage_id;
 		new_message->external_body_url= ms_strdup(argv[8]);
 		new_message->appdata = ms_strdup(argv[10]);
+		new_message->message_id = ms_strdup(argv[12]);
+		linphone_chat_message_set_content_type(new_message, argv[13]);
+		new_message->is_secured = (bool_t)atoi(argv[14]);
 
 		if (argv[11] != NULL) {
 			int id = atoi(argv[11]);
@@ -234,6 +260,20 @@ static int create_chat_message(void *data, int argc, char **argv, char **colName
 				fetch_content_from_database(cr->lc->db, new_message, id);
 			}
 		}
+
+		/* Fix content type for old messages that were stored without it */
+		if (new_message->content_type == NULL) {
+			if (new_message->file_transfer_information != NULL) {
+				new_message->content_type = ms_strdup("application/vnd.gsma.rcs-ft-http+xml");
+			} else if (new_message->external_body_url != NULL) {
+				new_message->content_type = ms_strdup("message/external-body");
+			} else {
+				new_message->content_type = ms_strdup("text/plain");
+			}
+		}
+
+		/* Add the new message to the weak messages list. */
+		linphone_chat_room_add_weak_message(cr, new_message);
 	}
 	cr->messages_hist=bctbx_list_prepend(cr->messages_hist,new_message);
 
@@ -309,18 +349,21 @@ unsigned int linphone_chat_message_store(LinphoneChatMessage *msg){
 
 		peer=linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(msg->chat_room));
 		local_contact=linphone_address_as_string_uri_only(linphone_chat_message_get_local_address(msg));
-		buf = sqlite3_mprintf("INSERT INTO history VALUES(NULL,%Q,%Q,%i,%Q,%Q,%i,%i,%Q,%lld,%Q,%i);",
+		buf = sqlite3_mprintf("INSERT INTO history VALUES(NULL,%Q,%Q,%i,%Q,%Q,%i,%i,%Q,%lld,%Q,%i,%Q,%Q,%i);",
 						local_contact,
 						peer,
 						msg->dir,
 						msg->message,
 						"-1", /* use UTC field now */
-						msg->is_read,
+						FALSE, /* use state == LinphoneChatMessageStateDisplayed now */
 						msg->state,
 						msg->external_body_url,
 						(int64_t)msg->time,
 						msg->appdata,
-						content_id
+						content_id,
+						msg->message_id,
+						msg->content_type,
+						(int)msg->is_secured
 					);
 		linphone_sql_request(lc->db,buf);
 		sqlite3_free(buf);
@@ -331,6 +374,41 @@ unsigned int linphone_chat_message_store(LinphoneChatMessage *msg){
 	return id;
 }
 
+void linphone_chat_message_store_update(LinphoneChatMessage *msg) {
+	LinphoneCore *lc = linphone_chat_room_get_core(msg->chat_room);
+
+	if (lc->db) {
+		char *peer;
+		char *local_contact;
+		char *buf;
+
+		peer = linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(msg->chat_room));
+		local_contact = linphone_address_as_string_uri_only(linphone_chat_message_get_local_address(msg));
+		buf = sqlite3_mprintf("UPDATE history SET"
+			" localContact = %Q,"
+			" remoteContact = %Q,"
+			" message = %Q,"
+			" status = %i,"
+			" appdata = %Q,"
+			" messageId = %Q,"
+			" content_type = %Q"
+			" WHERE (id = %u);",
+			local_contact,
+			peer,
+			msg->message,
+			msg->state,
+			msg->appdata,
+			msg->message_id,
+			msg->content_type,
+			msg->storage_id
+		);
+		linphone_sql_request(lc->db, buf);
+		sqlite3_free(buf);
+		ms_free(local_contact);
+		ms_free(peer);
+	}
+}
+
 void linphone_chat_message_store_state(LinphoneChatMessage *msg){
 	LinphoneCore *lc=msg->chat_room->lc;
 	if (lc->db){
@@ -353,7 +431,7 @@ void linphone_chat_message_store_appdata(LinphoneChatMessage* msg){
 
 void linphone_chat_room_mark_as_read(LinphoneChatRoom *cr){
 	LinphoneCore *lc=linphone_chat_room_get_core(cr);
-	int read=1;
+	bctbx_list_t *item;
 	char *peer;
 	char *buf;
 
@@ -363,12 +441,26 @@ void linphone_chat_room_mark_as_read(LinphoneChatRoom *cr){
 	if(linphone_chat_room_get_unread_messages_count(cr) == 0) return;
 
 	peer=linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
-	buf=sqlite3_mprintf("UPDATE history SET read=%i WHERE remoteContact = %Q;",
-				   read,peer);
+	buf = sqlite3_mprintf("SELECT * FROM history WHERE remoteContact = %Q AND direction = %i AND status != %i", peer, LinphoneChatMessageIncoming, LinphoneChatMessageStateDisplayed);
+	linphone_sql_request_message(lc->db, buf, cr);
+	sqlite3_free(buf);
+	for (item = cr->messages_hist; item != NULL; item = bctbx_list_next(item)) {
+		LinphoneChatMessage *cm = (LinphoneChatMessage *)bctbx_list_get_data(item);
+		linphone_chat_message_send_display_notification(cm);
+	}
+	bctbx_list_free_with_data(cr->messages_hist, (bctbx_list_free_func)linphone_chat_message_unref);
+	cr->messages_hist = NULL;
+	buf=sqlite3_mprintf("UPDATE history SET status=%i WHERE remoteContact=%Q AND direction=%i;",
+		LinphoneChatMessageStateDisplayed, peer, LinphoneChatMessageIncoming);
 	linphone_sql_request(lc->db,buf);
 	sqlite3_free(buf);
 	ms_free(peer);
 
+	if (cr->pending_message) {
+		linphone_chat_message_set_state(cr->pending_message, LinphoneChatMessageStateDisplayed);
+		linphone_chat_message_send_display_notification(cr->pending_message);
+	}
+
 	cr->unread_count = 0;
 }
 
@@ -388,6 +480,7 @@ static int linphone_chat_room_get_messages_count(LinphoneChatRoom *cr, bool_t un
 	int numrows=0;
 	char *peer;
 	char *buf;
+	char *option = NULL;
 	sqlite3_stmt *selectStatement;
 	int returnValue;
 
@@ -397,7 +490,10 @@ static int linphone_chat_room_get_messages_count(LinphoneChatRoom *cr, bool_t un
 	if(unread_only && cr->unread_count >= 0) return cr->unread_count;
 
 	peer=linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
-	buf=sqlite3_mprintf("SELECT count(*) FROM history WHERE remoteContact = %Q %s;",peer,unread_only?"AND read = 0":"");
+	if (unread_only) {
+		option = bctbx_strdup_printf("AND status!=%i AND direction=%i", LinphoneChatMessageStateDisplayed, LinphoneChatMessageIncoming);
+	}
+	buf=sqlite3_mprintf("SELECT count(*) FROM history WHERE remoteContact = %Q %s;",peer,unread_only?option:"");
 	returnValue = sqlite3_prepare_v2(lc->db,buf,-1,&selectStatement,NULL);
 	if (returnValue == SQLITE_OK){
 		if(sqlite3_step(selectStatement) == SQLITE_ROW){
@@ -410,7 +506,10 @@ static int linphone_chat_room_get_messages_count(LinphoneChatRoom *cr, bool_t un
 
 	/* no need to test the sign of cr->unread_count here
 	 * because it has been tested above */
-	if(unread_only) cr->unread_count = numrows;
+	if(unread_only) {
+		cr->unread_count = numrows;
+		if (option) bctbx_free(option);
+	}
 
 	return numrows;
 }
@@ -508,6 +607,7 @@ bctbx_list_t *linphone_chat_room_get_history_range(LinphoneChatRoom *cr, int sta
 		while (it) {
 			LinphoneChatMessage* msg = it->data;
 			if (msg->dir == LinphoneChatMessageOutgoing) {
+				if (msg->from != NULL) linphone_address_unref(msg->from);
 				msg->from = linphone_address_ref(local_addr);
 			} else {
 				msg->to = linphone_address_ref(local_addr);
@@ -527,7 +627,56 @@ bctbx_list_t *linphone_chat_room_get_history(LinphoneChatRoom *cr,int nb_message
 	return linphone_chat_room_get_history_range(cr, 0, nb_message-1);
 }
 
-void linphone_create_table(sqlite3* db){
+
+bctbx_list_t* linphone_chat_room_find_messages(LinphoneChatRoom *cr, const char *message_id) {
+	LinphoneCore *lc = linphone_chat_room_get_core(cr);
+	char *buf;
+	char *peer;
+	bctbx_list_t* messages;
+	
+	if (lc->db == NULL) return NULL;
+	peer = linphone_address_as_string_uri_only(linphone_chat_room_get_peer_address(cr));
+	cr->messages_hist = NULL;
+	buf = sqlite3_mprintf("SELECT * FROM history WHERE remoteContact = %Q AND messageId = %Q", peer, message_id);
+	linphone_sql_request_message(lc->db, buf, cr);
+	sqlite3_free(buf);
+	ms_free(peer);
+	messages = cr->messages_hist;
+	cr->messages_hist = NULL;
+	return messages;
+}
+
+LinphoneChatMessage * linphone_chat_room_find_message_with_dir(LinphoneChatRoom *cr, const char *message_id, LinphoneChatMessageDir dir) {
+	bctbx_list_t* messages = linphone_chat_room_find_messages(cr, message_id);
+	bctbx_list_t* it;
+	LinphoneChatMessage *ret = NULL;
+	for (it = messages; it != NULL; it = it->next) {
+		LinphoneChatMessage * cm = (LinphoneChatMessage*)it->data;
+		if (cm->dir == dir) {
+			linphone_chat_message_ref(cm);
+			ret = cm;
+			break;
+		}
+	}
+	if (messages)
+		bctbx_list_free_with_data(messages, (bctbx_list_free_func)linphone_chat_message_unref);
+	
+	return ret;
+
+}
+
+LinphoneChatMessage * linphone_chat_room_find_message(LinphoneChatRoom *cr, const char *message_id) {
+	bctbx_list_t* messages = linphone_chat_room_find_messages(cr, message_id);
+	LinphoneChatMessage *cm = NULL;
+	if (messages) {
+		cm = (LinphoneChatMessage *)bctbx_list_nth_data(messages, 0);
+		linphone_chat_message_ref(cm);
+		bctbx_list_free_with_data(messages, (bctbx_list_free_func)linphone_chat_message_unref);
+	}
+	return cm;
+}
+
+static void linphone_create_history_table(sqlite3* db){
 	char* errmsg=NULL;
 	int ret;
 	ret=sqlite3_exec(db,"CREATE TABLE IF NOT EXISTS history ("
@@ -609,8 +758,9 @@ static void linphone_migrate_timestamps(sqlite3* db){
 	}
 }
 
-void linphone_update_table(sqlite3* db) {
+static void linphone_update_history_table(sqlite3* db) {
 	char* errmsg=NULL;
+	char *buf;
 	int ret;
 
 	// for image url storage
@@ -681,6 +831,46 @@ void linphone_update_table(sqlite3* db) {
 			ms_debug("Table history content successfully for lime key storage data.");
 		}
 	}
+
+	// new field for message_id
+	ret = sqlite3_exec(db, "ALTER TABLE history ADD COLUMN messageId TEXT;", NULL, NULL, &errmsg);
+	if (ret != SQLITE_OK) {
+		ms_message("Table already up to date: %s", errmsg);
+		sqlite3_free(errmsg);
+	} else {
+		ms_message("Table history updated successfully for messageId data.");
+	}
+
+	// Convert is_read to LinphoneChatMessageStateDisplayed
+	buf = sqlite3_mprintf("UPDATE history SET status=%i WHERE read=1 AND direction=%i;", LinphoneChatMessageStateDisplayed, LinphoneChatMessageIncoming);
+	linphone_sql_request(db, buf);
+	sqlite3_free(buf);
+
+	/* New field for content type */
+	ret = sqlite3_exec(db, "ALTER TABLE history ADD COLUMN content_type TEXT;", NULL, NULL, &errmsg);
+	if (ret != SQLITE_OK) {
+		ms_message("Table already up to date: %s", errmsg);
+		sqlite3_free(errmsg);
+	} else {
+		ms_message("Table history updated successfully for content_type data.");
+	}
+
+	// new field for secured flag
+	ret = sqlite3_exec(db, "ALTER TABLE history ADD COLUMN is_secured INTEGER DEFAULT 0;", NULL, NULL, &errmsg);
+	if (ret != SQLITE_OK) {
+		ms_message("Table already up to date: %s", errmsg);
+		sqlite3_free(errmsg);
+	} else {
+		ms_message("Table history updated successfully for is_secured data.");
+	}
+}
+
+static void linphone_fix_outgoing_messages_state(sqlite3* db) {
+	/* Convert Idle and InProgress states of outgoing messages to NotDelivered */
+	char *buf = sqlite3_mprintf("UPDATE history SET status=%i WHERE direction=%i AND (status=%i OR status=%i);",
+		LinphoneChatMessageStateNotDelivered, LinphoneChatMessageOutgoing, LinphoneChatMessageStateIdle, LinphoneChatMessageStateInProgress);
+	linphone_sql_request(db, buf);
+	sqlite3_free(buf);
 }
 
 void linphone_message_storage_init_chat_rooms(LinphoneCore *lc) {
@@ -730,8 +920,9 @@ void linphone_core_message_storage_init(LinphoneCore *lc){
 
 	linphone_message_storage_activate_debug(db, lc->debug_storage);
 
-	linphone_create_table(db);
-	linphone_update_table(db);
+	linphone_create_history_table(db);
+	linphone_update_history_table(db);
+	linphone_fix_outgoing_messages_state(db);
 	lc->db=db;
 
 	// Create a chatroom for each contact in the chat history
@@ -764,7 +955,11 @@ bctbx_list_t *linphone_chat_room_get_history(LinphoneChatRoom *cr,int nb_message
 	return NULL;
 }
 
-LINPHONE_PUBLIC bctbx_list_t *linphone_chat_room_get_history_range(LinphoneChatRoom *cr, int begin, int end){
+bctbx_list_t *linphone_chat_room_get_history_range(LinphoneChatRoom *cr, int begin, int end){
+	return NULL;
+}
+
+LinphoneChatMessage * linphone_chat_room_find_message(LinphoneChatRoom *cr, const char *message_id) {
 	return NULL;
 }
 
@@ -794,4 +989,7 @@ int linphone_chat_room_get_history_size(LinphoneChatRoom *cr){
 	return 0;
 }
 
+void linphone_chat_message_store_update(LinphoneChatMessage *msg){
+}
+
 #endif
