@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "sal/sal.h"
@@ -58,7 +58,7 @@ static PayloadType * opus_match(MSOfferAnswerContext *ctx, const bctbx_list_t *l
 }
 
 static MSOfferAnswerContext *opus_offer_answer_create_context(void){
-	static MSOfferAnswerContext opus_oa = {opus_match, NULL};
+	static MSOfferAnswerContext opus_oa = {opus_match, NULL, NULL};
 	return &opus_oa;
 }
 
@@ -84,7 +84,7 @@ static PayloadType * g729A_match(MSOfferAnswerContext *ctx, const bctbx_list_t *
 }
 
 static MSOfferAnswerContext *g729a_offer_answer_create_context(void){
-	static MSOfferAnswerContext g729_oa = {g729A_match, NULL};
+	static MSOfferAnswerContext g729_oa = {g729A_match, NULL, NULL};
 	return &g729_oa;
 }
 
@@ -122,7 +122,7 @@ static PayloadType * red_match(MSOfferAnswerContext *ctx, const bctbx_list_t *lo
 }
 
 static MSOfferAnswerContext *red_offer_answer_create_context(void){
-	static MSOfferAnswerContext red_oa = {red_match, NULL};
+	static MSOfferAnswerContext red_oa = {red_match, NULL, NULL};
 	return &red_oa;
 }
 
@@ -494,7 +494,7 @@ static void initiate_incoming(MSFactory *factory, const SalStreamDescription *lo
 	}
 
 	if (remote_offer->haveZrtpHash == 1) {
-		if (ms_zrtp_available()) { /* if ZRTP is available, set the zrtp hash even if it is not selected */
+		if (local_cap->zrtphash[0] != 0) { /* if ZRTP is available, set the zrtp hash even if it is not selected */
 			strncpy((char *)(result->zrtphash), (char *)(local_cap->zrtphash), sizeof(local_cap->zrtphash));
 			result->haveZrtpHash =  1;
 		}
