@@ -14,47 +14,18 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 #include "private.h"
 #include "xml2lpc.h"
+#include "linphone/lpconfig.h"
 
 #define XML2LPC_CALLBACK_BUFFER_SIZE  1024
 
-static void xml2lpc_callback(void *ctx, xml2lpc_log_level level, const char *fmt, va_list list) {
-	char buffer[XML2LPC_CALLBACK_BUFFER_SIZE];
-	vsnprintf(buffer, XML2LPC_CALLBACK_BUFFER_SIZE, fmt, list);
-
-	if (level == XML2LPC_ERROR)
-		ms_error("%s", buffer);
-	else if (level == XML2LPC_WARNING)
-		ms_warning("%s", buffer);
-	/*else
-		ms_message("%s", buffer); // Don't log debug messages */
-}
 
 static void linphone_remote_provisioning_apply(LinphoneCore *lc, const char *xml) {
-	xml2lpc_context *context = xml2lpc_context_new(xml2lpc_callback, lc);
-	int result = xml2lpc_set_xml_string(context, xml);
-	char * error_msg = NULL;
-	if (result == 0) {
-		LpConfig * lpc = linphone_core_get_config(lc);
-		result = xml2lpc_convert(context, lpc);
-		if (result == 0) {
-			// if the remote provisioning added a proxy config and none was set before, set it
-			if (lp_config_has_section(lpc, "proxy_0") && lp_config_get_int(lpc, "sip", "default_proxy", -1) == -1){
-				lp_config_set_int(lpc, "sip", "default_proxy", 0);
-			}
-			lp_config_sync(lpc);
+	const char* error_msg = _linphone_config_load_from_xml_string(linphone_core_get_config(lc), xml);
 
-		} else {
-			error_msg = "xml to lpc failed";
-		}
-	} else {
-		error_msg = "invalid xml";
-	}
-
-	xml2lpc_context_destroy(context);
 	linphone_configuring_terminated(lc
 									,error_msg ? LinphoneConfiguringFailed : LinphoneConfiguringSuccessful
 									, error_msg);
@@ -123,7 +94,10 @@ int linphone_remote_provisioning_download_and_apply(LinphoneCore *lc, const char
 
 		lc->provisioning_http_listener = belle_http_request_listener_create_from_callbacks(&belle_request_listener, lc);
 
-		request=belle_http_request_create("GET",uri, NULL);
+		request=belle_http_request_create("GET"
+											, uri
+											, belle_sip_header_create("User-Agent",linphone_core_get_user_agent(lc))
+											, NULL);
 
 		return belle_http_provider_send_request(lc->http_provider, request, lc->provisioning_http_listener);
 	} else {
@@ -135,7 +109,7 @@ int linphone_remote_provisioning_download_and_apply(LinphoneCore *lc, const char
 	}
 }
 
-int linphone_core_set_provisioning_uri(LinphoneCore *lc, const char *remote_provisioning_uri) {
+LinphoneStatus linphone_core_set_provisioning_uri(LinphoneCore *lc, const char *remote_provisioning_uri) {
 	belle_generic_uri_t *uri=remote_provisioning_uri?belle_generic_uri_parse(remote_provisioning_uri):NULL;
 	if (!remote_provisioning_uri||uri) {
 		lp_config_set_string(lc->config,"misc","config-uri",remote_provisioning_uri);
