@@ -14,10 +14,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include <ctype.h>
 
@@ -73,7 +73,7 @@ static int sip_login_do_login(SipSetupContext * ctx, const char *uri, const char
 	linphone_proxy_config_enable_register(cfg,TRUE);
 	linphone_proxy_config_done(cfg);
 	ms_free(tmp);
-	linphone_address_destroy(parsed_uri);
+	linphone_address_unref(parsed_uri);
 	ms_message("SipLogin: done");
 	return 0;
 }
