@@ -14,22 +14,22 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #ifndef LINPHONE_UPNP_H
 #define LINPHONE_UPNP_H
 
 #include "mediastreamer2/upnp_igd.h"
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "sal/sal.h"
 
 typedef struct _UpnpSession UpnpSession;
 typedef struct _UpnpContext UpnpContext;
 
-int linphone_core_update_local_media_description_from_upnp(SalMediaDescription *desc, UpnpSession *session);
-int linphone_core_update_upnp_from_remote_media_description(LinphoneCall *call, const SalMediaDescription *md);
-int linphone_core_update_upnp(LinphoneCore *lc, LinphoneCall *call);
+int linphone_call_update_local_media_description_from_upnp(SalMediaDescription *desc, UpnpSession *session);
+int linphone_call_update_upnp_from_remote_media_description(LinphoneCall *call, const SalMediaDescription *md);
+int linphone_call_update_upnp(LinphoneCall *call);
 
 int linphone_upnp_call_process(LinphoneCall *call);
 UpnpSession* linphone_upnp_session_new(LinphoneCall *call);
@@ -43,6 +43,6 @@ LinphoneUpnpState linphone_upnp_context_get_state(UpnpContext *ctx);
 const char *linphone_upnp_context_get_external_ipaddress(UpnpContext *ctx);
 int linphone_upnp_context_get_external_port(UpnpContext *ctx);
 bool_t linphone_upnp_context_is_ready_for_register(UpnpContext *ctx);
-void linphone_core_update_upnp_state_in_call_stats(LinphoneCall *call);
+void linphone_call_update_upnp_state_in_call_stats(LinphoneCall *call);
 
 #endif //LINPHONE_UPNP_H
