@@ -14,41 +14,39 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#include "vcard.h"
+#include "vcard_private.h"
 #include "belcard/belcard.hpp"
 #include "belcard/belcard_parser.hpp"
 #include "sal/sal.h"
 #include <bctoolbox/crypto.h>
+#include "private.h"
+#include "linphone/factory.h"
+#include "linphone/wrapper_utils.h"
+
+#define VCARD_MD5_HASH_SIZE 16
+
+using namespace std;
 
 struct _LinphoneVcardContext {
-	belcard::BelCardParser *parser;
+	shared_ptr<belcard::BelCardParser> parser;
 	void *user_data;
 };
 
-struct _LinphoneVcard {
-	shared_ptr<belcard::BelCard> belCard;
-	char *etag;
-	char *url;
-	unsigned char *md5;
-};
-
-#ifdef __cplusplus
 extern "C" {
-#endif
 
 LinphoneVcardContext* linphone_vcard_context_new(void) {
 	LinphoneVcardContext* context = ms_new0(LinphoneVcardContext, 1);
-	context->parser = new belcard::BelCardParser();
+	context->parser = belcard::BelCardParser::getInstance();
 	context->user_data = NULL;
 	return context;
 }
 
 void linphone_vcard_context_destroy(LinphoneVcardContext *context) {
 	if (context) {
-		if (context->parser) delete context->parser;
+		context->parser = nullptr;
 		ms_free(context);
 	}
 }
@@ -62,31 +60,82 @@ void linphone_vcard_context_set_user_data(LinphoneVcardContext *context, void *d
 	if (context) context->user_data = data;
 }
 
-LinphoneVcard* linphone_vcard_new(void) {
-	LinphoneVcard* vCard = (LinphoneVcard*) ms_new0(LinphoneVcard, 1);
+} // extern "C"
+
+
+struct _LinphoneVcard {
+	belle_sip_object_t base;
+	shared_ptr<belcard::BelCard> belCard;
+	char *etag;
+	char *url;
+	unsigned char md5[VCARD_MD5_HASH_SIZE];
+	bctbx_list_t *sip_addresses_cache;
+};
+
+extern "C" {
+
+static void _linphone_vcard_uninit(LinphoneVcard *vCard) {
+	if (vCard->etag) ms_free(vCard->etag);
+	if (vCard->url) ms_free(vCard->url);
+	linphone_vcard_clean_cache(vCard);
+	vCard->belCard.reset();
+}
+
+BELLE_SIP_DECLARE_VPTR_NO_EXPORT(LinphoneVcard);
+BELLE_SIP_DECLARE_NO_IMPLEMENTED_INTERFACES(LinphoneVcard);
+BELLE_SIP_INSTANCIATE_VPTR(LinphoneVcard, belle_sip_object_t,
+	_linphone_vcard_uninit, // destroy
+	NULL, // clone
+	NULL, // Marshall
+	FALSE
+);
+
+LinphoneVcard* _linphone_vcard_new(void) {
+	LinphoneVcard* vCard = belle_sip_object_new(LinphoneVcard);
 	vCard->belCard = belcard::BelCardGeneric::create<belcard::BelCard>();
 	return vCard;
 }
 
+LinphoneVcard *linphone_vcard_new(void) {
+	return _linphone_vcard_new();
+}
+
 static LinphoneVcard* linphone_vcard_new_from_belcard(shared_ptr<belcard::BelCard> belcard) {
-	LinphoneVcard* vCard = (LinphoneVcard*) ms_new0(LinphoneVcard, 1);
+	LinphoneVcard* vCard = belle_sip_object_new(LinphoneVcard);
 	vCard->belCard = belcard;
 	return vCard;
 }
 
 void linphone_vcard_free(LinphoneVcard *vCard) {
-	if (!vCard) return;
-	if (vCard->etag) ms_free(vCard->etag);
-	if (vCard->url) ms_free(vCard->url);
-	vCard->belCard.reset();
-	ms_free(vCard);
+	belle_sip_object_unref((belle_sip_object_t *)vCard);
+}
+
+LinphoneVcard *linphone_vcard_ref(LinphoneVcard *vCard) {
+	return (LinphoneVcard *)belle_sip_object_ref((belle_sip_object_t *)vCard);
+}
+
+void linphone_vcard_unref(LinphoneVcard *vCard) {
+	belle_sip_object_unref((belle_sip_object_t *)vCard);
+}
+
+LinphoneVcard *linphone_vcard_clone(const LinphoneVcard *vCard) {
+	LinphoneVcard *copy = belle_sip_object_new(LinphoneVcard);
+
+	copy->belCard = belcard::BelCardParser::getInstance()->parseOne(vCard->belCard->toFoldedString());
+
+	if (vCard->url) copy->url = ms_strdup(vCard->url);
+	if (vCard->etag) copy->etag = ms_strdup(vCard->etag);
+
+	memcpy(copy->md5, vCard->md5, sizeof *vCard->md5);
+
+	return copy;
 }
 
 bctbx_list_t* linphone_vcard_context_get_vcard_list_from_file(LinphoneVcardContext *context, const char *filename) {
 	bctbx_list_t *result = NULL;
 	if (context && filename) {
 		if (!context->parser) {
-			context->parser = new belcard::BelCardParser();
+			context->parser = belcard::BelCardParser::getInstance();
 		}
 		shared_ptr<belcard::BelCardList> belCards = context->parser->parseFile(filename);
 		if (belCards) {
@@ -104,7 +153,7 @@ bctbx_list_t* linphone_vcard_context_get_vcard_list_from_buffer(LinphoneVcardCon
 	bctbx_list_t *result = NULL;
 	if (context && buffer) {
 		if (!context->parser) {
-			context->parser = new belcard::BelCardParser();
+			context->parser = belcard::BelCardParser::getInstance();
 		}
 		shared_ptr<belcard::BelCardList> belCards = context->parser->parse(buffer);
 		if (belCards) {
@@ -122,7 +171,7 @@ LinphoneVcard* linphone_vcard_context_get_vcard_from_buffer(LinphoneVcardContext
 	LinphoneVcard *vCard = NULL;
 	if (context && buffer) {
 		if (!context->parser) {
-			context->parser = new belcard::BelCardParser();
+			context->parser = belcard::BelCardParser::getInstance();
 		}
 		shared_ptr<belcard::BelCard> belCard = context->parser->parseOne(buffer);
 		if (belCard) {
@@ -136,28 +185,87 @@ LinphoneVcard* linphone_vcard_context_get_vcard_from_buffer(LinphoneVcardContext
 
 const char * linphone_vcard_as_vcard4_string(LinphoneVcard *vCard) {
 	if (!vCard) return NULL;
-	
+
 	return vCard->belCard->toFoldedString().c_str();
 }
 
+void *linphone_vcard_get_belcard(LinphoneVcard *vcard) {
+	return &vcard->belCard;
+}
+
 void linphone_vcard_set_full_name(LinphoneVcard *vCard, const char *name) {
 	if (!vCard || !name) return;
-	
-	shared_ptr<belcard::BelCardFullName> fn = belcard::BelCardGeneric::create<belcard::BelCardFullName>();
-	fn->setValue(name);
-	vCard->belCard->setFullName(fn);
+
+	if (vCard->belCard->getFullName()) {
+		vCard->belCard->getFullName()->setValue(name);
+	} else {
+		shared_ptr<belcard::BelCardFullName> fn = belcard::BelCardGeneric::create<belcard::BelCardFullName>();
+		fn->setValue(name);
+		vCard->belCard->setFullName(fn);
+	}
 }
 
 const char* linphone_vcard_get_full_name(const LinphoneVcard *vCard) {
 	if (!vCard) return NULL;
-	
+
 	const char *result = vCard->belCard->getFullName() ? vCard->belCard->getFullName()->getValue().c_str() : NULL;
 	return result;
 }
 
+void linphone_vcard_set_skip_validation(LinphoneVcard *vCard, bool_t skip) {
+	if (!vCard || !vCard->belCard) return;
+
+	vCard->belCard->setSkipFieldValidation((skip == TRUE) ? true : false);
+}
+
+bool_t linphone_vcard_get_skip_validation(const LinphoneVcard *vCard) {
+	if (!vCard) return FALSE;
+
+	bool_t result = vCard->belCard->getSkipFieldValidation();
+	return result;
+}
+
+void linphone_vcard_set_family_name(LinphoneVcard *vCard, const char *name) {
+	if (!vCard || !name) return;
+
+	if (vCard->belCard->getName()) {
+		vCard->belCard->getName()->setFamilyName(name);
+	} else {
+		shared_ptr<belcard::BelCardName> n = belcard::BelCardGeneric::create<belcard::BelCardName>();
+		n->setFamilyName(name);
+		vCard->belCard->setName(n);
+	}
+}
+
+const char* linphone_vcard_get_family_name(const LinphoneVcard *vCard) {
+	if (!vCard) return NULL;
+
+	const char *result = vCard->belCard->getName() ? vCard->belCard->getName()->getFamilyName().c_str() : NULL;
+	return result;
+}
+
+void linphone_vcard_set_given_name(LinphoneVcard *vCard, const char *name) {
+	if (!vCard || !name) return;
+
+	if (vCard->belCard->getName()) {
+		vCard->belCard->getName()->setGivenName(name);
+	} else {
+		shared_ptr<belcard::BelCardName> n = belcard::BelCardGeneric::create<belcard::BelCardName>();
+		n->setGivenName(name);
+		vCard->belCard->setName(n);
+	}
+}
+
+const char* linphone_vcard_get_given_name(const LinphoneVcard *vCard) {
+	if (!vCard) return NULL;
+
+	const char *result = vCard->belCard->getName() ? vCard->belCard->getName()->getGivenName().c_str() : NULL;
+	return result;
+}
+
 void linphone_vcard_add_sip_address(LinphoneVcard *vCard, const char *sip_address) {
 	if (!vCard || !sip_address) return;
-	
+
 	shared_ptr<belcard::BelCardImpp> impp = belcard::BelCardGeneric::create<belcard::BelCardImpp>();
 	impp->setValue(sip_address);
 	vCard->belCard->addImpp(impp);
@@ -165,7 +273,7 @@ void linphone_vcard_add_sip_address(LinphoneVcard *vCard, const char *sip_addres
 
 void linphone_vcard_remove_sip_address(LinphoneVcard *vCard, const char *sip_address) {
 	if (!vCard) return;
-	
+
 	shared_ptr<belcard::BelCardImpp> impp;
 	for (auto it = vCard->belCard->getImpp().begin(); it != vCard->belCard->getImpp().end(); ++it) {
 		const char *value = (*it)->getValue().c_str();
@@ -181,7 +289,7 @@ void linphone_vcard_remove_sip_address(LinphoneVcard *vCard, const char *sip_add
 
 void linphone_vcard_edit_main_sip_address(LinphoneVcard *vCard, const char *sip_address) {
 	if (!vCard || !sip_address) return;
-	
+
 	if (vCard->belCard->getImpp().size() > 0) {
 		const shared_ptr<belcard::BelCardImpp> impp = vCard->belCard->getImpp().front();
 		impp->setValue(sip_address);
@@ -192,22 +300,22 @@ void linphone_vcard_edit_main_sip_address(LinphoneVcard *vCard, const char *sip_
 	}
 }
 
-bctbx_list_t* linphone_vcard_get_sip_addresses(const LinphoneVcard *vCard) {
-	bctbx_list_t *result = NULL;
+const bctbx_list_t* linphone_vcard_get_sip_addresses(LinphoneVcard *vCard) {
 	if (!vCard) return NULL;
-	
-	for (auto it = vCard->belCard->getImpp().begin(); it != vCard->belCard->getImpp().end(); ++it) {
-		const char *value = (*it)->getValue().c_str();
-		if (strncmp(value, "sip:", 4) == 0) {
-			result = bctbx_list_append(result, (char *)value);
+	if (!vCard->sip_addresses_cache) {
+		for (auto it = vCard->belCard->getImpp().begin(); it != vCard->belCard->getImpp().end(); ++it) {
+			LinphoneAddress* addr = linphone_address_new((*it)->getValue().c_str());
+			if (addr) {
+				vCard->sip_addresses_cache = bctbx_list_append(vCard->sip_addresses_cache, addr);
+			}
 		}
 	}
-	return result;
+	return vCard->sip_addresses_cache;
 }
 
 void linphone_vcard_add_phone_number(LinphoneVcard *vCard, const char *phone) {
 	if (!vCard || !phone) return;
-	
+
 	shared_ptr<belcard::BelCardPhoneNumber> phone_number = belcard::BelCardGeneric::create<belcard::BelCardPhoneNumber>();
 	phone_number->setValue(phone);
 	vCard->belCard->addPhoneNumber(phone_number);
@@ -215,7 +323,7 @@ void linphone_vcard_add_phone_number(LinphoneVcard *vCard, const char *phone) {
 
 void linphone_vcard_remove_phone_number(LinphoneVcard *vCard, const char *phone) {
 	if (!vCard) return;
-	
+
 	shared_ptr<belcard::BelCardPhoneNumber> tel;
 	for (auto it = vCard->belCard->getPhoneNumbers().begin(); it != vCard->belCard->getPhoneNumbers().end(); ++it) {
 		const char *value = (*it)->getValue().c_str();
@@ -232,7 +340,7 @@ void linphone_vcard_remove_phone_number(LinphoneVcard *vCard, const char *phone)
 bctbx_list_t* linphone_vcard_get_phone_numbers(const LinphoneVcard *vCard) {
 	bctbx_list_t *result = NULL;
 	if (!vCard) return NULL;
-	
+
 	for (auto it = vCard->belCard->getPhoneNumbers().begin(); it != vCard->belCard->getPhoneNumbers().end(); ++it) {
 		const char *value = (*it)->getValue().c_str();
 		result = bctbx_list_append(result, (char *)value);
@@ -242,7 +350,7 @@ bctbx_list_t* linphone_vcard_get_phone_numbers(const LinphoneVcard *vCard) {
 
 void linphone_vcard_set_organization(LinphoneVcard *vCard, const char *organization) {
 	if (!vCard) return;
-	
+
 	if (vCard->belCard->getOrganizations().size() > 0) {
 		const shared_ptr<belcard::BelCardOrganization> org = vCard->belCard->getOrganizations().front();
 		org->setValue(organization);
@@ -258,13 +366,13 @@ const char* linphone_vcard_get_organization(const LinphoneVcard *vCard) {
 		const shared_ptr<belcard::BelCardOrganization> org = vCard->belCard->getOrganizations().front();
 		return org->getValue().c_str();
 	}
-	
+
 	return NULL;
 }
 
 bool_t linphone_vcard_generate_unique_id(LinphoneVcard *vCard) {
 	char uuid[64];
-	
+
 	if (vCard) {
 		if (linphone_vcard_get_uid(vCard)) {
 			return FALSE;
@@ -281,7 +389,7 @@ bool_t linphone_vcard_generate_unique_id(LinphoneVcard *vCard) {
 
 void linphone_vcard_set_uid(LinphoneVcard *vCard, const char *uid) {
 	if (!vCard || !uid) return;
-	
+
 	shared_ptr<belcard::BelCardUniqueId> uniqueId = belcard::BelCardGeneric::create<belcard::BelCardUniqueId>();
 	uniqueId->setValue(uid);
 	vCard->belCard->setUniqueId(uniqueId);
@@ -326,38 +434,27 @@ const char* linphone_vcard_get_url(const LinphoneVcard *vCard) {
 	return vCard->url;
 }
 
-#define VCARD_MD5_HASH_SIZE 16
-
 void linphone_vcard_compute_md5_hash(LinphoneVcard *vCard) {
-	unsigned char digest[VCARD_MD5_HASH_SIZE];
 	const char *text = NULL;
-	if (!vCard) {
-		return;
-	}
+	if (!vCard) return;
 	text = linphone_vcard_as_vcard4_string(vCard);
-	bctbx_md5((unsigned char *)text, strlen(text), digest);
-	vCard->md5 = (unsigned char *)ms_malloc(sizeof(digest));
-	memcpy(vCard->md5, digest, sizeof(digest));
+	bctbx_md5((unsigned char *)text, strlen(text), vCard->md5);
 }
 
 bool_t linphone_vcard_compare_md5_hash(LinphoneVcard *vCard) {
-	unsigned char *previous_md5 = vCard->md5;
-	unsigned char *new_md5 = NULL;
-	int result = -1;
-	
-	if (!previous_md5) {
-		return result;
-	}
-	
+	unsigned char previous_md5[VCARD_MD5_HASH_SIZE];
+	memcpy(previous_md5, vCard->md5, VCARD_MD5_HASH_SIZE);
 	linphone_vcard_compute_md5_hash(vCard);
-	new_md5 = vCard->md5;
-	result = memcmp(new_md5, previous_md5, VCARD_MD5_HASH_SIZE);
-	
-	ms_free(previous_md5);
-	ms_free(new_md5);
-	return result;
+	return memcmp(vCard->md5, previous_md5, VCARD_MD5_HASH_SIZE);
 }
 
-#ifdef __cplusplus
+bool_t linphone_core_vcard_supported(void) {
+	return TRUE;
 }
-#endif
\ No newline at end of file
+
+void linphone_vcard_clean_cache(LinphoneVcard *vCard) {
+	if (vCard->sip_addresses_cache) bctbx_list_free_with_data(vCard->sip_addresses_cache, (void (*)(void*))linphone_address_unref);
+	vCard->sip_addresses_cache = NULL;
+}
+
+} // extern "C"
