@@ -14,10 +14,10 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 
 #include <string.h>
@@ -88,6 +88,9 @@ static void format_request(LinphoneXmlRpcRequest *request) {
 		return;
 	}
 
+	/* autoindent so that logs are human-readable, as SIP sip on-purpose */
+	xmlTextWriterSetIndent(writer, 1);
+
 	err = xmlTextWriterStartDocument(writer, "1.0", "UTF-8", NULL);
 	if (err >= 0) {
 		err = xmlTextWriterStartElement(writer, (const xmlChar *)"methodCall");
@@ -115,7 +118,7 @@ static void format_request(LinphoneXmlRpcRequest *request) {
 				err = xmlTextWriterWriteElement(writer, (const xmlChar *)"int", (const xmlChar *)si);
 				break;
 			case LinphoneXmlRpcArgString:
-				err = xmlTextWriterWriteElement(writer, (const xmlChar *)"string", (const xmlChar *)arg->data.s);
+				err = xmlTextWriterWriteElement(writer, (const xmlChar *)"string", arg->data.s ? (const xmlChar *)arg->data.s : (const xmlChar *)"");
 				break;
 		}
 		if (err >= 0) {
@@ -154,12 +157,23 @@ static void free_arg(LinphoneXmlRpcArg *arg) {
 	belle_sip_free(arg);
 }
 
+static bool_t linphone_xml_rpc_request_aborted(LinphoneXmlRpcRequest *req){
+	LinphoneXmlRpcSession *session = (LinphoneXmlRpcSession*) belle_sip_object_data_get(BELLE_SIP_OBJECT(req), "session");
+	if (!session){
+		ms_error("linphone_xml_rpc_request_aborted(): no session, this should not happen.");
+		return FALSE;
+	}
+	return session->released;
+}
+
 static void process_io_error_from_post_xml_rpc_request(void *data, const belle_sip_io_error_event_t *event) {
 	LinphoneXmlRpcRequest *request = (LinphoneXmlRpcRequest *)data;
 	ms_error("I/O Error during XML-RPC request sending");
-	request->status = LinphoneXmlRpcStatusFailed;
-	if (request->callbacks->response != NULL) {
-		request->callbacks->response(request);
+	if (!linphone_xml_rpc_request_aborted(request)){
+		request->status = LinphoneXmlRpcStatusFailed;
+			if (request->callbacks->response != NULL) {
+			request->callbacks->response(request);
+		}
 	}
 	linphone_xml_rpc_request_unref(request);
 }
@@ -167,9 +181,11 @@ static void process_io_error_from_post_xml_rpc_request(void *data, const belle_s
 static void process_auth_requested_from_post_xml_rpc_request(void *data, belle_sip_auth_event_t *event) {
 	LinphoneXmlRpcRequest *request = (LinphoneXmlRpcRequest *)data;
 	ms_error("Authentication error during XML-RPC request sending");
-	request->status = LinphoneXmlRpcStatusFailed;
-	if (request->callbacks->response != NULL) {
-		request->callbacks->response(request);
+	if (!linphone_xml_rpc_request_aborted(request)){
+		request->status = LinphoneXmlRpcStatusFailed;
+		if (request->callbacks->response != NULL) {
+			request->callbacks->response(request);
+		}
 	}
 	linphone_xml_rpc_request_unref(request);
 }
@@ -212,7 +228,7 @@ end:
 }
 
 static void notify_xml_rpc_error(LinphoneXmlRpcRequest *request) {
-	request->status = LinphoneXmlRpcStatusOk;
+	request->status = LinphoneXmlRpcStatusFailed;
 	if (request->callbacks->response != NULL) {
 		request->callbacks->response(request);
 	}
@@ -222,11 +238,12 @@ static void process_response_from_post_xml_rpc_request(void *data, const belle_h
 	LinphoneXmlRpcRequest *request = (LinphoneXmlRpcRequest *)data;
 
 	/* Check the answer code */
-	if (event->response) {
+	if (!linphone_xml_rpc_request_aborted(request) && event->response) {
 		int code = belle_http_response_get_status_code(event->response);
 		if (code == 200) { /* Valid response from the server. */
 			parse_valid_xml_rpc_response(request, belle_sip_message_get_body((belle_sip_message_t *)event->response));
 		} else {
+			ms_error("process_response_from_post_xml_rpc_request(): error code = %i", code);
 			notify_xml_rpc_error(request);
 		}
 	}
@@ -234,7 +251,7 @@ static void process_response_from_post_xml_rpc_request(void *data, const belle_h
 }
 
 
-static LinphoneXmlRpcRequest * _linphone_xml_rpc_request_new(const char *method, LinphoneXmlRpcArgType return_type) {
+static LinphoneXmlRpcRequest * _linphone_xml_rpc_request_new(LinphoneXmlRpcArgType return_type, const char *method) {
 	LinphoneXmlRpcRequest *request = belle_sip_object_new(LinphoneXmlRpcRequest);
 	request->callbacks = linphone_xml_rpc_request_cbs_new();
 	request->status = LinphoneXmlRpcStatusPending;
@@ -289,18 +306,18 @@ BELLE_SIP_INSTANCIATE_VPTR(LinphoneXmlRpcSession, belle_sip_object_t,
 );
 
 
-LinphoneXmlRpcRequest * linphone_xml_rpc_request_new(const char *method, LinphoneXmlRpcArgType return_type) {
-	LinphoneXmlRpcRequest *request = _linphone_xml_rpc_request_new(method, return_type);
+LinphoneXmlRpcRequest * linphone_xml_rpc_request_new(LinphoneXmlRpcArgType return_type, const char *method) {
+	LinphoneXmlRpcRequest *request = _linphone_xml_rpc_request_new(return_type, method);
 	format_request(request);
 	return request;
 }
 
-LinphoneXmlRpcRequest * linphone_xml_rpc_request_new_with_args(const char *method, LinphoneXmlRpcArgType return_type, ...) {
+LinphoneXmlRpcRequest * linphone_xml_rpc_request_new_with_args(LinphoneXmlRpcArgType return_type, const char *method, ...) {
 	bool_t cont = TRUE;
 	va_list args;
 	LinphoneXmlRpcArgType arg_type;
-	LinphoneXmlRpcRequest *request = _linphone_xml_rpc_request_new(method, return_type);
-	va_start(args, return_type);
+	LinphoneXmlRpcRequest *request = _linphone_xml_rpc_request_new(return_type, method);
+	va_start(args, method);
 	while (cont) {
 		arg_type = va_arg(args, LinphoneXmlRpcArgType);
 		switch (arg_type) {
@@ -411,6 +428,7 @@ void linphone_xml_rpc_session_send_request(LinphoneXmlRpcSession *session, Linph
 	if (!req) {
 		belle_sip_object_unref(uri);
 		process_io_error_from_post_xml_rpc_request(request, NULL);
+		return;
 	}
 	data = linphone_xml_rpc_request_get_content(request);
 	bh = belle_sip_memory_body_handler_new_copy_from_buffer(data, strlen(data), NULL, NULL);
@@ -420,4 +438,14 @@ void linphone_xml_rpc_session_send_request(LinphoneXmlRpcSession *session, Linph
 	cbs.process_auth_requested = process_auth_requested_from_post_xml_rpc_request;
 	l = belle_http_request_listener_create_from_callbacks(&cbs, request);
 	belle_http_provider_send_request(session->core->http_provider, req, l);
+	/*ensure that the listener object will be destroyed with the request*/
+	belle_sip_object_data_set(BELLE_SIP_OBJECT(request), "listener", l, belle_sip_object_unref);
+	/*prevent destruction of the session while there are still pending http requests*/
+	belle_sip_object_data_set(BELLE_SIP_OBJECT(request), "session", belle_sip_object_ref(session), belle_sip_object_unref);
 }
+
+void linphone_xml_rpc_session_release(LinphoneXmlRpcSession *session){
+	session->released = TRUE;
+	belle_sip_object_unref(session);
+}
+
