@@ -17,21 +17,21 @@ along with this library; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
 
-#ifndef COMMAND_CONFIG_H_
-#define COMMAND_CONFIG_H_
+#ifndef LINPHONE_DAEMON_COMMAND_CONFIG_H_
+#define LINPHONE_DAEMON_COMMAND_CONFIG_H_
 
-#include "../daemon.h"
+#include "daemon.h"
 
 class ConfigGetCommand: public DaemonCommand {
 public:
 	ConfigGetCommand();
-	virtual void exec(Daemon *app, const char *args);
+	virtual void exec(Daemon *app, const std::string& args);
 };
 
 class ConfigSetCommand: public DaemonCommand {
 public:
 	ConfigSetCommand();
-	virtual void exec(Daemon *app, const char *args);
+	virtual void exec(Daemon *app, const std::string& args);
 };
 
-#endif //COMMAND_IPV6_H_
+#endif // LINPHONE_DAEMON_COMMAND_CONFIG_H_
