@@ -14,13 +14,13 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include <glib/gstdio.h>
 
 #include "linphone.h"
-#include "linphonecore_utils.h"
+#include "linphone/core_utils.h"
 #include "mediastreamer2/mediastream.h"
 #include "mediastreamer2/msvolume.h"
 
