@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "linphone.h"
@@ -89,12 +89,15 @@ static void linphone_gtk_set_selection_to_uri_bar(GtkTreeView *treeview){
 	LinphoneFriend *lf=NULL;
 	gchar* friend;
 	select = gtk_tree_view_get_selection (treeview);
-	if (gtk_tree_selection_get_selected (select, &model, &iter))
-	{
+	if (gtk_tree_selection_get_selected (select, &model, &iter)) {
+		const LinphoneAddress *addr;
 		gtk_tree_model_get (model, &iter,FRIEND_ID , &lf, -1);
-		friend=linphone_address_as_string(linphone_friend_get_address(lf));
-		gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar")),friend);
-		ms_free(friend);
+		addr = linphone_friend_get_address(lf);
+		if (addr) {
+			friend=linphone_address_as_string(addr);
+			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(linphone_gtk_get_main_window(),"uribar")),friend);
+			ms_free(friend);
+		}
 	}
 }
 
@@ -175,11 +178,15 @@ gboolean linphone_gtk_on_key_press(GtkWidget *widget, GdkEvent *event, gpointer
 				LinphoneChatRoom *cr;
 				do{
 					if (index == key) {
-						const LinphoneAddress *uri;
+						const LinphoneAddress *addr;
 						gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
 						gtk_tree_model_get (model, &iter,FRIEND_ID , &lf, -1);
-						uri = linphone_friend_get_address(lf);
-						if (lf != NULL) linphone_gtk_friend_list_set_chat_conversation(uri);
+						if (lf != NULL) {
+							addr = linphone_friend_get_address(lf);
+							if (addr != NULL) {
+								linphone_gtk_friend_list_set_chat_conversation(addr);
+							}
+						}
 						if (cr != NULL){
 							linphone_gtk_mark_chat_read(cr);
 							linphone_gtk_friend_list_update_button_display(GTK_TREE_VIEW(friendlist));
@@ -215,16 +222,18 @@ void linphone_gtk_delete_history(GtkWidget *button){
 		if(chat_view!=NULL){
 			const LinphoneAddress *from=linphone_gtk_friend_list_get_active_address();
 			const LinphoneAddress *addr=linphone_friend_get_address(lf);
-			if(linphone_address_weak_equal(from,addr)){
-				GtkTextView *text_view=GTK_TEXT_VIEW(linphone_gtk_get_widget(chat_view,"textview"));
-				GtkTextIter start;
-				GtkTextIter end;
-				GtkTextBuffer *text_buffer;
+			if (addr != NULL) {
+				if(linphone_address_weak_equal(from,addr)){
+					GtkTextView *text_view=GTK_TEXT_VIEW(linphone_gtk_get_widget(chat_view,"textview"));
+					GtkTextIter start;
+					GtkTextIter end;
+					GtkTextBuffer *text_buffer;
 
-				text_buffer=gtk_text_view_get_buffer(text_view);
-				gtk_text_buffer_get_bounds(text_buffer, &start, &end);
-				gtk_text_buffer_delete (text_buffer, &start, &end);
-				g_object_set_data(G_OBJECT(chat_view),"from_message",NULL);
+					text_buffer=gtk_text_view_get_buffer(text_view);
+					gtk_text_buffer_get_bounds(text_buffer, &start, &end);
+					gtk_text_buffer_delete (text_buffer, &start, &end);
+					g_object_set_data(G_OBJECT(chat_view),"from_message",NULL);
+				}
 			}
 		}
 		linphone_gtk_show_friends();
@@ -298,7 +307,7 @@ static gboolean grab_focus(GtkWidget *w){
 void linphone_gtk_friend_list_set_active_address(const LinphoneAddress *addr){
 	GtkWidget *w=linphone_gtk_get_main_window();
 	GtkWidget *friendlist=linphone_gtk_get_widget(w,"contact_list");
-	g_object_set_data_full(G_OBJECT(friendlist),"from", addr ? linphone_address_clone(addr) : NULL, (GDestroyNotify)linphone_address_destroy);
+	g_object_set_data_full(G_OBJECT(friendlist),"from", addr ? linphone_address_clone(addr) : NULL, (GDestroyNotify)linphone_address_unref);
 }
 
 const LinphoneAddress *linphone_gtk_friend_list_get_active_address(void){
@@ -327,18 +336,20 @@ void linphone_gtk_friend_list_set_chat_conversation(const LinphoneAddress *la){
 		store=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(friendlist)));
 		if (gtk_tree_model_get_iter_first(model,&iter)) {
 			do{
-				const LinphoneAddress *uri;
+				const LinphoneAddress *addr;
 				gtk_tree_model_get(model, &iter,FRIEND_ID , &lf, -1);
-				uri=linphone_friend_get_address(lf);
-				if (linphone_address_weak_equal(uri,la)){
-					gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
-					if(cr==NULL){
-						cr=linphone_gtk_create_chatroom(uri);
-						gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
+				addr=linphone_friend_get_address(lf);
+				if (addr != NULL) {
+					if (linphone_address_weak_equal(addr,la)){
+						gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
+						if(cr==NULL){
+							cr=linphone_gtk_create_chatroom(addr);
+							gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
+						}
+						linphone_gtk_friend_list_set_active_address(addr);
+						gtk_tree_selection_select_iter(selection, &iter);
+						break;
 					}
-					linphone_gtk_friend_list_set_active_address(uri);
-					gtk_tree_selection_select_iter(selection, &iter);
-					break;
 				}
 			}while(gtk_tree_model_iter_next(model,&iter));
 		}
@@ -388,25 +399,27 @@ void linphone_gtk_chat_selected(GtkWidget *item){
 	store=GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(item)));
 	if (gtk_tree_selection_get_selected (select, &model, &iter)){
 		GtkNotebook *notebook=(GtkNotebook *)linphone_gtk_get_widget(w,"viewswitch");
-		const LinphoneAddress *uri;
+		const LinphoneAddress *addr;
 		gtk_tree_model_get (model, &iter,FRIEND_ID , &lf, -1);
 		gtk_tree_model_get (model, &iter,FRIEND_CHATROOM , &cr, -1);
-		uri=linphone_friend_get_address(lf);
-		if(cr==NULL){
-			cr=linphone_gtk_create_chatroom(uri);
-			gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
+		addr=linphone_friend_get_address(lf);
+		if (addr != NULL) {
+			if(cr==NULL){
+				cr=linphone_gtk_create_chatroom(addr);
+				gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
+			}
+			page=GTK_WIDGET(g_object_get_data(G_OBJECT(friendlist),"chatview"));
+			linphone_gtk_friend_list_set_active_address(addr);
+			if(page==NULL){
+				page=linphone_gtk_init_chatroom(cr,addr);
+				g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)page);
+			} else {
+				linphone_gtk_load_chatroom(cr,addr,page);
+			}
+			linphone_gtk_mark_chat_read(cr);
+			gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,page));
+			g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(page,"text_entry"));
 		}
-		page=GTK_WIDGET(g_object_get_data(G_OBJECT(friendlist),"chatview"));
-		linphone_gtk_friend_list_set_active_address(uri);
-		if(page==NULL){
-			page=linphone_gtk_init_chatroom(cr,uri);
-			g_object_set_data(G_OBJECT(friendlist),"chatview",(gpointer)page);
-		} else {
-			linphone_gtk_load_chatroom(cr,uri,page);
-		}
-		linphone_gtk_mark_chat_read(cr);
-		gtk_notebook_set_current_page(notebook,gtk_notebook_page_num(notebook,page));
-		g_idle_add((GSourceFunc)grab_focus,linphone_gtk_get_widget(page,"text_entry"));
 	}
 }
 
@@ -503,7 +516,7 @@ static void icon_press_handler(GtkEntry *entry){
 			linphone_friend_set_address(lf,addr);
 			linphone_gtk_show_contact(lf, w);
 		}
-		linphone_address_destroy(addr);
+		linphone_address_unref(addr);
 	}
 }
 
@@ -589,8 +602,12 @@ void linphone_gtk_friend_list_on_name_column_clicked(GtkTreeModel *model){
 static int get_friend_weight(const LinphoneFriend *lf){
 	int w=0;
 	LinphoneCore *lc=linphone_gtk_get_core();
-	LinphoneChatRoom *cr=linphone_core_get_chat_room(lc,linphone_friend_get_address(lf));
+	const LinphoneAddress *addr = linphone_friend_get_address(lf);
+	LinphoneChatRoom *cr = NULL;
 
+	if (addr != NULL) {
+		cr = linphone_core_get_chat_room(lc, addr);
+	}
 	if (cr && linphone_chat_room_get_unread_messages_count(cr)>0){
 		w+=2000;
 	}
@@ -611,7 +628,7 @@ static int get_friend_weight(const LinphoneFriend *lf){
 }
 
 static int friend_compare_func(const LinphoneFriend *lf1, const LinphoneFriend *lf2){
-	int w1,w2;
+	int w1,w2,ret;
 	w1=get_friend_weight(lf1);
 	w2=get_friend_weight(lf2);
 	if (w1==w2){
@@ -619,13 +636,22 @@ static int friend_compare_func(const LinphoneFriend *lf1, const LinphoneFriend *
 		const LinphoneAddress *addr1,*addr2;
 		addr1=linphone_friend_get_address(lf1);
 		addr2=linphone_friend_get_address(lf2);
-		u1=linphone_address_get_username(addr1);
-		u2=linphone_address_get_username(addr2);
-		if (u1 && u2) return strcasecmp(u1,u2);
-		if (u1) return 1;
-		else return -1;
+		if ((addr1 == NULL) && (addr2 == NULL)) return 0;
+		if ((addr1 == NULL) && (addr2 != NULL)) return -1;
+		if ((addr1 != NULL) && (addr2 == NULL)) return 1;
+		u1=linphone_friend_get_name(lf1) ? linphone_friend_get_name(lf1) : linphone_address_get_display_name(addr1) ? linphone_address_get_display_name(addr1) : linphone_address_get_username(addr1);
+		u2=linphone_friend_get_name(lf2) ? linphone_friend_get_name(lf2) :linphone_address_get_display_name(addr2) ? linphone_address_get_display_name(addr2) : linphone_address_get_username(addr2);
+		if (u1 && u2) {
+			ret = strcasecmp(u1,u2);
+		} else if (u1) {
+			ret = 1;
+		} else {
+			ret = -1;
+		}
+	} else {
+		ret = w2-w1;
 	}
-	return w2-w1;
+	return ret;
 }
 
 static bctbx_list_t *sort_friend_list(const bctbx_list_t *friends){
@@ -743,34 +769,37 @@ void linphone_gtk_show_friends(void){
 
 	for(itf=sorted;itf!=NULL;itf=bctbx_list_next(itf)){
 		LinphoneFriend *lf=(LinphoneFriend*)itf->data;
-		const LinphoneAddress *f_uri=linphone_friend_get_address(lf);
-		char *uri=linphone_address_as_string(f_uri);
+		const LinphoneAddress *f_addr=linphone_friend_get_address(lf);
 		const char *name=linphone_friend_get_name(lf);
+		char *uri = NULL;
 		const char *display=name;
 		char *escaped=NULL;
 		int nbmsg=0;
 
 		//BuddyInfo *bi;
 		gboolean send_subscribe=linphone_friend_get_send_subscribe(lf);
-		if (display==NULL || display[0]=='\0') {
-			display=linphone_address_get_username(f_uri);
+		if (f_addr != NULL) uri = linphone_address_as_string(f_addr);
+		if ((display==NULL || display[0]=='\0') && (f_addr != NULL)) {
+			display=linphone_address_get_username(f_addr);
 		}
 		gtk_list_store_append(store,&iter);
 		gtk_list_store_set(store,&iter,FRIEND_NAME, display,FRIEND_ID,lf,
 				FRIEND_PRESENCE_IMG, send_subscribe ? status_to_icon_name(linphone_friend_get_status(lf)) : NULL,
 				FRIEND_CHAT,"linphone-chat-nothing", -1);
 
-		cr=linphone_gtk_create_chatroom(f_uri);
-		gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
-		nbmsg=linphone_chat_room_get_unread_messages_count(cr);
-		if(nbmsg != 0){
-			gtk_list_store_set(store,&iter,FRIEND_CHAT,"linphone-chat-new-message",
-					   FRIEND_CHAT_BUTTON_VISIBLE, TRUE, -1);
+		if (f_addr != NULL) {
+			cr=linphone_gtk_create_chatroom(f_addr);
+			gtk_list_store_set(store,&iter,FRIEND_CHATROOM,cr,-1);
+			nbmsg=linphone_chat_room_get_unread_messages_count(cr);
+			if(nbmsg != 0){
+				gtk_list_store_set(store,&iter,FRIEND_CHAT,"linphone-chat-new-message",
+						FRIEND_CHAT_BUTTON_VISIBLE, TRUE, -1);
+			}
+			escaped=g_markup_escape_text(uri,-1);
+			gtk_list_store_set(store,&iter,FRIEND_SIP_ADDRESS,escaped,-1);
+			g_free(escaped);
+			ms_free(uri);
 		}
-		escaped=g_markup_escape_text(uri,-1);
-		gtk_list_store_set(store,&iter,FRIEND_SIP_ADDRESS,escaped,-1);
-		g_free(escaped);
-		ms_free(uri);
 	}
 	bctbx_list_free(sorted);
 }
@@ -779,12 +808,14 @@ void linphone_gtk_show_contact(LinphoneFriend *lf, GtkWidget *parent){
 	GtkWidget *w = linphone_gtk_create_window("contact", parent);
 	char *uri;
 	const char *name = linphone_friend_get_name(lf);
-	const LinphoneAddress *f_uri = linphone_friend_get_address(lf);
+	const LinphoneAddress *f_addr = linphone_friend_get_address(lf);
 
-	uri=linphone_address_as_string_uri_only(f_uri);
-	if (uri) {
-		gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(w,"sip_address")),uri);
-		ms_free(uri);
+	if (f_addr != NULL) {
+		uri=linphone_address_as_string_uri_only(f_addr);
+		if (uri) {
+			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(w,"sip_address")),uri);
+			ms_free(uri);
+		}
 	}
 
 	if (name){
@@ -848,7 +879,7 @@ void linphone_gtk_contact_ok(GtkWidget *button){
 			linphone_core_add_friend(linphone_gtk_get_core(),lf);
 		}
 	}
-	linphone_address_destroy(friend_address);
+	linphone_address_unref(friend_address);
 	linphone_gtk_show_friends();
 	gtk_widget_destroy(w);
 }
