@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 /*
 *  C Implementation: incall_frame
@@ -126,7 +126,7 @@ static void linphone_gtk_in_call_set_animation_spinner(GtkWidget *callview){
 
 static void linphone_gtk_transfer_call(LinphoneCall *dest_call){
 	LinphoneCall *call=linphone_gtk_get_currently_displayed_call(NULL);
-	if (call) linphone_core_transfer_call_to_another(linphone_gtk_get_core(),call,dest_call);
+	if (call) linphone_call_transfer_to_another(call,dest_call);
 }
 
 void transfer_button_clicked(GtkWidget *button, gpointer call_ref){
@@ -171,7 +171,6 @@ void linphone_gtk_enable_transfer_button(LinphoneCore *lc, gboolean value){
 
 static void conference_button_clicked(GtkWidget *button, gpointer call_ref){
 	gtk_widget_set_sensitive(button,FALSE);
-	g_object_set_data(G_OBJECT(linphone_gtk_get_main_window()),"conf_frame",NULL);
 	linphone_core_add_all_to_conference(linphone_gtk_get_core());
 
 }
@@ -243,8 +242,10 @@ static const char *upnp_state_to_string(LinphoneUpnpState ice_state){
 }
 
 static void _refresh_call_stats(GtkWidget *callstats, LinphoneCall *call){
-	const LinphoneCallStats *as=linphone_call_get_audio_stats(call);
-	const LinphoneCallStats *vs=linphone_call_get_video_stats(call);
+	LinphoneUpnpState upnp_state;
+	LinphoneIceState ice_state;
+	LinphoneCallStats *as=linphone_call_get_audio_stats(call);
+	LinphoneCallStats *vs=linphone_call_get_video_stats(call);
 	const char *audio_media_connectivity = _("Direct or through server");
 	const char *video_media_connectivity = _("Direct or through server");
 	const LinphoneCallParams *curparams=linphone_call_get_current_params(call);
@@ -256,7 +257,7 @@ static void _refresh_call_stats(GtkWidget *callstats, LinphoneCall *call){
 	gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"rtp_profile")),tmp);
 	g_free(tmp);
 	tmp=g_strdup_printf(_("download: %f\nupload: %f (kbit/s)"),
-		as->download_bandwidth,as->upload_bandwidth);
+		linphone_call_stats_get_download_bandwidth(as),linphone_call_stats_get_upload_bandwidth(as));
 	gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"audio_bandwidth_usage")),tmp);
 	g_free(tmp);
 	if (has_video){
@@ -267,7 +268,7 @@ static void _refresh_call_stats(GtkWidget *callstats, LinphoneCall *call){
 		gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_size_recv")),size_r);
 		gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_size_sent")),size_s);
 
-		tmp=g_strdup_printf(_("download: %f\nupload: %f (kbit/s)"),vs->download_bandwidth,vs->upload_bandwidth);
+		tmp=g_strdup_printf(_("download: %f\nupload: %f (kbit/s)"),linphone_call_stats_get_download_bandwidth(vs),linphone_call_stats_get_upload_bandwidth(vs));
 		g_free(size_r);
 		g_free(size_s);
 	} else {
@@ -275,27 +276,33 @@ static void _refresh_call_stats(GtkWidget *callstats, LinphoneCall *call){
 	}
 	gtk_label_set_markup(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_bandwidth_usage")),tmp);
 	if (tmp) g_free(tmp);
-	if(as->upnp_state != LinphoneUpnpStateNotAvailable && as->upnp_state != LinphoneUpnpStateIdle) {
-		audio_media_connectivity = upnp_state_to_string(as->upnp_state);
-	} else if(as->ice_state != LinphoneIceStateNotActivated) {
-		audio_media_connectivity = ice_state_to_string(as->ice_state);
+	upnp_state = linphone_call_stats_get_upnp_state(as);
+	ice_state = linphone_call_stats_get_ice_state(as);
+	if(upnp_state != LinphoneUpnpStateNotAvailable && upnp_state != LinphoneUpnpStateIdle) {
+		audio_media_connectivity = upnp_state_to_string(upnp_state);
+	} else if(ice_state != LinphoneIceStateNotActivated) {
+		audio_media_connectivity = ice_state_to_string(ice_state);
 	}
 	gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"audio_media_connectivity")),audio_media_connectivity);
 
 	if (has_video){
-		if(vs->upnp_state != LinphoneUpnpStateNotAvailable && vs->upnp_state != LinphoneUpnpStateIdle) {
-				video_media_connectivity = upnp_state_to_string(vs->upnp_state);
-		} else if(vs->ice_state != LinphoneIceStateNotActivated) {
-			video_media_connectivity = ice_state_to_string(vs->ice_state);
+		upnp_state = linphone_call_stats_get_upnp_state(vs);
+		ice_state = linphone_call_stats_get_ice_state(vs);
+		if(upnp_state != LinphoneUpnpStateNotAvailable && upnp_state != LinphoneUpnpStateIdle) {
+				video_media_connectivity = upnp_state_to_string(upnp_state);
+		} else if(ice_state != LinphoneIceStateNotActivated) {
+			video_media_connectivity = ice_state_to_string(ice_state);
 		}
 	}else video_media_connectivity=NULL;
 	gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"video_media_connectivity")),video_media_connectivity);
 
-	if (as->round_trip_delay>0){
-		tmp=g_strdup_printf(_("%.3f seconds"),as->round_trip_delay);
+	if (linphone_call_stats_get_round_trip_delay(as)>0){
+		tmp=g_strdup_printf(_("%.3f seconds"),linphone_call_stats_get_round_trip_delay(as));
 		gtk_label_set_text(GTK_LABEL(linphone_gtk_get_widget(callstats,"round_trip_time")),tmp);
 		g_free(tmp);
 	}
+	linphone_call_stats_unref(as);
+	linphone_call_stats_unref(vs);
 }
 
 static gboolean refresh_call_stats(GtkWidget *callstats){
@@ -464,8 +471,8 @@ static void video_button_clicked(GtkWidget *button, LinphoneCall *call){
 	LinphoneCallParams *params = linphone_core_create_call_params(lc, call);
 	gtk_widget_set_sensitive(button,FALSE);
 	linphone_call_params_enable_video(params, adding);
-	linphone_core_update_call(lc,call,params);
-	linphone_call_params_destroy(params);
+	linphone_call_update(call,params);
+	linphone_call_params_unref(params);
 }
 
 void linphone_gtk_update_video_button(LinphoneCall *call){
@@ -949,11 +956,11 @@ void linphone_gtk_hold_clicked(GtkButton *button){
 	if (!call) return;
 	if(!active)
 	{
-		linphone_core_pause_call(linphone_gtk_get_core(),call);
+		linphone_call_pause(call);
 	}
 	else
 	{
-		linphone_core_resume_call(linphone_gtk_get_core(),call);
+		linphone_call_resume(call);
 	}
 }
 
@@ -1014,7 +1021,7 @@ void linphone_gtk_record_call_toggled(GtkWidget *button){
 			linphone_core_stop_conference_recording(lc);
 
 	}
-	gtk_label_set_markup(GTK_LABEL(label),message);
+	gtk_label_set_markup(GTK_LABEL(label),g_locale_to_utf8(message, -1, NULL, NULL, NULL));
 	g_free(message);
 }
 
