@@ -14,12 +14,12 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "linphone.h"
-#include "linphone_tunnel.h"
-#include "lpconfig.h"
+#include "linphone/tunnel.h"
+#include "linphone/lpconfig.h"
 #include "config.h"
 
 void linphone_gtk_fill_combo_box(GtkWidget *combo, const char **devices, const char *selected, DeviceCap cap){
@@ -287,7 +287,7 @@ void linphone_gtk_update_my_contact(GtkWidget *w){
 	gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"sip_address")),contact);
 	linphone_core_set_primary_contact(linphone_gtk_get_core(),contact);
 	ms_free(contact);
-	linphone_address_destroy(parsed);
+	linphone_address_unref(parsed);
 	linphone_gtk_load_identities();
 }
 
@@ -464,8 +464,17 @@ void linphone_gtk_alsa_special_device_changed(GtkWidget *w){
 }
 
 void linphone_gtk_cam_changed(GtkWidget *w){
+	LinphoneCall *call;
+	LinphoneCore *lc = linphone_gtk_get_core();
 	gchar *sel=gtk_combo_box_get_active_text(GTK_COMBO_BOX(w));
-	linphone_core_set_video_device(linphone_gtk_get_core(),sel);
+	if (sel){
+		if (strcmp(sel, linphone_core_get_video_device(lc)) != 0){
+			linphone_core_set_video_device(lc,sel);
+			if ((call = linphone_core_get_current_call(lc)) != NULL) {
+				linphone_call_update(call, NULL);
+			}
+		}
+	}
 	g_free(sel);
 }
 
@@ -638,7 +647,7 @@ static void linphone_gtk_init_codec_list(GtkTreeView *listview){
 }
 
 
-const char *get_codec_color(LinphoneCore *lc, PayloadType *pt){
+const char *get_codec_color(LinphoneCore *lc, OrtpPayloadType *pt){
 	const gchar *color;
 	if (linphone_core_check_payload_type_usability(lc,pt)) color="blue";
 		else color="red";
@@ -664,7 +673,7 @@ static void linphone_gtk_show_codecs(GtkTreeView *listview, const bctbx_list_t *
 		const gchar *color;
 		const char *params="";
 
-		struct _PayloadType *pt=(struct _PayloadType *)elem->data;
+		OrtpPayloadType *pt=(OrtpPayloadType *)elem->data;
 
 		color=get_codec_color(linphone_gtk_get_core(),pt);
 		if (linphone_core_payload_type_enabled(linphone_gtk_get_core(),pt)) status=_("Enabled");
@@ -807,7 +816,7 @@ static void linphone_gtk_codec_move(GtkWidget *button, int dir, int type){ /* 0=
 			else before=sel_elem;
 			codec_list=bctbx_list_insert(codec_list,before,pt);
 		}
-		codec_list=bctbx_list_remove_link(codec_list,sel_elem);
+		codec_list=bctbx_list_erase_link(codec_list,sel_elem);
 		if (pt->type==PAYLOAD_VIDEO)
 			linphone_core_set_video_codecs(lc,codec_list);
 		else linphone_core_set_audio_codecs(lc,codec_list);
@@ -974,7 +983,7 @@ static void update_proxy_transport(GtkWidget *w){
 		}else{
 			fill_transport_combo_box(combo,linphone_address_get_transport(laddr),TRUE);
 		}
-		linphone_address_destroy(laddr);
+		linphone_address_unref(laddr);
 	}
 }
 
@@ -998,7 +1007,7 @@ void linphone_gtk_proxy_transport_changed(GtkWidget *combo){
 			gtk_entry_set_text(GTK_ENTRY(proxy),newaddr);
 			ms_free(newaddr);
 		}
-		linphone_address_destroy(laddr);
+		linphone_address_unref(laddr);
 	}
 	g_object_set_data(G_OBJECT(w),"combo-updating",GINT_TO_POINTER(0));
 }
@@ -1074,7 +1083,7 @@ void linphone_gtk_proxy_ok(GtkButton *button){
 					linphone_proxy_config_set_server_addr(cfg,tmp);
 					ms_free(tmp);
 				}
-				linphone_address_destroy(laddr);
+				linphone_address_unref(laddr);
 			}
 		}
 	}
@@ -1297,7 +1306,7 @@ static void linphone_gtk_media_encryption_changed(GtkWidget *combo){
 			gtk_widget_set_sensitive(mandatory_box,TRUE);
 		}else if (strcasecmp(selected,"ZRTP")==0){
 			linphone_core_set_media_encryption(lc,LinphoneMediaEncryptionZRTP);
-			gtk_widget_set_sensitive(mandatory_box,FALSE);
+			gtk_widget_set_sensitive(mandatory_box,TRUE);
 		} else {
 			linphone_core_set_media_encryption(lc,LinphoneMediaEncryptionNone);
 			gtk_widget_set_sensitive(mandatory_box,FALSE);
@@ -1389,7 +1398,7 @@ static void linphone_gtk_show_media_encryption(GtkWidget *pb){
 			case LinphoneMediaEncryptionZRTP:
 				if (zrtp_id!=-1) {
 					gtk_combo_box_set_active(GTK_COMBO_BOX(combo),zrtp_id);
-					linphone_gtk_set_media_encryption_mandatory_sensitive(pb,FALSE);
+					linphone_gtk_set_media_encryption_mandatory_sensitive(pb,TRUE);
 				}
 			break;
 		}
@@ -1397,16 +1406,9 @@ static void linphone_gtk_show_media_encryption(GtkWidget *pb){
 	}
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(pb,"media_encryption_mandatory_checkbox")),
 				     linphone_core_is_media_encryption_mandatory(lc));
-	
-	gtk_widget_set_visible (linphone_gtk_get_widget(pb,"chat_lime_label"), 0);
-	gtk_widget_set_visible (linphone_gtk_get_widget(pb,"chat_lime_combo"), 0);
-	
-	if (linphone_gtk_get_ui_config_int("lime_experimental_feature", 0)){
-		gtk_widget_set_visible (linphone_gtk_get_widget(pb,"chat_lime_label"), 1);
-		gtk_widget_set_visible (linphone_gtk_get_widget(pb,"chat_lime_combo"), 1);
-		gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(pb,"chat_lime_combo")), linphone_core_lime_enabled(lc));
-		gtk_widget_set_sensitive(linphone_gtk_get_widget(pb,"chat_lime_combo"), linphone_core_lime_available(lc));
-	}
+
+	gtk_combo_box_set_active(GTK_COMBO_BOX(linphone_gtk_get_widget(pb,"chat_lime_combo")), linphone_core_lime_enabled(lc));
+	gtk_widget_set_sensitive(linphone_gtk_get_widget(pb,"chat_lime_combo"), linphone_core_lime_available(lc));
 
 	g_object_unref(G_OBJECT(model));
 }
@@ -1428,9 +1430,11 @@ void linphone_gtk_fill_soundcards(GtkWidget *pb){
 }
 
 void linphone_gtk_fill_webcams(GtkWidget *pb){
+#ifdef VIDEO_ENABLED
 	LinphoneCore *lc=linphone_gtk_get_core();
 	linphone_gtk_fill_combo_box(linphone_gtk_get_widget(pb,"webcams"),linphone_core_get_video_devices(lc),
 					linphone_core_get_video_device(lc),CAP_IGNORE);
+#endif
 }
 
 void linphone_gtk_fill_video_renderers(GtkWidget *pb){
@@ -1459,7 +1463,7 @@ void linphone_gtk_fill_video_renderers(GtkWidget *pb){
 		GtkTreeIter iter;
 
 		/* do not offer the user to select combo 'decoding/rendering' filter */
-		if (desc->enc_fmt != NULL)
+		if (desc->enc_fmt != NULL || (desc->flags & MS_FILTER_IS_ENABLED) == 0)
 			continue;
 
 		gtk_list_store_append(store,&iter);
@@ -1668,6 +1672,9 @@ void linphone_gtk_show_parameters(void){
 	if(!linphone_core_upnp_available()) {
 		gtk_widget_hide(linphone_gtk_get_widget(pb,"use_upnp"));
 	}
+	gtk_widget_hide(linphone_gtk_get_widget(pb, "use_nat_address"));
+	gtk_widget_hide(linphone_gtk_get_widget(pb, "nat_address"));
+	gtk_widget_hide(linphone_gtk_get_widget(pb, "nat_address_label"));
 
 	mtu=linphone_core_get_mtu(lc);
 	if (mtu<=0){
@@ -1689,6 +1696,9 @@ void linphone_gtk_show_parameters(void){
 	linphone_gtk_fill_video_sizes(linphone_gtk_get_widget(pb,"video_size"));
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(linphone_gtk_get_widget(pb,"video_framerate")),
 				linphone_core_get_preferred_framerate(lc));
+#ifndef VIDEO_ENABLED
+	gtk_widget_set_visible(linphone_gtk_get_widget(pb, "camera_preview_button"),FALSE);
+#endif
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(pb,"echo_cancelation")),
 					linphone_core_echo_cancellation_enabled(lc));
 
@@ -1705,7 +1715,7 @@ void linphone_gtk_show_parameters(void){
 		}
 		if (linphone_address_get_username(contact))
 			gtk_entry_set_text(GTK_ENTRY(linphone_gtk_get_widget(pb,"username")),linphone_address_get_username(contact));
-		linphone_address_destroy(contact);
+		linphone_address_unref(contact);
 	}
 #ifdef BUILD_WIZARD
 	gtk_widget_show(linphone_gtk_get_widget(pb,"wizard"));
@@ -1809,10 +1819,16 @@ void linphone_gtk_edit_tunnel(GtkButton *button){
 	if (port==0) port=443;
 	gtk_spin_button_set_value(GTK_SPIN_BUTTON(linphone_gtk_get_widget(w,"port")), port);
 
-	if (linphone_tunnel_get_mode(tunnel)){
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"radio_enable")),1);
-	} else{
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"radio_disable")),1);
+	switch(linphone_tunnel_get_mode(tunnel)){
+		case LinphoneTunnelModeDisable:
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"radio_disable")),1);
+		break;
+		case LinphoneTunnelModeEnable:
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"radio_enable")),1);
+		break;
+		case LinphoneTunnelModeAuto:
+			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"tunnel_autodetect")),1);
+		break;
 	}
 	{
 		const char *proxy=NULL,*username=NULL,*password=NULL;
@@ -1840,11 +1856,13 @@ void linphone_gtk_tunnel_ok(GtkButton *button){
 
 	gint port = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(linphone_gtk_get_widget(w,"port")));
 	gboolean enabled=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"radio_enable")));
+	gboolean autodetect=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w,"tunnel_autodetect")));
 	const char *host=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"host")));
 	const char *http_host=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"http_host")));
 	gint http_port = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(linphone_gtk_get_widget(w,"http_port")));
 	const char *username=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"username")));
 	const char *password=gtk_entry_get_text(GTK_ENTRY(linphone_gtk_get_widget(w,"password")));
+	LinphoneTunnelMode mode = LinphoneTunnelModeDisable;
 
 	if (tunnel==NULL) return;
 	if (host && *host=='\0') host=NULL;
@@ -1853,7 +1871,13 @@ void linphone_gtk_tunnel_ok(GtkButton *button){
 	linphone_tunnel_config_set_host(config, host);
 	linphone_tunnel_config_set_port(config, port);
 	linphone_tunnel_add_server(tunnel, config);
-	linphone_tunnel_set_mode(tunnel, (enabled ? LinphoneTunnelModeEnable : LinphoneTunnelModeDisable));
+
+	if (enabled){
+		mode = LinphoneTunnelModeEnable;
+	}else if (autodetect){
+		mode = LinphoneTunnelModeAuto;
+	}
+	linphone_tunnel_set_mode(tunnel, mode);
 	linphone_tunnel_set_http_proxy(tunnel,http_host,http_port,username,password);
 
 	gtk_widget_destroy(w);
