@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "linphone.h"
@@ -31,9 +31,9 @@ static LinphoneAccountCreator * linphone_gtk_assistant_get_creator(GtkWidget *w)
 	return (LinphoneAccountCreator *)g_object_get_data(G_OBJECT(w), "creator");
 }
 
-static void linphone_gtk_create_account_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
+static void linphone_gtk_create_account_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status, const char* resp) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
-	if (status == LinphoneAccountCreatorAccountCreated) {
+	if (status == LinphoneAccountCreatorStatusAccountCreated) {
 		// Go to page_6_linphone_account_validation_wait
 		gtk_assistant_set_current_page(GTK_ASSISTANT(assistant), 6);
 	} else { // Error when attempting to create the account
@@ -48,9 +48,9 @@ static void create_account(GtkWidget *assistant) {
 	linphone_account_creator_create_account(creator);
 }
 
-static void linphone_gtk_test_account_validation_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
+static void linphone_gtk_test_account_validation_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status, const char* resp) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
-	if (status == LinphoneAccountCreatorAccountValidated) {
+	if (status == LinphoneAccountCreatorStatusAccountActivated) {
 		// Go to page_9_finish
 		gtk_assistant_set_current_page(GTK_ASSISTANT(assistant), 9);
 	} else {
@@ -61,7 +61,7 @@ static void linphone_gtk_test_account_validation_cb(LinphoneAccountCreator *crea
 
 static void check_account_validation(GtkWidget *assistant) {
 	LinphoneAccountCreator *creator = linphone_gtk_assistant_get_creator(assistant);
-	linphone_account_creator_test_validation(creator);
+	linphone_account_creator_is_account_activated(creator);
 }
 
 void linphone_gtk_assistant_closed(GtkWidget *w) {
@@ -154,11 +154,11 @@ static int linphone_gtk_assistant_forward(int curpage, gpointer data) {
 			GtkEntry *password_entry = GTK_ENTRY(linphone_gtk_get_widget(w, "p4_entry_password1"));
 			GtkEntry *username_entry = GTK_ENTRY(linphone_gtk_get_widget(w, "p4_entry_username"));
 			GtkEntry *email_entry = GTK_ENTRY(linphone_gtk_get_widget(w, "p4_entry_email"));
-			GtkToggleButton *newsletter = GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w, "p4_check_newsletter"));
+			//GtkToggleButton *newsletter = GTK_TOGGLE_BUTTON(linphone_gtk_get_widget(w, "p4_check_newsletter"));
 			linphone_account_creator_set_username(creator, gtk_entry_get_text(username_entry));
 			linphone_account_creator_set_password(creator, gtk_entry_get_text(password_entry));
 			linphone_account_creator_set_email(creator, gtk_entry_get_text(email_entry));
-			linphone_account_creator_enable_newsletter_subscription(creator, gtk_toggle_button_get_active(newsletter));
+			//linphone_account_creator_enable_newsletter_subscription(creator, gtk_toggle_button_get_active(newsletter));
 			curpage = 5; // Go to page_5_linphone_account_creation_in_progress
 			break;
 		}
@@ -192,8 +192,12 @@ void linphone_gtk_external_account_configuration_changed(GtkEntry *entry) {
 	gtk_assistant_set_page_complete(GTK_ASSISTANT(assistant), page, external_account_configuration_complete(page) > 0);
 }
 
+static bool_t check_username_validity(const char *username) {
+	return username  && g_regex_match_simple("^[a-zA-Z]+[a-zA-Z0-9.\\-_]{3,}$", username, 0, 0);
+}
+
 void linphone_gtk_account_configuration_changed(GtkEntry *entry, GtkAssistant *assistant) {
-	gboolean complete = (gtk_entry_get_text_length(entry) > 0);
+	gboolean complete = check_username_validity(gtk_entry_get_text(entry));
 	GtkWidget *page = gtk_assistant_get_nth_page(assistant, gtk_assistant_get_current_page(assistant));
 	gtk_assistant_set_page_complete(assistant, page, complete);
 }
@@ -211,11 +215,11 @@ static gboolean update_interface_with_username_availability(GtkWidget *page) {
 	GtkLabel* usernameError = GTK_LABEL(linphone_gtk_get_widget(assistant, "p4_label_error"));
 	int account_status = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(page), "is_username_used"));
 
-	if (account_status == LinphoneAccountCreatorAccountNotExist) {
+	if (account_status == LinphoneAccountCreatorStatusAccountNotExist) {
 		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(1));
 		gtk_image_set_from_stock(isUsernameOk, GTK_STOCK_OK, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_label_set_text(usernameError, "");
-	} else if (account_status == LinphoneAccountCreatorAccountExist) {
+	} else if (account_status == LinphoneAccountCreatorStatusAccountExist) {
 		gtk_label_set_text(usernameError, _("Username is already in use!"));
 		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(0));
 		gtk_image_set_from_stock(isUsernameOk, GTK_STOCK_NO, GTK_ICON_SIZE_LARGE_TOOLBAR);
@@ -228,7 +232,7 @@ static gboolean update_interface_with_username_availability(GtkWidget *page) {
 	return FALSE;
 }
 
-static void linphone_gtk_test_account_existence_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status) {
+static void linphone_gtk_test_account_existence_cb(LinphoneAccountCreator *creator, LinphoneAccountCreatorStatus status, const char* resp) {
 	GtkWidget *assistant = (GtkWidget *)linphone_account_creator_get_user_data(creator);
 	GtkWidget *page = gtk_assistant_get_nth_page(GTK_ASSISTANT(assistant), gtk_assistant_get_current_page(GTK_ASSISTANT(assistant)));
 	g_object_set_data(G_OBJECT(page), "is_username_used", GINT_TO_POINTER(status));
@@ -239,7 +243,7 @@ static gboolean check_username_availability(GtkWidget *assistant) {
 	LinphoneAccountCreator *creator = linphone_gtk_assistant_get_creator(assistant);
 	GtkWidget *page = gtk_assistant_get_nth_page(GTK_ASSISTANT(assistant), gtk_assistant_get_current_page(GTK_ASSISTANT(assistant)));
 	g_object_set_data(G_OBJECT(page), "usernameAvailabilityTimerID", GUINT_TO_POINTER(0));
-	linphone_account_creator_test_existence(creator);
+	linphone_account_creator_is_account_exist(creator);
 	return FALSE;
 }
 
@@ -256,7 +260,7 @@ void linphone_gtk_account_creation_username_changed(GtkEntry *entry) {
 	linphone_account_creator_set_domain(creator, "sip.linphone.org");
 	linphone_account_creator_set_route(creator, "sip.linphone.org");
 
-	if (g_regex_match_simple("^[a-zA-Z]+[a-zA-Z0-9.\\-_]{3,}$", gtk_entry_get_text(username), 0, 0)) {
+	if (check_username_validity(gtk_entry_get_text(username))) {
 		guint timerID = GPOINTER_TO_UINT(g_object_get_data(G_OBJECT(page), "usernameAvailabilityTimerID"));
 		if (timerID > 0) {
 			g_source_remove(timerID);
@@ -266,7 +270,7 @@ void linphone_gtk_account_creation_username_changed(GtkEntry *entry) {
 	} else {
 		if (gtk_entry_get_text_length(username) < LOGIN_MIN_SIZE) {
 			gtk_label_set_text(usernameError, "Username is too short");
-		} else if (!g_regex_match_simple("^[a-zA-Z]+[a-zA-Z0-9.\\-_]{3,}$", gtk_entry_get_text(username), 0, 0)) {
+		} else if (!check_username_validity(gtk_entry_get_text(username))) {
 			gtk_label_set_text(usernameError, "Unauthorized username");
 		}
 		g_object_set_data(G_OBJECT(page), "is_username_available", GINT_TO_POINTER(0));
@@ -319,11 +323,11 @@ void linphone_gtk_account_creation_password_changed(GtkEntry *entry) {
 }
 
 static void linphone_gtk_assistant_init(GtkWidget *w) {
-	LinphoneAccountCreator *creator = linphone_account_creator_new(linphone_gtk_get_core(), "https://www.linphone.org/wizard.php");
+	LinphoneAccountCreator *creator = linphone_account_creator_new(linphone_gtk_get_core(), "https://subscribe.linphone.org:444/wizard.php");
 	LinphoneAccountCreatorCbs *cbs = linphone_account_creator_get_callbacks(creator);
 	linphone_account_creator_set_user_data(creator, w);
-	linphone_account_creator_cbs_set_existence_tested(cbs, linphone_gtk_test_account_existence_cb);
-	linphone_account_creator_cbs_set_validation_tested(cbs, linphone_gtk_test_account_validation_cb);
+	linphone_account_creator_cbs_set_is_account_exist(cbs, linphone_gtk_test_account_existence_cb);
+	linphone_account_creator_cbs_set_is_account_activated(cbs, linphone_gtk_test_account_validation_cb);
 	linphone_account_creator_cbs_set_create_account(cbs, linphone_gtk_create_account_cb);
 	g_object_set_data(G_OBJECT(w), "creator", creator);
 
