@@ -1,6 +1,6 @@
 #include "linphone.h"
 
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 static GList *pixmaps_directories = NULL;
 
@@ -193,6 +193,9 @@ void linphone_gtk_set_ui_config(const char *key , const char * val){
 const char *linphone_gtk_get_sound_path(const char *name){
 	static char *ret=NULL;
 	const char *file;
+	const char *sound_dir;
+	LinphoneFactory *factory = linphone_factory_get();
+
 	file=linphone_gtk_get_ui_config(name,NULL);
 	if (file==NULL){
 		char *dirname=g_path_get_dirname(name);
@@ -207,7 +210,8 @@ const char *linphone_gtk_get_sound_path(const char *name){
 		g_free(ret);
 		ret=NULL;
 	}
-	ret=g_build_filename(PACKAGE_SOUND_DIR,name,NULL);
+	sound_dir = linphone_factory_get_sound_resources_dir(factory);
+	ret=g_build_filename(sound_dir,name,NULL);
 	return ret;
 }
 
@@ -236,14 +240,14 @@ void linphone_gtk_visibility_set(const char *hiddens, const char *window_name, G
 	int len;
 	for(b=i=hiddens;*i!='\0';++i){
 		if (*i==' '){
-			len=MIN(i-b,sizeof(item)-1);
+			len=MIN(i-b,(int)sizeof(item)-1);
 			strncpy(item,b,len);
 			item[len]='\0';
 			b=i+1;
 			parse_item(item,window_name,w,show);
 		}
 	}
-	len=MIN(i-b,sizeof(item)-1);
+	len=MIN(i-b,(int)sizeof(item)-1);
 	if (len>0){
 		strncpy(item,b,len);
 		item[len]='\0';
