@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 #include "linphone.h"
@@ -42,7 +42,7 @@ static GtkTargetEntry targets[] = {
 
 static void set_video_controls_position(GtkWidget *video_window);
 
-static void on_end_of_play(LinphonePlayer *player, void *user_data){
+static void on_end_of_play(LinphonePlayer *player){
 	linphone_player_close(player);
 }
 
@@ -61,9 +61,10 @@ static void drag_data_received(GtkWidget *widget, GdkDragContext *context, gint
 			datalen--;
 		}
 		if (player){
-
+			LinphonePlayerCbs *cbs = linphone_player_get_callbacks(player);
+			linphone_player_cbs_set_eof_reached(cbs, on_end_of_play);
 			const char* filepath = (strstr(path,"file://")==path) ? path+strlen("file://") : path;
-			if (linphone_player_open(player,filepath,on_end_of_play,NULL)==0){
+			if (linphone_player_open(player,filepath)==0){
 
 				linphone_player_start(player);
 			}else{
@@ -183,7 +184,7 @@ static void on_controls_response(GtkWidget *dialog, int response_id, GtkWidget *
 		case GTK_RESPONSE_REJECT:
 		{
 			LinphoneCall *call=(LinphoneCall*)g_object_get_data(G_OBJECT(video_window),"call");
-			linphone_core_terminate_call(linphone_gtk_get_core(),call);
+			linphone_call_terminate(call);
 		}
 		break;
 		case GTK_RESPONSE_APPLY:
