@@ -14,7 +14,7 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 /**
@@ -35,7 +35,15 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "belle-sip/belle-sip.h"
 
 #ifndef LINPHONE_PUBLIC
-	#define LINPHONE_PUBLIC MS2_PUBLIC
+#if defined(_MSC_VER)
+#ifdef LINPHONE_EXPORTS
+#define LINPHONE_PUBLIC	__declspec(dllexport)
+#else
+#define LINPHONE_PUBLIC	__declspec(dllimport)
+#endif
+#else
+#define LINPHONE_PUBLIC
+#endif
 #endif
 
 #ifdef __cplusplus
@@ -125,13 +133,18 @@ void sal_address_set_transport(SalAddress* addr,SalTransport transport);
 void sal_address_set_transport_name(SalAddress* addr,const char* transport);
 void sal_address_set_method_param(SalAddress *addr, const char *method);
 void sal_address_set_params(SalAddress *addr, const char *params);
+bool_t sal_address_has_param(const SalAddress *addr, const char *name);
+const char * sal_address_get_param(const SalAddress *addr, const char *name);
+void sal_address_set_uri_param(SalAddress *addr, const char *name, const char *value);
 void sal_address_set_uri_params(SalAddress *addr, const char *params);
-bool_t sal_address_has_uri_param(SalAddress *addr, const char *name);
+bool_t sal_address_has_uri_param(const SalAddress *addr, const char *name);
+const char * sal_address_get_uri_param(const SalAddress *addr, const char *name);
 bool_t sal_address_is_ipv6(const SalAddress *addr);
 bool_t sal_address_is_sip(const SalAddress *addr);
 void sal_address_set_password(SalAddress *addr, const char *passwd);
 const char *sal_address_get_password(const SalAddress *addr);
 void sal_address_set_header(SalAddress *addr, const char *header_name, const char *header_value);
+const char *sal_address_get_header(const SalAddress *addr, const char *name);
 
 LINPHONE_PUBLIC Sal * sal_init(MSFactory *factory);
 LINPHONE_PUBLIC void sal_uninit(Sal* sal);
@@ -267,6 +280,7 @@ typedef struct SalStreamDescription{
 	SalMulticastRole multicast_role;
 } SalStreamDescription;
 
+const char *sal_multicast_role_to_string(SalMulticastRole role);
 const char *sal_stream_description_get_type_as_string(const SalStreamDescription *desc);
 const char *sal_stream_description_get_proto_as_string(const SalStreamDescription *desc);
 
@@ -301,11 +315,6 @@ typedef struct SalMessage{
 	time_t time;
 }SalMessage;
 
-typedef struct SalIsComposing {
-	const char *from;
-	const char *text;
-} SalIsComposing;
-
 #define SAL_MEDIA_DESCRIPTION_MAX_MESSAGE_ATTRIBUTES 5
 
 SalMediaDescription *sal_media_description_new(void);
@@ -350,6 +359,7 @@ typedef struct SalOpBase{
 	SalAddress* to_address;
 	char *origin;
 	SalAddress* origin_address;
+	SalAddress* diversion_address;
 	char *remote_ua;
 	SalAddress* remote_contact_address;
 	char *remote_contact;
@@ -400,7 +410,9 @@ typedef struct SalErrorInfo{
 	char *status_string;
 	int protocol_code;
 	char *warnings;
+	char *protocol;
 	char *full_string; /*concatenation of status_string + warnings*/
+	struct SalErrorInfo *sub_sei;
 }SalErrorInfo;
 
 typedef enum SalPresenceStatus{
@@ -435,11 +447,11 @@ typedef enum SalSubscribeStatus{
 	SalSubscribeTerminated
 }SalSubscribeStatus;
 
-typedef enum SalTextDeliveryStatus{
-	SalTextDeliveryInProgress,
-	SalTextDeliveryDone,
-	SalTextDeliveryFailed
-}SalTextDeliveryStatus;
+typedef enum SalMessageDeliveryStatus{
+	SalMessageDeliveryInProgress,
+	SalMessageDeliveryDone,
+	SalMessageDeliveryFailed
+}SalMessageDeliveryStatus;
 
 /**
  * auth event mode
@@ -479,11 +491,13 @@ typedef struct SalAuthInfo{
 typedef void (*SalOnCallReceived)(SalOp *op);
 typedef void (*SalOnCallRinging)(SalOp *op);
 typedef void (*SalOnCallAccepted)(SalOp *op);
-typedef void (*SalOnCallAck)(SalOp *op);
+typedef void (*SalOnCallAckReceived)(SalOp *op, SalCustomHeader *ack);
+typedef void (*SalOnCallAckBeingSent)(SalOp *op, SalCustomHeader *ack);
 typedef void (*SalOnCallUpdating)(SalOp *op, bool_t is_update);/*< Called when a reINVITE/UPDATE is received*/
 typedef void (*SalOnCallTerminated)(SalOp *op, const char *from);
 typedef void (*SalOnCallFailure)(SalOp *op);
 typedef void (*SalOnCallReleased)(SalOp *salop);
+typedef void (*SalOnCallCancelDone)(SalOp *salop);
 typedef void (*SalOnAuthRequestedLegacy)(SalOp *op, const char *realm, const char *username);
 typedef bool_t (*SalOnAuthRequested)(Sal *sal,SalAuthInfo* info);
 typedef void (*SalOnAuthFailure)(SalOp *op, SalAuthInfo* info);
@@ -492,11 +506,10 @@ typedef void (*SalOnRegisterFailure)(SalOp *op);
 typedef void (*SalOnVfuRequest)(SalOp *op);
 typedef void (*SalOnDtmfReceived)(SalOp *op, char dtmf);
 typedef void (*SalOnRefer)(Sal *sal, SalOp *op, const char *referto);
-typedef void (*SalOnTextReceived)(SalOp *op, const SalMessage *msg);
-typedef void (*SalOnTextDeliveryUpdate)(SalOp *op, SalTextDeliveryStatus);
-typedef void (*SalOnIsComposingReceived)(SalOp *op, const SalIsComposing *is_composing);
+typedef void (*SalOnMessageReceived)(SalOp *op, const SalMessage *msg);
+typedef void (*SalOnMessageDeliveryUpdate)(SalOp *op, SalMessageDeliveryStatus);
 typedef void (*SalOnNotifyRefer)(SalOp *op, SalReferStatus state);
-typedef void (*SalOnSubscribeResponse)(SalOp *op, SalSubscribeStatus status);
+typedef void (*SalOnSubscribeResponse)(SalOp *op, SalSubscribeStatus status, int will_retry);
 typedef void (*SalOnNotify)(SalOp *op, SalSubscribeStatus status, const char *event, SalBodyHandler *body);
 typedef void (*SalOnSubscribeReceived)(SalOp *salop, const char *event, const SalBodyHandler *body);
 typedef void (*SalOnIncomingSubscribeClosed)(SalOp *salop);
@@ -516,22 +529,24 @@ typedef void (*SalOnExpire)(SalOp *salop);
 
 typedef struct SalCallbacks{
 	SalOnCallReceived call_received;
+	SalOnCallReceived call_rejected;
 	SalOnCallRinging call_ringing;
 	SalOnCallAccepted call_accepted;
-	SalOnCallAck call_ack;
+	SalOnCallAckReceived call_ack_received;
+	SalOnCallAckBeingSent call_ack_being_sent;
 	SalOnCallUpdating call_updating;
 	SalOnCallTerminated call_terminated;
 	SalOnCallFailure call_failure;
 	SalOnCallReleased call_released;
+	SalOnCallCancelDone call_cancel_done;
 	SalOnAuthFailure auth_failure;
 	SalOnRegisterSuccess register_success;
 	SalOnRegisterFailure register_failure;
 	SalOnVfuRequest vfu_request;
 	SalOnDtmfReceived dtmf_received;
 	SalOnRefer refer_received;
-	SalOnTextReceived text_received;
-	SalOnTextDeliveryUpdate text_delivery_update;
-	SalOnIsComposingReceived is_composing_received;
+	SalOnMessageReceived message_received;
+	SalOnMessageDeliveryUpdate message_delivery_update;
 	SalOnNotifyRefer notify_refer;
 	SalOnSubscribeReceived subscribe_received;
 	SalOnIncomingSubscribeClosed incoming_subscribe_closed;
@@ -575,6 +590,20 @@ void sal_certificates_chain_parse_file(SalAuthInfo* auth_info, const char* path,
  */
 void sal_signing_key_parse_file(SalAuthInfo* auth_info, const char* path, const char *passwd);
 
+/** Parse a buffer containing either a certificate chain order in PEM format or a single DER cert
+ * @param auth_info structure where to store the result of parsing
+ * @param buffer the buffer to parse
+ * @param format either PEM or DER
+ */
+void sal_certificates_chain_parse(SalAuthInfo* auth_info, const char* buffer, SalCertificateRawFormat format);
+
+/**
+ * Parse a buffer containing either a private or public rsa key
+ * @param auth_info structure where to store the result of parsing
+ * @param passwd password (optionnal)
+ */
+void sal_signing_key_parse(SalAuthInfo* auth_info, const char* buffer, const char *passwd);
+
 /**
  * Parse a directory for files containing certificate with the given subject CNAME
  * @param[out]	certificate_pem				the address of a string to store the certificate in PEM format. To be freed by caller
@@ -614,6 +643,8 @@ void sal_use_tcp_tls_keepalive(Sal *ctx, bool_t enabled);
 int sal_set_tunnel(Sal *ctx, void *tunnelclient);
 /*Default value is true*/
 void sal_enable_sip_update_method(Sal *ctx,bool_t value);
+bool_t sal_is_content_type_supported(const Sal *sal, const char *content_type);
+void sal_add_content_type_support(Sal *sal, const char *content_type);
 
 /**
  * returns keepalive period in ms
@@ -626,11 +657,12 @@ void sal_use_one_matching_codec_policy(Sal *ctx, bool_t one_matching_codec);
 void sal_use_rport(Sal *ctx, bool_t use_rports);
 void sal_enable_auto_contacts(Sal *ctx, bool_t enabled);
 void sal_set_root_ca(Sal* ctx, const char* rootCa);
+void sal_set_root_ca_data(Sal* ctx, const char* data);
 const char *sal_get_root_ca(Sal* ctx);
 void sal_verify_server_certificates(Sal *ctx, bool_t verify);
 void sal_verify_server_cn(Sal *ctx, bool_t verify);
 void sal_set_ssl_config(Sal *ctx, void *ssl_config);
-void sal_set_uuid(Sal*ctx, const char *uuid);
+LINPHONE_PUBLIC void sal_set_uuid(Sal*ctx, const char *uuid);
 int sal_create_uuid(Sal*ctx, char *uuid, size_t len);
 int sal_generate_uuid(char *uuid, size_t len);
 LINPHONE_PUBLIC void sal_enable_test_features(Sal*ctx, bool_t enabled);
@@ -653,10 +685,12 @@ void sal_op_set_from(SalOp *op, const char *from);
 void sal_op_set_from_address(SalOp *op, const SalAddress *from);
 void sal_op_set_to(SalOp *op, const char *to);
 void sal_op_set_to_address(SalOp *op, const SalAddress *to);
+void sal_op_set_diversion_address(SalOp *op, const SalAddress *diversion);
 SalOp *sal_op_ref(SalOp* h);
 void sal_op_stop_refreshing(SalOp *op);
 int sal_op_refresh(SalOp *op);
 
+void sal_op_kill_dialog(SalOp *op);
 void sal_op_release(SalOp *h);
 /*same as release, but does not stop refresher if any*/
 void* sal_op_unref(SalOp* op);
@@ -669,6 +703,7 @@ const char *sal_op_get_from(const SalOp *op);
 const SalAddress *sal_op_get_from_address(const SalOp *op);
 const char *sal_op_get_to(const SalOp *op);
 const SalAddress *sal_op_get_to_address(const SalOp *op);
+const SalAddress *sal_op_get_diversion_address(const SalOp *op);
 const SalAddress *sal_op_get_contact_address(const SalOp *op);
 const MSList* sal_op_get_route_addresses(const SalOp *op);
 const char *sal_op_get_proxy(const SalOp *op);
@@ -690,14 +725,16 @@ const SalAddress* sal_op_get_service_route(const SalOp *op);
 void sal_op_set_service_route(SalOp *op,const SalAddress* service_route);
 
 void sal_op_set_manual_refresher_mode(SalOp *op, bool_t enabled);
-bool_t sal_op_is_ipv6(SalOp *op);
+int sal_op_get_address_family(SalOp *op);
 /*returns TRUE if there is no pending request that may block a future one */
 bool_t sal_op_is_idle(SalOp *op);
 
 const SalErrorInfo *sal_error_info_none(void);
 LINPHONE_PUBLIC const SalErrorInfo *sal_op_get_error_info(const SalOp *op);
+const SalErrorInfo *sal_op_get_reason_error_info(const SalOp *op);
 void sal_error_info_reset(SalErrorInfo *ei);
-void sal_error_info_set(SalErrorInfo *ei, SalReason reason, int code, const char *status_string, const char *warning);
+void sal_error_info_init_to_null(SalErrorInfo *sei);
+void sal_error_info_set(SalErrorInfo *ei, SalReason reason, const char *protocol, int code, const char *status_string, const char *warning);
 
 /*entity tag used for publish (see RFC 3903)*/
 const char *sal_op_get_entity_tag(const SalOp* op);
@@ -712,7 +749,10 @@ int sal_call_notify_ringing(SalOp *h, bool_t early_media);
 /*accept an incoming call or, during a call accept a reINVITE*/
 int sal_call_accept(SalOp*h);
 int sal_call_decline(SalOp *h, SalReason reason, const char *redirection /*optional*/);
+int sal_call_decline_with_error_info(SalOp *h, const SalErrorInfo* info, const char *redirection /*optional*/);
 int sal_call_update(SalOp *h, const char *subject, bool_t no_user_consent);
+void sal_call_cancel_invite(SalOp *op);
+void sal_call_cancel_invite_with_info(SalOp* op, const SalErrorInfo *info);
 SalMediaDescription * sal_call_get_remote_media_description(SalOp *h);
 LINPHONE_PUBLIC SalMediaDescription * sal_call_get_final_media_description(SalOp *h);
 int sal_call_refer(SalOp *h, const char *refer_to);
@@ -724,10 +764,16 @@ int sal_call_set_referer(SalOp *h, SalOp *refered_call);
 SalOp *sal_call_get_replaces(SalOp *h);
 int sal_call_send_dtmf(SalOp *h, char dtmf);
 int sal_call_terminate(SalOp *h);
+int sal_call_terminate_with_error(SalOp *op, const SalErrorInfo *info);
 bool_t sal_call_autoanswer_asked(SalOp *op);
 void sal_call_send_vfu_request(SalOp *h);
 int sal_call_is_offerer(const SalOp *h);
 int sal_call_notify_refer_state(SalOp *h, SalOp *newcall);
+bool_t sal_call_compare_op(const SalOp *op1, const SalOp *op2);
+LINPHONE_PUBLIC bool_t sal_call_dialog_request_pending(const SalOp *op);
+const char * sal_call_get_local_tag(SalOp *op);
+const char * sal_call_get_remote_tag(SalOp *op);
+void sal_call_set_replaces(SalOp *op, const char *call_id, const char *from_tag, const char *to_tag);
 /* Call test API */
 
 
@@ -806,16 +852,15 @@ SalPrivacy sal_op_get_privacy(const SalOp* op);
 /*misc*/
 void sal_get_default_local_ip(Sal *sal, int address_family, char *ip, size_t iplen);
 
-typedef void (*SalResolverCallback)(void *data, const char *name, struct addrinfo *ai_list);
 
-typedef struct SalResolverContext SalResolverContext;
-
-LINPHONE_PUBLIC SalResolverContext * sal_resolve_a(Sal* sal, const char *name, int port, int family, SalResolverCallback cb, void *data);
-LINPHONE_PUBLIC SalResolverContext * sal_resolve(Sal *sal, const char *service, const char *transport, const char *name, int port, int family, SalResolverCallback cb, void *data);
-//void sal_resolve_cancel(Sal *sal, SalResolverContext *ctx);
+LINPHONE_PUBLIC belle_sip_resolver_context_t * sal_resolve_a(Sal* sal, const char *name, int port, int family, belle_sip_resolver_callback_t cb, void *data);
+LINPHONE_PUBLIC belle_sip_resolver_context_t * sal_resolve(Sal *sal, const char *service, const char *transport, const char *name, int port, int family, belle_sip_resolver_callback_t cb, void *data);
 
+SalCustomHeader *sal_custom_header_ref(SalCustomHeader *ch);
+void sal_custom_header_unref(SalCustomHeader *ch);
 SalCustomHeader *sal_custom_header_append(SalCustomHeader *ch, const char *name, const char *value);
 const char *sal_custom_header_find(const SalCustomHeader *ch, const char *name);
+SalCustomHeader *sal_custom_header_remove(SalCustomHeader *ch, const char *name);
 void sal_custom_header_free(SalCustomHeader *ch);
 SalCustomHeader *sal_custom_header_clone(const SalCustomHeader *ch);
 
@@ -902,8 +947,9 @@ const char * sal_body_handler_get_header(const SalBodyHandler *body_handler, con
 /*this function parses a document with key=value pairs separated by new lines, and extracts the value for a given key*/
 int sal_lines_get_value(const char *data, const char *key, char *value, size_t value_size);
 
-LINPHONE_PUBLIC belle_sip_stack_t *sal_get_belle_sip_stack(Sal *sal);
-char* sal_op_get_public_uri(SalOp *sal);
+LINPHONE_PUBLIC void *sal_get_stack_impl(Sal *sal);
+const char* sal_op_get_public_address(SalOp *sal, int *port);
+const char* sal_op_get_local_address(SalOp *sal, int *port);
 
 unsigned long sal_begin_background_task(const char *name, void (*max_time_reached)(void *), void *data);
 void sal_end_background_task(unsigned long id);
