@@ -26,11 +26,12 @@ struct _Account{
 	int registered;
 	int done;
 	int created;
+	char *phone_alias;
 };
 
 typedef struct _Account Account;
 
-Account *account_new(LinphoneAddress *identity, const char *unique_id){
+static Account *account_new(LinphoneAddress *identity, const char *unique_id){
 	char *modified_username;
 	Account *obj=ms_new0(Account,1);
 
@@ -38,6 +39,7 @@ Account *account_new(LinphoneAddress *identity, const char *unique_id){
 	belle_sip_object_inhibit_leak_detector(TRUE);
 	obj->identity=linphone_address_clone(identity);
 	obj->password=sal_get_random_token(8);
+	obj->phone_alias = NULL;
 	obj->modified_identity=linphone_address_clone(identity);
 	modified_username=ms_strdup_printf("%s_%s",linphone_address_get_username(identity), unique_id);
 	linphone_address_set_username(obj->modified_identity, modified_username);
@@ -121,7 +123,7 @@ static void account_created_auth_requested_cb(LinphoneCore *lc, const char *user
 }
 // TEMPORARY CODE: remove line above when flexisip is updated, this is not needed anymore!
 
-void account_create_on_server(Account *account, const LinphoneProxyConfig *refcfg){
+void account_create_on_server(Account *account, const LinphoneProxyConfig *refcfg, const char* phone_alias){
 	LinphoneCoreVTable vtable={0};
 	LinphoneCore *lc;
 	LinphoneAddress *tmp_identity=linphone_address_clone(account->modified_identity);
@@ -129,12 +131,14 @@ void account_create_on_server(Account *account, const LinphoneProxyConfig *refcf
 	LinphoneAuthInfo *ai;
 	char *tmp;
 	LinphoneAddress *server_addr;
-	LCSipTransports tr;
+	LinphoneSipTransports tr;
+	char *chatdb;
 
 	vtable.registration_state_changed=account_created_on_server_cb;
 	// TEMPORARY CODE: remove line below when flexisip is updated, this is not needed anymore!
 	vtable.auth_info_requested=account_created_auth_requested_cb;
 	lc=configure_lc_from(&vtable,bc_tester_get_resource_dir_prefix(),NULL,account);
+	chatdb = ms_strdup(linphone_core_get_chat_database_path(lc));
 	tr.udp_port=LC_SIP_TRANSPORT_RANDOM;
 	tr.tcp_port=LC_SIP_TRANSPORT_RANDOM;
 	tr.tls_port=LC_SIP_TRANSPORT_RANDOM;
@@ -144,6 +148,7 @@ void account_create_on_server(Account *account, const LinphoneProxyConfig *refcf
 	linphone_address_set_secure(tmp_identity, FALSE);
 	linphone_address_set_password(tmp_identity,account->password);
 	linphone_address_set_header(tmp_identity,"X-Create-Account","yes");
+	if (phone_alias) linphone_address_set_header(tmp_identity, "X-Phone-Alias", phone_alias);
 	tmp=linphone_address_as_string(tmp_identity);
 	linphone_proxy_config_set_identity(cfg,tmp);
 	ms_free(tmp);
@@ -178,7 +183,7 @@ void account_create_on_server(Account *account, const LinphoneProxyConfig *refcf
 				NULL,
 				account->password,NULL,NULL,linphone_address_get_domain(account->modified_identity));
 	linphone_core_add_auth_info(lc,ai);
-	linphone_auth_info_destroy(ai);
+	linphone_auth_info_unref(ai);
 
 	if (wait_for_until(lc,NULL,&account->registered,1,3000)==FALSE){
 		ms_fatal("Account for %s is not working on server.", linphone_proxy_config_get_identity(refcfg));
@@ -188,36 +193,40 @@ void account_create_on_server(Account *account, const LinphoneProxyConfig *refcf
 	if (wait_for_until(lc,NULL,&account->done,1,3000)==FALSE){
 		ms_error("Account creation could not clean the registration context.");
 	}
-	linphone_core_destroy(lc);
+	linphone_core_unref(lc);
+	unlink(chatdb);
+	ms_free(chatdb);
 }
 
-LinphoneAddress *account_manager_check_account(AccountManager *m, LinphoneProxyConfig *cfg){
+static LinphoneAddress *account_manager_check_account(AccountManager *m, LinphoneProxyConfig *cfg,const char* phone_alias){
 	LinphoneCore *lc=linphone_proxy_config_get_core(cfg);
 	const char *identity=linphone_proxy_config_get_identity(cfg);
 	LinphoneAddress *id_addr=linphone_address_new(identity);
 	Account *account=account_manager_get_account(m,id_addr);
 	LinphoneAuthInfo *ai;
-	char *tmp;
 	bool_t create_account=FALSE;
 	const LinphoneAuthInfo *original_ai = linphone_core_find_auth_info(lc
 																		,NULL
 																		, linphone_address_get_username(id_addr)
 																		, linphone_address_get_domain(id_addr));
 
-	if (!account){
+	if (!account||(phone_alias&&(!account->phone_alias||strcmp(phone_alias,account->phone_alias)!=0))){
+		if (account) {
+			m->accounts=bctbx_list_remove(m->accounts,account);
+			account_destroy(account);
+		}
 		account=account_new(id_addr,m->unique_id);
+		account->phone_alias=ms_strdup(phone_alias);
 		ms_message("No account for %s exists, going to create one.",identity);
 		create_account=TRUE;
 		m->accounts=bctbx_list_append(m->accounts,account);
 	}
 	/*modify the username of the identity of the proxy config*/
 	linphone_address_set_username(id_addr, linphone_address_get_username(account->modified_identity));
-	tmp=linphone_address_as_string(id_addr);
-	linphone_proxy_config_set_identity(cfg,tmp);
-	ms_free(tmp);
+	linphone_proxy_config_set_identity_address(cfg, id_addr);
 
 	if (create_account){
-		account_create_on_server(account,cfg);
+		account_create_on_server(account,cfg,phone_alias);
 	}
 
 	/*remove previous auth info to avoid mismatching*/
@@ -228,7 +237,7 @@ LinphoneAddress *account_manager_check_account(AccountManager *m, LinphoneProxyC
 				NULL,
 				account->password,NULL,NULL,linphone_address_get_domain(account->modified_identity));
 	linphone_core_add_auth_info(lc,ai);
-	linphone_auth_info_destroy(ai);
+	linphone_auth_info_unref(ai);
 
 	linphone_address_unref(id_addr);
 	return account->modified_identity;
@@ -237,9 +246,12 @@ LinphoneAddress *account_manager_check_account(AccountManager *m, LinphoneProxyC
 void linphone_core_manager_check_accounts(LinphoneCoreManager *m){
 	const bctbx_list_t *it;
 	AccountManager *am=account_manager_get();
-
+	int logmask = ortp_get_log_level_mask(NULL);
+	
+	if (!liblinphonetester_show_account_manager_logs) linphone_core_set_log_level_mask(ORTP_ERROR|ORTP_FATAL);
 	for(it=linphone_core_get_proxy_config_list(m->lc);it!=NULL;it=it->next){
 		LinphoneProxyConfig *cfg=(LinphoneProxyConfig *)it->data;
-		account_manager_check_account(am,cfg);
+		account_manager_check_account(am,cfg,m->phone_alias);
 	}
+	if (!liblinphonetester_show_account_manager_logs) linphone_core_set_log_level_mask(logmask);
 }
