@@ -19,8 +19,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "linphonecore.h"
-#include "lpconfig.h"
+#include "linphone/core.h"
+#include "linphone/lpconfig.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 #include "mediastreamer2/msutils.h"
@@ -50,14 +50,14 @@ static void call_waiting_indication_with_param(bool_t enable_caller_privacy) {
 	lcs=bctbx_list_append(lcs,laure->lc);
 
 	BC_ASSERT_TRUE(call_with_caller_params(marie,pauline,marie_params));
-	linphone_call_params_destroy(marie_params);
+	linphone_call_params_unref(marie_params);
 	pauline_called_by_marie=linphone_core_get_current_call(pauline->lc);
 
 	if (enable_caller_privacy)
 		linphone_call_params_set_privacy(laure_params,LinphonePrivacyId);
 
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_invite_address_with_params(laure->lc,pauline->identity,laure_params));
-	linphone_call_params_destroy(laure_params);
+	linphone_call_params_unref(laure_params);
 
 	BC_ASSERT_TRUE(wait_for(laure->lc
 							,pauline->lc
@@ -77,7 +77,7 @@ static void call_waiting_indication_with_param(bool_t enable_caller_privacy) {
 		if (call != pauline_called_by_marie) {
 			/*fine, this is the call waiting*/
 			pauline_called_by_laure=call;
-			linphone_core_accept_call(pauline->lc,pauline_called_by_laure);
+			linphone_call_accept(pauline_called_by_laure);
 		}
 	}
 
@@ -134,7 +134,7 @@ static void second_call_rejection(bool_t second_without_audio){
 	linphone_call_params_enable_audio(params, !second_without_audio);
 	marie_call = linphone_core_invite_with_params(marie->lc, "sip:laure_non_exstent@test.linphone.org", params);
 
-	linphone_call_params_destroy(params);
+	linphone_call_params_unref(params);
 
 	if (second_without_audio){
 		BC_ASSERT_PTR_NOT_NULL(marie_call);
@@ -150,7 +150,7 @@ static void second_call_rejection(bool_t second_without_audio){
 	pauline_call = linphone_core_get_current_call(pauline->lc);
 	BC_ASSERT_PTR_NOT_NULL(pauline_call);
 	if (pauline_call){
-		linphone_core_accept_call(pauline->lc, pauline_call);
+		linphone_call_accept(pauline_call);
 	}
 	BC_ASSERT_TRUE(wait_for(marie->lc
 			,pauline->lc
@@ -195,7 +195,7 @@ static void incoming_call_accepted_when_outgoing_call_in_state(LinphoneCallState
 								,1));
 
 		if (state==LinphoneCallOutgoingEarlyMedia)
-			linphone_core_accept_early_media(pauline->lc,linphone_core_get_current_call(pauline->lc));
+			linphone_call_accept_early_media(linphone_core_get_current_call(pauline->lc));
 
 		BC_ASSERT_EQUAL(marie->stat.number_of_LinphoneCallOutgoingProgress,1, int, "%d");
 		BC_ASSERT_TRUE(wait_for(marie->lc
@@ -241,7 +241,7 @@ static void incoming_call_accepted_when_outgoing_call_in_outgoing_ringing_early_
 	incoming_call_accepted_when_outgoing_call_in_state(LinphoneCallOutgoingEarlyMedia);
 }
 
-static void simple_conference_base(LinphoneCoreManager* marie, LinphoneCoreManager* pauline, LinphoneCoreManager* laure, LinphoneCoreManager *focus) {
+static void simple_conference_base(LinphoneCoreManager* marie, LinphoneCoreManager* pauline, LinphoneCoreManager* laure, LinphoneCoreManager *focus, bool_t pause_and_hangup) {
 	stats initial_marie_stat;
 	stats initial_pauline_stat;
 	stats initial_laure_stat;
@@ -254,19 +254,21 @@ static void simple_conference_base(LinphoneCoreManager* marie, LinphoneCoreManag
 	bool_t is_remote_conf;
 	bool_t focus_is_up = (focus && ((LinphoneConferenceServer *)focus)->reg_state == LinphoneRegistrationOk);
 	bctbx_list_t* lcs=bctbx_list_append(NULL,marie->lc);
+	
 	lcs=bctbx_list_append(lcs,pauline->lc);
 	lcs=bctbx_list_append(lcs,laure->lc);
-	if(focus) lcs=bctbx_list_append(lcs,focus->lc);
+	if (focus) lcs=bctbx_list_append(lcs,focus->lc);
 
 	is_remote_conf = (strcmp(lp_config_get_string(marie->lc->config, "misc", "conference_type", "local"), "remote") == 0);
 	if(is_remote_conf) BC_ASSERT_PTR_NOT_NULL(focus);
 
-	BC_ASSERT_TRUE(call(marie,pauline));
+	if (!BC_ASSERT_TRUE(call(marie,pauline))) goto end;
+	
 	marie_call_pauline=linphone_core_get_current_call(marie->lc);
 	pauline_called_by_marie=linphone_core_get_current_call(pauline->lc);
 	BC_ASSERT_TRUE(pause_call_1(marie,marie_call_pauline,pauline,pauline_called_by_marie));
 
-	BC_ASSERT_TRUE(call(marie,laure));
+	if (!BC_ASSERT_TRUE(call(marie,laure))) goto end;
 	initial_marie_stat=marie->stat;
 	initial_pauline_stat=pauline->stat;
 	initial_laure_stat=laure->stat;
@@ -286,8 +288,8 @@ static void simple_conference_base(LinphoneCoreManager* marie, LinphoneCoreManag
 			BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneCallError, initial_marie_stat.number_of_LinphoneCallError+1, 5000));
 			BC_ASSERT_PTR_NULL(linphone_core_get_conference(marie->lc));
 			BC_ASSERT_EQUAL(linphone_core_terminate_conference(marie->lc), -1, int, "%d");
-			linphone_core_terminate_call(marie->lc, marie_call_pauline);
-			linphone_core_terminate_call(marie->lc, marie_call_laure);
+			linphone_call_terminate(marie_call_pauline);
+			linphone_call_terminate(marie_call_laure);
 			BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneCallEnd, initial_marie_stat.number_of_LinphoneCallEnd+2, 10000));
 			BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallEnd, initial_pauline_stat.number_of_LinphoneCallEnd+1, 5000));
 			BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneCallEnd, initial_laure_stat.number_of_LinphoneCallEnd+1, 5000));
@@ -342,8 +344,16 @@ static void simple_conference_base(LinphoneCoreManager* marie, LinphoneCoreManag
 	BC_ASSERT_PTR_NOT_NULL(conference = linphone_core_get_conference(marie->lc));
 	if(conference) {
 		bctbx_list_t *participants = linphone_conference_get_participants(conference);
-		BC_ASSERT_EQUAL(bctbx_list_size(participants), 2, int, "%d");
-		bctbx_list_free_with_data(participants, (void(*)(void *))linphone_address_destroy);
+		BC_ASSERT_EQUAL((unsigned int)bctbx_list_size(participants), 2, unsigned int, "%u");
+		bctbx_list_free_with_data(participants, (void(*)(void *))linphone_address_unref);
+	}
+
+	if (pause_and_hangup) {
+		linphone_core_pause_call(marie->lc, marie_call_laure);
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallPaused,2,10000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallPausedByRemote,1,10000));
+		linphone_core_remove_from_conference(marie->lc, marie_call_laure);
+		linphone_core_terminate_call(marie->lc, marie_call_laure);
 	}
 
 	linphone_core_terminate_conference(marie->lc);
@@ -365,12 +375,86 @@ static void simple_conference(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
 	LinphoneCoreManager* laure = linphone_core_manager_new( "laure_rc_udp");
-	simple_conference_base(marie,pauline,laure, NULL);
+	simple_conference_base(marie,pauline,laure, NULL, FALSE);
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 	linphone_core_manager_destroy(laure);
 }
 
+
+static void simple_conference_from_scratch(void){
+	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
+	LinphoneCoreManager* laure = linphone_core_manager_new( "laure_rc_udp");
+	LinphoneConference *conf;
+	LinphoneConferenceParams *conf_params;
+	LinphoneCall *pauline_call, *laure_call;
+	bctbx_list_t *participants = NULL;
+	bctbx_list_t *lcs = NULL;
+	
+	lcs = bctbx_list_append(lcs, marie->lc);
+	lcs = bctbx_list_append(lcs, pauline->lc);
+	lcs = bctbx_list_append(lcs, laure->lc);
+	
+	/*marie creates the conference*/
+	conf_params = linphone_core_create_conference_params(marie->lc);
+	linphone_conference_params_enable_video(conf_params, FALSE);
+	conf = linphone_core_create_conference_with_params(marie->lc, conf_params);
+	linphone_conference_params_unref(conf_params);
+	
+	participants = bctbx_list_append(participants, pauline->identity);
+	participants = bctbx_list_append(participants, laure->identity);
+	
+	linphone_conference_invite_participants(conf, participants, NULL);
+	
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallOutgoingProgress,2,2000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallIncomingReceived,1,10000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallIncomingReceived,1,10000));
+	
+	pauline_call = linphone_core_get_current_call(pauline->lc);
+	laure_call = linphone_core_get_current_call(laure->lc);
+	
+	BC_ASSERT_PTR_NOT_NULL(pauline_call);
+	BC_ASSERT_PTR_NOT_NULL(laure_call);
+	
+	if (pauline_call && laure_call){
+		const bctbx_list_t *marie_calls, *it;
+		linphone_call_accept(pauline_call);
+		linphone_call_accept(laure_call);
+		
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,2,10000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallStreamsRunning,2,3000));
+		
+		/*make sure that the two calls from Marie's standpoint are in conference*/
+		marie_calls = linphone_core_get_calls(marie->lc);
+		BC_ASSERT_EQUAL((int)bctbx_list_size(marie_calls), 2, int, "%i");
+		for (it = marie_calls; it != NULL; it = it->next){
+			BC_ASSERT_TRUE(linphone_call_params_get_local_conference_mode(linphone_call_get_current_params((LinphoneCall*)it->data)) == TRUE);
+		}
+		/*wait a bit for the conference audio processing to run, despite we do not test it for the moment*/
+		wait_for_list(lcs,NULL,0,5000);
+		
+		linphone_core_terminate_conference(marie->lc);
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,2,5000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,10000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallEnd,1,10000));
+		
+		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallReleased,2,1000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallReleased,1,1000));
+		BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallReleased,1,1000));
+	}
+	linphone_conference_unref(conf);
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+	linphone_core_manager_destroy(laure);
+	
+	bctbx_list_free(participants);
+	bctbx_list_free(lcs);
+}
+
+
+
+
 static void simple_encrypted_conference_with_ice(LinphoneMediaEncryption mode) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
@@ -387,7 +471,7 @@ static void simple_encrypted_conference_with_ice(LinphoneMediaEncryption mode) {
 		linphone_core_set_media_encryption(pauline->lc,mode);
 		linphone_core_set_media_encryption(laure->lc,mode);
 
-		simple_conference_base(marie,pauline,laure,NULL);
+		simple_conference_base(marie,pauline,laure,NULL,FALSE);
 	} else {
 		ms_warning("No [%s] support available",linphone_media_encryption_to_string(mode));
 		BC_PASS("Passed");
@@ -401,10 +485,20 @@ static void simple_encrypted_conference_with_ice(LinphoneMediaEncryption mode) {
 static void simple_conference_with_ice(void) {
 	simple_encrypted_conference_with_ice(LinphoneMediaEncryptionNone);
 }
+
 static void simple_zrtp_conference_with_ice(void) {
 	simple_encrypted_conference_with_ice(LinphoneMediaEncryptionZRTP);
 }
 
+static void conference_hang_up_call_on_hold(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new("pauline_tcp_rc");
+	LinphoneCoreManager* laure = linphone_core_manager_new("laure_rc_udp");
+	simple_conference_base(marie, pauline, laure, NULL, TRUE);
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+	linphone_core_manager_destroy(laure);
+}
 
 static void simple_call_transfer(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
@@ -429,7 +523,7 @@ static void simple_call_transfer(void) {
 	reset_counters(&laure->stat);
 
 
-	linphone_core_transfer_call(pauline->lc,pauline_called_by_marie,laure_identity);
+	linphone_call_transfer(pauline_called_by_marie,laure_identity);
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallRefered,1,2000));
 	/*marie pausing pauline*/
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallPausing,1,2000));
@@ -444,7 +538,7 @@ static void simple_call_transfer(void) {
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallIncomingReceived,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallOutgoingRinging,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneTransferCallOutgoingProgress,1,2000));
-	linphone_core_accept_call(laure->lc,linphone_core_get_current_call(laure->lc));
+	linphone_call_accept(linphone_core_get_current_call(laure->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallConnected,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallStreamsRunning,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,1,2000));
@@ -489,11 +583,11 @@ static void unattended_call_transfer(void) {
 	reset_counters(&pauline->stat);
 	reset_counters(&laure->stat);
 
-	linphone_core_transfer_call(marie->lc,pauline_called_by_marie,laure_identity);
+	linphone_call_transfer(pauline_called_by_marie,laure_identity);
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallRefered,1,2000));
 
 	/*marie ends the call  */
-	linphone_core_terminate_call(marie->lc,pauline_called_by_marie);
+	linphone_call_terminate(pauline_called_by_marie);
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,2000));
 
 	/*Pauline starts the transfer*/
@@ -501,7 +595,7 @@ static void unattended_call_transfer(void) {
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallOutgoingProgress,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallIncomingReceived,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallOutgoingRinging,1,2000));
-	linphone_core_accept_call(laure->lc,linphone_core_get_current_call(laure->lc));
+	linphone_call_accept(linphone_core_get_current_call(laure->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallConnected,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallStreamsRunning,1,2000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallConnected,1,2000));
@@ -533,7 +627,7 @@ static void unattended_call_transfer_with_error(void) {
 		reset_counters(&marie->stat);
 		reset_counters(&pauline->stat);
 
-		linphone_core_transfer_call(marie->lc,pauline_called_by_marie,"unknown_user");
+		linphone_call_transfer(pauline_called_by_marie,"unknown_user");
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallRefered,1,2000));
 
 		/*Pauline starts the transfer*/
@@ -559,7 +653,7 @@ static void unattended_call_transfer_with_error(void) {
 }
 
 
-static void call_transfer_existing_call_outgoing_call(void) {
+static void call_transfer_existing_call(bool_t outgoing_call) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
 	LinphoneCoreManager* laure = linphone_core_manager_new( "laure_rc_udp");
@@ -584,11 +678,20 @@ static void call_transfer_existing_call_outgoing_call(void) {
 			goto end;
 		}
 
-		/*marie call laure*/
-		if (!BC_ASSERT_TRUE(call(marie,laure))) {
-			end_call(marie, pauline);
-			goto end;
+		if (outgoing_call) {
+			/*marie call laure*/
+			if (!BC_ASSERT_TRUE(call(marie,laure))) {
+				end_call(marie, pauline);
+				goto end;
+			}
+		} else {
+			/*laure call pauline*/
+			if (!BC_ASSERT_TRUE(call(laure,marie))) {
+				end_call(marie,pauline);
+				goto end;
+			}
 		}
+			
 		marie_call_laure=linphone_core_get_current_call(marie->lc);
 		laure_called_by_marie=linphone_core_get_current_call(laure->lc);
 
@@ -600,7 +703,7 @@ static void call_transfer_existing_call_outgoing_call(void) {
 		reset_counters(&laure->stat);
 
 
-		linphone_core_transfer_call_to_another(marie->lc,marie_call_pauline,marie_call_laure);
+		linphone_call_transfer_to_another(marie_call_pauline,marie_call_laure);
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallRefered,1,2000));
 
 		/*pauline pausing marie*/
@@ -618,7 +721,7 @@ static void call_transfer_existing_call_outgoing_call(void) {
 			lcall = (LinphoneCall*)calls->data;
 			if (linphone_call_get_state(lcall) == LinphoneCallIncomingReceived) {
 				BC_ASSERT_PTR_EQUAL(linphone_call_get_replaced_call(lcall),laure_called_by_marie);
-				linphone_core_accept_call(laure->lc,lcall);
+				linphone_call_accept(lcall);
 				break;
 			}
 		}
@@ -642,6 +745,83 @@ end:
 	bctbx_list_free(lcs);
 }
 
+static void call_transfer_existing_call_outgoing_call(void) {
+	call_transfer_existing_call(TRUE);
+}
+
+static void call_transfer_existing_call_incoming_call(void) {
+	call_transfer_existing_call(FALSE);
+}
+
+static void call_transfer_existing_ringing_call(void) {
+	LinphoneCoreManager *marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_tcp_rc");
+	LinphoneCoreManager *laure = linphone_core_manager_new("laure_rc_udp");
+	LinphoneCall *marie_call_pauline;
+	LinphoneCall *pauline_called_by_marie;
+	LinphoneCall *marie_call_laure;
+	LinphoneCall *lcall;
+	bool_t call_ok = TRUE;
+	const bctbx_list_t *calls;
+	stats initial_marie_stats;
+	bctbx_list_t *lcs = bctbx_list_append(NULL, marie->lc);
+	lcs = bctbx_list_append(lcs, pauline->lc);
+	lcs = bctbx_list_append(lcs, laure->lc);
+
+	/* marie calls pauline */
+	BC_ASSERT_TRUE((call_ok = call(marie, pauline)));
+	if (call_ok) {
+		marie_call_pauline = linphone_core_get_current_call(marie->lc);
+		pauline_called_by_marie = linphone_core_get_current_call(pauline->lc);
+		/* marie pauses pauline */
+		if (!BC_ASSERT_TRUE(pause_call_1(marie, marie_call_pauline, pauline, pauline_called_by_marie))) goto end;
+
+		initial_marie_stats = marie->stat;
+		BC_ASSERT_PTR_NOT_NULL((marie_call_laure = linphone_core_invite_address(marie->lc, laure->identity)));
+		if (!marie_call_laure) goto end;
+		BC_ASSERT_TRUE(wait_for(marie->lc, laure->lc, &marie->stat.number_of_LinphoneCallOutgoingRinging, initial_marie_stats.number_of_LinphoneCallOutgoingRinging + 1));
+		linphone_call_transfer_to_another(marie_call_pauline, marie_call_laure);
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallRefered, 1, 2000));
+
+		/* pauline pausing marie */
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallPausing, 1, 4000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallPaused, 1, 4000));
+		/* pauline calling laure */
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallOutgoingProgress, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneTransferCallOutgoingInit, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneCallIncomingReceived, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallOutgoingRinging, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneTransferCallOutgoingProgress, 1, 2000));
+
+		/* laure accepts call */
+		for (calls = linphone_core_get_calls(laure->lc); calls != NULL; calls = calls->next) {
+			lcall = (LinphoneCall*)calls->data;
+			if (linphone_call_get_state(lcall) == LinphoneCallIncomingReceived) {
+				linphone_call_accept(lcall);
+				break;
+			}
+		}
+		BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneCallConnected, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneCallStreamsRunning, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallConnected, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallStreamsRunning, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneTransferCallConnected, 1, 2000));
+
+		/* terminate marie to pauline/laure call */
+		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallEnd, 1, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneCallEnd, 2, 2000));
+		BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneCallEnd, 1, 2000));
+
+		end_call(pauline, laure);
+	}
+
+end:
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(laure);
+	linphone_core_manager_destroy(pauline);
+	bctbx_list_free(lcs);
+}
+
 static void eject_from_3_participants_conference(LinphoneCoreManager *marie, LinphoneCoreManager *pauline, LinphoneCoreManager *laure, LinphoneCoreManager *focus) {
 	stats initial_marie_stat;
 	stats initial_pauline_stat;
@@ -716,7 +896,7 @@ static void eject_from_3_participants_conference(LinphoneCoreManager *marie, Lin
 		BC_ASSERT_TRUE(wait_for_list(lcs,&laure->stat.number_of_LinphoneCallStreamsRunning,3,10000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallStreamsRunning,5,10000));
 		BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(marie->lc));
-		BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_calls(marie->lc)), 2, int, "%d");
+		BC_ASSERT_EQUAL((unsigned int)bctbx_list_size(linphone_core_get_calls(marie->lc)), 2, unsigned int, "%u");
 		BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(pauline->lc));
 		BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(laure->lc));
 	} else {
@@ -775,7 +955,9 @@ static void eject_from_4_participants_conference(void) {
 	pauline_called_by_marie=linphone_core_get_current_call(pauline->lc);
 	BC_ASSERT_TRUE(pause_call_1(marie,marie_call_pauline,pauline,pauline_called_by_marie));
 
-	BC_ASSERT_TRUE(call(marie,michelle));
+	if (!BC_ASSERT_TRUE(call(marie,michelle)))
+		goto end;
+
 	marie_call_michelle=linphone_core_get_current_call(marie->lc);
 	michelle_called_by_marie=linphone_core_get_current_call(michelle->lc);
 	BC_ASSERT_TRUE(pause_call_1(marie,marie_call_michelle,michelle,michelle_called_by_marie));
@@ -811,7 +993,7 @@ static void eject_from_4_participants_conference(void) {
 	BC_ASSERT_PTR_NULL(linphone_core_get_current_call(marie->lc));
 	BC_ASSERT_TRUE(linphone_core_is_in_conference(marie->lc));
 	BC_ASSERT_EQUAL(linphone_core_get_conference_size(marie->lc),3, int, "%d");
-	BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_calls(marie->lc)), 3, int, "%d");
+	BC_ASSERT_EQUAL((unsigned int)bctbx_list_size(linphone_core_get_calls(marie->lc)), 3, unsigned int, "%u");
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(pauline->lc));
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(laure->lc));
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(michelle->lc));
@@ -851,8 +1033,6 @@ void simple_remote_conference(void) {
 	LinphoneProxyConfig *laure_proxy_config = linphone_core_get_default_proxy_config(((LinphoneCoreManager *)laure)->lc);
 	const char *laure_proxy_uri = linphone_proxy_config_get_server_addr(laure_proxy_config);
 	const char *focus_uri = linphone_proxy_config_get_identity(focus_proxy_config);
-	int laure_n_register = laure->stat.number_of_LinphoneRegistrationOk;
-	bctbx_list_t *lcs = NULL;
 
 	lp_config_set_string(marie_config, "misc", "conference_type", "remote");
 	lp_config_set_string(marie_config, "misc", "conference_focus_addr", focus_uri);
@@ -860,11 +1040,8 @@ void simple_remote_conference(void) {
 	linphone_proxy_config_edit(laure_proxy_config);
 	linphone_proxy_config_set_route(laure_proxy_config, laure_proxy_uri);
 	linphone_proxy_config_done(laure_proxy_config);
-	lcs = bctbx_list_append(lcs, laure->lc);
-	BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneRegistrationOk, laure_n_register+1, 5000));
-	bctbx_list_free(lcs);
 
-	simple_conference_base(marie, pauline, laure, (LinphoneCoreManager *)focus);
+	simple_conference_base(marie, pauline, laure, (LinphoneCoreManager *)focus, FALSE);
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
@@ -882,8 +1059,6 @@ void simple_remote_conference_shut_down_focus(void) {
 	LinphoneProxyConfig *laure_proxy_config = linphone_core_get_default_proxy_config(((LinphoneCoreManager *)laure)->lc);
 	const char *laure_proxy_uri = linphone_proxy_config_get_server_addr(laure_proxy_config);
 	const char *focus_uri = linphone_proxy_config_get_identity(focus_proxy_config);
-	int laure_n_register = laure->stat.number_of_LinphoneRegistrationOk;
-	bctbx_list_t *lcs = NULL;
 
 	lp_config_set_string(marie_config, "misc", "conference_type", "remote");
 	lp_config_set_string(marie_config, "misc", "conference_focus_addr", focus_uri);
@@ -891,11 +1066,8 @@ void simple_remote_conference_shut_down_focus(void) {
 	linphone_proxy_config_edit(laure_proxy_config);
 	linphone_proxy_config_set_route(laure_proxy_config, laure_proxy_uri);
 	linphone_proxy_config_done(laure_proxy_config);
-	lcs = bctbx_list_append(lcs, laure->lc);
-	BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneRegistrationOk, laure_n_register+1, 5000));
-	bctbx_list_free(lcs);
 
-	simple_conference_base(marie, pauline, laure, (LinphoneCoreManager *)focus);
+	simple_conference_base(marie, pauline, laure, (LinphoneCoreManager *)focus, FALSE);
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
@@ -913,8 +1085,6 @@ void eject_from_3_participants_remote_conference(void) {
 	LinphoneProxyConfig *laure_proxy_config = linphone_core_get_default_proxy_config(((LinphoneCoreManager *)laure)->lc);
 	const char *laure_proxy_uri = linphone_proxy_config_get_server_addr(laure_proxy_config);
 	const char *focus_uri = linphone_proxy_config_get_identity(focus_proxy_config);
-	int laure_n_register = laure->stat.number_of_LinphoneRegistrationOk;
-	bctbx_list_t *lcs = NULL;
 
 	lp_config_set_string(marie_config, "misc", "conference_type", "remote");
 	lp_config_set_string(marie_config, "misc", "conference_focus_addr", focus_uri);
@@ -922,9 +1092,6 @@ void eject_from_3_participants_remote_conference(void) {
 	linphone_proxy_config_edit(laure_proxy_config);
 	linphone_proxy_config_set_route(laure_proxy_config, laure_proxy_uri);
 	linphone_proxy_config_done(laure_proxy_config);
-	lcs = bctbx_list_append(lcs, laure->lc);
-	BC_ASSERT_TRUE(wait_for_list(lcs, &laure->stat.number_of_LinphoneRegistrationOk, laure_n_register+1, 5000));
-	bctbx_list_free(lcs);
 
 	eject_from_3_participants_conference(marie, pauline, laure, (LinphoneCoreManager *)focus);
 
@@ -934,6 +1101,44 @@ void eject_from_3_participants_remote_conference(void) {
 	linphone_conference_server_destroy(focus);
 }
 
+void do_not_stop_ringing_when_declining_one_of_two_incoming_calls(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
+	LinphoneCoreManager* laure = linphone_core_manager_new( "laure_rc_udp");
+	LinphoneCall* pauline_called_by_marie;
+	LinphoneCall* pauline_called_by_laure;
+	LinphoneCallParams *laure_params=linphone_core_create_call_params(laure->lc, NULL);
+	LinphoneCallParams *marie_params=linphone_core_create_call_params(marie->lc, NULL);
+
+	BC_ASSERT_PTR_NOT_NULL(linphone_core_invite_address_with_params(laure->lc,pauline->identity,laure_params));
+	linphone_call_params_unref(laure_params);
+
+	BC_ASSERT_TRUE(wait_for(laure->lc
+							,pauline->lc
+							,&pauline->stat.number_of_LinphoneCallIncomingReceived
+							,1));
+	pauline_called_by_laure=linphone_core_get_current_call(pauline->lc);
+
+	BC_ASSERT_PTR_NOT_NULL(linphone_core_invite_address_with_params(marie->lc,pauline->identity,marie_params));
+	linphone_call_params_unref(marie_params);
+
+	BC_ASSERT_TRUE(wait_for(marie->lc
+							,pauline->lc
+							,&pauline->stat.number_of_LinphoneCallIncomingReceived
+							,2));
+	pauline_called_by_marie=linphone_core_get_current_call(marie->lc);
+	linphone_call_decline(pauline_called_by_laure, LinphoneReasonDeclined);
+	BC_ASSERT_TRUE(wait_for(laure->lc,pauline->lc,&pauline->stat.number_of_LinphoneCallEnd,1));
+
+	BC_ASSERT_TRUE(linphone_ringtoneplayer_is_started(pauline->lc->ringtoneplayer));
+	linphone_call_terminate(pauline_called_by_marie);
+	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&pauline->stat.number_of_LinphoneCallEnd,2));
+
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+	linphone_core_manager_destroy(laure);
+}
+
 test_t multi_call_tests[] = {
 	TEST_NO_TAG("Call waiting indication", call_waiting_indication),
 	TEST_NO_TAG("Call waiting indication with privacy", call_waiting_indication_with_privacy),
@@ -943,17 +1148,22 @@ test_t multi_call_tests[] = {
 	TEST_NO_TAG("Incoming call accepted when outgoing call in outgoing ringing", incoming_call_accepted_when_outgoing_call_in_outgoing_ringing),
 	TEST_NO_TAG("Incoming call accepted when outgoing call in outgoing ringing early media", incoming_call_accepted_when_outgoing_call_in_outgoing_ringing_early_media),
 	TEST_NO_TAG("Simple conference", simple_conference),
+	TEST_NO_TAG("Simple conference established from scractch", simple_conference_from_scratch),
 	TEST_ONE_TAG("Simple conference with ICE", simple_conference_with_ice, "ICE"),
 	TEST_ONE_TAG("Simple ZRTP conference with ICE", simple_zrtp_conference_with_ice, "ICE"),
 	TEST_NO_TAG("Eject from 3 participants conference", eject_from_3_participants_local_conference),
 	TEST_NO_TAG("Eject from 4 participants conference", eject_from_4_participants_conference),
+	TEST_NO_TAG("Conference pause and terminate call", conference_hang_up_call_on_hold),
 	TEST_NO_TAG("Simple call transfer", simple_call_transfer),
 	TEST_NO_TAG("Unattended call transfer", unattended_call_transfer),
 	TEST_NO_TAG("Unattended call transfer with error", unattended_call_transfer_with_error),
 	TEST_NO_TAG("Call transfer existing call outgoing call", call_transfer_existing_call_outgoing_call),
+	TEST_NO_TAG("Call transfer existing call incoming call", call_transfer_existing_call_incoming_call),
+	TEST_NO_TAG("Call transfer existing ringing call", call_transfer_existing_ringing_call),
 	TEST_NO_TAG("Simple remote conference", simple_remote_conference),
 	TEST_NO_TAG("Simple remote conference with shut down focus", simple_remote_conference_shut_down_focus),
 	TEST_NO_TAG("Eject from 3 participants in remote conference", eject_from_3_participants_remote_conference),
+	TEST_NO_TAG("Do not stop ringing when declining one of two incoming calls", do_not_stop_ringing_when_declining_one_of_two_incoming_calls),
 };
 
 test_suite_t multi_call_test_suite = {"Multi call", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
