@@ -17,11 +17,17 @@
 */
 
 
-#include "linphonecore.h"
-#include "lpconfig.h"
+#include "linphone/core.h"
+#include "linphone/lpconfig.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
+static void setPublish(LinphoneProxyConfig * proxy_config, bool_t enable) {
+	linphone_proxy_config_edit(proxy_config);
+	linphone_proxy_config_enable_publish(proxy_config, enable);
+	linphone_proxy_config_done(proxy_config);
+}
+
 static void subscribe_forking(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneCoreManager* pauline = linphone_core_manager_new( transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
@@ -41,7 +47,7 @@ static void subscribe_forking(void) {
 
 	lev=linphone_core_subscribe(marie->lc,pauline->identity,"dodo",expires,content);
 
-	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionOutgoingInit,1,1000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionOutgoingProgress,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneSubscriptionIncomingReceived,1,3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_LinphoneSubscriptionIncomingReceived,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionActive,1,1000));
@@ -221,7 +227,7 @@ static void call_forking(void){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallIncomingReceived,1,3000));
 
 	/*marie accepts the call on its first device*/
-	linphone_core_accept_call(marie->lc,linphone_core_get_current_call(marie->lc));
+	linphone_call_accept(linphone_core_get_current_call(marie->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallConnected,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,1,1000));
@@ -231,7 +237,7 @@ static void call_forking(void){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallEnd,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallEnd,1,1000));
 
-	linphone_core_terminate_call(pauline->lc,linphone_core_get_current_call(pauline->lc));
+	linphone_call_terminate(linphone_core_get_current_call(pauline->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,1,1000));
 
@@ -270,13 +276,13 @@ static void call_forking_with_urgent_reply(void){
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallIncomingReceived,1,1000));
 
 		/*marie accepts the call on its first device*/
-		linphone_core_accept_call(marie->lc,linphone_core_get_current_call(marie->lc));
+		linphone_call_accept(linphone_core_get_current_call(marie->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallConnected,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 
-		linphone_core_terminate_call(pauline->lc,linphone_core_get_current_call(pauline->lc));
+		linphone_call_terminate(linphone_core_get_current_call(pauline->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,1,1000));
 	}
@@ -312,7 +318,7 @@ static void call_forking_cancelled(void){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallIncomingReceived,1,1000));
 
 	/*pauline finally cancels the call*/
-	linphone_core_terminate_call(pauline->lc,linphone_core_get_current_call(pauline->lc));
+	linphone_call_terminate(linphone_core_get_current_call(pauline->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,1000));
 
 	/*all devices should stop ringing*/
@@ -352,7 +358,7 @@ static void call_forking_declined(bool_t declined_globaly){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallIncomingReceived,1,1000));
 
 	/*marie finally declines the call*/
-	linphone_core_decline_call(marie->lc,linphone_core_get_current_call(marie->lc),
+	linphone_call_decline(linphone_core_get_current_call(marie->lc),
 		declined_globaly ? LinphoneReasonDeclined : LinphoneReasonBusy
 	);
 
@@ -364,13 +370,13 @@ static void call_forking_declined(bool_t declined_globaly){
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallEnd,1,1000));
 	}else{
 		/*pauline should continue ringing and be able to hear a call taken by marie2 */
-		linphone_core_accept_call(marie2->lc, linphone_core_get_current_call(marie2->lc));
+		linphone_call_accept(linphone_core_get_current_call(marie2->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,2000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallStreamsRunning,1,2000));
 		liblinphone_tester_check_rtcp(pauline,marie2);
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,1,3000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie3->stat.number_of_LinphoneCallEnd,1,3000));
-		linphone_core_terminate_call(marie2->lc,linphone_core_get_current_call(marie2->lc));
+		linphone_call_terminate(linphone_core_get_current_call(marie2->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallEnd,1,3000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,3000));
 	}
@@ -423,7 +429,7 @@ static void call_forking_with_push_notification_single(void){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallOutgoingRinging,1,1000));
 
 	/*marie accepts the call*/
-	linphone_core_accept_call(marie->lc,linphone_core_get_current_call(marie->lc));
+	linphone_call_accept(linphone_core_get_current_call(marie->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallConnected,1,5000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,1,1000));
@@ -431,7 +437,7 @@ static void call_forking_with_push_notification_single(void){
 
 	liblinphone_tester_check_rtcp(pauline,marie);
 
-	linphone_core_terminate_call(pauline->lc,linphone_core_get_current_call(pauline->lc));
+	linphone_call_terminate(linphone_core_get_current_call(pauline->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,5000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,1,5000));
 
@@ -471,7 +477,7 @@ static void call_forking_with_push_notification_multiple(void){
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallIncomingReceived,1,5000));
 
 	/*marie2 accepts the call*/
-	linphone_core_accept_call(marie2->lc,linphone_core_get_current_call(marie2->lc));
+	linphone_call_accept(linphone_core_get_current_call(marie2->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallConnected,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallConnected,1,1000));
@@ -482,7 +488,7 @@ static void call_forking_with_push_notification_multiple(void){
 
 	liblinphone_tester_check_rtcp(pauline,marie2);
 
-	linphone_core_terminate_call(pauline->lc,linphone_core_get_current_call(pauline->lc));
+	linphone_call_terminate(linphone_core_get_current_call(pauline->lc));
 
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallEnd,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneCallEnd,1,1000));
@@ -568,7 +574,7 @@ static void early_media_call_forking(void) {
 	linphone_call_params_enable_video(params,TRUE);
 
 	linphone_core_invite_address_with_params(pauline->lc,marie->identity,params);
-	linphone_call_params_destroy(params);
+	linphone_call_params_unref(params);
 
 	BC_ASSERT_TRUE(wait_for_list(lcs, &marie->stat.number_of_LinphoneCallIncomingEarlyMedia,1,3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallIncomingEarlyMedia,1,3000));
@@ -584,7 +590,7 @@ static void early_media_call_forking(void) {
 	BC_ASSERT_GREATER(linphone_core_manager_get_mean_audio_down_bw(marie2), 60, int, "%d");
 	BC_ASSERT_LOWER(linphone_core_manager_get_mean_audio_down_bw(marie2), 99, int, "%d");
 
-	linphone_core_accept_call(marie->lc,linphone_core_get_current_call(marie->lc));
+	linphone_call_accept(linphone_core_get_current_call(marie->lc));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallStreamsRunning,1,3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline->stat.number_of_LinphoneCallStreamsRunning,1,3000));
 
@@ -628,7 +634,7 @@ static void call_with_sips(void){
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphoneCallIncomingReceived,1,1000));
 
 		/*pauline accepts the call */
-		linphone_core_accept_call(pauline1->lc,linphone_core_get_current_call(pauline1->lc));
+		linphone_call_accept(linphone_core_get_current_call(pauline1->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphoneCallConnected,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphoneCallStreamsRunning,1,1000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallConnected,1,1000));
@@ -637,7 +643,7 @@ static void call_with_sips(void){
 		/*pauline2 should not have ring*/
 		BC_ASSERT_EQUAL(pauline2->stat.number_of_LinphoneCallIncomingReceived, 0, int, "%d");
 
-		linphone_core_terminate_call(pauline1->lc,linphone_core_get_current_call(pauline1->lc));
+		linphone_call_terminate(linphone_core_get_current_call(pauline1->lc));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_LinphoneCallEnd,1,3000));
 		BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneCallEnd,1,3000));
 
@@ -675,6 +681,7 @@ static void call_with_sips_not_achievable(void){
 		if (ei){
 			BC_ASSERT_EQUAL(linphone_error_info_get_reason(ei), LinphoneReasonTemporarilyUnavailable, int, "%d");
 		}
+		linphone_call_unref(call);
 
 		linphone_core_manager_destroy(marie);
 		linphone_core_manager_destroy(pauline1);
@@ -702,7 +709,7 @@ static bool_t is_remote_contact_ipv6(LinphoneCall *call){
 		if (ct_addr){
 			ret = strchr(linphone_address_get_domain(ct_addr),':') != NULL;
 		}
-		linphone_address_destroy(ct_addr);
+		linphone_address_unref(ct_addr);
 	}
 	return ret;
 }
@@ -723,7 +730,7 @@ static void _call_with_ipv6(bool_t caller_with_ipv6, bool_t callee_with_ipv6) {
 	marie = linphone_core_manager_new2( "marie_rc", FALSE);
 	linphone_core_enable_ipv6(marie->lc, caller_with_ipv6);
 	linphone_core_manager_start(marie, TRUE);
-	
+
 	pauline = linphone_core_manager_new2( transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc", FALSE);
 	linphone_core_enable_ipv6(pauline->lc, callee_with_ipv6);
 	linphone_core_manager_start(pauline, TRUE);
@@ -739,13 +746,13 @@ static void _call_with_ipv6(bool_t caller_with_ipv6, bool_t callee_with_ipv6) {
 		/*check that the remote contact is IPv6*/
 		BC_ASSERT_EQUAL(is_remote_contact_ipv6(pauline_call), caller_with_ipv6, int, "%i");
 		BC_ASSERT_EQUAL(is_remote_contact_ipv6(marie_call), callee_with_ipv6, int, "%i");
-		
+
 		/*check that the RTP destinations are IPv6 (flexisip should propose an IPv6 relay for parties with IPv6)*/
 		BC_ASSERT_EQUAL(is_sending_ipv6(marie_call->sessions[0].rtp_session, FALSE), caller_with_ipv6, int, "%i");
 		BC_ASSERT_EQUAL(is_sending_ipv6(marie_call->sessions[0].rtp_session, TRUE), caller_with_ipv6, int, "%i");
 		BC_ASSERT_EQUAL(is_sending_ipv6(pauline_call->sessions[0].rtp_session, FALSE), callee_with_ipv6, int, "%i");
 		BC_ASSERT_EQUAL(is_sending_ipv6(pauline_call->sessions[0].rtp_session, TRUE), callee_with_ipv6, int, "%i");
-		
+
 	}
 
 	liblinphone_tester_check_rtcp(marie,pauline);
@@ -805,8 +812,8 @@ static void file_transfer_message_rcs_to_external_body_client(void) {
 		linphone_core_set_file_transfer_server(pauline->lc,"https://www.linphone.org:444/lft.php");
 
 		/* create a chatroom on pauline's side */
-
 		chat_room = linphone_core_get_chat_room(pauline->lc, marie->identity);
+
 		/* create a file transfer message */
 		content = linphone_core_create_content(pauline->lc);
 		linphone_content_set_type(content,"image");
@@ -925,6 +932,7 @@ static void file_transfer_message_external_body_to_rcs_client(void) {
 static void dos_module_trigger(void) {
 	LinphoneChatRoom *chat_room;
 	int i = 0;
+	int dummy = 0;
 	const char* passmsg = "This one should pass through";
 	int number_of_messge_to_send = 100;
 	LinphoneChatMessage * chat_msg = NULL;
@@ -941,12 +949,12 @@ static void dos_module_trigger(void) {
 		sprintf(msg, "Flood message number %i", i);
 		chat_msg = linphone_chat_room_create_message(chat_room, msg);
 		linphone_chat_room_send_chat_message(chat_room, chat_msg);
-		ms_usleep(10000);
+		wait_for_until(marie->lc, pauline->lc, &dummy, 1, 10);
 		i++;
 	} while (i < number_of_messge_to_send);
 	// At this point we should be banned for a minute
 
-	ms_usleep(65000000); // Wait several seconds to ensure we are not banned anymore
+	wait_for_until(marie->lc, pauline->lc, &dummy, 1, 65000);; // Wait several seconds to ensure we are not banned anymore
 	BC_ASSERT_LOWER(marie->stat.number_of_LinphoneMessageReceived, number_of_messge_to_send, int, "%d");
 
 	reset_counters(&marie->stat);
@@ -962,7 +970,7 @@ static void dos_module_trigger(void) {
 	linphone_core_manager_destroy(pauline);
 }
 
-
+#if HAVE_SIPP
 static void test_subscribe_notify_with_sipp_publisher(void) {
 	char *scen;
 	FILE * sipp_out;
@@ -992,7 +1000,7 @@ static void test_subscribe_notify_with_sipp_publisher(void) {
 
 	sip_example_org = linphone_core_manager_resolve(marie, marie->identity);
 	sipp_out = sip_start(scen, linphone_address_get_username(marie->identity), linphone_auth_info_get_passwd(marie_auth), sip_example_org);
-	linphone_address_destroy(sip_example_org);
+	linphone_address_unref(sip_example_org);
 
 	if (sipp_out) {
 		/*wait for marie status*/
@@ -1004,7 +1012,8 @@ static void test_subscribe_notify_with_sipp_publisher(void) {
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
-	//does not work because sipp seams not able to manage 2 call  id in case file
+
+//does not work because sipp seams not able to manage 2 call  id in case file
 #if 0
 static void test_subscribe_notify_with_sipp_publisher_double_publish(void) {
 	char *scen;
@@ -1046,19 +1055,15 @@ static void test_subscribe_notify_with_sipp_publisher_double_publish(void) {
 	linphone_core_manager_destroy(pauline);
 }
 #endif
+#endif
 
 static void test_publish_unpublish(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
-	LinphoneProxyConfig* proxy;
+	LinphoneProxyConfig* proxy = linphone_core_get_default_proxy_config(marie->lc);
 
-	proxy = linphone_core_get_default_proxy_config(marie->lc);
-	linphone_proxy_config_edit(proxy);
-	linphone_proxy_config_enable_publish(proxy,TRUE);
-	linphone_proxy_config_done(proxy);
+	setPublish(proxy, TRUE);
 	wait_for(marie->lc, NULL, NULL, 0);
-	linphone_proxy_config_edit(proxy);
-	linphone_proxy_config_enable_publish(proxy,FALSE);
-	linphone_proxy_config_done(proxy);
+	setPublish(proxy, FALSE);
 	wait_for(marie->lc, NULL, NULL, 0);
 	linphone_core_manager_destroy(marie);
 }
@@ -1074,6 +1079,7 @@ static void test_list_subscribe (void) {
 					"<list>\n"
 						"\t<entry uri=\"%s\" />\n"
 						"\t<entry uri=\"%s\" />\n"
+						"\t<entry uri=\"sip:+33952@toto.com;user=phone\" />\n"
 					"</list>\n"
 					"</resource-lists>\n";
 
@@ -1085,7 +1091,6 @@ static void test_list_subscribe (void) {
 	char * subscribe_content = ms_strdup_printf(list,pauline_uri,laure_uri);
 	LinphoneContent* content = linphone_core_create_content(marie->lc);
 	LinphoneAddress *list_name = linphone_address_new("sip:mescops@sip.example.org");
-	LinphoneProxyConfig* proxy_config;
 	int dummy=0;
 
 	ms_free(pauline_uri);
@@ -1107,24 +1112,18 @@ static void test_list_subscribe (void) {
 
 	linphone_event_send_subscribe(lev,content);
 
-	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionOutgoingInit,1,1000));
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionOutgoingProgress,1,1000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionActive,1,5000));
 
 	/*make sure marie receives first notification before terminating*/
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_NotifyReceived,1,5000));
 	/*dummy wait to avoid derred notify*/
 	wait_for_list(lcs,&dummy,1,2000);
-	proxy_config = linphone_core_get_default_proxy_config(pauline->lc);
-	linphone_proxy_config_edit(proxy_config);
-	linphone_proxy_config_enable_publish(proxy_config,TRUE);
-	linphone_proxy_config_done(proxy_config);
+	setPublish(linphone_core_get_default_proxy_config(pauline->lc), TRUE);
 
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_NotifyReceived,2,5000));
 
-	proxy_config = linphone_core_get_default_proxy_config(laure->lc);
-	linphone_proxy_config_edit(proxy_config);
-	linphone_proxy_config_enable_publish(proxy_config,TRUE);
-	linphone_proxy_config_done(proxy_config);
+	setPublish(linphone_core_get_default_proxy_config(laure->lc), TRUE);
 	/*make sure notify is not sent "imadiatly but defered*/
 	BC_ASSERT_FALSE(wait_for_list(lcs,&marie->stat.number_of_NotifyReceived,3,1000));
 
@@ -1135,13 +1134,14 @@ static void test_list_subscribe (void) {
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphoneSubscriptionTerminated,1,5000));
 
 	ms_free(subscribe_content);
-	linphone_address_destroy(list_name);
+	linphone_address_unref(list_name);
 	linphone_content_unref(content);
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 	linphone_core_manager_destroy(laure);
 }
 
+#if HAVE_SIPP
 static void test_subscribe_on_wrong_dialog(void) {
 	char *scen;
 	FILE * sipp_out;
@@ -1153,7 +1153,7 @@ static void test_subscribe_on_wrong_dialog(void) {
 	scen = bc_tester_res("sipp/subscribe_on_wrong_dialog.xml");
 	sip_example_org = linphone_core_manager_resolve(marie, marie->identity);
 	sipp_out = sip_start(scen, linphone_address_get_username(marie->identity),linphone_auth_info_get_passwd(marie_auth), sip_example_org);
-	linphone_address_destroy(sip_example_org);
+	linphone_address_unref(sip_example_org);
 
 	if (sipp_out) {
 		/*wait for marie status*/
@@ -1162,7 +1162,223 @@ static void test_subscribe_on_wrong_dialog(void) {
 
 	linphone_core_manager_destroy(marie);
 }
+#endif
 
+static void test_list_subscribe_wrong_body(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
+
+	LinphoneEvent *lev;
+	LinphoneAddress *sub_addr = linphone_address_new("sip:rls@sip.example.com");
+
+	lev=linphone_core_create_subscribe(marie->lc,sub_addr,"presence",60);
+
+	linphone_event_add_custom_header(lev,"Supported","eventlist");
+	linphone_event_add_custom_header(lev,"Accept","application/pidf+xml, application/rlmi+xml");
+	linphone_event_add_custom_header(lev,"Content-Disposition", "recipient-list");
+	linphone_event_add_custom_header(lev,"Require", "recipient-list-subscribe");
+	linphone_event_add_custom_header(lev,"Content-type", "application/resource-lists+xml");
+
+	linphone_event_send_subscribe(lev,NULL);
+
+	BC_ASSERT_TRUE(wait_for_until(marie->lc,NULL,&marie->stat.number_of_LinphoneSubscriptionOutgoingProgress,1,1000));
+	BC_ASSERT_FALSE(wait_for_until(marie->lc,NULL,&marie->stat.number_of_LinphoneSubscriptionActive,1,2000));
+
+	linphone_event_terminate(lev);
+	linphone_core_manager_destroy(marie);
+	linphone_address_unref(sub_addr);
+}
+
+
+static void redis_publish_subscribe(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	LinphoneCoreManager* marie2 = NULL;
+	LinphoneAddress *marie_identity = linphone_address_ref(marie->identity);
+
+	linphone_core_set_network_reachable(marie->lc, FALSE);
+	linphone_core_manager_destroy(marie);
+
+	linphone_core_invite_address(pauline->lc, marie_identity);
+	BC_ASSERT_TRUE(wait_for_until(pauline->lc, NULL, &pauline->stat.number_of_LinphoneCallOutgoingProgress, 1, 3000));
+
+	marie2 = linphone_core_manager_new("marie2_rc");
+	BC_ASSERT_TRUE(wait_for_until(marie2->lc, NULL, &marie2->stat.number_of_LinphoneCallIncomingReceived, 1, 3000));
+
+	linphone_address_unref(marie_identity);
+	linphone_core_manager_destroy(pauline);
+	linphone_core_manager_destroy(marie2);
+}
+
+
+static void tls_authentication_requested_good(LinphoneCore *lc, LinphoneAuthInfo *auth_info, LinphoneAuthMethod method) {
+	if (method == LinphoneAuthTls){
+
+		char *cert = bc_tester_res("certificates/client/cert2.pem");
+		char *key = bc_tester_res("certificates/client/key2.pem");
+
+		linphone_auth_info_set_tls_cert_path(auth_info, cert);
+		linphone_auth_info_set_tls_key_path(auth_info, key);
+		linphone_core_add_auth_info(lc, auth_info);
+		bc_free(cert);
+		bc_free(key);
+	}
+}
+
+static void tls_authentication_requested_bad(LinphoneCore *lc, LinphoneAuthInfo *auth_info, LinphoneAuthMethod method) {
+	if (method == LinphoneAuthTls){
+
+		char *cert = bc_tester_res("certificates/client/cert2-signed-by-other-ca.pem");
+		char *key = bc_tester_res("certificates/client/key2.pem");
+
+		linphone_auth_info_set_tls_cert_path(auth_info, cert);
+		linphone_auth_info_set_tls_key_path(auth_info, key);
+		linphone_core_add_auth_info(lc, auth_info);
+		bc_free(cert);
+		bc_free(key);
+	}
+}
+
+static void tls_client_auth_try_register(const char *identity, bool_t with_good_cert, bool_t must_work){
+	LinphoneCoreManager *lcm;
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	LinphoneProxyConfig *cfg;
+
+	lcm = linphone_core_manager_new(NULL);
+
+	linphone_core_cbs_set_authentication_requested(cbs, with_good_cert ? tls_authentication_requested_good : tls_authentication_requested_bad);
+	linphone_core_add_callbacks(lcm->lc, cbs);
+	linphone_core_cbs_unref(cbs);
+	cfg = linphone_core_create_proxy_config(lcm->lc);
+
+	linphone_proxy_config_set_server_addr(cfg, "sip:sip2.linphone.org:5063;transport=tls");
+	linphone_proxy_config_enable_register(cfg, TRUE);
+	linphone_proxy_config_set_identity(cfg, identity);
+	linphone_core_add_proxy_config(lcm->lc, cfg);
+	if (must_work){
+		BC_ASSERT_TRUE(wait_for(lcm->lc, NULL, &lcm->stat.number_of_LinphoneRegistrationOk, 1));
+		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationFailed,0, int, "%d");
+		BC_ASSERT_EQUAL(lcm->stat.number_of_auth_info_requested,1, int, "%d");
+	}else{
+		BC_ASSERT_TRUE(wait_for(lcm->lc, NULL, &lcm->stat.number_of_LinphoneRegistrationFailed, 1));
+		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationOk,0, int, "%d");
+		/*we should expect 2 "auth_requested": one for the TLS certificate, another one because the server rejects the REGISTER with 401.*/
+		/*If the certificate isn't recognized at all, the connection will not happen and no SIP response will be received from server.*/
+		if (with_good_cert) BC_ASSERT_EQUAL(lcm->stat.number_of_auth_info_requested,2, int, "%d");
+		else BC_ASSERT_EQUAL(lcm->stat.number_of_auth_info_requested,1, int, "%d");
+	}
+
+	linphone_proxy_config_unref(cfg);
+	linphone_core_manager_destroy(lcm);
+}
+
+void tls_client_auth_bad_certificate_cn(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		/*first register to the proxy with galadrielle's identity, and authenticate by supplying galadrielle's certificate.
+		* It must work.*/
+		tls_client_auth_try_register("sip:galadrielle@sip.example.org", TRUE, TRUE);
+		/*now do the same thing, but trying to register as "Arwen". It must fail.*/
+		tls_client_auth_try_register("sip:arwen@sip.example.org", TRUE, FALSE);
+	}
+}
+
+void tls_client_auth_bad_certificate(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		/*first register to the proxy with galadrielle's identity, and authenticate by supplying galadrielle's certificate.
+		* It must work.*/
+		tls_client_auth_try_register("sip:galadrielle@sip.example.org", FALSE, FALSE);
+	}
+}
+
+static void on_eof(LinphonePlayer *player){
+	LinphonePlayerCbs *cbs = linphone_player_get_callbacks(player);
+	LinphoneCoreManager *marie=(LinphoneCoreManager*)linphone_player_cbs_get_user_data(cbs);
+	marie->stat.number_of_player_eof++;
+}
+
+void transcoder_tester(void) {
+	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
+	LinphoneCoreManager* pauline = linphone_core_manager_new( "pauline_tcp_rc");
+
+	LinphonePlayer *player;
+	char *hellopath = bc_tester_res("sounds/ahbahouaismaisbon.wav");
+	char *recordpath = bc_tester_file("record-call_with_file_player.wav");
+
+	bool_t call_ok;
+	double similar=1;
+	const double threshold = 0.8;
+
+	linphone_core_set_user_agent(marie->lc,"Transcoded Linphone",NULL);
+	linphone_core_set_user_agent(pauline->lc,"Transcoded Linphone",NULL);
+
+	disable_all_audio_codecs_except_one(marie->lc,"pcmu",-1);
+	disable_all_audio_codecs_except_one(pauline->lc,"pcma",-1);
+
+	/*caller uses files instead of soundcard in order to avoid mixing soundcard input with file played using call's player*/
+	linphone_core_use_files(marie->lc,TRUE);
+	linphone_core_set_play_file(marie->lc,NULL);
+
+	/*callee is recording and plays file*/
+	linphone_core_use_files(pauline->lc,TRUE);
+	linphone_core_set_play_file(pauline->lc,NULL);
+	linphone_core_set_record_file(pauline->lc,recordpath);
+
+	BC_ASSERT_TRUE((call_ok=call(marie,pauline)));
+	if (!call_ok) goto end;
+	player=linphone_call_get_player(linphone_core_get_current_call(marie->lc));
+	BC_ASSERT_PTR_NOT_NULL(player);
+	if (player){
+		LinphonePlayerCbs *cbs = linphone_player_get_callbacks(player);
+		linphone_player_cbs_set_eof_reached(cbs, on_eof);
+		linphone_player_cbs_set_user_data(cbs, marie);
+		BC_ASSERT_EQUAL(linphone_player_open(player,hellopath),0, int, "%d");
+		BC_ASSERT_EQUAL(linphone_player_start(player),0, int, "%d");
+	}
+	/* This assert should be modified to be at least as long as the WAV file */
+	BC_ASSERT_TRUE(wait_for_until(pauline->lc,marie->lc,&marie->stat.number_of_player_eof,1,10000));
+	/*wait one second more for transmission to be fully ended (transmission time + jitter buffer)*/
+	wait_for_until(pauline->lc,marie->lc,NULL,0,1000);
+
+	end_call(marie, pauline);
+	/*cannot run on iphone simulator because locks main loop beyond permitted time (should run
+	on another thread) */
+	BC_ASSERT_EQUAL(ms_audio_diff(hellopath,recordpath,&similar,&audio_cmp_params,NULL,NULL), 0, int, "%d");
+
+	BC_ASSERT_GREATER(similar, threshold, double, "%g");
+	BC_ASSERT_LOWER(similar, 1.0, double, "%g");
+	if (similar >= threshold && similar <= 1.0) {
+		remove(recordpath);
+	}
+
+end:
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+	ms_free(recordpath);
+	ms_free(hellopath);
+}
+
+void test_removing_old_tport(void) {
+	bctbx_list_t* lcs;
+	LinphoneCoreManager* marie2;
+	LinphoneCoreManager* marie1 = linphone_core_manager_new("marie_rc");
+	lcs=bctbx_list_append(NULL,marie1->lc);
+
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie1->stat.number_of_LinphoneRegistrationOk,1,5000));
+
+	marie2 = ms_new0(LinphoneCoreManager, 1);
+	linphone_core_manager_init(marie2, "marie_rc", NULL);
+	sal_set_uuid(marie2->lc->sal, linphone_config_get_string(linphone_core_get_config(marie1->lc),"misc", "uuid", "0"));
+	linphone_core_manager_start(marie2, TRUE);
+	lcs=bctbx_list_append(lcs, marie2->lc);
+	linphone_core_refresh_registers(marie2->lc);
+
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie2->stat.number_of_LinphoneRegistrationOk,1,5000));
+
+	BC_ASSERT_TRUE(wait_for_list(lcs,&marie1->stat.number_of_LinphoneRegistrationProgress,2,5000));
+
+	linphone_core_manager_destroy(marie1);
+	linphone_core_manager_destroy(marie2);
+	bctbx_list_free(lcs);
+}
 
 test_t flexisip_tests[] = {
 	TEST_ONE_TAG("Subscribe forking", subscribe_forking, "LeaksMemory"),
@@ -1179,20 +1395,31 @@ test_t flexisip_tests[] = {
 	TEST_NO_TAG("Call forking not responded", call_forking_not_responded),
 	TEST_NO_TAG("Early-media call forking", early_media_call_forking),
 	TEST_NO_TAG("Call with sips", call_with_sips),
-	TEST_ONE_TAG("Call with sips not achievable", call_with_sips_not_achievable, "LeaksMemory"),
+	TEST_NO_TAG("Call with sips not achievable", call_with_sips_not_achievable),
 	TEST_NO_TAG("Call ipv6 to ipv6", call_with_ipv6),
 	TEST_NO_TAG("Call ipv6 to ipv4", call_ipv6_to_ipv4),
 	TEST_NO_TAG("Call ipv4 to ipv6", call_ipv4_to_ipv6),
+#if HAVE_SIPP
 	TEST_ONE_TAG("Subscribe Notify with sipp publisher", test_subscribe_notify_with_sipp_publisher, "LeaksMemory"),
 	/*TEST_ONE_TAG("Subscribe Notify with sipp double publish", test_subscribe_notify_with_sipp_publisher_double_publish, "LeaksMemory"),*/
+#endif
 	TEST_NO_TAG("Publish/unpublish", test_publish_unpublish),
-	TEST_ONE_TAG("List subscribe", test_list_subscribe,"LeaksMemory"),
+	TEST_NO_TAG("List subscribe", test_list_subscribe),
+	TEST_NO_TAG("List subscribe without body", test_list_subscribe_wrong_body),
 	TEST_NO_TAG("File transfer message rcs to external body client", file_transfer_message_rcs_to_external_body_client),
 	TEST_ONE_TAG("File transfer message external body to rcs client", file_transfer_message_external_body_to_rcs_client, "LeaksMemory"),
 	TEST_ONE_TAG("File transfer message external body to external body client", file_transfer_message_external_body_to_external_body_client, "LeaksMemory"),
 	TEST_NO_TAG("DoS module trigger by sending a lot of chat messages", dos_module_trigger),
-	TEST_NO_TAG("Subscribe on wrong dialog", test_subscribe_on_wrong_dialog)
+#if HAVE_SIPP
+	TEST_NO_TAG("Subscribe on wrong dialog", test_subscribe_on_wrong_dialog),
+#endif
+	TEST_ONE_TAG("Redis Publish/subscribe", redis_publish_subscribe, "Skip"),
+	TEST_NO_TAG("TLS authentication - client rejected due to CN mismatch", tls_client_auth_bad_certificate_cn),
+	TEST_NO_TAG("TLS authentication - client rejected due to unrecognized certificate chain", tls_client_auth_bad_certificate),
+	TEST_NO_TAG("Transcoder", transcoder_tester),
+	TEST_NO_TAG("Removing old tport on flexisip for the same client", test_removing_old_tport)
 };
 
+
 test_suite_t flexisip_test_suite = {"Flexisip", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
 									sizeof(flexisip_tests) / sizeof(flexisip_tests[0]), flexisip_tests};
