@@ -20,7 +20,7 @@
 	#define _XOPEN_SOURCE 700 // To have definition of strptime, snprintf and getline
 #endif
 #include <time.h>
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
@@ -30,9 +30,9 @@
 
 
 /*getline is POSIX 2008, not available on many systems.*/
-#if defined(ANDROID) || defined(_WIN32) || defined(__QNX__)
+#if (defined(__ANDROID__) && !defined(__LP64__)) || defined(_WIN32) || defined(__QNX__)
 /* This code is public domain -- Will Hartung 4/9/09 */
-static size_t getline(char **lineptr, size_t *n, FILE *stream) {
+static ssize_t getline(char **lineptr, size_t *n, FILE *stream) {
 	char *bufptr = NULL;
 	char *p = bufptr;
 	size_t size;
@@ -84,7 +84,7 @@ static size_t getline(char **lineptr, size_t *n, FILE *stream) {
 	*lineptr = bufptr;
 	*n = size;
 
-	return p - bufptr - 1;
+	return (ssize_t)(p - bufptr) - 1;
 }
 #endif
 
@@ -293,9 +293,10 @@ static void upload_collected_traces(void)  {
 	if (transport_supported(LinphoneTransportTls)) {
 		LinphoneCoreManager* marie = setup(LinphoneLogCollectionEnabled);
 		int waiting = 100;
-		LinphoneCoreVTable *v_table = linphone_core_v_table_new();
-		v_table->log_collection_upload_state_changed = logCollectionUploadStateChangedCb;
-		linphone_core_add_listener(marie->lc, v_table);
+		LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+		linphone_core_cbs_set_log_collection_upload_state_changed(cbs, logCollectionUploadStateChangedCb);
+		linphone_core_add_callbacks(marie->lc, cbs);
+		linphone_core_cbs_unref(cbs);
 
 		linphone_core_set_log_collection_max_file_size(5000);
 		linphone_core_set_log_collection_upload_server_url(marie->lc,"https://www.linphone.org:444/lft.php");
