@@ -18,8 +18,8 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "linphonecore.h"
-#include "lpconfig.h"
+#include "linphone/core.h"
+#include "linphone/lpconfig.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
@@ -35,8 +35,7 @@ static int get_codec_position(const MSList *l, const char *mime_type, int rate){
 
 /*check basic things about codecs at startup: order and enablement*/
 static void start_with_no_config(void){
-	LinphoneCoreVTable vtable={0};
-	LinphoneCore *lc=linphone_core_new(&vtable, NULL, NULL, NULL);
+	LinphoneCore *lc=linphone_factory_create_core(linphone_factory_get(), NULL, NULL, NULL);
 	const MSList *codecs=linphone_core_get_audio_codecs(lc);
 	int opus_codec_pos;
 	int speex_codec_pos=get_codec_position(codecs, "speex", 8000);
@@ -51,7 +50,7 @@ static void start_with_no_config(void){
 	if (pt) {
 		BC_ASSERT_TRUE(linphone_core_payload_type_enabled(lc, pt));
 	}
-	linphone_core_destroy(lc);
+	linphone_core_unref(lc);
 }
 
 static void check_payload_type_numbers(LinphoneCall *call1, LinphoneCall *call2, int expected_number){
@@ -59,13 +58,13 @@ static void check_payload_type_numbers(LinphoneCall *call1, LinphoneCall *call2,
 	const PayloadType *pt=linphone_call_params_get_used_audio_codec(params);
 	BC_ASSERT_PTR_NOT_NULL(pt);
 	if (pt){
-		BC_ASSERT_EQUAL(linphone_core_get_payload_type_number(linphone_call_get_core(call1),pt),expected_number, int, "%d");
+		BC_ASSERT_EQUAL(payload_type_get_number(pt),expected_number, int, "%d");
 	}
 	params=linphone_call_get_current_params(call2);
 	pt=linphone_call_params_get_used_audio_codec(params);
 	BC_ASSERT_PTR_NOT_NULL(pt);
 	if (pt){
-		BC_ASSERT_EQUAL(linphone_core_get_payload_type_number(linphone_call_get_core(call1),pt),expected_number, int, "%d");
+		BC_ASSERT_EQUAL(payload_type_get_number(pt),expected_number, int, "%d");
 	}
 }
 
@@ -81,9 +80,7 @@ static void simple_call_with_different_codec_mappings(void) {
 	disable_all_audio_codecs_except_one(pauline->lc,"pcmu",-1);
 
 	/*marie set a fantasy number to PCMU*/
-	linphone_core_set_payload_type_number(marie->lc,
-		linphone_core_find_payload_type(marie->lc, "PCMU", 8000, -1),
-		104);
+	payload_type_set_number(linphone_core_find_payload_type(marie->lc, "PCMU", 8000, -1), 104);
 
 	BC_ASSERT_TRUE(call(marie,pauline));
 	pauline_call=linphone_core_get_current_call(pauline->lc);
@@ -92,7 +89,7 @@ static void simple_call_with_different_codec_mappings(void) {
 		LinphoneCallParams *params;
 		check_payload_type_numbers(linphone_core_get_current_call(marie->lc), pauline_call, 104);
 		/*make a reinvite in the other direction*/
-		linphone_core_update_call(pauline->lc, pauline_call,
+		linphone_call_update(pauline_call,
 			params=linphone_core_create_call_params(pauline->lc, pauline_call));
 		linphone_call_params_unref(params);
 		BC_ASSERT_TRUE(wait_for(pauline->lc,marie->lc,&pauline->stat.number_of_LinphoneCallUpdating,1));
@@ -401,9 +398,9 @@ static void savpf_dtls_to_avpf_video_call(void) {
 #endif
 
 #ifdef VIDEO_ENABLED
-static LinphonePayloadType * configure_core_for_avpf_and_video(LinphoneCore *lc) {
+static OrtpPayloadType * configure_core_for_avpf_and_video(LinphoneCore *lc) {
 	LinphoneProxyConfig *lpc;
-	LinphonePayloadType *lpt;
+	OrtpPayloadType *pt;
 	LinphoneVideoPolicy policy = { 0 };
 
 	policy.automatically_initiate = TRUE;
@@ -415,13 +412,13 @@ static LinphonePayloadType * configure_core_for_avpf_and_video(LinphoneCore *lc)
 	linphone_core_enable_video_capture(lc, TRUE);
 	linphone_core_enable_video_display(lc, TRUE);
 	linphone_core_set_video_policy(lc, &policy);
-	lpt = linphone_core_find_payload_type(lc, "VP8", 90000, -1);
-	if (lpt == NULL) {
+	pt = linphone_core_find_payload_type(lc, "VP8", 90000, -1);
+	if (pt == NULL) {
 		ms_warning("VP8 codec not available.");
 	} else {
 		disable_all_video_codecs_except_one(lc, "VP8");
 	}
-	return lpt;
+	return pt;
 }
 
 static void check_avpf_features(LinphoneCore *lc, unsigned char expected_features) {
@@ -444,19 +441,20 @@ static void check_avpf_features(LinphoneCore *lc, unsigned char expected_feature
 static void compatible_avpf_features(void) {
 	LinphoneCoreManager *marie = linphone_core_manager_new("marie_rc");
 	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_tcp_rc");
-	LinphonePayloadType *lpt;
+	OrtpPayloadType *pt;
 	bool_t call_ok;
 
 	if (configure_core_for_avpf_and_video(marie->lc) == NULL) goto end;
-	lpt = configure_core_for_avpf_and_video(pauline->lc);
+	
+	pt = configure_core_for_avpf_and_video(pauline->lc);
 
 	BC_ASSERT_TRUE((call_ok=call(marie, pauline)));
 	if (!call_ok) goto end;
 
 	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &marie->stat.number_of_LinphoneCallStreamsRunning, 1));
 	BC_ASSERT_TRUE(wait_for(marie->lc, pauline->lc, &pauline->stat.number_of_LinphoneCallStreamsRunning, 1));
-	check_avpf_features(marie->lc, lpt->avpf.features);
-	check_avpf_features(pauline->lc, lpt->avpf.features);
+	check_avpf_features(marie->lc, pt->avpf.features);
+	check_avpf_features(pauline->lc, pt->avpf.features);
 
 	end_call(marie,pauline);
 end:
@@ -467,12 +465,13 @@ end:
 static void incompatible_avpf_features(void) {
 	LinphoneCoreManager *marie = linphone_core_manager_new("marie_rc");
 	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_tcp_rc");
-	LinphonePayloadType *lpt;
+	OrtpPayloadType *pt;
 	bool_t call_ok;
 
 	if (configure_core_for_avpf_and_video(marie->lc) == NULL) goto end;
-	lpt = configure_core_for_avpf_and_video(pauline->lc);
-	lpt->avpf.features = PAYLOAD_TYPE_AVPF_NONE;
+	
+	pt = configure_core_for_avpf_and_video(pauline->lc);
+	pt->avpf.features = PAYLOAD_TYPE_AVPF_NONE;
 
 	BC_ASSERT_TRUE(call_ok=call(marie, pauline));
 	if (!call_ok) goto end;
