@@ -17,7 +17,7 @@
 */
 
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
@@ -37,10 +37,13 @@ static LinphoneCoreManager* presence_linphone_core_manager_new(char* username) {
 
 
 void new_subscription_requested(LinphoneCore *lc, LinphoneFriend *lf, const char *url){
-	char* from=linphone_address_as_string(linphone_friend_get_address(lf));
 	stats* counters;
-	ms_message("New subscription request from [%s] url [%s]",from,url);
-	ms_free(from);
+	const LinphoneAddress *addr = linphone_friend_get_address(lf);
+	if (addr != NULL) {
+		char* from=linphone_address_as_string(addr);
+		ms_message("New subscription request from [%s] url [%s]",from,url);
+		ms_free(from);
+	}
 	counters = get_stats(lc);
 	counters->number_of_NewSubscriptionRequest++;
 	linphone_core_add_friend(lc,lf); /*accept subscription*/
@@ -48,11 +51,13 @@ void new_subscription_requested(LinphoneCore *lc, LinphoneFriend *lf, const char
 
 void notify_presence_received(LinphoneCore *lc, LinphoneFriend * lf) {
 	stats* counters;
-	
 	unsigned int i;
-	char* from=linphone_address_as_string(linphone_friend_get_address(lf));
-	ms_message("New Notify request from [%s] ",from);
-	ms_free(from);
+	const LinphoneAddress *addr = linphone_friend_get_address(lf);
+	if (addr != NULL) {
+		char* from=linphone_address_as_string(addr);
+		ms_message("New Notify request from [%s] ",from);
+		ms_free(from);
+	}
 	counters = get_stats(lc);
 	counters->number_of_NotifyPresenceReceived++;
 	counters->last_received_presence = linphone_friend_get_presence_model(lf);
@@ -63,82 +68,93 @@ void notify_presence_received(LinphoneCore *lc, LinphoneFriend * lf) {
 	} else {
 		ms_error("Unexpected basic status [%i]",linphone_presence_model_get_basic_status(counters->last_received_presence));
 	}
-	for (i=0;i<linphone_presence_model_get_nb_activities(counters->last_received_presence); i++) {
-		LinphonePresenceActivity *activity = linphone_presence_model_get_nth_activity(counters->last_received_presence, i);
-		switch (linphone_presence_activity_get_type(activity)) {
-			case LinphonePresenceActivityOffline:
-				counters->number_of_LinphonePresenceActivityOffline++; break;
-			case LinphonePresenceActivityOnline:
-				counters->number_of_LinphonePresenceActivityOnline++; break;
-			case LinphonePresenceActivityAppointment:
-				counters->number_of_LinphonePresenceActivityAppointment++; break;
-			case LinphonePresenceActivityAway:
-				counters->number_of_LinphonePresenceActivityAway++; break;
-			case LinphonePresenceActivityBreakfast:
-				counters->number_of_LinphonePresenceActivityBreakfast++; break;
-			case LinphonePresenceActivityBusy:
-				counters->number_of_LinphonePresenceActivityBusy++; break;
-			case LinphonePresenceActivityDinner:
-				counters->number_of_LinphonePresenceActivityDinner++; break;
-			case LinphonePresenceActivityHoliday:
-				counters->number_of_LinphonePresenceActivityHoliday++; break;
-			case LinphonePresenceActivityInTransit:
-				counters->number_of_LinphonePresenceActivityInTransit++; break;
-			case LinphonePresenceActivityLookingForWork:
-				counters->number_of_LinphonePresenceActivityLookingForWork++; break;
-			case LinphonePresenceActivityLunch:
-				counters->number_of_LinphonePresenceActivityLunch++; break;
-			case LinphonePresenceActivityMeal:
-				counters->number_of_LinphonePresenceActivityMeal++; break;
-			case LinphonePresenceActivityMeeting:
-				counters->number_of_LinphonePresenceActivityMeeting++; break;
-			case LinphonePresenceActivityOnThePhone:
-				counters->number_of_LinphonePresenceActivityOnThePhone++; break;
-			case LinphonePresenceActivityOther:
-				counters->number_of_LinphonePresenceActivityOther++; break;
-			case LinphonePresenceActivityPerformance:
-				counters->number_of_LinphonePresenceActivityPerformance++; break;
-			case LinphonePresenceActivityPermanentAbsence:
-				counters->number_of_LinphonePresenceActivityPermanentAbsence++; break;
-			case LinphonePresenceActivityPlaying:
-				counters->number_of_LinphonePresenceActivityPlaying++; break;
-			case LinphonePresenceActivityPresentation:
-				counters->number_of_LinphonePresenceActivityPresentation++; break;
-			case LinphonePresenceActivityShopping:
-				counters->number_of_LinphonePresenceActivityShopping++; break;
-			case LinphonePresenceActivitySleeping:
-				counters->number_of_LinphonePresenceActivitySleeping++; break;
-			case LinphonePresenceActivitySpectator:
-				counters->number_of_LinphonePresenceActivitySpectator++; break;
-			case LinphonePresenceActivitySteering:
-				counters->number_of_LinphonePresenceActivitySteering++; break;
-			case LinphonePresenceActivityTravel:
-				counters->number_of_LinphonePresenceActivityTravel++; break;
-			case LinphonePresenceActivityTV:
-				counters->number_of_LinphonePresenceActivityTV++; break;
-			case LinphonePresenceActivityUnknown:
-				counters->number_of_LinphonePresenceActivityUnknown++; break;
-			case LinphonePresenceActivityVacation:
-				counters->number_of_LinphonePresenceActivityVacation++; break;
-			case LinphonePresenceActivityWorking:
-				counters->number_of_LinphonePresenceActivityWorking++; break;
-			case LinphonePresenceActivityWorship:
-				counters->number_of_LinphonePresenceActivityWorship++; break;
+	if (linphone_presence_model_get_nb_activities(counters->last_received_presence) > 0) {
+		for (i=0;counters->last_received_presence&&i<linphone_presence_model_get_nb_activities(counters->last_received_presence); i++) {
+			LinphonePresenceActivity *activity = linphone_presence_model_get_nth_activity(counters->last_received_presence, i);
+			switch (linphone_presence_activity_get_type(activity)) {
+				case LinphonePresenceActivityAppointment:
+					counters->number_of_LinphonePresenceActivityAppointment++; break;
+				case LinphonePresenceActivityAway:
+					counters->number_of_LinphonePresenceActivityAway++; break;
+				case LinphonePresenceActivityBreakfast:
+					counters->number_of_LinphonePresenceActivityBreakfast++; break;
+				case LinphonePresenceActivityBusy:
+					counters->number_of_LinphonePresenceActivityBusy++; break;
+				case LinphonePresenceActivityDinner:
+					counters->number_of_LinphonePresenceActivityDinner++; break;
+				case LinphonePresenceActivityHoliday:
+					counters->number_of_LinphonePresenceActivityHoliday++; break;
+				case LinphonePresenceActivityInTransit:
+					counters->number_of_LinphonePresenceActivityInTransit++; break;
+				case LinphonePresenceActivityLookingForWork:
+					counters->number_of_LinphonePresenceActivityLookingForWork++; break;
+				case LinphonePresenceActivityLunch:
+					counters->number_of_LinphonePresenceActivityLunch++; break;
+				case LinphonePresenceActivityMeal:
+					counters->number_of_LinphonePresenceActivityMeal++; break;
+				case LinphonePresenceActivityMeeting:
+					counters->number_of_LinphonePresenceActivityMeeting++; break;
+				case LinphonePresenceActivityOnThePhone:
+					counters->number_of_LinphonePresenceActivityOnThePhone++; break;
+				case LinphonePresenceActivityOther:
+					counters->number_of_LinphonePresenceActivityOther++; break;
+				case LinphonePresenceActivityPerformance:
+					counters->number_of_LinphonePresenceActivityPerformance++; break;
+				case LinphonePresenceActivityPermanentAbsence:
+					counters->number_of_LinphonePresenceActivityPermanentAbsence++; break;
+				case LinphonePresenceActivityPlaying:
+					counters->number_of_LinphonePresenceActivityPlaying++; break;
+				case LinphonePresenceActivityPresentation:
+					counters->number_of_LinphonePresenceActivityPresentation++; break;
+				case LinphonePresenceActivityShopping:
+					counters->number_of_LinphonePresenceActivityShopping++; break;
+				case LinphonePresenceActivitySleeping:
+					counters->number_of_LinphonePresenceActivitySleeping++; break;
+				case LinphonePresenceActivitySpectator:
+					counters->number_of_LinphonePresenceActivitySpectator++; break;
+				case LinphonePresenceActivitySteering:
+					counters->number_of_LinphonePresenceActivitySteering++; break;
+				case LinphonePresenceActivityTravel:
+					counters->number_of_LinphonePresenceActivityTravel++; break;
+				case LinphonePresenceActivityTV:
+					counters->number_of_LinphonePresenceActivityTV++; break;
+				case LinphonePresenceActivityUnknown:
+					counters->number_of_LinphonePresenceActivityUnknown++; break;
+				case LinphonePresenceActivityVacation:
+					counters->number_of_LinphonePresenceActivityVacation++; break;
+				case LinphonePresenceActivityWorking:
+					counters->number_of_LinphonePresenceActivityWorking++; break;
+				case LinphonePresenceActivityWorship:
+					counters->number_of_LinphonePresenceActivityWorship++; break;
+			}
 		}
+	} else {
+		if (linphone_presence_model_get_basic_status(counters->last_received_presence) == LinphonePresenceBasicStatusOpen)
+			counters->number_of_LinphonePresenceActivityOnline++;
+		else
+			counters->number_of_LinphonePresenceActivityOffline++;
 	}
 }
 
+void notify_presence_received_for_uri_or_tel(LinphoneCore *lc, LinphoneFriend *lf, const char *uri_or_tel, const LinphonePresenceModel *presence) {
+	stats *counters = get_stats(lc);
+	ms_message("Presence notification for URI or phone number [%s]", uri_or_tel);
+	counters->number_of_NotifyPresenceReceivedForUriOrTel++;
+}
+
 static void simple_publish_with_expire(int expires) {
 	LinphoneCoreManager* marie = linphone_core_manager_new( "marie_rc");
 	LinphoneProxyConfig* proxy;
 	LinphonePresenceModel* presence;
-	LinphoneCoreVTable *vtable = linphone_core_v_table_new();
-	vtable->publish_state_changed = linphone_publish_state_changed;
-	_linphone_core_add_listener(marie->lc, vtable, TRUE, TRUE );
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	
+	linphone_core_cbs_set_publish_state_changed(cbs, linphone_publish_state_changed);
+	_linphone_core_add_callbacks(marie->lc, cbs, TRUE);
+	linphone_core_cbs_unref(cbs);
 
 	proxy = linphone_core_get_default_proxy_config(marie->lc);
 	linphone_proxy_config_edit(proxy);
-	if (expires >0) {
+	if (expires > 0) {
 		linphone_proxy_config_set_publish_expires(proxy,expires);
 	}
 	linphone_proxy_config_enable_publish(proxy,TRUE);
@@ -147,8 +163,10 @@ static void simple_publish_with_expire(int expires) {
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,1));
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,1));
 
-	presence =linphone_presence_model_new_with_activity(LinphonePresenceActivityOffline,NULL);
+	presence = linphone_presence_model_new();
+	linphone_presence_model_set_basic_status(presence, LinphonePresenceBasicStatusClosed);
 	linphone_core_set_presence_model(marie->lc,presence);
+	linphone_presence_model_unref(presence);
 
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,2));
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,2));
@@ -180,10 +198,10 @@ static void simple_publish_with_expire(int expires) {
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishProgress,4));
 	BC_ASSERT_TRUE(wait_for(marie->lc,marie->lc,&marie->stat.number_of_LinphonePublishOk,4));
 
-	linphone_core_manager_destroy(marie);
-	/*fixme we should wait untill 200okBC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishCleared,2,int,"%i");*/
+	linphone_core_manager_stop(marie);
+	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishCleared,3,int,"%i"); /*yes it is 3 because when we change the expires, a new LinphoneEvent is created*/
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishOk,4,int,"%i");
-
+	linphone_core_manager_destroy(marie);
 }
 
 static void simple_publish(void) {
@@ -194,6 +212,31 @@ static void publish_with_expires(void) {
 	simple_publish_with_expire(2);
 }
 
+static void publish_with_dual_identity(void) {
+	LinphoneCoreManager* pauline = linphone_core_manager_new("multi_account_rc");
+	const bctbx_list_t* proxies;
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	
+	linphone_core_cbs_set_publish_state_changed(cbs, linphone_publish_state_changed);
+	_linphone_core_add_callbacks(pauline->lc, cbs, TRUE);
+	linphone_core_cbs_unref(cbs);
+	
+	for (proxies = linphone_core_get_proxy_config_list(pauline->lc); proxies!=NULL; proxies = proxies->next) {
+		LinphoneProxyConfig *proxy = (LinphoneProxyConfig *) proxies->data;
+		linphone_proxy_config_edit(proxy);
+		linphone_proxy_config_enable_publish(proxy,TRUE);
+		linphone_proxy_config_done(proxy);
+	}
+	
+	BC_ASSERT_TRUE(wait_for(pauline->lc,pauline->lc,&pauline->stat.number_of_LinphonePublishProgress,4));
+	BC_ASSERT_TRUE(wait_for(pauline->lc,pauline->lc,&pauline->stat.number_of_LinphonePublishOk,4));
+	
+	linphone_core_manager_stop(pauline);
+	BC_ASSERT_EQUAL(pauline->stat.number_of_LinphonePublishCleared,4,int,"%i");
+	BC_ASSERT_EQUAL(pauline->stat.number_of_LinphonePublishOk,4,int,"%i");
+	linphone_core_manager_destroy(pauline);
+	
+}
 static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,LinphoneCoreManager* callee_mgr) {
 	stats initial_caller=caller_mgr->stat;
 	stats initial_callee=callee_mgr->stat;
@@ -224,6 +267,9 @@ static bool_t subscribe_to_callee_presence(LinphoneCoreManager* caller_mgr,Linph
 	return result;
 
 }
+
+/* BEWARE this test will fail if the machine it is run on is behind an active firewall not sending ICMP errors on incoming connections! */
+/* It will create a leak of NOTIFY because during marie error after network switch off, Pauline will never received the acknowledgement of the notify ...*/
 static void subscribe_failure_handle_by_app(void) {
 	LinphoneCoreManager* marie = linphone_core_manager_new("marie_rc");
 	LinphoneCoreManager* pauline = linphone_core_manager_new("pauline_tcp_rc");
@@ -243,6 +289,7 @@ static void subscribe_failure_handle_by_app(void) {
 	sal_set_recv_error(marie->lc->sal, 1);
 
 	lf = linphone_core_get_friend_by_address(marie->lc,lf_identity);
+	ms_free(lf_identity);
 	BC_ASSERT_PTR_NOT_NULL(lf);
 	linphone_friend_edit(lf);
 	linphone_friend_enable_subscribes(lf,FALSE); /*disable subscription*/
@@ -273,49 +320,49 @@ static void simple_subscribe(void) {
 	linphone_core_manager_destroy(pauline);
 }
 static void simple_subscribe_with_early_notify(void) {
-	
+
 	LinphoneCoreManager* marie = presence_linphone_core_manager_new("marie");
 	LinphoneCoreManager* pauline = presence_linphone_core_manager_new("pauline");
 	LinphoneAddress *marie_identity_addr = linphone_address_clone(marie->identity);
 	LpConfig *pauline_lp;
-	
+
 	char* pauline_identity=linphone_address_as_string_uri_only(pauline->identity);
 	char* marie_identity;
-	
+
 	LinphoneFriend* pauline_s_friend;
 	LinphoneFriend* marie_s_friend=linphone_core_create_friend_with_address(marie->lc,pauline_identity);
-	
+
 	pauline_lp = linphone_core_get_config(pauline->lc);
 	lp_config_set_int(pauline_lp,"sip","notify_pending_state",1);
-	
+
 	linphone_friend_edit(marie_s_friend);
 	linphone_friend_enable_subscribes(marie_s_friend,TRUE);
 	linphone_friend_done(marie_s_friend);
 	linphone_core_add_friend(marie->lc,marie_s_friend);
 	ms_free(pauline_identity);
-	
-	
+
+
 	/*to simulate pending state.*/
 
 	linphone_address_set_port(marie_identity_addr,0);
 	marie_identity=linphone_address_as_string_uri_only(marie_identity_addr);
 	pauline_s_friend=linphone_core_create_friend_with_address(pauline->lc,marie_identity);
 	linphone_core_add_friend(pauline->lc,pauline_s_friend);
-	
+
 	ms_free(marie_identity);
 
 	BC_ASSERT_TRUE(wait_for(marie->lc,pauline->lc,&marie->stat.number_of_NotifyPresenceReceived,1));
 	BC_ASSERT_EQUAL(linphone_friend_get_subscription_state(marie_s_friend), LinphoneSubscriptionPending,int, "%d");
-	
+
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,marie->stat.number_of_LinphonePresenceActivityOnline+1);
-	
+
 	BC_ASSERT_EQUAL(marie->stat.number_of_NotifyPresenceReceived,2, int, "%d");
-	
+
 	linphone_friend_unref(marie_s_friend);
 	linphone_friend_unref(pauline_s_friend);
 	linphone_address_unref(marie_identity_addr);
 	linphone_core_manager_destroy(marie);
-	
+
 	linphone_core_manager_destroy(pauline);
 }
 
@@ -377,6 +424,7 @@ static void presence_information(void) {
 	/* Presence activity without description. */
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivityDinner, NULL);
 	linphone_core_set_presence_model(pauline->lc, presence);
+	linphone_presence_model_unref(presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityDinner,1);
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityDinner, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
@@ -388,6 +436,7 @@ static void presence_information(void) {
 	/* Presence activity with description. */
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivitySteering, bike_description);
 	linphone_core_set_presence_model(pauline->lc, presence);
+	linphone_presence_model_unref(presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivitySteering,1);
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivitySteering, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
@@ -400,6 +449,7 @@ static void presence_information(void) {
 	/* Presence activity with description and note. */
 	presence = linphone_presence_model_new_with_activity_and_note(LinphonePresenceActivityVacation, NULL, vacation_note, vacation_lang);
 	linphone_core_set_presence_model(pauline->lc, presence);
+	linphone_presence_model_unref(presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityVacation,1);
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityVacation, 1, int, "%d");
 	activity = linphone_presence_model_get_activity(marie->stat.last_received_presence);
@@ -421,6 +471,7 @@ static void presence_information(void) {
 	presence = linphone_presence_model_new_with_activity(LinphonePresenceActivityOnThePhone, NULL);
 	linphone_presence_model_set_contact(presence, contact);
 	linphone_core_set_presence_model(pauline->lc, presence);
+	linphone_presence_model_unref(presence);
 	wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnThePhone,1);
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityOnThePhone, 1, int, "%d");
 	contact2 = linphone_presence_model_get_contact(presence);
@@ -438,6 +489,7 @@ static void presence_information(void) {
 	BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePresenceActivityShopping, 1, int, "%d");
 	presence_timestamp = linphone_presence_model_get_timestamp(presence);
 	BC_ASSERT_GREATER((unsigned)presence_timestamp , (unsigned)current_timestamp, unsigned, "%u");
+	linphone_presence_model_unref(presence);
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
@@ -464,7 +516,7 @@ static void subscribe_presence_forked(void){
 
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline1->stat.number_of_NewSubscriptionRequest,1, 10000));
 	BC_ASSERT_TRUE(wait_for_list(lcs,&pauline2->stat.number_of_NewSubscriptionRequest,1, 2000));
-	
+
 	/*we should get only one notify*/
 	BC_ASSERT_TRUE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,1, 10000));
 	BC_ASSERT_FALSE(wait_for_list(lcs,&marie->stat.number_of_LinphonePresenceActivityOnline,2, 2000));
@@ -529,11 +581,13 @@ static void simple_subscribe_with_friend_from_rc(void) {
 	LinphoneCoreManager *marie = presence_linphone_core_manager_new_with_rc_name("marie", "pauline_as_friend_rc");
 	LinphoneFriend *pauline_as_friend;
 
-	BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_friend_list(marie->lc)), 1, int , "%i");
+	BC_ASSERT_EQUAL((unsigned int)bctbx_list_size(linphone_core_get_friend_list(marie->lc)), 1, unsigned int , "%u");
 
 	if (bctbx_list_size(linphone_core_get_friend_list(marie->lc))>0) {
 		pauline_as_friend = (LinphoneFriend*)linphone_core_get_friend_list(marie->lc)->data;
+		linphone_friend_edit(pauline_as_friend);
 		linphone_friend_set_address(pauline_as_friend, pauline->identity); /*hack to update addr with port number*/
+		linphone_friend_done(pauline_as_friend);
 	}
 
 	BC_ASSERT_TRUE (wait_for(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePresenceActivityOnline,1));
@@ -552,8 +606,9 @@ test_t presence_tests[] = {
 	TEST_ONE_TAG("Simple Subscribe", simple_subscribe,"presence"),
 	TEST_ONE_TAG("Simple Subscribe with early NOTIFY", simple_subscribe_with_early_notify,"presence"),
 	TEST_NO_TAG("Simple Subscribe with friend from rc", simple_subscribe_with_friend_from_rc),
-	TEST_ONE_TAG("Simple Publish", simple_publish, "LeaksMemory"),
-	TEST_ONE_TAG("Simple Publish with expires", publish_with_expires, "LeaksMemory"),
+	TEST_NO_TAG("Simple Publish", simple_publish),
+	TEST_NO_TAG("Publish with 2 identities", publish_with_dual_identity),
+	TEST_NO_TAG("Simple Publish with expires", publish_with_expires),
 	/*TEST_ONE_TAG("Call with presence", call_with_presence, "LeaksMemory"),*/
 	TEST_NO_TAG("Unsubscribe while subscribing", unsubscribe_while_subscribing),
 	TEST_NO_TAG("Presence information", presence_information),
