@@ -17,7 +17,7 @@
 */
 
 #include <stdio.h>
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
@@ -197,7 +197,7 @@ static void quality_reporting_not_sent_if_low_bandwidth(void) {
 		BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishProgress,0, int, "%d");
 		BC_ASSERT_EQUAL(marie->stat.number_of_LinphonePublishOk,0, int, "%d");
 	}
-	linphone_call_params_destroy(marie_params);
+	linphone_call_params_unref(marie_params);
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
@@ -307,8 +307,11 @@ static void quality_reporting_session_report_if_video_stopped(void) {
 		BC_ASSERT_TRUE(linphone_call_params_video_enabled(linphone_call_get_current_params(call_pauline)));
 
 		/*remove video*/
+		linphone_call_params_unref(pauline_params);
+		pauline_params=linphone_core_create_call_params(pauline->lc, call_pauline);
 		linphone_call_params_enable_video(pauline_params,FALSE);
-		linphone_core_update_call(pauline->lc,call_pauline,pauline_params);
+		linphone_call_update(call_pauline,pauline_params);
+		linphone_call_params_unref(pauline_params);
 
 		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishProgress,1,10000));
 		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishOk,1,10000));
@@ -320,8 +323,8 @@ static void quality_reporting_session_report_if_video_stopped(void) {
 		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishProgress,2,5000));
 		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishOk,2,5000));
 	}
-	linphone_call_params_destroy(marie_params);
-	linphone_call_params_destroy(pauline_params);
+	linphone_call_params_unref(marie_params);
+	
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
@@ -340,9 +343,10 @@ static void quality_reporting_sent_using_custom_route(void) {
 	LinphoneCall* call_marie = NULL;
 	LinphoneCall* call_pauline = NULL;
 
-	LinphoneCoreVTable publish_vtable = {0};
-	publish_vtable.publish_state_changed = publish_report_with_route_state_changed;
-	linphone_core_add_listener(marie->lc, &publish_vtable);
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	linphone_core_cbs_set_publish_state_changed(cbs, publish_report_with_route_state_changed);
+	linphone_core_add_callbacks(marie->lc, cbs);
+	linphone_core_cbs_unref(cbs);
 
 	//INVALID collector: sip.linphone.org do not collect reports, so it will throw a 404 Not Found error
 	linphone_proxy_config_set_quality_reporting_collector(linphone_core_get_default_proxy_config(marie->lc), "sip:sip.linphone.org");
@@ -383,7 +387,7 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 
 	if (create_call_for_quality_reporting_tests(marie, pauline, &call_marie, &call_pauline, marie_params, pauline_params))  {
 		linphone_reporting_set_on_report_send(call_marie, on_report_send_mandatory);
-		linphone_proxy_config_set_quality_reporting_interval(call_marie->dest_proxy, 1);
+		linphone_proxy_config_set_quality_reporting_interval(call_marie->dest_proxy, 3);
 
 		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,NULL,0,3000));
 		BC_ASSERT_TRUE(linphone_call_params_video_enabled(linphone_call_get_current_params(call_pauline)));
@@ -393,8 +397,8 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 		BC_ASSERT_PTR_NOT_NULL(linphone_core_get_current_call(pauline->lc));
 
 		// PUBLISH submission to the collector should be ok
-		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishProgress,1,60000));
-		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishOk,1,60000));
+		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishProgress,1,5000));
+		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishOk,1,10000));
 
 		pauline_chat_room = linphone_call_get_chat_room(call_pauline);
 		BC_ASSERT_PTR_NOT_NULL(pauline_chat_room);
@@ -406,23 +410,60 @@ static void quality_reporting_interval_report_video_and_rtt(void) {
 
 			for (i = 0; i < strlen(message); i++) {
 				linphone_chat_message_put_char(rtt_message, message[i]);
-				BC_ASSERT_TRUE(wait_for_until(pauline->lc, marie->lc, &marie->stat.number_of_LinphoneIsComposingActiveReceived, i+1, 1000));
+				BC_ASSERT_TRUE(wait_for_until(pauline->lc, marie->lc, &marie->stat.number_of_LinphoneIsComposingActiveReceived, (int)i+1, 1000));
 				BC_ASSERT_EQUAL(linphone_chat_room_get_char(marie_chat_room), message[i], char, "%c");
 			}
 			linphone_chat_room_send_chat_message(pauline_chat_room, rtt_message);
 		}
-
+		
 		end_call(marie, pauline);
+		/*wait that all publish complete*/
+		BC_ASSERT_TRUE(wait_for_until(marie->lc,pauline->lc,&marie->stat.number_of_LinphonePublishOk,marie->stat.number_of_LinphonePublishProgress,60000));
 	}
 
-	linphone_call_params_destroy(marie_params);
-	linphone_call_params_destroy(pauline_params);
+	linphone_call_params_unref(marie_params);
+	linphone_call_params_unref(pauline_params);
 
 	linphone_core_manager_destroy(marie);
 	linphone_core_manager_destroy(pauline);
 }
 #endif
 
+static void video_bandwidth_estimation(void){
+	LinphoneCoreManager *marie = linphone_core_manager_new("marie_rc");
+	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_rc");
+	LinphoneVideoPolicy pol = {0};
+	OrtpNetworkSimulatorParams simparams = { 0 };
+	
+	linphone_core_set_video_device(marie->lc, "Mire: Mire (synthetic moving picture)");
+	linphone_core_enable_video_capture(marie->lc, TRUE);
+	linphone_core_enable_video_display(marie->lc, TRUE);
+	linphone_core_enable_video_capture(pauline->lc, TRUE);
+	linphone_core_enable_video_display(pauline->lc, TRUE);
+	
+	pol.automatically_accept = TRUE;
+	pol.automatically_initiate = TRUE;
+	linphone_core_set_video_policy(marie->lc, &pol);
+	linphone_core_set_video_policy(pauline->lc, &pol);
+	
+	linphone_core_set_preferred_video_size_by_name(marie->lc, "vga");
+	simparams.mode = OrtpNetworkSimulatorOutbound;
+	simparams.enabled = TRUE;
+	simparams.max_bandwidth = 300000;
+	linphone_core_set_network_simulator_params(marie->lc, &simparams);
+	
+	if (BC_ASSERT_TRUE(call(marie, pauline))){
+		/*wait for the first TMMBR*/
+		BC_ASSERT_TRUE(wait_for_until(marie->lc, pauline->lc, &marie->stat.number_of_tmmbr_received, 1, 50000));
+		BC_ASSERT_GREATER((float)marie->stat.last_tmmbr_value_received, 270000.f, float, "%f");
+		BC_ASSERT_LOWER((float)marie->stat.last_tmmbr_value_received, 330000.f, float, "%f");
+		
+		end_call(marie, pauline);
+	}
+	linphone_core_manager_destroy(marie);
+	linphone_core_manager_destroy(pauline);
+}
+
 test_t quality_reporting_tests[] = {
 	TEST_NO_TAG("Not used if no config", quality_reporting_not_used_without_config),
 	TEST_NO_TAG("Call term session report not sent if call did not start", quality_reporting_not_sent_if_call_not_started),
@@ -434,7 +475,8 @@ test_t quality_reporting_tests[] = {
 	TEST_NO_TAG("Interval report if interval is configured with video and realtime text", quality_reporting_interval_report_video_and_rtt),
 	TEST_NO_TAG("Session report sent if video stopped during call", quality_reporting_session_report_if_video_stopped),
 	#endif
-	TEST_NO_TAG("Sent using custom route", quality_reporting_sent_using_custom_route)
+	TEST_NO_TAG("Sent using custom route", quality_reporting_sent_using_custom_route),
+	TEST_NO_TAG("Video bandwidth estimation", video_bandwidth_estimation)
 };
 
 test_suite_t quality_reporting_test_suite = {"QualityReporting", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
