@@ -17,31 +17,28 @@
 */
 
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
 
-
-static void auth_info_requested(LinphoneCore *lc, const char *realm, const char *username, const char *domain) {
-	LinphoneAuthInfo *info;
-	info=linphone_auth_info_new(test_username,NULL,test_password,NULL,realm,domain); /*create authentication structure from identity*/
-	linphone_core_add_auth_info(lc,info); /*add authentication info to LinphoneCore*/
+static void authentication_requested(LinphoneCore *lc, LinphoneAuthInfo *auth_info, LinphoneAuthMethod method) {
+	linphone_auth_info_set_passwd(auth_info, test_password);
+	linphone_core_add_auth_info(lc, auth_info); /*add authentication info to LinphoneCore*/
 }
 
-
-
 static LinphoneCoreManager* create_lcm_with_auth(unsigned int with_auth) {
-	LinphoneCoreManager* lcm=linphone_core_manager_new(NULL);
+	LinphoneCoreManager* lcm = linphone_core_manager_new(NULL);
 
 	if (with_auth) {
-		LinphoneCoreVTable* vtable = linphone_core_v_table_new();
-		vtable->auth_info_requested=auth_info_requested;
-		linphone_core_add_listener(lcm->lc,vtable);
+		LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+		linphone_core_cbs_set_authentication_requested(cbs, authentication_requested);
+		linphone_core_add_callbacks(lcm->lc, cbs);
+		linphone_core_cbs_unref(cbs);
 	}
 
 	/*to allow testing with 127.0.0.1*/
-	linphone_core_set_network_reachable(lcm->lc,TRUE);
+	linphone_core_set_network_reachable(lcm->lc, TRUE);
 	return lcm;
 }
 
@@ -65,7 +62,6 @@ void registration_state_changed(struct _LinphoneCore *lc, LinphoneProxyConfig *c
 		default:
 			BC_FAIL("unexpected event");break;
 		}
-
 }
 
 static void register_with_refresh_base_3(LinphoneCore* lc
@@ -73,7 +69,7 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 											,const char* domain
 											,const char* route
 											,bool_t late_auth_info
-											,LCSipTransports transport
+											,LinphoneTransports *transport
 											,LinphoneRegistrationState expected_final_state) {
 	int retry=0;
 	char* addr;
@@ -88,7 +84,7 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 
 	counters = get_stats(lc);
 	reset_counters(counters);
-	linphone_core_set_sip_transports(lc,&transport);
+	linphone_core_set_transports(lc, transport);
 
 	proxy_cfg = linphone_proxy_config_new();
 
@@ -106,7 +102,7 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 	} else {
 		linphone_proxy_config_set_server_addr(proxy_cfg,server_addr);
 	}
-	linphone_address_destroy(from);
+	linphone_address_unref(from);
 
 	linphone_core_add_proxy_config(lc,proxy_cfg);
 	linphone_core_set_default_proxy(lc,proxy_cfg);
@@ -119,6 +115,7 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 				BC_ASSERT_EQUAL(linphone_proxy_config_get_error(proxy_cfg),LinphoneReasonUnauthorized, int, "%d");
 				info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
 				linphone_core_add_auth_info(lc,info); /*add authentication info to LinphoneCore*/
+				linphone_auth_info_unref(info);
 			}
 		}
 		if (linphone_proxy_config_get_error(proxy_cfg) == LinphoneReasonBadCredentials
@@ -136,7 +133,7 @@ static void register_with_refresh_base_3(LinphoneCore* lc
 	} else
 		/*checking to be done outside this functions*/
 	BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationCleared,0, int, "%d");
-	linphone_proxy_config_destroy(proxy_cfg);
+	linphone_proxy_config_unref(proxy_cfg);
 }
 
 static void register_with_refresh_base_2(LinphoneCore* lc
@@ -144,12 +141,17 @@ static void register_with_refresh_base_2(LinphoneCore* lc
 											,const char* domain
 											,const char* route
 											,bool_t late_auth_info
-											,LCSipTransports transport) {
+											,LinphoneTransports *transport) {
 	register_with_refresh_base_3(lc, refresh, domain, route, late_auth_info, transport,LinphoneRegistrationOk );
 }
 static void register_with_refresh_base(LinphoneCore* lc, bool_t refresh,const char* domain,const char* route) {
-	LCSipTransports transport = {5070,5070,0,5071};
+	LinphoneTransports *transport = linphone_transports_new();
+	linphone_transports_set_udp_port(transport, 5070);
+	linphone_transports_set_tcp_port(transport, 5070);
+	linphone_transports_set_tls_port(transport, 5071);
+	linphone_transports_set_dtls_port(transport, 0);
 	register_with_refresh_base_2(lc,refresh,domain,route,FALSE,transport);
+	linphone_transports_unref(transport);
 }
 
 static void register_with_refresh(LinphoneCoreManager* lcm, bool_t refresh,const char* domain,const char* route) {
@@ -167,7 +169,7 @@ static void register_with_refresh_with_send_error(void) {
 	char route[256];
 	sprintf(route,"sip:%s",test_route);
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
-
+	linphone_auth_info_unref(info);
 	register_with_refresh_base(lcm->lc,TRUE,auth_domain,route);
 	/*simultate a network error*/
 	sal_set_send_error(lcm->lc->sal, -1);
@@ -238,21 +240,18 @@ static void change_expires(void){
 	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 
 	linphone_proxy_config_edit(proxy_config);
-	reset_counters(counters); /*clear stats*/
 
 	/*nothing is supposed to arrive until done*/
 	BC_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationCleared,1,3000));
 
 	linphone_proxy_config_set_expires(proxy_config,3);
-
+	reset_counters(counters); /*clear stats*/
 	linphone_proxy_config_done(proxy_config);
 	BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,1));
 	/*wait 2s without receive refresh*/
 	BC_ASSERT_FALSE(wait_for_until(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,2,2000));
 	/* now, it should be ok*/
 	BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,2));
-
-
 	linphone_core_manager_destroy(lcm);
 }
 
@@ -287,10 +286,13 @@ static void simple_tcp_register(void){
 static void simple_tcp_register_compatibility_mode(void){
 	char route[256];
 	LinphoneCoreManager* lcm;
-	LCSipTransports transport = {0,5070,0,0};
+	LinphoneTransports *transport = NULL;
 	sprintf(route,"sip:%s",test_route);
 	lcm = create_lcm();
+	transport = linphone_transports_new();
+	linphone_transports_set_tcp_port(transport, 5070);
 	register_with_refresh_base_2(lcm->lc,FALSE,test_domain,route,FALSE,transport);
+	linphone_transports_unref(transport);
 	linphone_core_manager_destroy(lcm);
 }
 
@@ -312,6 +314,7 @@ static void simple_authenticated_register(void){
 	char route[256];
 	sprintf(route,"sip:%s",test_route);
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
+	linphone_auth_info_unref(info);
 	counters = &lcm->stat;
 	register_with_refresh(lcm,FALSE,auth_domain,route);
 	BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,0, int, "%d");
@@ -328,6 +331,7 @@ static void ha1_authenticated_register(void){
 	info=linphone_auth_info_new(test_username,NULL,NULL,ha1,auth_domain,NULL); /*create authentication structure from identity*/
 	sprintf(route,"sip:%s",test_route);
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
+	linphone_auth_info_unref(info);
 	counters = &lcm->stat;
 	register_with_refresh(lcm,FALSE,auth_domain,route);
 	BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,0, int, "%d");
@@ -336,7 +340,7 @@ static void ha1_authenticated_register(void){
 
 static void authenticated_register_with_no_initial_credentials(void){
 	LinphoneCoreManager *lcm;
-	LinphoneCoreVTable* vtable = linphone_core_v_table_new();
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
 	stats* counters;
 	char route[256];
 
@@ -344,8 +348,9 @@ static void authenticated_register_with_no_initial_credentials(void){
 
 	lcm = linphone_core_manager_new(NULL);
 
-	vtable->auth_info_requested=auth_info_requested;
-	linphone_core_add_listener(lcm->lc,vtable);
+	linphone_core_cbs_set_authentication_requested(cbs, authentication_requested);
+	linphone_core_add_callbacks(lcm->lc, cbs);
+	linphone_core_cbs_unref(cbs);
 
 	counters= get_stats(lcm->lc);
 	counters->number_of_auth_info_requested=0;
@@ -359,14 +364,19 @@ static void authenticated_register_with_late_credentials(void){
 	LinphoneCoreManager *lcm;
 	stats* counters;
 	char route[256];
-	LCSipTransports transport = {5070,5070,0,5071};
+	LinphoneTransports *transport = NULL;
 
 	sprintf(route,"sip:%s",test_route);
 
 	lcm =  linphone_core_manager_new(NULL);
+	transport = linphone_transports_new();
+	linphone_transports_set_udp_port(transport, 5070);
+	linphone_transports_set_tcp_port(transport, 5070);
+	linphone_transports_set_dtls_port(transport, 5071);
 
 	counters = get_stats(lcm->lc);
 	register_with_refresh_base_2(lcm->lc,FALSE,auth_domain,route,TRUE,transport);
+	linphone_transports_unref(transport);
 	BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,1, int, "%d");
 	linphone_core_manager_destroy(lcm);
 }
@@ -395,11 +405,11 @@ static void authenticated_register_with_provided_credentials(void){
 	linphone_proxy_config_set_expires(cfg,1);
 	linphone_proxy_config_set_route(cfg, test_route);
 	linphone_proxy_config_set_server_addr(cfg,test_route);
-	linphone_address_destroy(from);
+	linphone_address_unref(from);
 
 	ai = linphone_auth_info_new(test_username, NULL, test_password, NULL, NULL, NULL);
 	linphone_core_add_auth_info(lcm->lc, ai);
-
+	linphone_auth_info_unref(ai);
 	linphone_core_add_proxy_config(lcm->lc, cfg);
 
 	BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,1));
@@ -415,7 +425,7 @@ static void authenticated_register_with_provided_credentials(void){
 static void authenticated_register_with_wrong_late_credentials(void){
 	LinphoneCoreManager *lcm;
 	stats* counters;
-	LCSipTransports transport = {5070,5070,0,5071};
+	LinphoneTransports *transport = NULL;
 	char route[256];
 	const char* saved_test_passwd=test_password;
 	char* wrong_passwd="mot de pass tout pourri";
@@ -425,9 +435,15 @@ static void authenticated_register_with_wrong_late_credentials(void){
 	sprintf(route,"sip:%s",test_route);
 
 	lcm =  linphone_core_manager_new(NULL);
+	transport = linphone_transports_new();
+	linphone_transports_set_udp_port(transport, 5070);
+	linphone_transports_set_tcp_port(transport, 5070);
+	linphone_transports_set_tls_port(transport, 5071);
+	linphone_transports_set_dtls_port(transport, 0);
 
 	counters = get_stats(lcm->lc);
 	register_with_refresh_base_3(lcm->lc,FALSE,auth_domain,route,TRUE,transport,LinphoneRegistrationFailed);
+	linphone_transports_unref(transport);
 	BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,2, int, "%d");
 	BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,2, int, "%d");
 	BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress,2, int, "%d");
@@ -438,19 +454,25 @@ static void authenticated_register_with_wrong_late_credentials(void){
 
 static void authenticated_register_with_wrong_credentials_with_params_base(const char* user_agent,LinphoneCoreManager *lcm) {
 	stats* counters;
-	LCSipTransports transport = {5070,5070,0,5071};
+	LinphoneTransports *transport = linphone_transports_new();
 	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,"wrong passwd",NULL,auth_domain,NULL); /*create authentication structure from identity*/
 	char route[256];
 
 	sprintf(route,"sip:%s",test_route);
+	linphone_transports_set_udp_port(transport, 5070);
+	linphone_transports_set_tcp_port(transport, 5070);
+	linphone_transports_set_tls_port(transport, 5071);
+	linphone_transports_set_dtls_port(transport, 0);
 
 	sal_set_refresher_retry_after(lcm->lc->sal,500);
 	if (user_agent) {
 		linphone_core_set_user_agent(lcm->lc,user_agent,NULL);
 	}
 	linphone_core_add_auth_info(lcm->lc,info); /*add wrong authentication info to LinphoneCore*/
+	linphone_auth_info_unref(info);
 	counters = get_stats(lcm->lc);
 	register_with_refresh_base_3(lcm->lc,TRUE,auth_domain,route,FALSE,transport,LinphoneRegistrationFailed);
+	linphone_transports_unref(transport);
 	//BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,3, int, "%d"); register_with_refresh_base_3 does not alow to precisely check number of number_of_auth_info_requested
 	/*wait for retry*/
 	BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_auth_info_requested,4));
@@ -467,7 +489,7 @@ static void authenticated_register_with_wrong_credentials_with_params_base(const
 			BC_ASSERT_PTR_NOT_NULL(phrase);
 			if (phrase) BC_ASSERT_STRING_EQUAL(phrase,"Forbidden");
 			BC_ASSERT_EQUAL(linphone_error_info_get_protocol_code(ei),403, int, "%d");
-			BC_ASSERT_PTR_NULL(linphone_error_info_get_details(ei));
+			BC_ASSERT_PTR_NULL(linphone_error_info_get_warnings(ei));
 		}
 
 	}
@@ -505,7 +527,7 @@ static LinphoneCoreManager* configure_lcm(void) {
 	if (transport_supported(LinphoneTransportTls)) {
 		LinphoneCoreManager *lcm=linphone_core_manager_new2( "multi_account_rc", FALSE);
 		stats *counters=&lcm->stat;
-		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
+		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,(int)bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
 		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0, int, "%d");
 		return lcm;
 	}
@@ -554,34 +576,79 @@ static void transport_change(void){
 	LinphoneCore* lc;
 	int register_ok;
 	stats* counters ;
-	LCSipTransports sip_tr;
-	LCSipTransports sip_tr_orig;
+	LinphoneTransports *sip_tr;
+	LinphoneTransports *sip_tr_orig;
 	int number_of_udp_proxy=0;
 	int total_number_of_proxies;
 
 	lcm=configure_lcm();
 	if (lcm) {
-		memset(&sip_tr,0,sizeof(sip_tr));
 		lc=lcm->lc;
+		sip_tr = linphone_transports_new();
 		counters = get_stats(lc);
 		register_ok=counters->number_of_LinphoneRegistrationOk;
 
 		number_of_udp_proxy=get_number_of_udp_proxy(lc);
-		total_number_of_proxies=bctbx_list_size(linphone_core_get_proxy_config_list(lc));
-		linphone_core_get_sip_transports(lc,&sip_tr_orig);
+		total_number_of_proxies=(int)bctbx_list_size(linphone_core_get_proxy_config_list(lc));
+		sip_tr_orig = linphone_core_get_transports(lc);
 
-		sip_tr.udp_port=sip_tr_orig.udp_port;
+		sip_tr->udp_port = sip_tr_orig->udp_port;
 
 		/*keep only udp*/
-		linphone_core_set_sip_transports(lc,&sip_tr);
+		linphone_core_set_transports(lc, sip_tr);
 		BC_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationOk,register_ok+number_of_udp_proxy));
 
 		BC_ASSERT_TRUE(wait_for(lc,lc,&counters->number_of_LinphoneRegistrationFailed,total_number_of_proxies-number_of_udp_proxy));
 
+		linphone_transports_unref(sip_tr);
+		linphone_transports_unref(sip_tr_orig);
 		linphone_core_manager_destroy(lcm);
 	}
 }
 
+static void transport_dont_bind(void){
+	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_tcp_rc");
+	stats* counters = &pauline->stat;
+	LinphoneTransports *tr = linphone_transports_new();
+	linphone_transports_set_tcp_port(tr, LC_SIP_TRANSPORT_DONTBIND);
+	linphone_transports_set_tls_port(tr, LC_SIP_TRANSPORT_DONTBIND);
+	
+	linphone_core_set_transports(pauline->lc, tr);
+	BC_ASSERT_TRUE(wait_for_until(pauline->lc,pauline->lc,&counters->number_of_LinphoneRegistrationOk,2,15000));
+	linphone_transports_unref(tr);
+	tr = linphone_core_get_transports_used(pauline->lc);
+	BC_ASSERT_EQUAL(tr->udp_port, 0, int, "%i");
+	BC_ASSERT_EQUAL(tr->tcp_port, LC_SIP_TRANSPORT_DONTBIND, int, "%i");
+	BC_ASSERT_EQUAL(tr->tls_port, LC_SIP_TRANSPORT_DONTBIND, int, "%i");
+	linphone_transports_unref(tr);
+	linphone_core_manager_destroy(pauline);
+}
+
+static void transport_busy(void){
+	LinphoneCoreManager *pauline = linphone_core_manager_new("pauline_tcp_rc");
+	LCSipTransports tr;
+	
+	memset(&tr, 0, sizeof(tr));
+	tr.udp_port = 5070;
+	tr.tcp_port = 5070;
+	tr.tls_port = 5071;
+	
+	linphone_core_set_sip_transports(pauline->lc, &tr);
+	
+	{
+		LinphoneCoreManager *marie = linphone_core_manager_new("marie_rc");
+		linphone_core_set_sip_transports(marie->lc, &tr);
+		memset(&tr, 0, sizeof(tr));
+		linphone_core_get_sip_transports_used(pauline->lc, &tr);
+		/*BC_ASSERT_EQUAL(tr.udp_port, 0, int, "%i");
+		BC_ASSERT_EQUAL(tr.tcp_port, 0, int, "%i");
+		BC_ASSERT_EQUAL(tr.tls_port, 0, int, "%i");*/
+		linphone_core_manager_destroy(marie);
+	}
+	
+	linphone_core_manager_destroy(pauline);
+}
+
 static void proxy_transport_change(void){
 	LinphoneCoreManager* lcm = create_lcm();
 	stats* counters = &lcm->stat;
@@ -590,7 +657,7 @@ static void proxy_transport_change(void){
 	char* addr_as_string;
 	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
-
+	linphone_auth_info_unref(info);
 	register_with_refresh_base(lcm->lc,FALSE,auth_domain,NULL);
 
 	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
@@ -613,7 +680,7 @@ static void proxy_transport_change(void){
 	/*as we change p[roxy server destination, we should'nt be notified about the clear*/
 	BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationCleared,0, int, "%d");
 	ms_free(addr_as_string);
-	linphone_address_destroy(addr);
+	linphone_address_unref(addr);
 	linphone_core_manager_destroy(lcm);
 
 }
@@ -623,12 +690,17 @@ static void proxy_transport_change_with_wrong_port(void) {
 	LinphoneProxyConfig* proxy_config;
 	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
 	char route[256];
-	LCSipTransports transport= {LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM};
+	LinphoneTransports *transport= linphone_transports_new();
 	sprintf(route,"sip:%s",test_route);
+	linphone_transports_set_udp_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_tcp_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_tls_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_dtls_port(transport, LC_SIP_TRANSPORT_RANDOM);
 
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
-
+	linphone_auth_info_unref(info);
 	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
+	linphone_transports_unref(transport);
 
 	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 	linphone_proxy_config_edit(proxy_config);
@@ -654,12 +726,17 @@ static void proxy_transport_change_with_wrong_port_givin_up(void) {
 	LinphoneProxyConfig* proxy_config;
 	LinphoneAuthInfo *info=linphone_auth_info_new(test_username,NULL,test_password,NULL,auth_domain,NULL); /*create authentication structure from identity*/
 	char route[256];
-	LCSipTransports transport= {LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM,LC_SIP_TRANSPORT_RANDOM};
+	LinphoneTransports *transport = linphone_transports_new();
 	sprintf(route,"sip:%s",test_route);
+	linphone_transports_set_udp_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_tcp_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_tls_port(transport, LC_SIP_TRANSPORT_RANDOM);
+	linphone_transports_set_dtls_port(transport, LC_SIP_TRANSPORT_RANDOM);
 
 	linphone_core_add_auth_info(lcm->lc,info); /*add authentication info to LinphoneCore*/
-
+	linphone_auth_info_unref(info);
 	register_with_refresh_base_3(lcm->lc, FALSE, auth_domain, "sip2.linphone.org:5987", 0,transport,LinphoneRegistrationProgress);
+	linphone_transports_unref(transport);
 
 	proxy_config = linphone_core_get_default_proxy_config(lcm->lc);
 	linphone_proxy_config_edit(proxy_config);
@@ -739,7 +816,7 @@ static void io_recv_error_late_recovery(void){
 		lc=lcm->lc;
 		sal_set_refresher_retry_after(lc->sal,1000);
 		counters=&lcm->stat;
-		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
+		BC_ASSERT_TRUE(wait_for(lcm->lc,lcm->lc,&counters->number_of_LinphoneRegistrationOk,(int)bctbx_list_size(linphone_core_get_proxy_config_list(lcm->lc))));
 
 
 		counters = get_stats(lc);
@@ -791,7 +868,7 @@ static void io_recv_error_without_active_register(void){
 
 		/*nothing should happen because no active registration*/
 		wait_for_until(lc,lc, &dummy, 1, 3000);
-		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress, bctbx_list_size(linphone_core_get_proxy_config_list(lc)), int, "%d");
+		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationProgress, (int)bctbx_list_size(linphone_core_get_proxy_config_list(lc)), int, "%d");
 
 		BC_ASSERT_EQUAL(counters->number_of_LinphoneRegistrationFailed,0,int,"%d");
 
@@ -816,14 +893,62 @@ static void tls_certificate_failure(void){
 		linphone_core_set_root_ca(lcm->lc,NULL); /*no root ca*/
 		linphone_core_refresh_registers(lcm->lc);
 		BC_ASSERT_TRUE(wait_for(lc,lc,&lcm->stat.number_of_LinphoneRegistrationFailed,2));
-		ms_free(rootcapath);
+		bc_free(rootcapath);
 		rootcapath = bc_tester_res("certificates/cn/cafile.pem"); /*good root ca*/
 		linphone_core_set_root_ca(lcm->lc,rootcapath);
 		linphone_core_refresh_registers(lcm->lc);
 		BC_ASSERT_TRUE(wait_for(lc,lc,&lcm->stat.number_of_LinphoneRegistrationOk,1));
 		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationFailed,2, int, "%d");
 		linphone_core_manager_destroy(lcm);
-		ms_free(rootcapath);
+		bc_free(rootcapath);
+	}
+}
+
+char *read_file(const char *path) {
+	long  numbytes = 0;
+	size_t readbytes;
+	char *buffer = NULL;
+	FILE *infile = fopen(path, "rb");
+	
+	BC_ASSERT_PTR_NOT_NULL(infile);
+	if (infile) {
+		fseek(infile, 0L, SEEK_END);
+		numbytes = ftell(infile);
+		fseek(infile, 0L, SEEK_SET);
+		buffer = (char*)ms_malloc((numbytes + 1) * sizeof(char));
+		readbytes = fread(buffer, sizeof(char), numbytes, infile);
+		fclose(infile);
+		buffer[readbytes] = '\0';
+	}
+	return buffer;
+}
+
+static void tls_certificate_data(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager* lcm;
+		LinphoneCore *lc;
+		char *rootcapath = bc_tester_res("certificates/cn/agent.pem"); /*bad root ca*/
+		char *data = read_file(rootcapath);
+
+		lcm = linphone_core_manager_new2("pauline_rc",FALSE);
+		lc = lcm->lc;
+		linphone_core_set_root_ca_data(lcm->lc, data);
+		linphone_core_set_network_reachable(lc, TRUE);
+		BC_ASSERT_TRUE(wait_for(lcm->lc, lcm->lc, &lcm->stat.number_of_LinphoneRegistrationFailed, 1));
+		linphone_core_set_root_ca_data(lcm->lc, NULL); /*no root ca*/
+		linphone_core_refresh_registers(lcm->lc);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &lcm->stat.number_of_LinphoneRegistrationFailed, 2));
+		bc_free(rootcapath);
+		ms_free(data);
+		rootcapath = bc_tester_res("certificates/cn/cafile.pem"); /*good root ca*/
+		data = read_file(rootcapath);
+		linphone_core_set_root_ca_data(lcm->lc, data);
+		linphone_core_refresh_registers(lcm->lc);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &lcm->stat.number_of_LinphoneRegistrationOk, 1));
+		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationFailed, 2, int, "%d");
+		linphone_core_manager_destroy(lcm);
+		bc_free(rootcapath);
+		ms_free(data);
 	}
 }
 
@@ -846,8 +971,8 @@ static void tls_with_non_tls_server(void){
 				,(linphone_address_get_port(addr)>0?linphone_address_get_port(addr):5060));
 		linphone_proxy_config_set_server_addr(proxy_cfg,tmp);
 		linphone_proxy_config_done(proxy_cfg);
-		linphone_address_destroy(addr);
-		BC_ASSERT_TRUE(wait_for_until(lc,lc,&lcm->stat.number_of_LinphoneRegistrationFailed,1,5000));
+		linphone_address_unref(addr);
+		BC_ASSERT_TRUE(wait_for_until(lc,lc,&lcm->stat.number_of_LinphoneRegistrationFailed,1,10000));
 		linphone_core_manager_destroy(lcm);
 	}
 }
@@ -865,7 +990,7 @@ static void tls_alt_name_register(void){
 		BC_ASSERT_TRUE(wait_for(lc,lc,&lcm->stat.number_of_LinphoneRegistrationOk,1));
 		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationFailed,0, int, "%d");
 		linphone_core_manager_destroy(lcm);
-		ms_free(rootcapath);
+		bc_free(rootcapath);
 	}
 }
 
@@ -882,23 +1007,180 @@ static void tls_wildcard_register(void){
 		BC_ASSERT_TRUE(wait_for(lc,lc,&lcm->stat.number_of_LinphoneRegistrationOk,2));
 		BC_ASSERT_EQUAL(lcm->stat.number_of_LinphoneRegistrationFailed,0, int, "%d");
 		linphone_core_manager_destroy(lcm);
-		ms_free(rootcapath);
+		bc_free(rootcapath);
 	}
 }
 
 static void redirect(void){
 	char route[256];
 	LinphoneCoreManager* lcm;
-	LCSipTransports transport = {-1,0,0,0};
+	LinphoneTransports *transport = NULL;
 	sprintf(route,"sip:%s:5064",test_route);
 	lcm = create_lcm();
 	if (lcm) {
+		transport = linphone_transports_new();
+		linphone_transports_set_udp_port(transport, -1);
 		linphone_core_set_user_agent(lcm->lc,"redirect",NULL);
 		register_with_refresh_base_2(lcm->lc,FALSE,test_domain,route,FALSE,transport);
+		linphone_transports_unref(transport);
 		linphone_core_manager_destroy(lcm);
 	}
 }
 
+static void tls_auth_global_client_cert(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *manager = ms_new0(LinphoneCoreManager, 1);
+		LpConfig *lpc = NULL;
+		char *cert_path = bc_tester_res("certificates/client/cert.pem");
+		char *key_path = bc_tester_res("certificates/client/key.pem");
+		linphone_core_manager_init(manager, "pauline_tls_client_rc", NULL);
+		lpc = manager->lc->config;
+		lp_config_set_string(lpc, "sip", "client_cert_chain", cert_path);
+		lp_config_set_string(lpc, "sip", "client_cert_key", key_path);
+		linphone_core_manager_start(manager, TRUE);
+		linphone_core_manager_destroy(manager);
+		bc_free(cert_path);
+		bc_free(key_path);
+	}
+}
+
+static void tls_auth_global_client_cert_api(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *pauline = linphone_core_manager_new2("pauline_tls_client_rc", FALSE);
+		char *cert_path = bc_tester_res("certificates/client/cert.pem");
+		char *key_path = bc_tester_res("certificates/client/key.pem");
+		char *cert = read_file(cert_path);
+		char *key = read_file(key_path);
+		LinphoneCore *lc = pauline->lc;
+		linphone_core_set_tls_cert(lc, cert);
+		linphone_core_set_tls_key(lc, key);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &pauline->stat.number_of_LinphoneRegistrationOk, 1));
+		linphone_core_manager_destroy(pauline);
+		ms_free(cert);
+		ms_free(key);
+		bc_free(cert_path);
+		bc_free(key_path);
+	}
+}
+
+static void tls_auth_global_client_cert_api_path(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *pauline = linphone_core_manager_new2("pauline_tls_client_rc", FALSE);
+		char *cert = bc_tester_res("certificates/client/cert.pem");
+		char *key = bc_tester_res("certificates/client/key.pem");
+		LinphoneCore *lc = pauline->lc;
+		linphone_core_set_tls_cert_path(lc, cert);
+		linphone_core_set_tls_key_path(lc, key);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &pauline->stat.number_of_LinphoneRegistrationOk, 1));
+		linphone_core_manager_destroy(pauline);
+		bc_free(cert);
+		bc_free(key);
+	}
+}
+
+static void tls_auth_info_client_cert_api(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *pauline = linphone_core_manager_new2("pauline_tls_client_rc", FALSE);
+		char *cert_path = bc_tester_res("certificates/client/cert.pem");
+		char *key_path = bc_tester_res("certificates/client/key.pem");
+		char *cert = read_file(cert_path);
+		char *key = read_file(key_path);
+		LinphoneCore *lc = pauline->lc;
+		LinphoneAuthInfo *authInfo = (LinphoneAuthInfo *)lc->auth_info->data;
+		linphone_auth_info_set_tls_cert(authInfo, cert);
+		linphone_auth_info_set_tls_key(authInfo, key);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &pauline->stat.number_of_LinphoneRegistrationOk, 1));
+		linphone_core_manager_destroy(pauline);
+		ms_free(cert);
+		ms_free(key);
+		bc_free(cert_path);
+		bc_free(key_path);
+	}
+}
+
+static void tls_auth_info_client_cert_api_path(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *pauline = linphone_core_manager_new2("pauline_tls_client_rc", FALSE);
+		char *cert = bc_tester_res("certificates/client/cert.pem");
+		char *key = bc_tester_res("certificates/client/key.pem");
+		LinphoneCore *lc = pauline->lc;
+		LinphoneAuthInfo *authInfo = (LinphoneAuthInfo *)lc->auth_info->data;
+		linphone_auth_info_set_tls_cert_path(authInfo, cert);
+		linphone_auth_info_set_tls_key_path(authInfo, key);
+		BC_ASSERT_TRUE(wait_for(lc, lc, &pauline->stat.number_of_LinphoneRegistrationOk, 1));
+		linphone_core_manager_destroy(pauline);
+		bc_free(cert);
+		bc_free(key);
+	}
+}
+
+static void authentication_requested_2(LinphoneCore *lc, LinphoneAuthInfo *auth_info, LinphoneAuthMethod method) {
+	char *cert = bc_tester_res("certificates/client/cert.pem");
+	char *key = bc_tester_res("certificates/client/key.pem");
+	BC_ASSERT_EQUAL(method, LinphoneAuthTls, int, "%i");
+	linphone_auth_info_set_tls_cert_path(auth_info, cert);
+	linphone_auth_info_set_tls_key_path(auth_info, key);
+	linphone_core_add_auth_info(lc, auth_info);
+	bc_free(cert);
+	bc_free(key);
+}
+
+static void tls_auth_info_client_cert_cb(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *lcm;
+		LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+		stats* counters;
+
+		lcm = linphone_core_manager_new(NULL);
+
+		linphone_core_cbs_set_authentication_requested(cbs, authentication_requested_2);
+		linphone_core_add_callbacks(lcm->lc, cbs);
+		linphone_core_cbs_unref(cbs);
+
+		counters= get_stats(lcm->lc);
+		counters->number_of_auth_info_requested=0;
+		register_with_refresh(lcm,FALSE,auth_domain,"sip2.linphone.org:5063;transport=tls");
+		BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,1, int, "%d");
+		linphone_core_manager_destroy(lcm);
+	}
+}
+
+static void authentication_requested_3(LinphoneCore *lc, LinphoneAuthInfo *auth_info, LinphoneAuthMethod method) {
+	char *cert_path = bc_tester_res("certificates/client/cert.pem");
+	char *key_path = bc_tester_res("certificates/client/key.pem");
+	char *cert = read_file(cert_path);
+	char *key = read_file(key_path);
+	BC_ASSERT_EQUAL(method, LinphoneAuthTls, int, "%i");
+	linphone_auth_info_set_tls_cert(auth_info, cert);
+	linphone_auth_info_set_tls_key(auth_info, key);
+	linphone_core_add_auth_info(lc, auth_info);
+	ms_free(cert);
+	ms_free(key);
+	bc_free(cert_path);
+	bc_free(key_path);
+}
+
+static void tls_auth_info_client_cert_cb_2(void) {
+	if (transport_supported(LinphoneTransportTls)) {
+		LinphoneCoreManager *lcm;
+		LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+		stats* counters;
+
+		lcm = linphone_core_manager_new(NULL);
+
+		linphone_core_cbs_set_authentication_requested(cbs, authentication_requested_3);
+		linphone_core_add_callbacks(lcm->lc, cbs);
+		linphone_core_cbs_unref(cbs);
+
+		counters= get_stats(lcm->lc);
+		counters->number_of_auth_info_requested=0;
+		register_with_refresh(lcm,FALSE,auth_domain,"sip2.linphone.org:5063;transport=tls");
+		BC_ASSERT_EQUAL(counters->number_of_auth_info_requested,1, int, "%d");
+		linphone_core_manager_destroy(lcm);
+	}
+}
+
+
 test_t register_tests[] = {
 	TEST_NO_TAG("Simple register", simple_register),
 	TEST_NO_TAG("Simple register unregister", simple_unregister),
@@ -909,6 +1191,7 @@ test_t register_tests[] = {
 	TEST_NO_TAG("TLS register with alt. name certificate", tls_alt_name_register),
 	TEST_NO_TAG("TLS register with wildcard certificate", tls_wildcard_register),
 	TEST_NO_TAG("TLS certificate not verified",tls_certificate_failure),
+	TEST_NO_TAG("TLS certificate given by string instead of file",tls_certificate_data),
 	TEST_NO_TAG("TLS with non tls server",tls_with_non_tls_server),
 	TEST_NO_TAG("Simple authenticated register", simple_authenticated_register),
 	TEST_NO_TAG("Ha1 authenticated register", ha1_authenticated_register),
@@ -924,6 +1207,8 @@ test_t register_tests[] = {
 	TEST_NO_TAG("Register with refresh and send error", register_with_refresh_with_send_error),
 	TEST_NO_TAG("Multi account", multiple_proxy),
 	TEST_NO_TAG("Transport changes", transport_change),
+	TEST_NO_TAG("Transport configured with dontbind option", transport_dont_bind),
+	TEST_NO_TAG("Transport busy", transport_busy),
 	TEST_NO_TAG("Proxy transport changes", proxy_transport_change),
 	TEST_NO_TAG("Proxy transport changes with wrong address at first", proxy_transport_change_with_wrong_port),
 	TEST_NO_TAG("Proxy transport changes with wrong address, giving up",proxy_transport_change_with_wrong_port_givin_up),
@@ -933,7 +1218,14 @@ test_t register_tests[] = {
 	TEST_NO_TAG("Io recv error with recovery", io_recv_error_retry_immediatly),
 	TEST_NO_TAG("Io recv error with late recovery", io_recv_error_late_recovery),
 	TEST_NO_TAG("Io recv error without active registration", io_recv_error_without_active_register),
-	TEST_NO_TAG("Simple redirect", redirect)
+	TEST_NO_TAG("Simple redirect", redirect),
+	TEST_NO_TAG("Global TLS client certificate authentication", tls_auth_global_client_cert),
+	TEST_NO_TAG("Global TLS client certificate authentication using API", tls_auth_global_client_cert_api),
+	TEST_NO_TAG("Global TLS client certificate authentication using API 2", tls_auth_global_client_cert_api_path),
+	TEST_NO_TAG("AuthInfo TLS client certificate authentication using API", tls_auth_info_client_cert_api),
+	TEST_NO_TAG("AuthInfo TLS client certificate authentication using API 2", tls_auth_info_client_cert_api_path),
+	TEST_NO_TAG("AuthInfo TLS client certificate authentication in callback", tls_auth_info_client_cert_cb),
+	TEST_NO_TAG("AuthInfo TLS client certificate authentication in callback 2", tls_auth_info_client_cert_cb_2),
 };
 
 test_suite_t register_test_suite = {"Register", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
