@@ -17,7 +17,7 @@
 */
 
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
@@ -108,13 +108,13 @@ static void remote_provisioning_file(void) {
 #if TARGET_OS_IPHONE
 	ms_message("Skipping remote provisioning from file on iOS");
 	return;
-#elif defined(ANDROID)
+#elif defined(__ANDROID__)
 	marie = linphone_core_manager_new2("marie_remote_localfile_android_rc", FALSE);
 #elif defined(LINPHONE_WINDOWS_UNIVERSAL)
 	marie = linphone_core_manager_new2("marie_remote_localfile_win10_rc", FALSE);
 #else
 	marie = ms_new0(LinphoneCoreManager, 1);
-	linphone_core_manager_init(marie, "marie_remote_localfile_rc");
+	linphone_core_manager_init(marie, "marie_remote_localfile_rc",NULL);
 	// fix relative path to absolute path
 	{
 		char* path = bc_tester_res("rcfiles/marie_remote_localfile2_rc");
