@@ -17,41 +17,38 @@
 */
 
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "liblinphone_tester.h"
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 #include "private.h"
 
 static void linphone_version_test(void){
 	const char *version=linphone_core_get_version();
 	/*make sure the git version is always included in the version number*/
+	BC_ASSERT_PTR_NOT_NULL(version);
 	BC_ASSERT_PTR_NULL(strstr(version,"unknown"));
 }
 
 static void core_init_test(void) {
-	LinphoneCoreVTable v_table;
 	LinphoneCore* lc;
-	memset (&v_table,0,sizeof(v_table));
-	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
+	lc = linphone_factory_create_core(linphone_factory_get(),NULL,NULL,NULL);
 	/* until we have good certificates on our test server... */
 	linphone_core_verify_server_certificates(lc,FALSE);
 	if (BC_ASSERT_PTR_NOT_NULL(lc)) {
-		linphone_core_destroy(lc);
+		linphone_core_unref(lc);
 	}
 }
 
 static void linphone_address_test(void) {
-	linphone_address_destroy(create_linphone_address(NULL));
+	linphone_address_unref(create_linphone_address(NULL));
 	BC_ASSERT_PTR_NULL(linphone_address_new("sip:@sip.linphone.org"));
 
 }
 
 static void core_sip_transport_test(void) {
-	LinphoneCoreVTable v_table;
 	LinphoneCore* lc;
 	LCSipTransports tr;
-	memset (&v_table,0,sizeof(v_table));
-	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
+	lc = linphone_factory_create_core(linphone_factory_get(),NULL,NULL,NULL);
 	if (!BC_ASSERT_PTR_NOT_NULL(lc)) return;
 	linphone_core_get_sip_transports(lc,&tr);
 	BC_ASSERT_EQUAL(tr.udp_port,5060, int, "%d"); /*default config*/
@@ -71,19 +68,16 @@ static void core_sip_transport_test(void) {
 	BC_ASSERT_EQUAL(lp_config_get_int(linphone_core_get_config(lc),"sip","sip_tcp_port",-2),LC_SIP_TRANSPORT_RANDOM, int, "%d");
 	BC_ASSERT_EQUAL(lp_config_get_int(linphone_core_get_config(lc),"sip","sip_tls_port",-2),LC_SIP_TRANSPORT_RANDOM, int, "%d");
 
-	linphone_core_destroy(lc);
+	linphone_core_unref(lc);
 }
 
-static void linphone_interpret_url_test(void)
-{
-	LinphoneCoreVTable v_table;
+static void linphone_interpret_url_test(void) {
 	LinphoneCore* lc;
 	const char* sips_address = "sips:margaux@sip.linphone.org";
 	LinphoneAddress* address;
 	LinphoneProxyConfig *proxy_config;
 	char *tmp;
-	memset ( &v_table,0,sizeof ( v_table ) );
-	lc = linphone_core_new ( &v_table,NULL,NULL,NULL );
+	lc = linphone_factory_create_core(linphone_factory_get(),NULL,NULL,NULL);
 	if (!BC_ASSERT_PTR_NOT_NULL( lc )) return;
 
 	proxy_config =linphone_core_create_proxy_config(lc);
@@ -99,14 +93,14 @@ static void linphone_interpret_url_test(void)
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sips");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "margaux");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
-	linphone_address_destroy(address);
+	linphone_address_unref(address);
 
 	address = linphone_core_interpret_url(lc,"23");
 	BC_ASSERT_PTR_NOT_NULL(address);
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sip");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "23");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
-	linphone_address_destroy(address);
+	linphone_address_unref(address);
 
 	address = linphone_core_interpret_url(lc,"#24");
 	BC_ASSERT_PTR_NOT_NULL(address);
@@ -115,16 +109,16 @@ static void linphone_interpret_url_test(void)
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
 	tmp = linphone_address_as_string(address);
 	BC_ASSERT_TRUE(strcmp (tmp,"sip:%2324@sip.linphone.org") == 0);
-	linphone_address_destroy(address);
+	linphone_address_unref(address);
 
 	address = linphone_core_interpret_url(lc,tmp);
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_scheme(address), "sip");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_username(address), "#24");
 	BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(address), "sip.linphone.org");
-	linphone_address_destroy(address);
+	linphone_address_unref(address);
 	ms_free(tmp);
 
-	linphone_core_destroy (lc);
+	linphone_core_unref(lc);
 }
 
 static void linphone_lpconfig_from_buffer(void){
@@ -186,6 +180,12 @@ static void linphone_lpconfig_from_xml_zerolen_value(void){
 
 	LinphoneCoreManager* mgr = linphone_core_manager_new2("empty_rc",FALSE);
 
+	/* BUG
+	 * This test makes a provisionning by xml outside of the Configuring state of the LinphoneCore.
+	 * It is leaking memory because the config is litterally erased and rewritten by the invocation
+	 * of the private function linphone_remote_provisioning_load_file .
+	 */ 
+	
 	BC_ASSERT_EQUAL(linphone_remote_provisioning_load_file(mgr->lc, xml_path), 0, int, "%d");
 
 	conf = mgr->lc->config;
@@ -218,12 +218,12 @@ void linphone_proxy_config_address_equal_test(void) {
 	BC_ASSERT_EQUAL(linphone_proxy_config_address_equal(c,f), LinphoneProxyConfigAddressDifferent, int, "%d");
 	BC_ASSERT_EQUAL(linphone_proxy_config_address_equal(e,f), LinphoneProxyConfigAddressWeakEqual, int, "%d");
 
-	linphone_address_destroy(a);
-	linphone_address_destroy(b);
-	linphone_address_destroy(c);
-	linphone_address_destroy(d);
-	linphone_address_destroy(e);
-	linphone_address_destroy(f);
+	linphone_address_unref(a);
+	linphone_address_unref(b);
+	linphone_address_unref(c);
+	linphone_address_unref(d);
+	linphone_address_unref(e);
+	linphone_address_unref(f);
 }
 
 void linphone_proxy_config_is_server_config_changed_test(void) {
@@ -269,13 +269,11 @@ void linphone_proxy_config_is_server_config_changed_test(void) {
 }
 
 static void chat_room_test(void) {
-	LinphoneCoreVTable v_table;
 	LinphoneCore* lc;
-	memset (&v_table,0,sizeof(v_table));
-	lc = linphone_core_new(&v_table,NULL,NULL,NULL);
+	lc = linphone_factory_create_core(linphone_factory_get(),NULL,NULL,NULL);
 	if (!BC_ASSERT_PTR_NOT_NULL(lc)) return;
 	BC_ASSERT_PTR_NOT_NULL(linphone_core_get_chat_room_from_uri(lc,"sip:toto@titi.com"));
-	linphone_core_destroy(lc);
+	linphone_core_unref(lc);
 }
 
 static void devices_reload_test(void) {
@@ -376,6 +374,25 @@ static void codec_setup(void){
 	
 }
 
+static void custom_tones_setup(void){
+	LinphoneCoreManager *mgr = linphone_core_manager_new2("empty_rc", FALSE);
+	const char *tone;
+	
+	linphone_core_set_tone(mgr->lc, LinphoneToneCallOnHold, "callonhold.wav");
+	tone = linphone_core_get_tone_file(mgr->lc, LinphoneToneCallOnHold);
+	BC_ASSERT_PTR_NOT_NULL(tone);
+	if (tone){
+		BC_ASSERT_STRING_EQUAL(tone, "callonhold.wav");
+	}
+	linphone_core_set_tone(mgr->lc, LinphoneToneCallOnHold, "callonhold2.wav");
+	tone = linphone_core_get_tone_file(mgr->lc, LinphoneToneCallOnHold);
+	BC_ASSERT_PTR_NOT_NULL(tone);
+	if (tone){
+		BC_ASSERT_STRING_EQUAL(tone, "callonhold2.wav");
+	}
+	linphone_core_manager_destroy(mgr);
+}
+
 test_t setup_tests[] = {
 	TEST_NO_TAG("Version check", linphone_version_test),
 	TEST_NO_TAG("Linphone Address", linphone_address_test),
@@ -387,11 +404,12 @@ test_t setup_tests[] = {
 	TEST_NO_TAG("LPConfig from buffer", linphone_lpconfig_from_buffer),
 	TEST_NO_TAG("LPConfig zero_len value from buffer", linphone_lpconfig_from_buffer_zerolen_value),
 	TEST_NO_TAG("LPConfig zero_len value from file", linphone_lpconfig_from_file_zerolen_value),
-	TEST_NO_TAG("LPConfig zero_len value from XML", linphone_lpconfig_from_xml_zerolen_value),
+	TEST_ONE_TAG("LPConfig zero_len value from XML", linphone_lpconfig_from_xml_zerolen_value, "LeaksMemory"),
 	TEST_NO_TAG("Chat room", chat_room_test),
 	TEST_NO_TAG("Devices reload", devices_reload_test),
 	TEST_NO_TAG("Codec usability", codec_usability_test),
-	TEST_NO_TAG("Codec setup", codec_setup)
+	TEST_NO_TAG("Codec setup", codec_setup),
+	TEST_NO_TAG("Custom tones setup", custom_tones_setup) 
 };
 
 test_suite_t setup_test_suite = {"Setup", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
