@@ -16,7 +16,7 @@
 	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 #include "mediastreamer2/stun.h"
@@ -47,11 +47,19 @@ static void linphone_stun_test_encode(void)
 
 static void linphone_stun_test_grab_ip(void)
 {
+	
 	LinphoneCoreManager* lc_stun = linphone_core_manager_new2("stun_rc", FALSE);
 	LinphoneCall dummy_call;
 	int ping_time;
 	int tmp = 0;
 
+	/*this test verifies the very basic STUN support of liblinphone, which is deprecated.
+	 * It works only in IPv4 mode and there is no plan to make it work over ipv6.*/
+	if (liblinphone_tester_ipv4_available()){
+		goto end;
+	}
+	linphone_core_enable_ipv6(lc_stun->lc, FALSE);
+	
 	memset(&dummy_call, 0, sizeof(LinphoneCall));
 	dummy_call.main_audio_stream_index = 0;
 	dummy_call.main_video_stream_index = 1;
@@ -85,6 +93,7 @@ static void linphone_stun_test_grab_ip(void)
 #endif
 	ms_message("STUN test result: local text port maps to %s:%i", dummy_call.tc.addr, dummy_call.tc.port);
 
+end:
 	linphone_core_manager_destroy(lc_stun);
 }
 
@@ -105,7 +114,7 @@ static void configure_nat_policy(LinphoneCore *lc, bool_t turn_enabled) {
 	linphone_core_set_nat_policy(lc, nat_policy);
 	linphone_core_add_auth_info(lc, auth_info);
 	linphone_nat_policy_unref(nat_policy);
-	linphone_auth_info_destroy(auth_info);
+	linphone_auth_info_unref(auth_info);
 }
 
 static void check_turn_context_statistics(MSTurnContext *turn_context, bool_t forced_relay) {
@@ -121,7 +130,7 @@ static void check_turn_context_statistics(MSTurnContext *turn_context, bool_t fo
 	}
 }
 
-static void ice_turn_call_base(bool_t video_enabled, bool_t forced_relay, bool_t caller_turn_enabled, bool_t callee_turn_enabled, bool_t rtcp_mux_enabled) {
+static void ice_turn_call_base(bool_t video_enabled, bool_t forced_relay, bool_t caller_turn_enabled, bool_t callee_turn_enabled, bool_t rtcp_mux_enabled, bool_t ipv6) {
 	LinphoneCoreManager *marie;
 	LinphoneCoreManager *pauline;
 	LinphoneCall *lcall;
@@ -129,11 +138,19 @@ static void ice_turn_call_base(bool_t video_enabled, bool_t forced_relay, bool_t
 	LinphoneMediaDirection expected_video_dir = LinphoneMediaDirectionInactive;
 	bctbx_list_t *lcs = NULL;
 
-	marie = linphone_core_manager_new("marie_rc");
+	marie = linphone_core_manager_new2("marie_rc", FALSE);
 	lcs = bctbx_list_append(lcs, marie->lc);
-	pauline = linphone_core_manager_new(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc");
+	pauline = linphone_core_manager_new2(transport_supported(LinphoneTransportTls) ? "pauline_rc" : "pauline_tcp_rc", FALSE);
 	lcs = bctbx_list_append(lcs, pauline->lc);
 
+	if (ipv6) {
+		linphone_core_enable_ipv6(marie->lc, TRUE);
+		linphone_core_enable_ipv6(pauline->lc, TRUE);
+	} else {
+		linphone_core_enable_ipv6(marie->lc, FALSE);
+		linphone_core_enable_ipv6(pauline->lc, FALSE);
+	}
+
 	configure_nat_policy(marie->lc, caller_turn_enabled);
 	configure_nat_policy(pauline->lc, callee_turn_enabled);
 	if (forced_relay == TRUE) {
@@ -148,8 +165,13 @@ static void ice_turn_call_base(bool_t video_enabled, bool_t forced_relay, bool_t
 		lp_config_set_int(linphone_core_get_config(pauline->lc), "rtp", "rtcp_mux", 1);
 	}
 
+	linphone_core_manager_start(marie, TRUE);
+	linphone_core_manager_start(pauline, TRUE);
+
 	if (video_enabled) {
 #ifdef VIDEO_ENABLED
+		linphone_core_set_video_device(pauline->lc,liblinphone_tester_mire_id);
+		linphone_core_set_video_device(marie->lc,liblinphone_tester_mire_id);
 		video_call_base_2(marie, pauline, FALSE, LinphoneMediaEncryptionNone, TRUE, TRUE);
 		expected_video_dir = LinphoneMediaDirectionSendRecv;
 #endif
@@ -187,31 +209,39 @@ static void ice_turn_call_base(bool_t video_enabled, bool_t forced_relay, bool_t
 }
 
 static void basic_ice_turn_call(void) {
-	ice_turn_call_base(FALSE, FALSE, TRUE, TRUE, FALSE);
+	ice_turn_call_base(FALSE, FALSE, TRUE, TRUE, FALSE, FALSE);
+}
+
+static void basic_ipv6_ice_turn_call(void) {
+	if (liblinphone_tester_ipv6_available()) {
+		ice_turn_call_base(FALSE, FALSE, TRUE, TRUE, FALSE, TRUE);
+	} else {
+		ms_warning("Test skipped, no ipv6 available");
+	}
 }
 
 #ifdef VIDEO_ENABLED
 static void video_ice_turn_call(void) {
-	ice_turn_call_base(TRUE, FALSE, TRUE, TRUE, FALSE);
+	ice_turn_call_base(TRUE, FALSE, TRUE, TRUE, FALSE, FALSE);
 }
 #endif
 
 static void relayed_ice_turn_call(void) {
-	ice_turn_call_base(FALSE, TRUE, TRUE, TRUE, FALSE);
+	ice_turn_call_base(FALSE, TRUE, TRUE, TRUE, FALSE, FALSE);
 }
 
 #ifdef VIDEO_ENABLED
 static void relayed_video_ice_turn_call(void) {
-	ice_turn_call_base(TRUE, TRUE, TRUE, TRUE, FALSE);
+	ice_turn_call_base(TRUE, TRUE, TRUE, TRUE, FALSE, FALSE);
 }
 #endif
 
 static void relayed_ice_turn_call_with_rtcp_mux(void) {
-	ice_turn_call_base(FALSE, TRUE, TRUE, TRUE, TRUE);
+	ice_turn_call_base(FALSE, TRUE, TRUE, TRUE, TRUE, FALSE);
 }
 
 static void relayed_ice_turn_to_ice_stun_call(void) {
-	ice_turn_call_base(FALSE, TRUE, TRUE, FALSE, FALSE);
+	ice_turn_call_base(FALSE, TRUE, TRUE, FALSE, FALSE, FALSE);
 }
 
 
@@ -219,6 +249,7 @@ test_t stun_tests[] = {
 	TEST_ONE_TAG("Basic Stun test (Ping/public IP)", linphone_stun_test_grab_ip, "STUN"),
 	TEST_ONE_TAG("STUN encode", linphone_stun_test_encode, "STUN"),
 	TEST_TWO_TAGS("Basic ICE+TURN call", basic_ice_turn_call, "ICE", "TURN"),
+	TEST_TWO_TAGS("Basic IPv6 ICE+TURN call", basic_ipv6_ice_turn_call, "ICE", "TURN"),
 #ifdef VIDEO_ENABLED
 	TEST_TWO_TAGS("Video ICE+TURN call", video_ice_turn_call, "ICE", "TURN"),
 	TEST_TWO_TAGS("Relayed video ICE+TURN call", relayed_video_ice_turn_call, "ICE", "TURN"),
