@@ -17,7 +17,8 @@
  */
 
 #include <stdio.h>
-#include "linphonecore.h"
+#include <stdlib.h>
+#include "linphone/core.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 #include <bctoolbox/tester.h>
@@ -45,6 +46,7 @@
 static int liblinphone_tester_keep_accounts_flag = 0;
 static int liblinphone_tester_keep_record_files = FALSE;
 static int liblinphone_tester_leak_detector_disabled = FALSE;
+bool_t liblinphone_tester_tls_support_disabled = FALSE;
 int manager_count = 0;
 int leaked_objects_count = 0;
 const MSAudioDiffParams audio_cmp_params = {10,2000};
@@ -55,7 +57,11 @@ const char* test_username="liblinphone_tester";
 const char* test_password="secret";
 const char* test_route="sip2.linphone.org";
 const char *userhostsfile = "tester_hosts";
-bool_t liblinphonetester_ipv6 = FALSE;
+bool_t liblinphonetester_ipv6 = TRUE;
+bool_t liblinphonetester_show_account_manager_logs = FALSE;
+int liblinphonetester_transport_timeout = 9000; /*milliseconds. it is set to such low value to workaround a problem with our Freebox v6 when connecting to Ipv6 addresses.
+			It was found that the freebox sometimes block SYN-ACK packets, which prevents connection to be succesful.
+			Thanks to the timeout, it will fallback to IPv4*/
 
 const char *liblinphone_tester_mire_id="Mire: Mire (synthetic moving picture)";
 
@@ -106,7 +112,7 @@ static void auth_info_requested(LinphoneCore *lc, const char *realm, const char
 
 void reset_counters( stats* counters) {
 	if (counters->last_received_chat_message) linphone_chat_message_unref(counters->last_received_chat_message);
-	if (counters->last_received_info_message) linphone_info_message_destroy(counters->last_received_info_message);
+	if (counters->last_received_info_message) linphone_info_message_unref(counters->last_received_info_message);
 	memset(counters,0,sizeof(stats));
 }
 
@@ -137,7 +143,7 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 	ringbackpath     = ms_strdup_printf("%s/sounds/ringback.wav", path);
 	nowebcampath     = ms_strdup_printf("%s/images/nowebcamCIF.jpg", path);
 	rootcapath       = ms_strdup_printf("%s/certificates/cn/cafile.pem", path);
-	dnsuserhostspath = ms_strdup_printf("%s/%s", path, userhostsfile);
+	dnsuserhostspath = userhostsfile[0]=='/' ? ms_strdup(userhostsfile) : ms_strdup_printf("%s/%s", path, userhostsfile);
 
 
 	if( config != NULL ) {
@@ -155,6 +161,7 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 	chatdb = ms_strdup_printf("%s/messages-%p.db",bc_tester_get_writable_dir_prefix(),lc);
 
 	linphone_core_enable_ipv6(lc, liblinphonetester_ipv6);
+	linphone_core_set_sip_transport_timeout(lc, liblinphonetester_transport_timeout);
 
 	sal_enable_test_features(lc->sal,TRUE);
 	sal_set_dns_user_hosts_file(lc->sal, dnsuserhostspath);
@@ -169,6 +176,7 @@ LinphoneCore* configure_lc_from(LinphoneCoreVTable* v_table, const char* path, c
 	ms_free(nowebcampath);
 	ms_free(rootcapath);
 	ms_free(dnsuserhostspath);
+	remove(chatdb);
 	ms_free(chatdb);
 
 	if( filepath ) ms_free(filepath);
@@ -262,11 +270,15 @@ LinphoneCoreManager *get_manager(LinphoneCore *lc){
 }
 
 bool_t transport_supported(LinphoneTransportType transport) {
-	Sal *sal = sal_init(NULL);
-	bool_t supported = sal_transport_available(sal,(SalTransport)transport);
-	if (!supported) ms_message("TLS transport not supported, falling back to TCP if possible otherwise skipping test.");
-	sal_uninit(sal);
-	return supported;
+	if ((transport == LinphoneTransportDtls || transport == LinphoneTransportTls) && liblinphone_tester_tls_support_disabled == TRUE) {
+		return FALSE;
+	} else {
+		Sal *sal = sal_init(NULL);
+		bool_t supported = sal_transport_available(sal,(SalTransport)transport);
+		if (!supported) ms_message("TLS transport not supported, falling back to TCP if possible otherwise skipping test.");
+		sal_uninit(sal);
+		return  supported;
+	}
 }
 
 #if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
@@ -277,10 +289,12 @@ bool_t transport_supported(LinphoneTransportType transport) {
 #else
 #pragma GCC diagnostic ignored "-Wdeprecated-declarations"
 #endif
-void linphone_core_manager_init(LinphoneCoreManager *mgr, const char* rc_file) {
+void linphone_core_manager_init(LinphoneCoreManager *mgr, const char* rc_file, const char* phone_alias) {
+	LinphoneImNotifPolicy *im_notif_policy;
 	char *rc_path = NULL;
 	char *hellopath = bc_tester_res("sounds/hello8000.wav");
-	mgr->number_of_cunit_error_at_creation =  bc_get_number_of_failures();
+
+	mgr->number_of_bcunit_error_at_creation =  bc_get_number_of_failures();
 	mgr->v_table.registration_state_changed=registration_state_changed;
 	mgr->v_table.auth_info_requested=auth_info_requested;
 	mgr->v_table.call_state_changed=call_state_changed;
@@ -289,6 +303,7 @@ void linphone_core_manager_init(LinphoneCoreManager *mgr, const char* rc_file) {
 	mgr->v_table.is_composing_received=is_composing_received;
 	mgr->v_table.new_subscription_requested=new_subscription_requested;
 	mgr->v_table.notify_presence_received=notify_presence_received;
+	mgr->v_table.notify_presence_received_for_uri_or_tel=notify_presence_received_for_uri_or_tel;
 	mgr->v_table.transfer_state_changed=linphone_transfer_state_changed;
 	mgr->v_table.info_received=info_message_received;
 	mgr->v_table.subscription_state_changed=linphone_subscription_state_change;
@@ -300,10 +315,19 @@ void linphone_core_manager_init(LinphoneCoreManager *mgr, const char* rc_file) {
 	mgr->v_table.dtmf_received=dtmf_received;
 	mgr->v_table.call_stats_updated=call_stats_updated;
 
+	mgr->phone_alias = phone_alias ? ms_strdup(phone_alias) : NULL;
+
 	reset_counters(&mgr->stat);
 	if (rc_file) rc_path = ms_strdup_printf("rcfiles/%s", rc_file);
 	mgr->lc=configure_lc_from(&mgr->v_table, bc_tester_get_resource_dir_prefix(), rc_path, mgr);
 	linphone_core_manager_check_accounts(mgr);
+	im_notif_policy = linphone_core_get_im_notif_policy(mgr->lc);
+	if (im_notif_policy != NULL) {
+		/* The IM notification policy can be NULL at this point in case of remote provisioning. */
+		linphone_im_notif_policy_clear(im_notif_policy);
+		linphone_im_notif_policy_set_send_is_composing(im_notif_policy, TRUE);
+		linphone_im_notif_policy_set_recv_is_composing(im_notif_policy, TRUE);
+	}
 
 	manager_count++;
 
@@ -358,7 +382,7 @@ void linphone_core_manager_start(LinphoneCoreManager *mgr, int check_for_proxies
 
 	/*BC_ASSERT_EQUAL(bctbx_list_size(linphone_core_get_proxy_config_list(lc)),proxy_count, int, "%d");*/
 	if (check_for_proxies){ /**/
-		proxy_count=bctbx_list_size(linphone_core_get_proxy_config_list(mgr->lc));
+		proxy_count=(int)bctbx_list_size(linphone_core_get_proxy_config_list(mgr->lc));
 	}else{
 		proxy_count=0;
 		/*this is to prevent registration to go on*/
@@ -379,7 +403,7 @@ void linphone_core_manager_start(LinphoneCoreManager *mgr, int check_for_proxies
 	proxy = linphone_core_get_default_proxy_config(mgr->lc);
 	if (proxy) {
 		if (mgr->identity){
-			linphone_address_destroy(mgr->identity);
+			linphone_address_unref(mgr->identity);
 		}
 		mgr->identity = linphone_address_clone(linphone_proxy_config_get_identity_address(proxy));
 		linphone_address_clean(mgr->identity);
@@ -390,55 +414,67 @@ void linphone_core_manager_start(LinphoneCoreManager *mgr, int check_for_proxies
 		/*now that stun server resolution is done, we can start registering*/
 		linphone_core_set_network_reachable(mgr->lc, TRUE);
 	}
+
 }
 
-LinphoneCoreManager* linphone_core_manager_new( const char* rc_file) {
+LinphoneCoreManager* linphone_core_manager_new3(const char* rc_file, int check_for_proxies, const char* phone_alias) {
 	LinphoneCoreManager *manager = ms_new0(LinphoneCoreManager, 1);
-	linphone_core_manager_init(manager, rc_file);
-	linphone_core_manager_start(manager, TRUE);
-	return manager;
-}
 
-LinphoneCoreManager* linphone_core_manager_new2(const char* rc_file, int check_for_proxies) {
-	LinphoneCoreManager *manager = ms_new0(LinphoneCoreManager, 1);
-	linphone_core_manager_init(manager, rc_file);
+	linphone_core_manager_init(manager, rc_file, phone_alias);
 	linphone_core_manager_start(manager, check_for_proxies);
 	return manager;
 }
 
+LinphoneCoreManager* linphone_core_manager_new2(const char* rc_file, int check_for_proxies) {
+	return linphone_core_manager_new3(rc_file, check_for_proxies, NULL);
+}
+
+LinphoneCoreManager* linphone_core_manager_new( const char* rc_file) {
+	return linphone_core_manager_new2(rc_file, TRUE);
+}
+
+
 void linphone_core_manager_stop(LinphoneCoreManager *mgr){
 	if (mgr->lc) {
-		linphone_core_destroy(mgr->lc);
-		mgr->lc=NULL;
+		const char *record_file = linphone_core_get_record_file(mgr->lc);
+		char *chatdb = ms_strdup(linphone_core_get_chat_database_path(mgr->lc));
+		if (!liblinphone_tester_keep_record_files && record_file && ortp_file_exist(record_file)==0) {
+			if ((bc_get_number_of_failures() - mgr->number_of_bcunit_error_at_creation)>0) {
+				ms_error("Test has failed, keeping recorded file [%s]", record_file);
+			}
+			else {
+				unlink(record_file);
+			}
+		}
+		linphone_core_unref(mgr->lc);
+		if (chatdb) {
+			if (ortp_file_exist(chatdb)==0) {
+				if (unlink(chatdb) != 0){
+					ms_error("Could not delete %s: %s", chatdb, strerror(errno));
+				}
+			}
+			ms_free(chatdb);
+		}
+		mgr->lc = NULL;
 	}
 }
 
 void linphone_core_manager_uninit(LinphoneCoreManager *mgr) {
+	int old_log_level = ortp_get_log_level_mask(NULL);
+	linphone_core_set_log_level(ORTP_ERROR);
+	if (mgr->phone_alias) {
+		ms_free(mgr->phone_alias);
+	}
 	if (mgr->stat.last_received_chat_message) {
 		linphone_chat_message_unref(mgr->stat.last_received_chat_message);
 	}
-	if (mgr->stat.last_received_info_message) linphone_info_message_destroy(mgr->stat.last_received_info_message);
-	if (mgr->lc){
-		const char *record_file=linphone_core_get_record_file(mgr->lc);
-		char *chatdb = ms_strdup(linphone_core_get_chat_database_path(mgr->lc));
-		if (!liblinphone_tester_keep_record_files && record_file){
-			if ((bc_get_number_of_failures()-mgr->number_of_cunit_error_at_creation)>0) {
-				ms_message ("Test has failed, keeping recorded file [%s]",record_file);
-			} else {
-				unlink(record_file);
-			}
-		}
-		linphone_core_destroy(mgr->lc);
-		if (chatdb) {
-			unlink(chatdb);
-			ms_free(chatdb);
-		}
-	}
+	if (mgr->stat.last_received_info_message) linphone_info_message_unref(mgr->stat.last_received_info_message);
 	if (mgr->identity) {
-		linphone_address_destroy(mgr->identity);
+		linphone_address_unref(mgr->identity);
 	}
 
 	manager_count--;
+	linphone_core_set_log_level(old_log_level);
 }
 
 void linphone_core_manager_wait_for_stun_resolution(LinphoneCoreManager *mgr) {
@@ -452,24 +488,27 @@ void linphone_core_manager_wait_for_stun_resolution(LinphoneCoreManager *mgr) {
 }
 
 void linphone_core_manager_destroy(LinphoneCoreManager* mgr) {
+	linphone_core_manager_stop(mgr);
 	linphone_core_manager_uninit(mgr);
 	ms_free(mgr);
 }
 
 int liblinphone_tester_ipv6_available(void){
-	struct addrinfo *ai=bctbx_ip_address_to_addrinfo(AF_INET6,SOCK_STREAM,"2a01:e00::2",53);
-	if (ai){
-		struct sockaddr_storage ss;
-		struct addrinfo src;
-		socklen_t slen=sizeof(ss);
-		char localip[128];
-		int port=0;
-		belle_sip_get_src_addr_for(ai->ai_addr,(socklen_t)ai->ai_addrlen,(struct sockaddr*) &ss,&slen,4444);
-		src.ai_addr=(struct sockaddr*) &ss;
-		src.ai_addrlen=slen;
-		bctbx_addrinfo_to_ip_address(&src,localip, sizeof(localip),&port);
-		freeaddrinfo(ai);
-		return strcmp(localip,"::1")!=0;
+	if (liblinphonetester_ipv6) {
+		struct addrinfo *ai=bctbx_ip_address_to_addrinfo(AF_INET6,SOCK_STREAM,"2a01:e00::2",53);
+		if (ai){
+			struct sockaddr_storage ss;
+			struct addrinfo src;
+			socklen_t slen=sizeof(ss);
+			char localip[128];
+			int port=0;
+			belle_sip_get_src_addr_for(ai->ai_addr,(socklen_t)ai->ai_addrlen,(struct sockaddr*) &ss,&slen,4444);
+			src.ai_addr=(struct sockaddr*) &ss;
+			src.ai_addrlen=slen;
+			bctbx_addrinfo_to_ip_address(&src,localip, sizeof(localip),&port);
+			freeaddrinfo(ai);
+			return strcmp(localip,"::1")!=0;
+		}
 	}
 	return FALSE;
 }
@@ -521,6 +560,7 @@ void liblinphone_tester_add_suites() {
 	bc_tester_add_suite(&message_test_suite);
 	bc_tester_add_suite(&presence_test_suite);
 	bc_tester_add_suite(&presence_server_test_suite);
+	bc_tester_add_suite(&account_creator_test_suite);
 #ifdef UPNP
 	bc_tester_add_suite(&upnp_test_suite);
 #endif
@@ -588,8 +628,8 @@ void liblinphone_tester_before_each(void) {
 
 static char* all_leaks_buffer = NULL;
 
-int liblinphone_tester_after_each(void) {
-	int err = 0;
+void liblinphone_tester_after_each(void) {
+	linphone_factory_clean();
 	if (!liblinphone_tester_leak_detector_disabled){
 		int leaked_objects = belle_sip_object_get_object_count() - leaked_objects_count;
 		if (leaked_objects > 0) {
@@ -613,11 +653,9 @@ int liblinphone_tester_after_each(void) {
 			// if the test is NOT marked as leaking memory and it actually is, we should make it fail
 			if (!leaks_expected && leaked_objects > 0) {
 				BC_FAIL("This test is leaking memory!");
-				err = 1;
 				// and reciprocally
 			} else if (leaks_expected && leaked_objects == 0) {
 				BC_FAIL("This test is not leaking anymore, please remove LeaksMemory tag!");
-				// err = 1; // do not force fail actually, because it can be some false positive warning
 			}
 		}
 	}
@@ -625,7 +663,6 @@ int liblinphone_tester_after_each(void) {
 	if (manager_count != 0) {
 		ms_fatal("%d Linphone core managers are still alive!", manager_count);
 	}
-	return err;
 }
 
 void liblinphone_tester_uninit(void) {
@@ -640,6 +677,7 @@ void liblinphone_tester_uninit(void) {
 
 static void check_ice_from_rtp(LinphoneCall *c1, LinphoneCall *c2, LinphoneStreamType stream_type) {
 	MediaStream *ms;
+	LinphoneCallStats *stats;
 	switch (stream_type) {
 	case LinphoneStreamTypeAudio:
 		ms=&c1->audiostream->ms;
@@ -656,26 +694,42 @@ static void check_ice_from_rtp(LinphoneCall *c1, LinphoneCall *c2, LinphoneStrea
 		return;
 	}
 
+	stats = linphone_call_get_audio_stats(c1);
+	if (stats->ice_state == LinphoneIceStateHostConnection && media_stream_started(ms)) {
+		struct sockaddr_storage remaddr;
+		socklen_t remaddrlen = sizeof(remaddr);
+		char ip[NI_MAXHOST] = { 0 };
+		int port = 0;
+		SalMediaDescription *result_desc;
+		char *expected_addr = NULL;
 
-	if (linphone_call_get_audio_stats(c1)->ice_state == LinphoneIceStateHostConnection && media_stream_started(ms)) {
-		char ip[16];
-		char port[8];
-		getnameinfo((const struct sockaddr *)&c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addr
-					, c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addrlen
-					, ip
-					, sizeof(ip)
-					, port
-					, sizeof(port)
-					, NI_NUMERICHOST|NI_NUMERICSERV);
-		BC_ASSERT_STRING_EQUAL(ip, c2->media_localip);
+		const LinphoneCallParams *cp1 = linphone_call_get_current_params(c1);
+		const LinphoneCallParams *cp2 = linphone_call_get_current_params(c2);
+		if (cp1->update_call_when_ice_completed && cp2->update_call_when_ice_completed) {
+			memset(&remaddr, 0, remaddrlen);
+			result_desc = sal_call_get_final_media_description(c2->op);
+			expected_addr = result_desc->streams[0].rtp_addr;
+			if (expected_addr[0] == '\0') expected_addr = result_desc->addr;
+			if ((strchr(expected_addr, ':') == NULL) && (c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addr.ss_family == AF_INET6)) {
+				bctbx_sockaddr_ipv6_to_ipv4((struct sockaddr *)&c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addr, (struct sockaddr *)&remaddr, &remaddrlen);
+			} else {
+				memcpy(&remaddr, &c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addr, c1->audiostream->ms.sessions.rtp_session->rtp.gs.rem_addrlen);
+			}
+			bctbx_sockaddr_to_ip_address((struct sockaddr *)&remaddr, remaddrlen, ip, sizeof(ip), &port);
+
+			BC_ASSERT_STRING_EQUAL(ip, expected_addr);
+		}
 	}
+	linphone_call_stats_unref(stats);
 }
+
 bool_t check_ice(LinphoneCoreManager* caller, LinphoneCoreManager* callee, LinphoneIceState state) {
 	LinphoneCall *c1,*c2;
+	bool_t global_success = TRUE;
 	bool_t audio_success=FALSE;
 	bool_t video_success=FALSE;
 	bool_t text_success=FALSE;
-	bool_t video_enabled, realtime_text_enabled;
+	bool_t audio_enabled, video_enabled, realtime_text_enabled;
 	MSTimeSpec ts;
 
 	c1=linphone_core_get_current_call(caller->lc);
@@ -689,35 +743,46 @@ bool_t check_ice(LinphoneCoreManager* caller, LinphoneCoreManager* callee, Linph
 
 	BC_ASSERT_EQUAL(linphone_call_params_video_enabled(linphone_call_get_current_params(c1)),linphone_call_params_video_enabled(linphone_call_get_current_params(c2)), int, "%d");
 	BC_ASSERT_EQUAL(linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c1)),linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c2)), int, "%d");
+	audio_enabled=linphone_call_params_audio_enabled(linphone_call_get_current_params(c1));
 	video_enabled=linphone_call_params_video_enabled(linphone_call_get_current_params(c1));
 	realtime_text_enabled=linphone_call_params_realtime_text_enabled(linphone_call_get_current_params(c1));
-	liblinphone_tester_clock_start(&ts);
-	do{
-		if ((c1 != NULL) && (c2 != NULL)) {
-			if (linphone_call_get_audio_stats(c1)->ice_state==state &&
-				linphone_call_get_audio_stats(c2)->ice_state==state ){
-				audio_success=TRUE;
-				check_ice_from_rtp(c1,c2,LinphoneStreamTypeAudio);
-				check_ice_from_rtp(c2,c1,LinphoneStreamTypeAudio);
-				break;
+	if (audio_enabled) {
+		liblinphone_tester_clock_start(&ts);
+		do{
+			if ((c1 != NULL) && (c2 != NULL)) {
+				LinphoneCallStats *stats1 = linphone_call_get_audio_stats(c1);
+				LinphoneCallStats *stats2 = linphone_call_get_audio_stats(c2);
+				if (stats1->ice_state==state &&
+					stats2->ice_state==state){
+					audio_success=TRUE;
+					check_ice_from_rtp(c1,c2,LinphoneStreamTypeAudio);
+					check_ice_from_rtp(c2,c1,LinphoneStreamTypeAudio);
+					break;
+				}
+				linphone_call_stats_unref(stats1);
+				linphone_call_stats_unref(stats2);
+				linphone_core_iterate(caller->lc);
+				linphone_core_iterate(callee->lc);
 			}
-			linphone_core_iterate(caller->lc);
-			linphone_core_iterate(callee->lc);
-		}
-		ms_usleep(20000);
-	}while(!liblinphone_tester_clock_elapsed(&ts,10000));
+			ms_usleep(20000);
+		}while(!liblinphone_tester_clock_elapsed(&ts,10000));
+	}
 
 	if (video_enabled){
 		liblinphone_tester_clock_start(&ts);
 		do{
 			if ((c1 != NULL) && (c2 != NULL)) {
-				if (linphone_call_get_video_stats(c1)->ice_state==state &&
-					linphone_call_get_video_stats(c2)->ice_state==state ){
+				LinphoneCallStats *stats1 = linphone_call_get_video_stats(c1);
+				LinphoneCallStats *stats2 = linphone_call_get_video_stats(c2);
+				if (stats1->ice_state==state &&
+					stats2->ice_state==state){
 					video_success=TRUE;
 					check_ice_from_rtp(c1,c2,LinphoneStreamTypeVideo);
 					check_ice_from_rtp(c2,c1,LinphoneStreamTypeVideo);
 					break;
 				}
+				linphone_call_stats_unref(stats1);
+				linphone_call_stats_unref(stats2);
 				linphone_core_iterate(caller->lc);
 				linphone_core_iterate(callee->lc);
 			}
@@ -729,13 +794,17 @@ bool_t check_ice(LinphoneCoreManager* caller, LinphoneCoreManager* callee, Linph
 		liblinphone_tester_clock_start(&ts);
 		do{
 			if ((c1 != NULL) && (c2 != NULL)) {
-				if (linphone_call_get_text_stats(c1)->ice_state==state &&
-					linphone_call_get_text_stats(c2)->ice_state==state ){
+				LinphoneCallStats *stats1 = linphone_call_get_text_stats(c1);
+				LinphoneCallStats *stats2 = linphone_call_get_text_stats(c2);
+				if (stats1->ice_state==state &&
+					stats2->ice_state==state){
 					text_success=TRUE;
 					check_ice_from_rtp(c1,c2,LinphoneStreamTypeText);
 					check_ice_from_rtp(c2,c1,LinphoneStreamTypeText);
 					break;
 				}
+				linphone_call_stats_unref(stats1);
+				linphone_call_stats_unref(stats2);
 				linphone_core_iterate(caller->lc);
 				linphone_core_iterate(callee->lc);
 			}
@@ -754,16 +823,19 @@ bool_t check_ice(LinphoneCoreManager* caller, LinphoneCoreManager* callee, Linph
 	}
 	linphone_call_unref(c1);
 	linphone_call_unref(c2);
-	return video_enabled ? (realtime_text_enabled ? text_success && audio_success && video_success : audio_success && video_success) : realtime_text_enabled ? text_success && audio_success : audio_success;
+	if (audio_enabled) global_success = global_success && audio_success;
+	if (video_enabled) global_success = global_success && video_success;
+	if (realtime_text_enabled) global_success = global_success && text_success;
+	return global_success;
 }
 
 static void linphone_conference_server_call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCallState cstate, const char *msg) {
-	LinphoneCoreVTable *vtable = linphone_core_get_current_vtable(lc);
-	LinphoneConferenceServer *conf_srv = (LinphoneConferenceServer *)vtable->user_data;
+	LinphoneCoreCbs *cbs = linphone_core_get_current_callbacks(lc);
+	LinphoneConferenceServer *conf_srv = (LinphoneConferenceServer *)linphone_core_cbs_get_user_data(cbs);
 
 	switch(cstate) {
 		case LinphoneCallIncomingReceived:
-			linphone_core_accept_call(lc, call);
+			linphone_call_accept(call);
 			break;
 
 		case LinphoneCallStreamsRunning:
@@ -799,18 +871,18 @@ static void linphone_conference_server_refer_received(LinphoneCore *core, const
 		linphone_address_clean(refer_to_addr);
 		uri = linphone_address_as_string_uri_only(refer_to_addr);
 		call = linphone_core_find_call_from_uri(core, uri);
-		if(call) linphone_core_terminate_call(core, call);
+		if(call) linphone_call_terminate(call);
 		ms_free(uri);
 	}
-	linphone_address_destroy(refer_to_addr);
+	linphone_address_unref(refer_to_addr);
 }
 
 static void linphone_conference_server_registration_state_changed(LinphoneCore *core,
 																  LinphoneProxyConfig *cfg,
 																  LinphoneRegistrationState cstate,
 																  const char *message) {
-	LinphoneCoreVTable *vtable = linphone_core_get_current_vtable(core);
-	LinphoneConferenceServer *m = (LinphoneConferenceServer *)linphone_core_v_table_get_user_data(vtable);
+	LinphoneCoreCbs *cbs = linphone_core_get_current_callbacks(core);
+	LinphoneConferenceServer *m = (LinphoneConferenceServer *)linphone_core_cbs_get_user_data(cbs);
 	if(cfg == linphone_core_get_default_proxy_config(core)) {
 		m->reg_state = cstate;
 	}
@@ -819,21 +891,26 @@ static void linphone_conference_server_registration_state_changed(LinphoneCore *
 LinphoneConferenceServer* linphone_conference_server_new(const char *rc_file, bool_t do_registration) {
 	LinphoneConferenceServer *conf_srv = (LinphoneConferenceServer *)ms_new0(LinphoneConferenceServer, 1);
 	LinphoneCoreManager *lm = (LinphoneCoreManager *)conf_srv;
-
-	conf_srv->vtable = linphone_core_v_table_new();
-	conf_srv->vtable->call_state_changed = linphone_conference_server_call_state_changed;
-	conf_srv->vtable->refer_received = linphone_conference_server_refer_received;
-	conf_srv->vtable->registration_state_changed = linphone_conference_server_registration_state_changed;
-	conf_srv->vtable->user_data = conf_srv;
+	LinphoneProxyConfig *proxy;
+	conf_srv->cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	linphone_core_cbs_set_call_state_changed(conf_srv->cbs, linphone_conference_server_call_state_changed);
+	linphone_core_cbs_set_refer_received(conf_srv->cbs, linphone_conference_server_refer_received);
+	linphone_core_cbs_set_registration_state_changed(conf_srv->cbs, linphone_conference_server_registration_state_changed);
+	linphone_core_cbs_set_user_data(conf_srv->cbs, conf_srv);
 	conf_srv->reg_state = LinphoneRegistrationNone;
-	linphone_core_manager_init(lm, rc_file);
-	linphone_core_add_listener(lm->lc, conf_srv->vtable);
+	linphone_core_manager_init(lm, rc_file,NULL);
+	if (!do_registration) {
+		proxy = linphone_core_get_default_proxy_config(lm->lc);
+		linphone_proxy_config_edit(proxy);
+		linphone_proxy_config_enable_register(proxy,FALSE);
+		linphone_proxy_config_done(proxy);
+	}
+	linphone_core_add_callbacks(lm->lc, conf_srv->cbs);
 	linphone_core_manager_start(lm, do_registration);
 	return conf_srv;
 }
 
 void linphone_conference_server_destroy(LinphoneConferenceServer *conf_srv) {
-	linphone_core_manager_uninit((LinphoneCoreManager *)conf_srv);
-	linphone_core_v_table_destroy(conf_srv->vtable);
-	ms_free(conf_srv);
+	linphone_core_cbs_unref(conf_srv->cbs);
+	linphone_core_manager_destroy((LinphoneCoreManager *)conf_srv);
 }
