@@ -18,32 +18,27 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include "linphonecore.h"
-#include "lpconfig.h"
+#include "linphone/core.h"
+#include "linphone/lpconfig.h"
 #include "private.h"
 #include "liblinphone_tester.h"
 
 /* Retrieve the public IP from a given hostname */
-static const char* get_ip_from_hostname(const char * tunnel_hostname){
+int get_ip_from_hostname(const char * tunnel_hostname, char *ip, size_t ip_size){
 	struct addrinfo hints;
-	struct addrinfo *res = NULL, *it = NULL;
-	struct sockaddr_in *add;
-	char * output = NULL;
+	struct addrinfo *res = NULL;
 	int err;
 	memset(&hints, 0, sizeof(hints));
 	hints.ai_family = AF_UNSPEC;
 	hints.ai_socktype = SOCK_STREAM;
 	if ((err = getaddrinfo(tunnel_hostname, NULL, &hints, &res))){
 		ms_error("error while retrieving IP from %s: %s", tunnel_hostname, gai_strerror(err));
-		return NULL;
+		return err;
 	}
 
-	for (it=res; it!=NULL; it=it->ai_next){
-		add = (struct sockaddr_in *) it->ai_addr;
-		output = inet_ntoa( add->sin_addr );
-	}
+	bctbx_addrinfo_to_ip_address(res, ip, ip_size, NULL);
 	freeaddrinfo(res);
-	return output;
+	return err;
 }
 static char* get_public_contact_ip(LinphoneCore* lc)  {
 	const LinphoneAddress * contact = linphone_proxy_config_get_contact(linphone_core_get_default_proxy_config(lc));
@@ -52,7 +47,7 @@ static char* get_public_contact_ip(LinphoneCore* lc)  {
 }
 
 
-static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_sip, LinphoneMediaEncryption encryption, bool_t with_video_and_ice) {
+static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_sip, LinphoneMediaEncryption encryption, bool_t with_video_and_ice, bool_t dual_socket) {
 	if (linphone_core_tunnel_available()){
 		LinphoneCoreManager *pauline = linphone_core_manager_new( "pauline_rc");
 		LinphoneCoreManager *marie = linphone_core_manager_new( "marie_rc");
@@ -60,9 +55,10 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 		LinphoneProxyConfig *proxy = linphone_core_get_default_proxy_config(pauline->lc);
 		LinphoneAddress *server_addr = linphone_address_new(linphone_proxy_config_get_server_addr(proxy));
 		LinphoneAddress *route = linphone_address_new(linphone_proxy_config_get_route(proxy));
-		const char * tunnel_ip = get_ip_from_hostname("tunnel.linphone.org");
+		char  tunnel_ip[64];
 		char *public_ip, *public_ip2=NULL;
-
+		BC_ASSERT_FALSE(get_ip_from_hostname("tunnel.linphone.org",tunnel_ip,sizeof(tunnel_ip)));
+						
 		BC_ASSERT_TRUE(wait_for(pauline->lc,NULL,&pauline->stat.number_of_LinphoneRegistrationOk,1));
 		public_ip = get_public_contact_ip(pauline->lc);
 		BC_ASSERT_STRING_NOT_EQUAL(public_ip, tunnel_ip);
@@ -80,13 +76,6 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 				linphone_core_find_payload_type(pauline->lc, "speex", 16000, 1), TRUE);
 			linphone_core_enable_payload_type(pauline->lc,
 				linphone_core_find_payload_type(pauline->lc, "G722", 8000, 1), TRUE);
-			linphone_core_enable_payload_type(marie->lc,
-				linphone_core_find_payload_type(marie->lc, "speex", 32000, 1), TRUE);
-			linphone_core_enable_payload_type(marie->lc,
-				linphone_core_find_payload_type(marie->lc, "speex", 16000, 1), TRUE);
-			linphone_core_enable_payload_type(marie->lc,
-				linphone_core_find_payload_type(marie->lc, "G722", 8000, 1), TRUE);
-
 		}
 
 		if (tunnel_mode != LinphoneTunnelModeDisable){
@@ -95,7 +84,18 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 
 			linphone_tunnel_config_set_host(config, "tunnel.linphone.org");
 			linphone_tunnel_config_set_port(config, 443);
-			linphone_tunnel_config_set_remote_udp_mirror_port(config, 12345);
+			if (!dual_socket) {
+				linphone_tunnel_config_set_host(config, "tunnel.linphone.org");
+				linphone_tunnel_config_set_port(config, 443);
+				linphone_tunnel_config_set_remote_udp_mirror_port(config, 12345);
+			} else {
+				linphone_tunnel_config_set_host(config, "94.23.19.176");
+				linphone_tunnel_config_set_port(config, 4443);
+				linphone_tunnel_config_set_host2(config, "188.165.40.171");
+				linphone_tunnel_config_set_port2(config, 4443);
+				linphone_tunnel_config_set_remote_udp_mirror_port(config, -1);
+				linphone_tunnel_enable_dual_mode(tunnel, TRUE);
+			}
 			linphone_tunnel_add_server(tunnel, config);
 			linphone_tunnel_set_mode(tunnel, tunnel_mode);
 			linphone_tunnel_enable_sip(tunnel, with_sip);
@@ -118,7 +118,11 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 				*/
 				ms_free(public_ip);
 				public_ip = get_public_contact_ip(pauline->lc);
-				BC_ASSERT_STRING_EQUAL(public_ip, tunnel_ip);
+				if (!dual_socket) {
+					BC_ASSERT_STRING_EQUAL(public_ip, tunnel_ip);
+				} else {
+					BC_ASSERT_STRING_EQUAL(public_ip, "94.23.19.176");
+				}
 			} else {
 				public_ip2 = get_public_contact_ip(pauline->lc);
 				BC_ASSERT_STRING_EQUAL(public_ip, public_ip2);
@@ -143,23 +147,27 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 					LinphoneAddress *tmp = linphone_address_new(remote_contact);
 					BC_ASSERT_PTR_NOT_NULL(tmp);
 					if (tmp){
-						BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(tmp), tunnel_ip);
-						linphone_address_destroy(tmp);
+						if (!dual_socket) {
+							BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(tmp), tunnel_ip);
+						} else {
+							BC_ASSERT_STRING_EQUAL(linphone_address_get_domain(tmp), "94.23.19.176");
+						}
+						linphone_address_unref(tmp);
 					}
 				}
 			}
 		}
 #ifdef VIDEO_ENABLED
 		if (with_video_and_ice){
-			BC_ASSERT_TRUE(add_video(pauline, marie, TRUE));
+			BC_ASSERT_TRUE(request_video(pauline, marie, TRUE));
 		}
 #endif
 		end_call(pauline,marie);
 
 		ms_free(public_ip);
 		if(public_ip2 != NULL) ms_free(public_ip2);
-		linphone_address_destroy(server_addr);
-		linphone_address_destroy(route);
+		linphone_address_unref(server_addr);
+		linphone_address_unref(route);
 		linphone_core_manager_destroy(pauline);
 		linphone_core_manager_destroy(marie);
 	}else{
@@ -169,30 +177,30 @@ static void call_with_tunnel_base(LinphoneTunnelMode tunnel_mode, bool_t with_si
 
 
 static void call_with_tunnel(void) {
-	call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionNone, FALSE);
+	call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionNone, FALSE, FALSE);
 }
 
 static void call_with_tunnel_srtp(void) {
-	call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionSRTP, FALSE);
+	call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionSRTP, FALSE, FALSE);
 }
 
 static void call_with_tunnel_without_sip(void) {
-	call_with_tunnel_base(LinphoneTunnelModeEnable, FALSE, LinphoneMediaEncryptionNone, FALSE);
+	call_with_tunnel_base(LinphoneTunnelModeEnable, FALSE, LinphoneMediaEncryptionNone, FALSE, FALSE);
 }
 
 static void call_with_tunnel_auto(void) {
-	call_with_tunnel_base(LinphoneTunnelModeAuto, TRUE, LinphoneMediaEncryptionNone, FALSE);
+	call_with_tunnel_base(LinphoneTunnelModeAuto, TRUE, LinphoneMediaEncryptionNone, FALSE, FALSE);
 }
 
 static void call_with_tunnel_auto_without_sip_with_srtp(void) {
-	call_with_tunnel_base(LinphoneTunnelModeAuto, FALSE, LinphoneMediaEncryptionSRTP, FALSE);
+	call_with_tunnel_base(LinphoneTunnelModeAuto, FALSE, LinphoneMediaEncryptionSRTP, FALSE, FALSE);
 }
 
 #ifdef VIDEO_ENABLED
 
 static void full_tunnel_video_ice_call(void){
 	if (linphone_core_tunnel_available()){
-		call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionNone, TRUE);
+		call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionNone, TRUE, FALSE);
 	}else
 		ms_warning("Could not test %s because tunnel functionality is not available",__FUNCTION__);
 }
@@ -252,9 +260,10 @@ static void register_on_second_tunnel(void) {
 		LinphoneTunnel *tunnel = linphone_core_get_tunnel(pauline->lc);
 		LinphoneTunnelConfig *config1 = linphone_tunnel_config_new();
 		LinphoneTunnelConfig *config2 = linphone_tunnel_config_new();
-		const char * tunnel_ip = get_ip_from_hostname("tunnel.linphone.org");
+		char tunnel_ip[64];
 		char* public_ip;
-
+		
+		BC_ASSERT_FALSE(get_ip_from_hostname("tunnel.linphone.org",tunnel_ip,sizeof(tunnel_ip)));
 		linphone_tunnel_simulate_udp_loss(tunnel, TRUE);
 
 		// add a first tunnel config with an invalid port
@@ -291,6 +300,20 @@ static void register_on_second_tunnel(void) {
 	}
 }
 
+static void dual_socket_mode(void) {
+	if (linphone_core_tunnel_available())
+		call_with_tunnel_base(LinphoneTunnelModeEnable, FALSE, LinphoneMediaEncryptionNone, FALSE, TRUE);
+	else
+		ms_warning("Could not test %s because tunnel functionality is not available",__FUNCTION__);
+}
+
+static void dual_socket_mode_with_sip(void) {
+	if (linphone_core_tunnel_available())
+		call_with_tunnel_base(LinphoneTunnelModeEnable, TRUE, LinphoneMediaEncryptionNone, FALSE, TRUE);
+	else
+		ms_warning("Could not test %s because tunnel functionality is not available",__FUNCTION__);
+}
+
 test_t tunnel_tests[] = {
 	TEST_NO_TAG("Simple", call_with_tunnel),
 	TEST_NO_TAG("With SRTP", call_with_tunnel_srtp),
@@ -308,6 +331,8 @@ test_t tunnel_tests[] = {
 	TEST_NO_TAG("ZRTP ice video call", tunnel_zrtp_video_ice_call),
 #endif
 	TEST_NO_TAG("Register on second tunnel", register_on_second_tunnel),
+	TEST_NO_TAG("Dual socket mode", dual_socket_mode),
+	TEST_NO_TAG("Dual socket mode with SIP", dual_socket_mode_with_sip),
 };
 
 test_suite_t tunnel_test_suite = {"Tunnel", NULL, NULL, liblinphone_tester_before_each, liblinphone_tester_after_each,
