@@ -20,9 +20,9 @@
 #if defined(VIDEO_ENABLED)
 
 
-#include "linphonecore.h"
+#include "linphone/core.h"
 #include "liblinphone_tester.h"
-#include "lpconfig.h"
+#include "linphone/lpconfig.h"
 
 #if __clang__ || ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4)
 #pragma GCC diagnostic push
@@ -133,7 +133,7 @@ static void early_media_video_call_state_changed(LinphoneCore *lc, LinphoneCall
 			linphone_call_params_enable_video(params, TRUE);
 			linphone_call_params_set_audio_direction(params, LinphoneMediaDirectionSendOnly);
 			linphone_call_params_set_video_direction(params, LinphoneMediaDirectionRecvOnly);
-			linphone_core_accept_early_media_with_params(lc, call, params);
+			linphone_call_accept_early_media_with_params(call, params);
 			linphone_call_params_unref(params);
 			break;
 		default:
@@ -151,7 +151,7 @@ static void early_media_video_call_state_changed_with_inactive_audio(LinphoneCor
 			linphone_call_params_enable_video(params, TRUE);
 			linphone_call_params_set_audio_direction(params, LinphoneMediaDirectionInactive);
 			linphone_call_params_set_video_direction(params, LinphoneMediaDirectionRecvOnly);
-			linphone_core_accept_early_media_with_params(lc, call, params);
+			linphone_call_accept_early_media_with_params(call, params);
 			linphone_call_params_unref(params);
 			break;
 		default:
@@ -201,7 +201,7 @@ static bool_t video_call_with_params(LinphoneCoreManager* caller_mgr, LinphoneCo
 	}
 
 	if (automatically_accept == TRUE) {
-		linphone_core_accept_call_with_params(callee_mgr->lc, linphone_core_get_current_call(callee_mgr->lc), callee_params);
+		linphone_call_accept_with_params(linphone_core_get_current_call(callee_mgr->lc), callee_params);
 
 		BC_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &callee_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
 		BC_ASSERT_TRUE(wait_for(callee_mgr->lc, caller_mgr->lc, &caller_mgr->stat.number_of_LinphoneCallConnected, initial_callee.number_of_LinphoneCallConnected + 1));
@@ -211,11 +211,12 @@ static bool_t video_call_with_params(LinphoneCoreManager* caller_mgr, LinphoneCo
 	return result;
 }
 
-static LinphoneCallParams * _configure_for_video(LinphoneCoreManager *manager, LinphoneCoreCallStateChangedCb cb) {
+static LinphoneCallParams * _configure_for_video(LinphoneCoreManager *manager, LinphoneCoreCbsCallStateChangedCb cb) {
 	LinphoneCallParams *params;
-	LinphoneCoreVTable *vtable = linphone_core_v_table_new();
-	vtable->call_state_changed = cb;
-	linphone_core_add_listener(manager->lc, vtable);
+	LinphoneCoreCbs *cbs = linphone_factory_create_core_cbs(linphone_factory_get());
+	linphone_core_cbs_set_call_state_changed(cbs, cb);
+	linphone_core_add_callbacks(manager->lc, cbs);
+	linphone_core_cbs_unref(cbs);
 	linphone_core_set_video_device(manager->lc, "StaticImage: Static picture");
 	linphone_core_enable_video_capture(manager->lc, TRUE);
 	linphone_core_enable_video_display(manager->lc, TRUE);
@@ -341,7 +342,7 @@ static void two_incoming_early_media_video_calls_test(void) {
 			LinphoneCallParams *params = linphone_call_params_copy(linphone_call_get_current_params(call));
 			linphone_call_params_set_audio_direction(params, LinphoneMediaDirectionSendRecv);
 			linphone_call_params_set_video_direction(params, LinphoneMediaDirectionSendRecv);
-			linphone_core_accept_call_with_params(marie->lc, call, params);
+			linphone_call_accept_with_params(call, params);
 			linphone_call_params_unref(params);
 
 			/* Wait for 5s. */
@@ -413,25 +414,35 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	LinphoneCallParams *pauline_params;
 	LinphoneCallParams *marie1_params;
 	LinphoneCallParams *marie2_params;
-	LinphoneVideoPolicy pol;
+	LinphoneVideoActivationPolicy *pol = linphone_video_activation_policy_new();
 	LinphoneCall *marie1_call;
 	LinphoneCall *marie2_call;
 	LinphoneCall *pauline_call;
 	LinphoneInfoMessage *info;
 	int dummy = 0;
-	pol.automatically_accept = 1;
-	pol.automatically_initiate = 1;
+	pol->automatically_accept = TRUE;
+	pol->automatically_initiate = TRUE;
+	LinphoneRange *port_range = NULL;
 
 	linphone_core_enable_video_capture(pauline->lc, TRUE);
 	linphone_core_enable_video_display(pauline->lc, TRUE);
 	linphone_core_enable_video_capture(marie1->lc, TRUE);
 	linphone_core_enable_video_display(marie1->lc, TRUE);
-	linphone_core_set_video_policy(marie1->lc, &pol);
+	linphone_core_set_video_activation_policy(marie1->lc, pol);
 	linphone_core_enable_video_capture(marie2->lc, TRUE);
 	linphone_core_enable_video_display(marie2->lc, TRUE);
-	linphone_core_set_video_policy(marie2->lc, &pol);
+	linphone_core_set_video_activation_policy(marie2->lc, pol);
+	linphone_video_activation_policy_unref(pol);
 	linphone_core_set_audio_port_range(marie2->lc, 40200, 40300);
+	port_range = linphone_core_get_audio_ports_range(marie2->lc);
+	BC_ASSERT_EQUAL(port_range->min, 40200, int, "%i");
+	BC_ASSERT_EQUAL(port_range->max, 40300, int, "%i");
+	linphone_range_unref(port_range);
 	linphone_core_set_video_port_range(marie2->lc, 40400, 40500);
+	port_range = linphone_core_get_video_ports_range(marie2->lc);
+	BC_ASSERT_EQUAL(port_range->min, 40400, int, "%i");
+	BC_ASSERT_EQUAL(port_range->max, 40500, int, "%i");
+	linphone_range_unref(port_range);
 
 	lcs = bctbx_list_append(lcs,marie1->lc);
 	lcs = bctbx_list_append(lcs,marie2->lc);
@@ -444,7 +455,7 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	marie2_params = configure_for_early_media_video_receiving_with_inactive_audio(marie2);
 
 	linphone_core_invite_address_with_params(pauline->lc, marie1->identity, pauline_params);
-	linphone_call_params_destroy(pauline_params);
+	linphone_call_params_unref(pauline_params);
 
 	BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallIncomingReceived, 1, 3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs, &marie2->stat.number_of_LinphoneCallIncomingReceived, 1, 3000));
@@ -470,21 +481,35 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	BC_ASSERT_PTR_NOT_NULL(marie2_call);
 
 	if (pauline_call && marie1_call && marie2_call) {
+		LinphoneCallStats *pauline_audio_stats, *marie1_audio_stats, *marie2_audio_stats;
+		LinphoneCallStats *pauline_video_stats, *marie1_video_stats, *marie2_video_stats;
 		linphone_call_set_next_video_frame_decoded_callback(pauline_call, linphone_call_iframe_decoded_cb, pauline->lc);
 
 		/* wait a bit that streams are established */
 		wait_for_list(lcs, &dummy, 1, 3000);
-		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(pauline_call)->download_bandwidth, 0, float, "%f");
-		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(marie1_call)->download_bandwidth, 0, float, "%f");
-		BC_ASSERT_EQUAL(linphone_call_get_audio_stats(marie2_call)->download_bandwidth, 0, float, "%f");
-		BC_ASSERT_LOWER(linphone_call_get_video_stats(pauline_call)->download_bandwidth, 11, float, "%f"); /* because of stun packets*/
-		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie1_call)->download_bandwidth, 0, float, "%f");
-		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie2_call)->download_bandwidth, 0, float, "%f");
+		pauline_audio_stats = linphone_call_get_audio_stats(pauline_call);
+		marie1_audio_stats = linphone_call_get_audio_stats(marie1_call);
+		marie2_audio_stats = linphone_call_get_audio_stats(marie2_call);
+		pauline_video_stats = linphone_call_get_video_stats(pauline_call);
+		marie1_video_stats = linphone_call_get_video_stats(marie1_call);
+		marie2_video_stats = linphone_call_get_video_stats(marie2_call);
+		BC_ASSERT_EQUAL(pauline_audio_stats->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_EQUAL(marie1_audio_stats->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_EQUAL(marie2_audio_stats->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_LOWER(pauline_video_stats->download_bandwidth, 11, float, "%f"); /* because of stun packets*/
+		BC_ASSERT_GREATER(marie1_video_stats->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(marie2_video_stats->download_bandwidth, 0, float, "%f");
 		BC_ASSERT_GREATER(marie1->stat.number_of_IframeDecoded, 1, int, "%i");
 		BC_ASSERT_GREATER(marie2->stat.number_of_IframeDecoded, 1, int, "%i");
+		linphone_call_stats_unref(pauline_audio_stats);
+		linphone_call_stats_unref(marie1_audio_stats);
+		linphone_call_stats_unref(marie2_audio_stats);
+		linphone_call_stats_unref(pauline_video_stats);
+		linphone_call_stats_unref(marie1_video_stats);
+		linphone_call_stats_unref(marie2_video_stats);
 
 		linphone_call_params_set_audio_direction(marie1_params, LinphoneMediaDirectionSendRecv);
-		linphone_core_accept_call_with_params(marie1->lc, linphone_core_get_current_call(marie1->lc), marie1_params);
+		linphone_call_accept_with_params(linphone_core_get_current_call(marie1->lc), marie1_params);
 		BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
 		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallStreamsRunning, 1, 3000));
 
@@ -493,15 +518,24 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 
 		/*wait a bit that streams are established*/
 		wait_for_list(lcs, &dummy, 1, 3000);
-		BC_ASSERT_GREATER(linphone_call_get_audio_stats(pauline_call)->download_bandwidth, 71, float, "%f");
-		BC_ASSERT_GREATER(linphone_call_get_audio_stats(marie1_call)->download_bandwidth, 71, float, "%f");
-		BC_ASSERT_GREATER(linphone_call_get_video_stats(pauline_call)->download_bandwidth, 0, float, "%f");
-		BC_ASSERT_GREATER(linphone_call_get_video_stats(marie1_call)->download_bandwidth, 0, float, "%f");
+		pauline_audio_stats = linphone_call_get_audio_stats(pauline_call);
+		marie1_audio_stats = linphone_call_get_audio_stats(marie1_call);
+		pauline_video_stats = linphone_call_get_video_stats(pauline_call);
+		marie1_video_stats = linphone_call_get_video_stats(marie1_call);
+		BC_ASSERT_GREATER(pauline_audio_stats->download_bandwidth, 71, float, "%f");
+		BC_ASSERT_GREATER(marie1_audio_stats->download_bandwidth, 71, float, "%f");
+		BC_ASSERT_GREATER(pauline_video_stats->download_bandwidth, 0, float, "%f");
+		BC_ASSERT_GREATER(marie1_video_stats->download_bandwidth, 0, float, "%f");
 		BC_ASSERT_GREATER(pauline->stat.number_of_IframeDecoded, 1, int, "%i");
+		linphone_call_stats_unref(pauline_audio_stats);
+		linphone_call_stats_unref(marie1_audio_stats);
+		linphone_call_stats_unref(pauline_video_stats);
+		linphone_call_stats_unref(marie1_video_stats);
 
 		/* send an INFO in reverse side to check that dialogs are properly established */
 		info = linphone_core_create_info_message(marie1->lc);
 		linphone_call_send_info_message(marie1_call, info);
+		linphone_info_message_unref(info);
 		BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_inforeceived, 1, 3000));
 	}
 
@@ -509,8 +543,8 @@ static void forked_outgoing_early_media_video_call_with_inactive_audio_test(void
 	BC_ASSERT_TRUE(wait_for_list(lcs, &pauline->stat.number_of_LinphoneCallEnd, 1, 3000));
 	BC_ASSERT_TRUE(wait_for_list(lcs, &marie1->stat.number_of_LinphoneCallEnd, 1, 3000));
 
-	linphone_call_params_destroy(marie1_params);
-	linphone_call_params_destroy(marie2_params);
+	linphone_call_params_unref(marie1_params);
+	linphone_call_params_unref(marie2_params);
 	bctbx_list_free(lcs);
 	linphone_core_manager_destroy(marie1);
 	linphone_core_manager_destroy(marie2);
@@ -540,7 +574,7 @@ static void enable_disable_camera_after_camera_switches(void) {
 		ms_message("Switching from [%s] to [%s]", currentCamId, newCamId);
 		linphone_core_set_video_device(marie->lc, newCamId);
 		if(call != NULL) {
-			linphone_core_update_call(marie->lc, call, NULL);
+			linphone_call_update(call, NULL);
 		}
 		BC_ASSERT_STRING_EQUAL(newCamId,ms_web_cam_get_string_id(linphone_call_get_video_device(call)));
 		linphone_call_enable_camera(call,FALSE);
