@@ -15,15 +15,11 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
 
 
-#ifdef IN_LINPHONE
-#include "linphonecore.h"
-#else
-#include "linphone/linphonecore.h"
-#endif
+#include "linphone/core.h"
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -44,9 +40,6 @@ static void dump_call_logs(int signum){
 	dump_stats=TRUE;
 }
 
-#endif
-#ifndef PACKAGE_DATA_DIR
-	#define PACKAGE_DATA_DIR  '.'
 #endif
 /*
  * Call state notification callback
@@ -62,7 +55,7 @@ static void call_state_changed(LinphoneCore *lc, LinphoneCall *call, LinphoneCal
 			linphone_call_params_set_audio_direction(call_params,LinphoneMediaDirectionSendOnly);
 			linphone_call_params_set_video_direction(call_params,LinphoneMediaDirectionSendOnly);
 			linphone_core_accept_call_with_params(lc,call,call_params);
-			linphone_call_params_destroy(call_params);
+			linphone_call_params_unref(call_params);
 		break;
 		default:
 			break;
@@ -147,7 +140,11 @@ int main(int argc, char *argv[]){
 	linphone_core_set_use_files(lc,TRUE);
 	linphone_core_enable_echo_cancellation(lc, FALSE); /*no need for local echo cancellation when playing files*/
 	if (!media_file){
-		linphone_core_set_play_file(lc,PACKAGE_DATA_DIR "/sounds/linphone/hello16000.wav");
+		LinphoneFactory *factory = linphone_factory_get();
+		const char *sound_resources_dir = linphone_factory_get_sound_resources_dir(factory);
+		char *play_file = bctbx_strdup_printf("%s/hello16000.wav", sound_resources_dir);
+		linphone_core_set_play_file(lc, play_file);
+		bctbx_free(play_file);
 		linphone_core_set_preferred_framerate(lc,5);
 	}else{
 		PayloadType *pt = linphone_core_find_payload_type(lc, "opus", 48000, -1);
